"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setWebProperty = void 0;
var contextInfo_1 = require("../../lib/contextInfo");
var web_1 = require("../../lib/web");
/**
 * Adds or updates a key/value pair for a web's property bag
 * This uses JSOM to set a web's property. The REST API doesn't support this at the moment.
 */
exports.setWebProperty = function (key, value, indexed, siteUrl) {
    var encodedKey = "vti_x005f_indexedpropertykeys";
    var decodedKey = "vti_indexedpropertykeys";
    // Return a promise
    return new Promise(function (resolve, reject) {
        // Gets the request digest value of the web
        var getRequestDigest = function () {
            // Return a promise
            return new Promise(function (resolve, reject) {
                // See if the site url exists
                if (siteUrl) {
                    contextInfo_1.ContextInfo.getWeb(siteUrl).execute(function (context) {
                        // Resolve the request
                        resolve(context.GetContextWebInformation.FormDigestValue);
                    }, reject);
                }
                else {
                    // Resolve the request w/ the default digest value
                    resolve(contextInfo_1.ContextInfo.formDigestValue);
                }
            });
        };
        // Get the request digest for the web
        getRequestDigest().then(function (requestDigest) {
            // Convert the key to a base64 string
            var idxKeyValue = "";
            for (var i = 0; i < key.length; i++) {
                idxKeyValue += key[i] + "\x00";
            }
            idxKeyValue = btoa(idxKeyValue);
            // See if a value exists
            if (value) {
                // Add the property
                web_1.Web(siteUrl, { requestDigest: requestDigest }).AllProperties().add(key, value).execute(function () {
                    // See if we are indexing the property
                    if (indexed) {
                        // Get the properties
                        web_1.Web(siteUrl, { requestDigest: requestDigest }).AllProperties().execute(function (propBag) {
                            // See if the indexed keys doesn't contain this value
                            var idxKeys = propBag[encodedKey] || "";
                            if (idxKeys.indexOf(idxKeyValue) < 0) {
                                // Append the value
                                web_1.Web(siteUrl, { requestDigest: requestDigest }).AllProperties().add(decodedKey, idxKeys + idxKeyValue + "|").execute(function () {
                                    // Resolve the request
                                    resolve();
                                }, function () {
                                    // Reject the request
                                    reject("Error setting the indexed keys property.");
                                });
                            }
                            else {
                                // Resolve the request
                                resolve();
                            }
                        });
                    }
                    else {
                        // Resolve the request
                        resolve();
                    }
                }, function () {
                    // Reject the request
                    reject("Error adding the web property.");
                });
            }
            else {
                // Remove the property
                web_1.Web(siteUrl, { requestDigest: requestDigest }).AllProperties().remove(key).execute(function () {
                    // See if we are indexing the property
                    if (indexed) {
                        // Get the properties
                        web_1.Web(siteUrl, { requestDigest: requestDigest }).AllProperties().execute(function (propBag) {
                            // See if the indexed keys contains the value
                            var idxKeys = propBag[encodedKey] || "";
                            if (idxKeys.indexOf(idxKeyValue) >= 0) {
                                // Remove the value
                                web_1.Web(siteUrl, { requestDigest: requestDigest }).AllProperties().add(decodedKey, idxKeys.replace(idxKeyValue + "|", "")).execute(function () {
                                    // Resolve the request
                                    resolve();
                                }, function () {
                                    // Reject the request
                                    reject("Error setting the indexed keys property.");
                                });
                            }
                            else {
                                // Resolve the request
                                resolve();
                            }
                        });
                    }
                    else {
                        // Resolve the request
                        resolve();
                    }
                }, function () {
                    // Reject the request
                    reject("Error removing the web property.");
                });
            }
        }, function () {
            // Reject the request
            reject("Unable to get the context of this web.");
        });
    });
};
