/*! For license information please see gd-sprest.js.LICENSE.txt */
(function(){var __webpack_modules__={"./build/helper/executor.js":function(__unused_webpack_module,exports){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.Executor = void 0;\n/**\r\n * Executor\r\n * @param methodParams - An array of parameters to execute in order synchronously.\r\n * @param method - The method to execute for each method parameter provided.\r\n * @param onExecuted - An optional event executed after the method completes. If a promise is returned, the executor will wait until it\'s resolved.\r\n */\n\nfunction Executor(methodParams, method, onExecuted) {\n  var _this = this;\n\n  if (methodParams === void 0) {\n    methodParams = [];\n  }\n\n  var _resolve = null;\n  var _reject = null; // Method to execute the methods\n\n  var executeMethods = function executeMethods(idx) {\n    if (idx === void 0) {\n      idx = 0;\n    } // Execute the method and see if a promise is returned\n\n\n    var returnVal = idx < methodParams.length ? method(methodParams[idx]) : null;\n\n    if (returnVal && returnVal.then) {\n      // Wait for the method to complete\n      returnVal.then(function () {\n        var args = [];\n\n        for (var _i = 0; _i < arguments.length; _i++) {\n          args[_i] = arguments[_i];\n        } // See if the on executed event exists\n\n\n        if (onExecuted) {\n          // Execute the method and see if a promise is returned\n          var returnVal_1 = onExecuted.apply(_this, args);\n\n          if (returnVal_1 && returnVal_1.then) {\n            // Wait for the method to complete\n            returnVal_1.then(function () {\n              // Execute the next method\n              executeMethods(idx + 1);\n            });\n          } else {\n            // Execute the next method\n            executeMethods(idx + 1);\n          }\n        } else {\n          // Execute the next method\n          executeMethods(idx + 1);\n        }\n      }, _reject);\n    } // Else, see if additional methods need to be executed\n    else if (idx < methodParams.length) {\n      // Execute the next method\n      executeMethods(idx + 1);\n    } // Else, resolve the promise\n    else {\n      _resolve();\n    }\n  }; // Return a promise\n\n\n  return new Promise(function (resolve, reject) {\n    // Set the resolve reference\n    _resolve = resolve;\n    _reject = reject; // See if params exist\n\n    if (methodParams.length > 0) {\n      // Execute the methods\n      executeMethods();\n    } else {\n      // resolve the promise\n      _resolve();\n    }\n  });\n}\n\nexports.Executor = Executor;\n\n//# sourceURL=webpack://gd-sprest/./build/helper/executor.js?')},"./build/helper/fieldSchemaXML.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.FieldSchemaXML = void 0;\n\nvar lib_1 = __webpack_require__(/*! ../lib */ "./build/lib/index.js");\n\nvar __1 = __webpack_require__(/*! .. */ "./build/index.js");\n\nvar spCfg_1 = __webpack_require__(/*! ./spCfg */ "./build/helper/spCfg.js");\n/**\r\n * Field Schema XML\r\n * Helper class for generating the field schema xml\r\n */\n\n\nexports.FieldSchemaXML = function (fieldInfo, targetWebUrl) {\n  var _resolve = null;\n  var _reject = null; // Returns the schema xml for a boolean field.\n\n  var createBoolean = function createBoolean(fieldInfo, props) {\n    var schemaXml = null; // Set the field type\n\n    props["Type"] = "Boolean"; // Generate the schema\n\n    schemaXml = "<Field " + toString(props) + ">";\n\n    if (fieldInfo.defaultValue) {\n      schemaXml += "<Default>" + fieldInfo.defaultValue + "</Default>";\n    }\n\n    schemaXml += "</Field>"; // Resolve the request\n\n    _resolve(schemaXml);\n  }; // Returns the schema xml for a calculated field.\n\n\n  var createCalculated = function createCalculated(fieldInfo, props) {\n    var schemaXml = null; // Set the field type\n\n    props["Type"] = "Calculated"; // Set the result type\n\n    switch (fieldInfo.resultType) {\n      case __1.SPTypes.FieldResultType.Boolean:\n        props["ResultType"] = "Boolean";\n        break;\n\n      case __1.SPTypes.FieldResultType.Currency:\n        props["ResultType"] = "Currency";\n\n        if (fieldInfo.lcid > 0) {\n          props["LCID"] = fieldInfo.lcid;\n        }\n\n        break;\n\n      case __1.SPTypes.FieldResultType.DateOnly:\n        props["Format"] = "DateOnly";\n        props["ResultType"] = "DateTime";\n        break;\n\n      case __1.SPTypes.FieldResultType.DateTime:\n        props["Format"] = "DateTime";\n        props["ResultType"] = "DateTime";\n        break;\n\n      case __1.SPTypes.FieldResultType.Number:\n        props["ResultType"] = "Number";\n\n        if (fieldInfo.decimals >= 0) {\n          props["Decimals"] = fieldInfo.decimals;\n        }\n\n        if (fieldInfo.numberType == __1.SPTypes.FieldNumberType.Percentage) {\n          props["Percentage"] = "TRUE";\n        }\n\n        break;\n\n      default:\n        props["ResultType"] = "Text";\n        break;\n    } // Generate the schema\n\n\n    schemaXml = "<Field " + toString(props) + ">";\n\n    if (fieldInfo.formula) {\n      schemaXml += "<Formula>" + fieldInfo.formula + "</Formula>";\n    }\n\n    if (fieldInfo.fieldRefs) {\n      schemaXml += "<FieldRefs>";\n\n      for (var i = 0; i < fieldInfo.fieldRefs.length; i++) {\n        schemaXml += "<FieldRef Name=\\"" + fieldInfo.fieldRefs[i] + "\\" />";\n      }\n\n      schemaXml += "</FieldRefs>";\n    }\n\n    schemaXml += "</Field>"; // Resolve the request\n\n    _resolve(schemaXml);\n  }; // Returns the schema xml for a choice field.\n\n\n  var createChoice = function createChoice(fieldInfo, props) {\n    var schemaXml = null; // Set the field type\n\n    props["Type"] = fieldInfo.multi ? "MultiChoice" : "Choice"; // Set the fill in choice property\n\n    if (typeof fieldInfo.fillInChoice === "boolean") {\n      props["FillInChoice"] = fieldInfo.fillInChoice ? "TRUE" : "FALSE";\n    } // Set the result type\n\n\n    switch (fieldInfo.format) {\n      case __1.SPTypes.ChoiceFormatType.Dropdown:\n        props["Format"] = "Dropdown";\n        break;\n\n      case __1.SPTypes.ChoiceFormatType.RadioButtons:\n        props["Format"] = "RadioButtons";\n        break;\n    } // Generate the schema\n\n\n    schemaXml = "<Field " + toString(props) + ">";\n\n    if (fieldInfo.defaultValue) {\n      schemaXml += "<Default>" + fieldInfo.defaultValue + "</Default>";\n    }\n\n    if (fieldInfo.choices) {\n      schemaXml += "<CHOICES>";\n\n      for (var i = 0; i < fieldInfo.choices.length; i++) {\n        schemaXml += "<CHOICE>" + fieldInfo.choices[i] + "</CHOICE>";\n      }\n\n      schemaXml += "</CHOICES>";\n    }\n\n    schemaXml += "</Field>"; // Resolve the request\n\n    _resolve(schemaXml);\n  }; // Returns the schema xml for a currency field.\n\n\n  var createCurrency = function createCurrency(fieldInfo, props) {\n    var schemaXml = null; // Set the field type\n\n    props["Type"] = "Currency"; // Set the number properties\n\n    if (fieldInfo.decimals >= 0) {\n      props["Decimals"] = fieldInfo.decimals;\n    }\n\n    if (fieldInfo.lcid > 0) {\n      props["LCID"] = fieldInfo.lcid;\n    }\n\n    if (fieldInfo.max != null) {\n      props["Max"] = fieldInfo.max;\n    }\n\n    if (fieldInfo.min != null) {\n      props["Min"] = fieldInfo.min;\n    } // Generate the schema\n\n\n    schemaXml = "<Field " + toString(props) + " />"; // Resolve the request\n\n    _resolve(schemaXml);\n  }; // Returns the schema xml for a date field.\n\n\n  var createDate = function createDate(fieldInfo, props) {\n    var schemaXml = null; // Set the field type\n\n    props["Type"] = "DateTime"; // Set the date/time properties\n\n    props["Format"] = fieldInfo.format == __1.SPTypes.DateFormat.DateTime ? "DateTime" : "DateOnly"; // Set the date/time display\n\n    switch (fieldInfo.displayFormat) {\n      case __1.SPTypes.FriendlyDateFormat.Disabled:\n        props["FriendlyDisplayFormat"] = "Disabled";\n        break;\n\n      case __1.SPTypes.FriendlyDateFormat.Relative:\n        props["FriendlyDisplayFormat"] = "Relative";\n        break;\n\n      case __1.SPTypes.FriendlyDateFormat.Unspecified:\n        props["FriendlyDisplayFormat"] = "Unspecified";\n        break;\n    } // Generate the schema\n\n\n    schemaXml = "<Field " + toString(props) + ">";\n\n    if (fieldInfo.defaultToday) {\n      schemaXml += "<Default>[today]</Default>";\n    } else if (fieldInfo.defaultValue) {\n      schemaXml += "<Default>" + fieldInfo.defaultValue + "</Default>";\n    }\n\n    if (fieldInfo.defaultFormula) {\n      schemaXml += "<DefaultFormula>" + fieldInfo.defaultFormula + "</DefaultFormula>";\n    }\n\n    schemaXml += "</Field>"; // Resolve the request\n\n    _resolve(schemaXml);\n  }; // Returns the schema xml for a geolocation field.\n\n\n  var createGeolocation = function createGeolocation(fieldInfo, props) {\n    var schemaXml = null; // Set the field type\n\n    props["Type"] = "Geolocation"; // Generate the schema\n\n    schemaXml = "<Field " + toString(props) + " />"; // Resolve the request\n\n    _resolve(schemaXml);\n  }; // Returns the schema xml for a guid field.\n\n\n  var createGuid = function createGuid(fieldInfo, props) {\n    var schemaXml = null; // Set the field type\n\n    props["Type"] = "Guid"; // Generate the schema\n\n    schemaXml = "<Field " + toString(props) + " />"; // Resolve the request\n\n    _resolve(schemaXml);\n  }; // Returns the schema xml for a image field.\n\n\n  var createImage = function createImage(fieldInfo, props) {\n    var schemaXml = null; // Set the field type\n\n    props["Type"] = "Thumbnail"; // Generate the schema\n\n    schemaXml = "<Field " + toString(props) + " />"; // Resolve the request\n\n    _resolve(schemaXml);\n  }; // Returns the schema xml for a lookup field.\n\n\n  var createLookup = function createLookup(fieldInfo, props) {\n    // Set the field type\n    props["Type"] = fieldInfo.multi ? "LookupMulti" : "Lookup"; // Update the relationship behavior\n\n    switch (fieldInfo.relationshipBehavior) {\n      case __1.SPTypes.RelationshipDeleteBehaviorType.Cascade:\n        props["RelationshipDeleteBehavior"] = "Cascade";\n        break;\n\n      case __1.SPTypes.RelationshipDeleteBehaviorType.None:\n        props["RelationshipDeleteBehavior"] = "None";\n        break;\n\n      case __1.SPTypes.RelationshipDeleteBehaviorType.Restrict:\n        props["RelationshipDeleteBehavior"] = "Restrict";\n        break;\n    } // Set the lookup properties\n\n\n    if (fieldInfo.fieldRef) {\n      props["FieldRef"] = fieldInfo.fieldRef;\n    }\n\n    if (fieldInfo.multi) {\n      props["Mult"] = "TRUE";\n    }\n\n    props["ShowField"] = fieldInfo.showField || "Title"; // See if the lookup name exists\n\n    if (fieldInfo.listName) {\n      // Get the web containing the list\n      lib_1.Web(fieldInfo.webUrl || targetWebUrl, {\n        disableCache: true\n      }) // Get the list\n      .Lists(fieldInfo.listName) // Set the query\n      .query({\n        Expand: ["ParentWeb"]\n      }) // Execute the request\n      .execute(function (list) {\n        // Set the list and web ids\n        props["List"] = "{" + list.Id + "}";\n\n        if (fieldInfo.webUrl) {\n          props["WebId"] = list.ParentWeb.Id;\n        } // Resolve the request\n\n\n        _resolve("<Field " + toString(props) + " />");\n      }, function () {\n        // Error getting the lookup list\n        _reject("Error getting the lookup list.");\n      });\n    } else {\n      // Set the list id\n      props["List"] = "{" + fieldInfo.listId.replace(/[\\{\\}]/g, "") + "}"; // Resolve the request\n\n      _resolve("<Field " + toString(props) + " />");\n    }\n  }; // Returns the schema xml for a managed metadata field.\n\n\n  var createMMS = function createMMS(fieldInfo, props) {\n    // Create the value field\n    var valueProps = {\n      ID: "{" + lib_1.ContextInfo.generateGUID() + "}",\n      Name: fieldInfo.name + "_0",\n      StaticName: fieldInfo.name + "_0",\n      DisplayName: fieldInfo.title + " Value",\n      Type: "Note",\n      Hidden: "TRUE",\n      Required: "FALSE",\n      ShowInViewForms: "FALSE",\n      CanToggleHidden: "TRUE"\n    }; // Generate the value field schema xml\n\n    var schemaXmlValue = "<Field " + toString(valueProps) + " />"; // Set the mms properties\n\n    props["Type"] = "TaxonomyFieldType";\n    props["ShowField"] = "Term" + (fieldInfo.locale ? fieldInfo.locale.toString() : "1033"); // Generate the mms field schema xml\n\n    var schemaXml = ["<Field " + toString(props) + ">", "<Customization>", "<ArrayOfProperty>", "<Property>", "<Name>TextField</Name>", "<Value xmlns:q6=\\"http://www.w3.org/2001/XMLSchema\\" p4:type=\\"q6:string\\" xmlns:p4=\\"http://www.w3.org/2001/XMLSchema-instance\\">" + valueProps.ID + "</Value>", "</Property>", "</ArrayOfProperty>", "</Customization>", "</Field>"].join(""); // Resolve the request\n\n    _resolve([schemaXmlValue, schemaXml]);\n  }; // Returns the schema xml for a note field.\n\n\n  var createNote = function createNote(fieldInfo, props) {\n    var schemaXml = null; // Set the field type\n\n    props["Type"] = "Note"; // Set the note properties\n\n    if (fieldInfo.appendFl) {\n      props["AppendOnly"] = "TRUE";\n    }\n\n    if (fieldInfo.noteType == __1.SPTypes.FieldNoteType.EnhancedRichText || fieldInfo.noteType == __1.SPTypes.FieldNoteType.RichText) {\n      props["RichText"] = "TRUE";\n    }\n\n    if (fieldInfo.noteType == __1.SPTypes.FieldNoteType.EnhancedRichText) {\n      props["RichTextMode"] = "FullHtml";\n    }\n\n    if (fieldInfo.numberOfLines > 0) {\n      props["NumLines"] = fieldInfo.numberOfLines;\n    }\n\n    if (fieldInfo.unlimited) {\n      props["UnlimitedLengthInDocumentLibrary"] = "TRUE";\n    } // Generate the schema\n\n\n    schemaXml = "<Field " + toString(props) + " />"; // Resolve the request\n\n    _resolve(schemaXml);\n  }; // Returns the schema xml for a number field.\n\n\n  var createNumber = function createNumber(fieldInfo, props) {\n    var schemaXml = null; // Set the field type\n\n    props["Type"] = "Number"; // Set the number properties\n\n    if (fieldInfo.decimals >= 0) {\n      props["Decimals"] = fieldInfo.decimals;\n    }\n\n    if (fieldInfo.max != null) {\n      props["Max"] = fieldInfo.max;\n    }\n\n    if (fieldInfo.min != null) {\n      props["Min"] = fieldInfo.min;\n    }\n\n    if (fieldInfo.numberType == __1.SPTypes.FieldNumberType.Integer) {\n      props["Decimals"] = 0;\n    }\n\n    if (fieldInfo.numberType == __1.SPTypes.FieldNumberType.Percentage) {\n      props["Percentage"] = "TRUE";\n    } // Generate the schema\n\n\n    schemaXml = "<Field " + toString(props) + ">";\n\n    if (fieldInfo.defaultValue) {\n      schemaXml += "<Default>" + fieldInfo.defaultValue + "</Default>";\n    }\n\n    schemaXml += "</Field>"; // Resolve the request\n\n    _resolve(schemaXml);\n  }; // Returns the schema xml for a text field.\n\n\n  var createText = function createText(fieldInfo, props) {\n    var schemaXml = null; // Set the field type\n\n    props["Type"] = "Text"; // Set the number properties\n\n    if (fieldInfo.maxLength != null) {\n      props["MaxLength"] = fieldInfo.maxLength;\n    } // Generate the schema\n\n\n    schemaXml = "<Field " + toString(props) + ">";\n\n    if (fieldInfo.defaultValue) {\n      schemaXml += "<Default>" + fieldInfo.defaultValue + "</Default>";\n    }\n\n    schemaXml += "</Field>"; // Resolve the request\n\n    _resolve(schemaXml);\n  }; // Returns the schema xml for a url field.\n\n\n  var createUrl = function createUrl(fieldInfo, props) {\n    var schemaXml = null; // Set the field type\n\n    props["Type"] = "URL"; // Set the url properties\n\n    props["Format"] = fieldInfo.format == __1.SPTypes.UrlFormatType.Image ? "Image" : "Hyperlink"; // Generate the schema\n\n    schemaXml = "<Field " + toString(props) + " />"; // Resolve the request\n\n    _resolve(schemaXml);\n  }; // Returns the schema xml for a user field.\n\n\n  var createUser = function createUser(fieldInfo, props) {\n    var schemaXml = null; // Set the field type\n\n    props["Type"] = "User"; // Set the user properties\n\n    if (fieldInfo.multi) {\n      props["Mult"] = "TRUE";\n    }\n\n    if (fieldInfo.selectionMode != null) {\n      props["UserSelectionMode"] = fieldInfo.selectionMode;\n    }\n\n    if (fieldInfo.selectionScope != null) {\n      props["UserSelectionScope"] = fieldInfo.selectionScope;\n    }\n\n    if (fieldInfo.showField != null) {\n      props["ShowField"] = fieldInfo.showField;\n    } // Generate the schema\n\n\n    schemaXml = "<Field " + toString(props) + " />"; // Resolve the request\n\n    _resolve(schemaXml);\n  }; // Method to convert the properties to a string\n\n\n  var toString = function toString(props) {\n    var properties = ""; // Parse the properties\n\n    for (var key in props) {\n      var value = props[key]; // Add the property\n\n      properties += (properties ? " " : "") + key + "=\\"" + props[key] + "\\"";\n    } // Return the string value\n\n\n    return properties;\n  }; // Return a promise\n\n\n  return new Promise(function (resolve, reject) {\n    // Set the resolve/reject methods\n    _resolve = resolve;\n    _reject = reject; // See if the schema xml has been defined\n\n    if (fieldInfo.schemaXml) {\n      // Resolve the promise\n      resolve(fieldInfo.schemaXml);\n    } else {\n      // Set the base properties\n      var props = {};\n      props["ID"] = fieldInfo.id || "{" + lib_1.ContextInfo.generateGUID() + "}";\n      props["Name"] = fieldInfo.name;\n      props["StaticName"] = fieldInfo.name;\n      props["DisplayName"] = fieldInfo.title || fieldInfo.name; // Set the optional properties\n\n      if (typeof fieldInfo.allowDeletion !== "undefined") {\n        props["AllowDeletion"] = fieldInfo.allowDeletion ? "TRUE" : "FALSE";\n      }\n\n      if (typeof fieldInfo.customFormatter !== "undefined") {\n        props["CustomFormatter"] = JSON.stringify(fieldInfo.customFormatter).replace(/"/g, "&quot;");\n      }\n\n      if (typeof fieldInfo.description !== "undefined") {\n        props["Description"] = fieldInfo.description;\n      }\n\n      if (typeof fieldInfo.enforceUniqueValues !== "undefined") {\n        props["EnforceUniqueValues"] = fieldInfo.enforceUniqueValues ? "TRUE" : "FALSE";\n      }\n\n      if (typeof fieldInfo.group !== "undefined") {\n        props["Group"] = fieldInfo.group;\n      }\n\n      if (typeof fieldInfo.jslink !== "undefined") {\n        props["JSLink"] = fieldInfo.jslink;\n      }\n\n      if (typeof fieldInfo.hidden !== "undefined") {\n        props["Hidden"] = fieldInfo.hidden ? "TRUE" : "FALSE";\n      }\n\n      if (typeof fieldInfo.indexed !== "undefined") {\n        props["Indexed"] = fieldInfo.indexed ? "TRUE" : "FALSE";\n      }\n\n      if (typeof fieldInfo.readOnly !== "undefined") {\n        props["ReadOnly"] = fieldInfo.readOnly ? "TRUE" : "FALSE";\n      }\n\n      if (typeof fieldInfo.required !== "undefined") {\n        props["Required"] = fieldInfo.required ? "TRUE" : "FALSE";\n      }\n\n      if (typeof fieldInfo.showInDisplayForm !== "undefined") {\n        props["ShowInDisplayForm"] = fieldInfo.showInDisplayForm ? "TRUE" : "FALSE";\n      }\n\n      if (typeof fieldInfo.showInEditForm !== "undefined") {\n        props["ShowInEditForm"] = fieldInfo.showInEditForm ? "TRUE" : "FALSE";\n      }\n\n      if (typeof fieldInfo.showInListSettings !== "undefined") {\n        props["ShowInListSettings"] = fieldInfo.showInListSettings ? "TRUE" : "FALSE";\n      }\n\n      if (typeof fieldInfo.showInNewForm !== "undefined") {\n        props["ShowInNewForm"] = fieldInfo.showInNewForm ? "TRUE" : "FALSE";\n      }\n\n      if (typeof fieldInfo.showInViewForms !== "undefined") {\n        props["ShowInViewForms"] = fieldInfo.showInViewForms ? "TRUE" : "FALSE";\n      }\n\n      if (typeof fieldInfo.sortable !== "undefined") {\n        props["Sortable"] = fieldInfo.sortable ? "TRUE" : "FALSE";\n      } // Set the type\n\n\n      switch (fieldInfo.type) {\n        // Boolean\n        case spCfg_1.SPCfgFieldType.Boolean:\n          createBoolean(fieldInfo, props);\n          break;\n        // Calculated\n\n        case spCfg_1.SPCfgFieldType.Calculated:\n          createCalculated(fieldInfo, props);\n          break;\n        // Choice\n\n        case spCfg_1.SPCfgFieldType.Choice:\n          createChoice(fieldInfo, props);\n          break;\n        // Currency\n\n        case spCfg_1.SPCfgFieldType.Currency:\n          createCurrency(fieldInfo, props);\n          break;\n        // Date/Time\n\n        case spCfg_1.SPCfgFieldType.Date:\n          createDate(fieldInfo, props);\n          break;\n        // Geolocation\n\n        case spCfg_1.SPCfgFieldType.Geolocation:\n          createGeolocation(fieldInfo, props);\n          break;\n        // Guid\n\n        case spCfg_1.SPCfgFieldType.Guid:\n          createGuid(fieldInfo, props);\n          break;\n        // Image\n\n        case spCfg_1.SPCfgFieldType.Image:\n          createImage(fieldInfo, props);\n          break;\n        // Lookup\n\n        case spCfg_1.SPCfgFieldType.Lookup:\n          createLookup(fieldInfo, props);\n          break;\n        // MMS\n\n        case spCfg_1.SPCfgFieldType.MMS:\n          createMMS(fieldInfo, props);\n          break;\n        // Note\n\n        case spCfg_1.SPCfgFieldType.Note:\n          createNote(fieldInfo, props);\n          break;\n        // Number\n\n        case spCfg_1.SPCfgFieldType.Number:\n          createNumber(fieldInfo, props);\n          break;\n        // Text\n\n        case spCfg_1.SPCfgFieldType.Text:\n          createText(fieldInfo, props);\n          break;\n        // URL\n\n        case spCfg_1.SPCfgFieldType.Url:\n          createUrl(fieldInfo, props);\n          break;\n        // User\n\n        case spCfg_1.SPCfgFieldType.User:\n          createUser(fieldInfo, props);\n          break;\n        // Field type not supported\n\n        default:\n          // Create a text field by default\n          createText(fieldInfo, props);\n          break;\n      }\n    }\n  });\n};\n\n//# sourceURL=webpack://gd-sprest/./build/helper/fieldSchemaXML.js?')},"./build/helper/index.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nvar __createBinding = this && this.__createBinding || (Object.create ? function (o, m, k, k2) {\n  if (k2 === undefined) k2 = k;\n  Object.defineProperty(o, k2, {\n    enumerable: true,\n    get: function get() {\n      return m[k];\n    }\n  });\n} : function (o, m, k, k2) {\n  if (k2 === undefined) k2 = k;\n  o[k2] = m[k];\n});\n\nvar __exportStar = this && this.__exportStar || function (m, exports) {\n  for (var p in m) {\n    if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);\n  }\n};\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\n\n__exportStar(__webpack_require__(/*! ./executor */ "./build/helper/executor.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./fieldSchemaXML */ "./build/helper/fieldSchemaXML.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./jslink */ "./build/helper/jslink.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./listForm */ "./build/helper/listForm.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./listFormField */ "./build/helper/listFormField.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./methods */ "./build/helper/methods/index.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./ribbonLink */ "./build/helper/ribbonLink.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./sbLink */ "./build/helper/sbLink.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./sp */ "./build/helper/sp/index.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./spCfg */ "./build/helper/spCfg.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./taxonomy */ "./build/helper/taxonomy.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./webpart */ "./build/helper/webpart.js"), exports);\n\n//# sourceURL=webpack://gd-sprest/./build/helper/index.js?')},"./build/helper/jslink.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.JSLink = void 0;\n\nvar lib_1 = __webpack_require__(/*! ../lib */ "./build/lib/index.js");\n\nvar sptypes_1 = __webpack_require__(/*! ../sptypes */ "./build/sptypes/index.js");\n/**\r\n * JSLink Helper Methods\r\n */\n\n\nexports.JSLink = {\n  // Hide event flag\n  _hideEventFl: false,\n\n  /**\r\n   * Field to Method Mapper\r\n   * 1 - Display Form\r\n   * 2 - Edit Form\r\n   * 3 - New Form\r\n   * 4 - View\r\n   */\n  _fieldToMethodMapper: {\n    \'Attachments\': {\n      4: lib_1.ContextInfo.window["RenderFieldValueDefault"],\n      1: lib_1.ContextInfo.window["SPFieldAttachments_Default"],\n      2: lib_1.ContextInfo.window["SPFieldAttachments_Default"],\n      3: lib_1.ContextInfo.window["SPFieldAttachments_Default"]\n    },\n    \'Boolean\': {\n      4: lib_1.ContextInfo.window["RenderFieldValueDefault"],\n      1: lib_1.ContextInfo.window["SPField_FormDisplay_DefaultNoEncode"],\n      2: lib_1.ContextInfo.window["SPFieldBoolean_Edit"],\n      3: lib_1.ContextInfo.window["SPFieldBoolean_Edit"]\n    },\n    \'Currency\': {\n      4: lib_1.ContextInfo.window["RenderFieldValueDefault"],\n      1: lib_1.ContextInfo.window["SPField_FormDisplay_Default"],\n      2: lib_1.ContextInfo.window["SPFieldNumber_Edit"],\n      3: lib_1.ContextInfo.window["SPFieldNumber_Edit"]\n    },\n    \'Calculated\': {\n      4: lib_1.ContextInfo.window["RenderFieldValueDefault"],\n      1: lib_1.ContextInfo.window["SPField_FormDisplay_Default"],\n      2: lib_1.ContextInfo.window["SPField_FormDisplay_Empty"],\n      3: lib_1.ContextInfo.window["SPField_FormDisplay_Empty"]\n    },\n    \'Choice\': {\n      4: lib_1.ContextInfo.window["RenderFieldValueDefault"],\n      1: lib_1.ContextInfo.window["SPField_FormDisplay_Default"],\n      2: lib_1.ContextInfo.window["SPFieldChoice_Edit"],\n      3: lib_1.ContextInfo.window["SPFieldChoice_Edit"]\n    },\n    \'Computed\': {\n      4: lib_1.ContextInfo.window["RenderFieldValueDefault"],\n      1: lib_1.ContextInfo.window["SPField_FormDisplay_Default"],\n      2: lib_1.ContextInfo.window["SPField_FormDisplay_Default"],\n      3: lib_1.ContextInfo.window["SPField_FormDisplay_Default"]\n    },\n    \'DateTime\': {\n      4: lib_1.ContextInfo.window["RenderFieldValueDefault"],\n      1: lib_1.ContextInfo.window["SPFieldDateTime_Display"],\n      2: lib_1.ContextInfo.window["SPFieldDateTime_Edit"],\n      3: lib_1.ContextInfo.window["SPFieldDateTime_Edit"]\n    },\n    \'File\': {\n      4: lib_1.ContextInfo.window["RenderFieldValueDefault"],\n      1: lib_1.ContextInfo.window["SPFieldFile_Display"],\n      2: lib_1.ContextInfo.window["SPFieldFile_Edit"],\n      3: lib_1.ContextInfo.window["SPFieldFile_Edit"]\n    },\n    \'Integer\': {\n      4: lib_1.ContextInfo.window["RenderFieldValueDefault"],\n      1: lib_1.ContextInfo.window["SPField_FormDisplay_Default"],\n      2: lib_1.ContextInfo.window["SPFieldNumber_Edit"],\n      3: lib_1.ContextInfo.window["SPFieldNumber_Edit"]\n    },\n    \'Lookup\': {\n      4: lib_1.ContextInfo.window["RenderFieldValueDefault"],\n      1: lib_1.ContextInfo.window["SPFieldLookup_Display"],\n      2: lib_1.ContextInfo.window["SPFieldLookup_Edit"],\n      3: lib_1.ContextInfo.window["SPFieldLookup_Edit"]\n    },\n    \'LookupMulti\': {\n      4: lib_1.ContextInfo.window["RenderFieldValueDefault"],\n      1: lib_1.ContextInfo.window["SPFieldLookup_Display"],\n      2: lib_1.ContextInfo.window["SPFieldLookup_Edit"],\n      3: lib_1.ContextInfo.window["SPFieldLookup_Edit"]\n    },\n    \'MultiChoice\': {\n      4: lib_1.ContextInfo.window["RenderFieldValueDefault"],\n      1: lib_1.ContextInfo.window["SPField_FormDisplay_Default"],\n      2: lib_1.ContextInfo.window["SPFieldMultiChoice_Edit"],\n      3: lib_1.ContextInfo.window["SPFieldMultiChoice_Edit"]\n    },\n    \'Note\': {\n      4: lib_1.ContextInfo.window["RenderFieldValueDefault"],\n      1: lib_1.ContextInfo.window["SPFieldNote_Display"],\n      2: lib_1.ContextInfo.window["SPFieldNote_Edit"],\n      3: lib_1.ContextInfo.window["SPFieldNote_Edit"]\n    },\n    \'Number\': {\n      4: lib_1.ContextInfo.window["RenderFieldValueDefault"],\n      1: lib_1.ContextInfo.window["SPField_FormDisplay_Default"],\n      2: lib_1.ContextInfo.window["SPFieldNumber_Edit"],\n      3: lib_1.ContextInfo.window["SPFieldNumber_Edit"]\n    },\n    \'Text\': {\n      4: lib_1.ContextInfo.window["RenderFieldValueDefault"],\n      1: lib_1.ContextInfo.window["SPField_FormDisplay_Default"],\n      2: lib_1.ContextInfo.window["SPFieldText_Edit"],\n      3: lib_1.ContextInfo.window["SPFieldText_Edit"]\n    },\n    \'URL\': {\n      4: lib_1.ContextInfo.window["RenderFieldValueDefault"],\n      1: lib_1.ContextInfo.window["SPFieldUrl_Display"],\n      2: lib_1.ContextInfo.window["SPFieldUrl_Edit"],\n      3: lib_1.ContextInfo.window["SPFieldUrl_Edit"]\n    },\n    \'User\': {\n      4: lib_1.ContextInfo.window["RenderFieldValueDefault"],\n      1: lib_1.ContextInfo.window["SPFieldUser_Display"],\n      2: lib_1.ContextInfo.window["SPClientPeoplePickerCSRTemplate"],\n      3: lib_1.ContextInfo.window["SPClientPeoplePickerCSRTemplate"]\n    },\n    \'UserMulti\': {\n      4: lib_1.ContextInfo.window["RenderFieldValueDefault"],\n      1: lib_1.ContextInfo.window["SPFieldUserMulti_Display"],\n      2: lib_1.ContextInfo.window["SPClientPeoplePickerCSRTemplate"],\n      3: lib_1.ContextInfo.window["SPClientPeoplePickerCSRTemplate"]\n    }\n  },\n\n  /**\r\n   * Methods\r\n   */\n\n  /**\r\n   * Disables edit for the specified field.\r\n   * @param ctx - The client context.\r\n   * @param field - The field to disable edit.\r\n   * @param requireValueFl - Flag to only disable the field, if a value exists.\r\n   */\n  disableEdit: function disableEdit(ctx, field, requireValueFl) {\n    var fieldValue = ctx.CurrentFieldValue; // Ensure a value exists\n\n    if (fieldValue) {\n      // Update the context, based on the field type\n      switch (ctx.CurrentFieldSchema.Type) {\n        case "MultiChoice":\n          var regExp = new RegExp(sptypes_1.SPTypes.ClientTemplatesUtility.UserLookupDelimitString, "g"); // Update the field value\n\n          fieldValue = ctx.CurrentFieldValue // Replace the delimiter\n          .replace(regExp, "; ") // Trim the delimiter from the beginning\n          .replace(/^; /g, "") // Trim the delimiter from the end\n          .replace(/; $/g, "");\n          break;\n\n        case "Note":\n          // Replace the return characters\n          fieldValue = "<div>" + ctx.CurrentFieldValue.replace(/\\n/g, "<br />") + "</div>";\n          break;\n\n        case "User":\n        case "UserMulti":\n          for (var i = 0; i < ctx.CurrentFieldValue.length; i++) {\n            var userValue = ctx.CurrentFieldValue[i]; // Add the user value\n\n            fieldValue += // User Lookup ID\n            userValue.EntityData.SPUserID + // Delimiter\n            sptypes_1.SPTypes.ClientTemplatesUtility.UserLookupDelimitString + // User Lookup Value\n            userValue.DisplayText + ( // Optional Delimiter\n            i == ctx.CurrentFieldValue.length - 1 ? "" : sptypes_1.SPTypes.ClientTemplatesUtility.UserLookupDelimitString);\n          }\n\n          break;\n      }\n\n      ; // Update the current field value\n\n      ctx.CurrentFieldValue = fieldValue;\n    } // Determine the control mode\n\n\n    var controlMode = sptypes_1.SPTypes.ControlMode.Display;\n\n    if (requireValueFl && (fieldValue == null || fieldValue == "")) {\n      // Inherit the control mode\n      controlMode = ctx.ControlMode;\n    } // Return the display value of the field\n\n\n    return exports.JSLink.renderField(ctx, field, controlMode);\n  },\n\n  /**\r\n   * Disable quick edit for the specified field.\r\n   * @param ctx - The client context.\r\n   * @param field - The field to disable edit.\r\n   */\n  disableQuickEdit: function disableQuickEdit(ctx, field) {\n    // Ensure we are in grid edit mode\n    if (ctx.inGridMode) {\n      // Disable editing for this field\n      field.AllowGridEditing = false;\n      return "";\n    } // Return the default field value html\n\n\n    return exports.JSLink.renderField(ctx, field);\n  },\n\n  /**\r\n   * Returns the list view.\r\n   * @param ctx - The client context.\r\n   */\n  getListView: function getListView(ctx) {\n    // Get the webpart\n    var wp = exports.JSLink.getWebPart(ctx);\n\n    if (wp) {\n      // Find the list form table\n      wp = wp.querySelector(".ms-formtable");\n    } // Return the list view\n\n\n    return wp;\n  },\n\n  /**\r\n   * Returns the list view items.\r\n   * @param ctx - The client context.\r\n   */\n  getListViewItems: function getListViewItems(ctx) {\n    // Return the list view items\n    return ctx.ListData ? ctx.ListData.Row : [];\n  },\n\n  /**\r\n   * Returns the selected list view items\r\n   */\n  getListViewSelectedItems: function getListViewSelectedItems() {\n    // Return the selected items\n    return lib_1.ContextInfo.window["SP"].ListOperation.Selection.getSelectedItems();\n  },\n\n  /**\r\n   * Returns the webpart containing the JSLink field/form/view.\r\n   * @param ctx - The client context.\r\n   */\n  getWebPart: function getWebPart(ctx) {\n    // Return the webpart\n    return lib_1.ContextInfo.document.querySelector("#WebPart" + (ctx.FormUniqueId || ctx.wpq));\n  },\n\n  /**\r\n   * Hides the specified field.\r\n   * @param ctx - The client context.\r\n   * @param field - The field to hide.\r\n   */\n  hideField: function hideField(ctx, field) {\n    // Ensure the hide event has been created\n    if (!exports.JSLink._hideEventFl) {\n      // Set the flag\n      exports.JSLink._hideEventFl = true; // Create the event\n\n      lib_1.ContextInfo.window.addEventListener("load", function () {\n        // Query for the elements to hide\n        var fieldElements = lib_1.ContextInfo.document.querySelectorAll(".hide-field");\n\n        for (var _i = 0, fieldElements_1 = fieldElements; _i < fieldElements_1.length; _i++) {\n          var fieldElement = fieldElements_1[_i]; // Get the parent row\n\n          var parentRow = fieldElement.parentNode && fieldElement.parentNode.parentNode ? fieldElement.parentNode.parentNode : null;\n\n          if (parentRow) {\n            // Ensure the parent row exists\n            if (fieldElement.parentNode.getAttribute("data-field-name") != parentRow.getAttribute("data-field-name")) {\n              // Find the parent row\n              while (parentRow && parentRow.nodeName.toLowerCase() != "tr") {\n                // Update the parent node\n                parentRow = parentRow.parentNode;\n              }\n            } // Hide the parent row\n\n\n            if (parentRow) {\n              parentRow.style.display = "none";\n            }\n          }\n        }\n      });\n    }\n  },\n\n  /**\r\n   * Registers the JSLink configuration\r\n   * @param cfg - The JSLink configuration.\r\n   */\n  register: function register(cfg) {\n    // Ensure a configuration exists\n    if (cfg) {\n      // Get the template manager\n      var templateManager = lib_1.ContextInfo.window.SPClientTemplates;\n      templateManager = templateManager ? templateManager.TemplateManager : null; // Ensure it exists\n\n      if (templateManager) {\n        // Apply the customization\n        templateManager.RegisterTemplateOverrides(cfg);\n      }\n    }\n  },\n\n  /**\r\n   * Removes the field and html from the page.\r\n   * @param ctx - The client context.\r\n   * @param field - The field to remove.\r\n   */\n  removeField: function removeField(ctx, field) {\n    // Hide the field\n    exports.JSLink.hideField(ctx, field); // Return an empty element\n\n    return "<div class=\'hide-field\'></div>";\n  },\n\n  /**\r\n   * Method to render the default html for a field.\r\n   * @param ctx - The client context.\r\n   * @param field - The form field.\r\n   * @param formType - The form type. (Display, Edit, New or View)\r\n   */\n  renderField: function renderField(ctx, field, formType) {\n    // Determine the field type\n    var fieldType = field ? field.Type : ctx.CurrentFieldSchema ? ctx.CurrentFieldSchema.Type : null; // Ensure the form type is set\n\n    formType = formType ? formType : ctx.ControlMode; // Ensure a field to method mapper exists\n\n    if (exports.JSLink._fieldToMethodMapper[fieldType] && exports.JSLink._fieldToMethodMapper[fieldType][formType]) {\n      // Return the default html for this field\n      var defaultHtml = exports.JSLink._fieldToMethodMapper[fieldType][formType](ctx);\n\n      if (defaultHtml) {\n        return defaultHtml;\n      }\n    } // Set the field renderer based on the field type\n\n\n    var field = ctx.CurrentFieldSchema;\n    var fieldRenderer = null;\n\n    switch (field.Type) {\n      case "AllDayEvent":\n        fieldRenderer = new lib_1.ContextInfo.window["AllDayEventFieldRenderer"](field.Name);\n        break;\n\n      case "Attachments":\n        fieldRenderer = new lib_1.ContextInfo.window["AttachmentFieldRenderer"](field.Name);\n        break;\n\n      case "BusinessData":\n        fieldRenderer = new lib_1.ContextInfo.window["BusinessDataFieldRenderer"](field.Name);\n        break;\n\n      case "Computed":\n        fieldRenderer = new lib_1.ContextInfo.window["ComputedFieldRenderer"](field.Name);\n        break;\n\n      case "CrossProjectLink":\n        fieldRenderer = new lib_1.ContextInfo.window["ProjectLinkFieldRenderer"](field.Name);\n        break;\n\n      case "Currency":\n        fieldRenderer = new lib_1.ContextInfo.window["NumberFieldRenderer"](field.Name);\n        break;\n\n      case "DateTime":\n        fieldRenderer = new lib_1.ContextInfo.window["DateTimeFieldRenderer"](field.Name);\n        break;\n\n      case "Lookup":\n        fieldRenderer = new lib_1.ContextInfo.window["LookupFieldRenderer"](field.Name);\n        break;\n\n      case "LookupMulti":\n        fieldRenderer = new lib_1.ContextInfo.window["LookupFieldRenderer"](field.Name);\n        break;\n\n      case "Note":\n        fieldRenderer = new lib_1.ContextInfo.window["NoteFieldRenderer"](field.Name);\n        break;\n\n      case "Number":\n        fieldRenderer = new lib_1.ContextInfo.window["NumberFieldRenderer"](field.Name);\n        break;\n\n      case "Recurrence":\n        fieldRenderer = new lib_1.ContextInfo.window["RecurrenceFieldRenderer"](field.Name);\n        break;\n\n      case "Text":\n        fieldRenderer = new lib_1.ContextInfo.window["TextFieldRenderer"](field.Name);\n        break;\n\n      case "URL":\n        fieldRenderer = new lib_1.ContextInfo.window["UrlFieldRenderer"](field.Name);\n        break;\n\n      case "User":\n        fieldRenderer = new lib_1.ContextInfo.window["UserFieldRenderer"](field.Name);\n        break;\n\n      case "UserMulti":\n        fieldRenderer = new lib_1.ContextInfo.window["UserFieldRenderer"](field.Name);\n        break;\n\n      case "WorkflowStatus":\n        fieldRenderer = new lib_1.ContextInfo.window["RawFieldRenderer"](field.Name);\n        break;\n    }\n\n    ; // Get the current item\n\n    var currentItem = ctx.CurrentItem || ctx.ListData.Items[0]; // Return the item\'s field value html\n\n    return fieldRenderer ? fieldRenderer.RenderField(ctx, field, currentItem, ctx.ListSchema) : currentItem[field.Name];\n  }\n};\n\n//# sourceURL=webpack://gd-sprest/./build/helper/jslink.js?')},"./build/helper/listForm.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.ListForm = void 0;\n\nvar __1 = __webpack_require__(/*! .. */ "./build/index.js");\n/**\r\n * List Form\r\n */\n\n\nexports.ListForm = {\n  // Method to create an instance of the list form\n  create: function create(props) {\n    var _info = null;\n    var _props = null;\n    var _reject = null;\n    var _resolve = null; // Save the properties\n\n    _props = props || {};\n    _props.fields = _props.fields; // Method to load the list data\n\n    var load = function load() {\n      // Clear the information\n      _info = {\n        item: _props.item,\n        query: _props.query || {},\n        requestDigest: _props.requestDigest\n      }; // Load the list data\n\n      loadListData().then( // Success\n      function () {\n        // See if the fields have been defined\n        if (_props.fields) {\n          // Process the fields\n          processFields(); // Load the item data\n\n          loadItem();\n        } else {\n          // Load the content type\n          loadContentType();\n        }\n      }, // Reject\n      _reject);\n    }; // Method to load a content type for the associated fields\n\n\n    var loadContentType = function loadContentType() {\n      // Load the content types\n      _info.list.ContentTypes() // Query for the default content type and expand the field links\n      .query({\n        Filter: _props.contentType ? "Name eq \'" + _props.contentType + "\'" : null,\n        Expand: ["FieldLinks"],\n        Select: ["*", "FieldLinks/DisplayName", "FieldLinks/Hidden", "FieldLinks/Name", "FieldLinks/ReadOnly", "FieldLinks/Required"],\n        Top: 1\n      }) // Execute the request, but wait for the previous one to be completed\n      .execute(function (ct) {\n        // Resolve the promise\n        loadDefaultFields(ct.results[0]);\n      }, _reject);\n    }; // Method to load the default fields\n\n\n    var loadDefaultFields = function loadDefaultFields(ct) {\n      var fields = ct ? ct.FieldLinks.results : [];\n      var formFields = {};\n      var formLinks = {}; // Parse the field links\n\n      for (var i = 0; i < fields.length; i++) {\n        var fieldLink = fields[i]; // Get the field\n\n        var field = _info.fields[fieldLink.Name];\n\n        if (field) {\n          // Skip the content type field\n          if (field.InternalName == "ContentType") {\n            continue;\n          } // Skip hidden fields\n\n\n          if (field.Hidden || fieldLink.Hidden) {\n            continue;\n          } // Save the form field and link\n\n\n          formFields[field.InternalName] = field;\n          formLinks[field.InternalName] = fieldLink;\n        }\n      } // Update the fields\n\n\n      _info.contentType = ct;\n      _info.fields = formFields;\n      _info.fieldLinks = formLinks; // Load the item data\n\n      loadItem();\n    }; // Method to load the field data\n\n\n    var loadFieldData = function loadFieldData(fields) {\n      // Clear the fields\n      _info.fields = {}; // Parse the fields\n\n      for (var i = 0; i < fields.results.length; i++) {\n        var field = fields.results[i]; // See if the exclude fields is defined\n\n        if (_props.excludeFields) {\n          var excludeField = false; // Parse the fields to exclude\n\n          for (var j = 0; j < _props.excludeFields.length; j++) {\n            // See if we are excluding this field\n            if (_props.excludeFields[j] == field.InternalName) {\n              // Set the flag\n              excludeField = true;\n              break;\n            }\n          } // See if we are excluding the field\n\n\n          if (excludeField) {\n            continue;\n          }\n        } // Save the field\n\n\n        _info.fields[field.InternalName] = field;\n      }\n    }; // Method to load the item\n\n\n    var loadItem = function loadItem() {\n      var reloadItem = false; // See if the item already exist\n\n      if (_info.item) {\n        // Parse the fields\n        for (var fieldName in _info.fields) {\n          var field = _info.fields[fieldName]; // See what type of field this is\n\n          switch (field.FieldTypeKind) {\n            // Lookup or User Field\n            case __1.SPTypes.FieldType.Lookup:\n            case __1.SPTypes.FieldType.User:\n              var fieldValue = _info.item[fieldName + "Id"]; // Ensure the value exists\n\n              if (fieldValue) {\n                // See if a value exists\n                if (fieldValue.results ? fieldValue.results.length > 0 : fieldValue > 0) {\n                  // Ensure the field data has been loaded\n                  if (_info.item[fieldName] == null) {\n                    // Set the flag\n                    reloadItem = true;\n                  }\n                }\n              }\n\n              break;\n            // Default\n\n            default:\n              // See if this is an taxonomy field\n              if (field.TypeAsString.indexOf("TaxonomyFieldType") == 0) {\n                var fieldValue_1 = _info.item[fieldName + "Id"]; // Ensure the value exists\n\n                if (fieldValue_1) {\n                  // See if a field value exists\n                  if (fieldValue_1.results ? fieldValue_1.results.length > 0 : fieldValue_1 != null) {\n                    // Parse the fields\n                    for (var fieldName_1 in _info.fields) {\n                      var valueField = _info.fields[fieldName_1]; // See if this is the value field\n\n                      if (valueField.InternalName == field.InternalName + "_0" || valueField.Title == field.InternalName + "_0") {\n                        // Ensure the value field is loaded\n                        if (_info.item[valueField.InternalName] == null) {\n                          // Set the flag\n                          reloadItem = true;\n                        }\n\n                        break;\n                      }\n                    }\n                  }\n                }\n              }\n\n              break;\n          } // See if we are reloading the item\n\n\n          if (reloadItem) {\n            break;\n          }\n        }\n      } // See if the item exists\n\n\n      if (_info.item && !reloadItem) {\n        // See if we are loading attachments\n        if (_props.loadAttachments && _info.attachments == null) {\n          // Load the attachments\n          exports.ListForm.loadAttachments(_props).then(function (attachments) {\n            // Set the attachments\n            _info.attachments = attachments; // Resolve the promise\n\n            _resolve(_info);\n          }, _reject);\n        } else {\n          // Resolve the promise\n          _resolve(_info);\n        }\n      } // Else, see if we are loading the list item\n      else if (reloadItem || _props.itemId > 0) {\n        // Update the item query\n        _info.query = exports.ListForm.generateODataQuery(_info, _props.loadAttachments); // Get the list item\n\n        _info.list.Items(reloadItem ? _props.item.Id : _props.itemId) // Set the query\n        .query(_info.query) // Execute the request\n        .execute(function (item) {\n          // Save the attachments\n          _info.attachments = item.AttachmentFiles.results; // Save the item\n\n          _info.item = item; // Refresh the item\n\n          exports.ListForm.refreshItem(_info).then(function (info) {\n            // Update the info\n            _info = info; // Resolve the promise\n\n            _resolve(_info);\n          }, _reject);\n        }, _reject);\n      } // Else, this is a new item\n      else {\n        // Default the attachments\n        _info.attachments = _props.loadAttachments ? [] : _info.attachments; // Resolve the promise\n\n        _resolve(_info);\n      }\n    }; // Method to load the list data\n\n\n    var loadListData = function loadListData() {\n      // Return a promise\n      return new Promise(function (resolve, reject) {\n        // See if the list & fields already exist\n        if (_info.list && _info.fields) {\n          // Resolve the promise\n          resolve();\n          return;\n        } // Get the web\n\n\n        __1.Web(_props.webUrl, {\n          disableCache: true,\n          requestDigest: _props.requestDigest\n        }) // Get the list\n        .Lists(_props.listName) // Execute the request\n        .execute(function (list) {\n          // Save the list and web url\n          _info.list = list;\n          _info.webUrl = _props.webUrl;\n        }, reject) // Load the fields\n        .Fields() // Execute the request\n        .execute(function (fields) {\n          // Load the field data\n          loadFieldData(fields); // Resolve the promise\n\n          resolve();\n        }, reject, true);\n      });\n    }; // Method to process the fields\n\n\n    var processFields = function processFields() {\n      var formFields = {}; // Parse the fields provided\n\n      for (var i = 0; i < _props.fields.length; i++) {\n        var field = _info.fields[_props.fields[i]]; // Ensure the field exists\n\n        if (field) {\n          // Save the field\n          formFields[field.InternalName] = field; // See if this is a taxonomy field\n\n          if (field.TypeAsString.indexOf("TaxonomyFieldType") == 0) {\n            // Parse the list fields\n            for (var fieldName in _info.fields) {\n              var valueField = _info.fields[fieldName]; // See if this is a value field\n\n              if (valueField.InternalName == field.InternalName + "_0" || valueField.Title == field.InternalName + "_0") {\n                // Include this field\n                formFields[valueField.InternalName] = valueField;\n                break;\n              }\n            }\n          }\n        }\n      } // Update the fields\n\n\n      _info.fields = formFields;\n    }; // Return a promise\n\n\n    return new Promise(function (resolve, reject) {\n      // Save the methods\n      _reject = reject;\n      _resolve = resolve; // Load the list data\n\n      load();\n    });\n  },\n  // Method to generate the odata query\n  generateODataQuery: function generateODataQuery(info, loadAttachments) {\n    if (loadAttachments === void 0) {\n      loadAttachments = false;\n    }\n\n    var query = info.query || {}; // Default the select query to get all the fields by default\n\n    query.Select = query.Select || ["*"];\n    query.Expand = query.Expand || []; // See if we are loading the attachments\n\n    if (loadAttachments) {\n      // Expand the attachment files collection\n      query.Expand.push("AttachmentFiles"); // Select the attachment files\n\n      query.Select.push("Attachments");\n      query.Select.push("AttachmentFiles");\n    } // Parse the fields\n\n\n    for (var fieldName in info.fields) {\n      var field = info.fields[fieldName]; // Skip the attachments field\n\n      if (fieldName == "Attachments") {\n        continue;\n      } // See if this is the file leaf ref\n\n\n      if (field.InternalName == "FileLeafRef") {\n        // Ensure the field is included\n        query.Select.push("FileLeafRef");\n        continue;\n      } // Update the query, based on the type\n\n\n      switch (field.FieldTypeKind) {\n        // Lookup Field\n        case __1.SPTypes.FieldType.Lookup:\n          var lookupField = field; // See if this is an associated lookup field\n\n          if (lookupField.PrimaryFieldId) {\n            // Parse the form fields to find the parent field\n            for (var parentFieldName in info.fields) {\n              var parentField = info.fields[parentFieldName]; // See if the parent field is being loaded\n\n              if (parentField.Id == lookupField.PrimaryFieldId) {\n                // Select the field\n                query.Select.push(parentField.InternalName + "/" + lookupField.LookupField);\n                break;\n              }\n            }\n          } else {\n            // Expand the field\n            query.Expand.push(field.InternalName); // Select the fields\n\n            query.Select.push(field.InternalName + "/Id");\n            query.Select.push(field.InternalName + "/" + field.LookupField);\n          }\n\n          break;\n        // User Field\n\n        case __1.SPTypes.FieldType.User:\n          // Expand the field\n          query.Expand.push(field.InternalName); // Select the fields\n\n          query.Select.push(field.InternalName + "/Id");\n          query.Select.push(field.InternalName + "/Title");\n          break;\n        // Default\n\n        default:\n          // See if this is an taxonomy field\n          if (field.TypeAsString.indexOf("TaxonomyFieldType") == 0) {\n            // Parse the fields\n            for (var fieldName_2 in info.fields) {\n              var valueField = info.fields[fieldName_2]; // See if this is the value field\n\n              if (valueField.InternalName == field.InternalName + "_0" || valueField.Title == field.InternalName + "_0") {\n                // Include the value field\n                query.Select.push(valueField.InternalName);\n                break;\n              }\n            }\n          }\n\n          break;\n      }\n    } // Return the query\n\n\n    return query;\n  },\n  // Method to load the item attachments\n  loadAttachments: function loadAttachments(info) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Ensure the item id exists\n      var itemId = info.item ? info.item.Id : info.itemId;\n\n      if (itemId > 0) {\n        // Get the web\n        __1.Web(info.webUrl, {\n          requestDigest: info.requestDigest\n        }) // Get the list\n        .Lists(info.listName) // Get the item\n        .Items(itemId) // Get the attachment files\n        .AttachmentFiles() // Execute the request\n        .execute(function (attachments) {\n          // Ensure the attachments exist\n          if (!attachments.existsFl) {\n            // Reject the promise\n            reject(attachments.response);\n            return;\n          } // Resolve the promise\n\n\n          resolve(attachments.results || []);\n        }, reject);\n      } else {\n        // Resolve the promise\n        resolve([]);\n      }\n    });\n  },\n  // Method to refresh an item\n  refreshItem: function refreshItem(info) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Update the query\n      info.query = exports.ListForm.generateODataQuery(info, info.attachments ? true : false); // Get the item\n\n      info.list.Items(info.item.Id).query(info.query).execute(function (item) {\n        // Update the item\n        info.item = item; // Get the item values\n\n        info.list.Items(item.Id).query({\n          Expand: ["FieldValuesAsText", "Folder"]\n        }).execute(function (item) {\n          // Set the values\n          info.itemFolder = item.Folder;\n          info.fieldValuesAsText = item.FieldValuesAsText;\n        }); // Get the html values for this item\n        // This is needed for complex field values\n\n        info.list.Items(item.Id).FieldValuesAsHtml().execute(function (values) {\n          // Set the values\n          info.fieldValuesAsHtml = values; // Resolve the promise\n\n          resolve(info);\n        }, true);\n      }, reject);\n    });\n  },\n  // Method to refresh the request digest value\n  refreshRequestDigest: function refreshRequestDigest(info) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Get the context info\n      __1.ContextInfo.getWeb(info.webUrl).execute(function (context) {\n        // Update the request digest value\n        info.item.updateRequestDigest(context.GetContextWebInformation.FormDigestValue);\n        info.list.updateRequestDigest(context.GetContextWebInformation.FormDigestValue); // Resolve the request\n\n        resolve();\n      }, reject);\n    });\n  },\n  // Method to remove attachments from an item\n  removeAttachment: function removeAttachment(info, fileName) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Ensure attachments exist\n      if (info.attachments) {\n        // Parse the attachments\n        for (var i = 0; i < info.attachments.length; i++) {\n          // See if this is the target attachment\n          var attachment = info.attachments[i];\n\n          if (attachment.FileName == fileName) {\n            // Get the web\n            __1.Web(info.webUrl, {\n              requestDigest: info.requestDigest\n            }) // Get the file\n            .getFileByServerRelativeUrl(attachment.ServerRelativeUrl) // Delete the file\n            ["delete"]() // Execute the request\n            .execute(function () {\n              // Resolve the promise\n              resolve(info);\n            }, reject); // Attachment found\n\n\n            return;\n          } // Attachment not found\n\n\n          reject("Attachment \'" + fileName + "\' was not found.");\n        }\n      } else {\n        // Attachments not loaded\n        reject("Attachment \'" + fileName + "\' was not found.");\n      }\n    });\n  },\n  // Method to save attachments to an existing item\n  saveAttachments: function saveAttachments(info, attachmentInfo) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      var itemId = info.item ? info.item.Id : info.itemId;\n\n      if (itemId > 0) {\n        // Get the web\n        var attachments = __1.Web(info.webUrl, {\n          requestDigest: info.requestDigest\n        }) // Get the lists\n        .Lists(info.listName) // Get the item\n        .Items(itemId) // Get the attachment files\n        .AttachmentFiles(); // Parse the attachment information\n\n\n        for (var i = 0; i < attachmentInfo.length; i++) {\n          var attachment = attachmentInfo[i]; // Add the attachment\n\n          attachments.add(attachment.name, attachment.data).execute(true);\n        } // Wait for the requests to complete\n\n\n        attachments.done(function () {\n          var args = [];\n\n          for (var _i = 0; _i < arguments.length; _i++) {\n            args[_i] = arguments[_i];\n          } // Resolve the promise\n\n\n          resolve.apply(args);\n        });\n      } else {\n        // Resolve the promise\n        resolve(null);\n      }\n    });\n  },\n  // Method to save a new or existing item\n  saveItem: function saveItem(info, formValues, checkItemVersion) {\n    if (formValues === void 0) {\n      formValues = {};\n    } // Return a promise\n\n\n    return new Promise(function (resolve, reject) {\n      // See if this is an existing item\n      if (info.item && info.item.update) {\n        // Set the request properties if we are checking the item version\n        var requestProps = checkItemVersion && info.item.etag ? {\n          requestHeader: {\n            "IF-MATCH": info.item.etag\n          }\n        } : null; // Update the item\n\n        __1.Web(info.webUrl, requestProps).Lists(info.list.Title).Items(info.item.Id).update(formValues).execute(function (response) {\n          // Refresh the item\n          exports.ListForm.refreshItem(info).then(function (info) {\n            // Resolve the promise\n            resolve(info);\n          }, reject);\n        }, reject);\n      } else {\n        // Set the metadata type\n        formValues["__metadata"] = {\n          type: info.list.ListItemEntityTypeFullName\n        }; // Add the item\n\n        info.list.Items().add(formValues).execute(function (item) {\n          // Update the info\n          info.item = item; // Refresh the item\n\n          exports.ListForm.refreshItem(info).then(function (info) {\n            // Resolve the promise\n            resolve(info);\n          }, reject);\n        }, reject);\n      }\n    });\n  },\n  // Method to show a file dialog\n  showFileDialog: function showFileDialog(accept, info, onSave) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Method to add an attachment\n      var addAttachment = function addAttachment(name, data, src) {\n        // Call the save event\n        onSave ? onSave({\n          name: name,\n          data: data,\n          src: src\n        }) : null; // Get the list\n\n        info.list // Get the item\n        .Items(info.item.Id) // Get the attachments\n        .AttachmentFiles() // Add the file\n        .add(name, data) // Execute the request\n        .execute(function () {\n          // Refresh the item\n          exports.ListForm.refreshItem(info).then(function (info) {\n            // Remove the element\n            document.body.removeChild(el); // Resolve the promise\n\n            resolve(info);\n          });\n        }, reject);\n      }; // Method to read the file\n\n\n      var readFile = function readFile(ev) {\n        // Get the source file\n        var srcFile = ev.target["files"][0];\n\n        if (srcFile) {\n          var reader = new FileReader(); // Set the file loaded event\n\n          reader.onloadend = function (ev) {\n            var ext = srcFile.name.split(".");\n            ext = ext[ext.length - 1].toLowerCase(); // See if the info exists\n\n            if (info) {\n              // Add the attachment\n              addAttachment(srcFile.name, ev.target.result, srcFile);\n            } else {\n              // Remove the element\n              document.body.removeChild(el); // Resolve the promise with the file information\n\n              resolve({\n                data: ev.target.result,\n                name: srcFile.name,\n                src: srcFile\n              });\n            }\n          }; // Set the error\n\n\n          reader.onerror = function (ev) {\n            // Remove the element\n            document.body.removeChild(el); // Reject the promise\n\n            reject(ev.target.error);\n          }; // Read the file\n\n\n          reader.readAsArrayBuffer(srcFile);\n        }\n      }; // Create the file element\n\n\n      var el = document.body.querySelector("#listform-attachment");\n\n      if (el == null) {\n        el = document.createElement("input"); // Set the properties\n\n        el.accept = accept ? accept.join(\',\') : null;\n        el.id = "listform-attachment";\n        el.type = "file";\n        el.hidden = true;\n        el.onchange = readFile; // Add the element to the body\n\n        document.body.appendChild(el);\n      } // Show the dialog\n\n\n      el.click();\n    });\n  }\n};\n\n//# sourceURL=webpack://gd-sprest/./build/helper/listForm.js?')},"./build/helper/listFormField.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.ListFormField = void 0;\n\nvar __1 = __webpack_require__(/*! .. */ "./build/index.js");\n/**\r\n * List Form Field\r\n */\n\n\nexports.ListFormField = {\n  // Method to create an instance of the list form field\n  create: function create(props) {\n    var _fieldInfo = props || {};\n\n    var _reject = null;\n    var _resolve = null; // Load the field\n\n    var load = function load() {\n      // See if the field exists\n      if (_fieldInfo.field) {\n        // Process the field\n        processField();\n      } // Else, load the field from the information provided\n      else {\n        // Get the web\n        __1.Web(_fieldInfo.webUrl) // Get the list\n        .Lists(_fieldInfo.listName) // Get the fields\n        .Fields() // Get the field by its internal name\n        .getByInternalNameOrTitle(_fieldInfo.name) // Execute the request\n        .execute(function (field) {\n          // Save the field\n          _fieldInfo.field = field; // Process the field\n\n          processField();\n        }, _reject);\n      }\n    }; // Method to proces the field and save its information\n\n\n    var processField = function processField() {\n      // Update the field information\n      _fieldInfo.defaultValue = _fieldInfo.field.DefaultValue;\n      _fieldInfo.readOnly = _fieldInfo.field.ReadOnlyField;\n      _fieldInfo.required = _fieldInfo.field.Required ? true : false;\n      _fieldInfo.title = _fieldInfo.field.Title;\n      _fieldInfo.type = _fieldInfo.field.FieldTypeKind;\n      _fieldInfo.typeAsString = _fieldInfo.field.TypeAsString; // Update the field info, based on the type\n\n      switch (_fieldInfo.type) {\n        // Choice\n        case __1.SPTypes.FieldType.Choice:\n        case __1.SPTypes.FieldType.MultiChoice:\n          var choices = _fieldInfo.field.Choices;\n          _fieldInfo.choices = (choices ? choices["results"] : null) || [];\n          _fieldInfo.multi = _fieldInfo.type == __1.SPTypes.FieldType.MultiChoice;\n          break;\n        // Date/Time\n\n        case __1.SPTypes.FieldType.DateTime:\n          var fldDate = _fieldInfo.field;\n          _fieldInfo.showTime = fldDate.DisplayFormat == __1.SPTypes.DateFormat.DateTime;\n          break;\n        // Lookup\n\n        case __1.SPTypes.FieldType.Lookup:\n          var fldLookup = _fieldInfo.field;\n          _fieldInfo.lookupField = fldLookup.LookupField;\n          _fieldInfo.lookupListId = fldLookup.LookupList;\n          _fieldInfo.lookupWebId = fldLookup.LookupWebId;\n          _fieldInfo.multi = fldLookup.AllowMultipleValues;\n          break;\n        // Number\n\n        case __1.SPTypes.FieldType.Number:\n          var fldNumber = _fieldInfo.field;\n          var startIdx = fldNumber.SchemaXml.indexOf(\'Decimals="\') + 10;\n          _fieldInfo.decimals = startIdx > 10 ? parseInt(fldNumber.SchemaXml.substr(startIdx, fldNumber.SchemaXml.substr(startIdx).indexOf(\'"\'))) : 0;\n          _fieldInfo.maxValue = fldNumber.MaximumValue;\n          _fieldInfo.minValue = fldNumber.MinimumValue;\n          _fieldInfo.showAsPercentage = fldNumber.SchemaXml.indexOf(\'Percentage="TRUE"\') > 0;\n          break;\n        // Note\n\n        case __1.SPTypes.FieldType.Note:\n          var fldNote = _fieldInfo.field;\n          _fieldInfo.multiline = true;\n          _fieldInfo.richText = fldNote.RichText;\n          _fieldInfo.rows = fldNote.NumberOfLines;\n          break;\n        // Text\n\n        case __1.SPTypes.FieldType.Text:\n          _fieldInfo.multiline = false;\n          _fieldInfo.richText = false;\n          _fieldInfo.rows = 1;\n          break;\n        // User\n\n        case __1.SPTypes.FieldType.User:\n          var fldUser = _fieldInfo.field;\n          _fieldInfo.allowGroups = fldUser.SelectionMode == __1.SPTypes.FieldUserSelectionType.PeopleAndGroups;\n          _fieldInfo.multi = fldUser.AllowMultipleValues;\n          break;\n        // Default\n\n        default:\n          // See if this is an MMS field\n          if (_fieldInfo.typeAsString.indexOf("TaxonomyFieldType") == 0) {\n            var fldMMS = _fieldInfo.field;\n            _fieldInfo.multi = fldMMS.AllowMultipleValues;\n            _fieldInfo.termId = fldMMS.IsAnchorValid ? fldMMS.AnchorId : fldMMS.TermSetId;\n            _fieldInfo.termSetId = fldMMS.TermSetId;\n            _fieldInfo.termStoreId = fldMMS.SspId;\n          }\n\n          break;\n      } // Resolve the promise\n\n\n      _resolve(_fieldInfo);\n    }; // Return a promise\n\n\n    return new Promise(function (resolve, reject) {\n      // Save the methods\n      _resolve = resolve;\n      _reject = reject; // See if the field exists\n\n      if (_fieldInfo.field) {\n        // Process the field\n        processField();\n      } else {\n        // Load the field\n        load();\n      }\n    });\n  },\n  // Method to get the folder to store the image field file in\n  getOrCreateImageFolder: function getOrCreateImageFolder(info) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Get the site assets library\n      (function () {\n        return new Promise(function (resolve) {\n          __1.Web(info.webUrl).Lists("Site Assets").execute( // Exists\n          function (lib) {\n            // Resolve the request\n            resolve(lib);\n          }, // Doesn\'t Exist\n          function () {\n            // Create the list\n            __1.Web(info.webUrl).Lists().add({\n              Title: "SiteAssets",\n              BaseTemplate: __1.SPTypes.ListTemplateType.DocumentLibrary\n            }).execute(function (lib) {\n              // Update the title\n              lib.update({\n                Title: "Site Assets"\n              }).execute(function () {\n                // Resolve the request\n                resolve(lib);\n              }, reject);\n            });\n          });\n        });\n      })().then(function (siteAssets) {\n        // Get the list id folder\n        siteAssets.RootFolder().Folders(info.list.Id).execute( // Exists\n        function (folder) {\n          // Resolve the request\n          resolve(folder);\n        }, // Doesn\'t Exist\n        function () {\n          // Create the folder\n          siteAssets.RootFolder().Folders().add(info.list.Id).execute(function (folder) {\n            // Resolve the request\n            resolve(folder);\n          }, reject);\n        });\n      });\n    });\n  },\n  // Method to load the lookup data\n  loadLookupData: function loadLookupData(info, queryTop) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Get the current site collection\n      __1.Site() // Get the web containing the lookup list\n      .openWebById(info.lookupWebId) // Execute the request\n      .execute(function (web) {\n        // See if there is a filter\n        var query = info.lookupFilter || {};\n\n        if (typeof query == "string") {\n          // Set the filter\n          query = {\n            Filter: query\n          };\n        } // Default the value if it hasn\'t been set\n\n\n        if (query.GetAllItems == null) {\n          // Set the default value\n          query.GetAllItems = true;\n        } // Default the value if it hasn\'t been set\n\n\n        if (query.OrderBy == null) {\n          // Set the default sort\n          query.OrderBy = ["Title"];\n        } // Default the value if it hasn\'t been set\n\n\n        if (query.Select == null) {\n          // Set the default value\n          query.Select = ["ID", info.lookupField];\n        } // Default the value if it hasn\'t been set\n\n\n        if (query.Top == null) {\n          // Set the default value\n          query.Top = queryTop > 0 && queryTop <= 5000 ? queryTop : 500;\n        } // Get the list\n\n\n        web.Lists() // Get the list by id\n        .getById(info.lookupListId) // Get the items\n        .Items() // Set the query\n        .query(query) // Execute the request\n        .execute(function (items) {\n          // Resolve the promise\n          resolve(items.results);\n        }, reject);\n      }, reject);\n    });\n  },\n  // Method to load the mms data\n  loadMMSData: function loadMMSData(info) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Load the term set\n      __1.Helper.Taxonomy.getTermSetById(info.termStoreId, info.termSetId).then(function (termSet) {\n        // Get the target root term\n        var root = __1.Helper.Taxonomy.findById(termSet, info.termId); // See if the root node doesn\'t exist\n\n\n        if (root == null) {\n          // Set the root to the term set\n          root = __1.Helper.Taxonomy.findById(termSet, info.termSetId);\n        } // Resolve the request\n\n\n        resolve(__1.Helper.Taxonomy.toArray(root));\n      }, reject);\n    });\n  },\n  // Method to load the mms value field\n  loadMMSValueField: function loadMMSValueField(info) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Get the web\n      __1.Web(info.webUrl) // Get the list\n      .Lists(info.listName) // Get the fields\n      .Fields() // Get the hidden field\n      .getByInternalNameOrTitle(info.name + "_0") // Execute the request\n      .execute( // Success\n      function (field) {\n        // Resolve the promise\n        resolve(field);\n      }, // Error\n      function () {\n        // Reject w/ a message\n        reject("Unable to find the hidden value field for \'" + info.name + "\'.");\n      });\n    });\n  }\n};\n\n//# sourceURL=webpack://gd-sprest/./build/helper/listFormField.js?')},"./build/helper/methods/addContentEditorWebPart.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.addContentEditorWebPart = void 0;\n\nvar webpart_1 = __webpack_require__(/*! ../webpart */ "./build/helper/webpart.js"); // Method to add a script editor webpart to the page\n\n\nexports.addContentEditorWebPart = function (url, wpProps) {\n  // Return a promise\n  return new Promise(function (resolve, reject) {\n    // Get the current context\n    var context = SP.ClientContext.get_current(); // Get the webpart manager for the page\n\n    var page = context.get_web().getFileByServerRelativeUrl(url);\n    var wpMgr = page.getLimitedWebPartManager(SP.WebParts.PersonalizationScope.shared); // Create the webpart\n\n    var wp = wpMgr.importWebPart(webpart_1.WebPart.generateContentEditorXML(wpProps)).get_webPart(); // Add the webpart to the page\n\n    wpMgr.addWebPart(wp, wpProps.zone || "", wpProps.index || 0); // Save the page\n\n    context.load(wp);\n    context.executeQueryAsync( // Success\n    function () {\n      // Resolve the promise\n      resolve();\n    }, // Error\n    function () {\n      var args = [];\n\n      for (var _i = 0; _i < arguments.length; _i++) {\n        args[_i] = arguments[_i];\n      } // Reject the promise\n\n\n      reject(args[1] ? args[1].get_message() : "");\n    });\n  });\n};\n\n//# sourceURL=webpack://gd-sprest/./build/helper/methods/addContentEditorWebPart.js?')},"./build/helper/methods/addPermissionLevel.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.addPermissionLevel = void 0;\n\nvar lib_1 = __webpack_require__(/*! ../../lib */ "./build/lib/index.js");\n/**\r\n * Adds a permission level to the current or specified web.\r\n */\n\n\nexports.addPermissionLevel = function (props) {\n  // Return a promise\n  return new Promise(function (resolve, reject) {\n    // Ensure the base permissions exist\n    if (SP && SP.BasePermissions) {\n      // Set the context and get the role definitions\n      var ctx = props.WebUrl ? new SP.ClientContext(props.WebUrl) : SP.ClientContext.get_current(); // Set the base permissions\n\n      var basePermissions = new SP.BasePermissions();\n      var permissions = props.Permissions || [];\n\n      for (var i = 0; i < permissions.length; i++) {\n        // Set the flag\n        basePermissions.set(permissions[i]);\n      } // Create the role definition\n\n\n      var roleDefInfo = new SP.RoleDefinitionCreationInformation();\n      roleDefInfo.set_basePermissions(basePermissions);\n      roleDefInfo.set_description(props.Description);\n      roleDefInfo.set_name(props.Name);\n      roleDefInfo.set_order(props.Order); // Add the role definition\n\n      var roleDef_1 = ctx.get_site().get_rootWeb().get_roleDefinitions().add(roleDefInfo);\n      ctx.load(roleDef_1); // Execute the request\n\n      ctx.executeQueryAsync(function () {\n        // Get the role definition\n        lib_1.Site(props.WebUrl).RootWeb().RoleDefinitions().getById(roleDef_1.get_id()).execute(function (roleDef) {\n          // Resolve the request\n          resolve(roleDef);\n        }, reject);\n      }, reject);\n    } else {\n      // Reject the request\n      reject("The \'SP\' core library is not available.");\n    }\n  });\n};\n\n//# sourceURL=webpack://gd-sprest/./build/helper/methods/addPermissionLevel.js?')},"./build/helper/methods/addScriptEditorWebPart.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.addScriptEditorWebPart = void 0;\n\nvar webpart_1 = __webpack_require__(/*! ../webpart */ "./build/helper/webpart.js"); // Method to add a script editor webpart to the page\n\n\nexports.addScriptEditorWebPart = function (url, wpProps) {\n  // Return a promise\n  return new Promise(function (resolve, reject) {\n    // Get the current context\n    var context = SP.ClientContext.get_current(); // Get the webpart manager for the page\n\n    var page = context.get_web().getFileByServerRelativeUrl(url);\n    var wpMgr = page.getLimitedWebPartManager(SP.WebParts.PersonalizationScope.shared); // Create the webpart\n\n    var wp = wpMgr.importWebPart(webpart_1.WebPart.generateScriptEditorXML(wpProps)).get_webPart(); // Add the webpart to the page\n\n    wpMgr.addWebPart(wp, wpProps.zone || "", wpProps.index || 0); // Save the page\n\n    context.load(wp);\n    context.executeQueryAsync( // Success\n    function () {\n      // Resolve the promise\n      resolve();\n    }, // Error\n    function () {\n      var args = [];\n\n      for (var _i = 0; _i < arguments.length; _i++) {\n        args[_i] = arguments[_i];\n      } // Reject the promise\n\n\n      reject(args[1] ? args[1].get_message() : "");\n    });\n  });\n};\n\n//# sourceURL=webpack://gd-sprest/./build/helper/methods/addScriptEditorWebPart.js?')},"./build/helper/methods/copyPermissionLevel.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.copyPermissionLevel = void 0;\n\nvar lib_1 = __webpack_require__(/*! ../../lib */ "./build/lib/index.js");\n\nvar sptypes_1 = __webpack_require__(/*! ../../sptypes */ "./build/sptypes/index.js");\n/**\r\n * Copies a permission level to the current or specified web.\r\n */\n\n\nexports.copyPermissionLevel = function (props) {\n  // Return a promise\n  return new Promise(function (resolve, reject) {\n    // Ensure the base permissions exist\n    if (SP && SP.BasePermissions) {\n      // Set the context and get the role definitions\n      var ctx_1 = props.WebUrl ? new SP.ClientContext(props.WebUrl) : SP.ClientContext.get_current(); // Get the base permission\n\n      var basePerm = ctx_1.get_site().get_rootWeb().get_roleDefinitions().getByName(props.BasePermission);\n      ctx_1.load(basePerm);\n      ctx_1.executeQueryAsync( // Success\n      function () {\n        // Copy the base permissions\n        var basePermissions = basePerm.get_basePermissions();\n        var permissions = new SP.BasePermissions();\n        var removePermissions = props.RemovePermissions || [];\n\n        for (var key in sptypes_1.SPTypes.BasePermissionTypes) {\n          var permission = sptypes_1.SPTypes.BasePermissionTypes[key]; // See if the base permission has this\n\n          if (basePermissions.has(permission) && removePermissions.indexOf(permission) < 0) {\n            // Set the permission\n            permissions.set(permission);\n          }\n        } // Parse the custom permissions to add\n\n\n        var newPermissions = props.AddPermissions || [];\n\n        for (var i = 0; i < newPermissions.length; i++) {\n          // Set the flag\n          permissions.set(newPermissions[i]);\n        } // Create the role definition\n\n\n        var roleDefInfo = new SP.RoleDefinitionCreationInformation();\n        roleDefInfo.set_basePermissions(permissions);\n        roleDefInfo.set_description(props.Description);\n        roleDefInfo.set_name(props.Name);\n        roleDefInfo.set_order(props.Order); // Add the role definition\n\n        var roleDef = ctx_1.get_site().get_rootWeb().get_roleDefinitions().add(roleDefInfo);\n        ctx_1.load(roleDef); // Execute the request\n\n        ctx_1.executeQueryAsync(function () {\n          // Get the role definition\n          lib_1.Site(props.WebUrl).RootWeb().RoleDefinitions().getById(roleDef.get_id()).execute(function (roleDef) {\n            // Resolve the request\n            resolve(roleDef);\n          }, reject);\n        }, reject);\n      }, // Error\n      function () {\n        // Reject the request\n        reject("Permission not found in site: " + props.WebUrl);\n      });\n    } else {\n      // Reject the request\n      reject("The \'SP\' core library is not available.");\n    }\n  });\n};\n\n//# sourceURL=webpack://gd-sprest/./build/helper/methods/copyPermissionLevel.js?')},"./build/helper/methods/createContentType.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.createContentType = void 0;\n\nvar lib_1 = __webpack_require__(/*! ../../lib */ "./build/lib/index.js");\n/**\r\n * Creates a content type in a web or specified list.\r\n * @param ctInfo - The content type information.\r\n * @param parentInfo - The parent content type id and url containing it.\r\n * @param webUrl - The relative url to create the content type in.\r\n * @param listName - The list name to add the content type to.\r\n */\n\n\nexports.createContentType = function (ctInfo, parentInfo, webUrl, listName) {\n  // Return a promise\n  return new Promise(function (resolve, reject) {\n    // Set the context\n    var ctx = webUrl ? new SP.ClientContext(webUrl) : SP.ClientContext.get_current(); // Get the parent content type\n\n    var parentContentType = (parentInfo.Url ? ctx.get_site().openWeb(parentInfo.Url) : ctx.get_web()).get_contentTypes().getById(parentInfo.Id); // Create the content type\n\n    var ct = new SP.ContentTypeCreationInformation();\n    ctInfo.Description != null ? ct.set_description(ctInfo.Description) : null;\n    ctInfo.Group != null ? ct.set_group(ctInfo.Group) : null;\n    ct.set_name(ctInfo.Name);\n    ct.set_parentContentType(parentContentType); // Add the content type\n\n    var src = listName ? ctx.get_web().get_lists().getByTitle(listName) : ctx.get_web();\n    var contentTypes = src.get_contentTypes();\n    contentTypes.add(ct);\n    ctx.load(contentTypes); // Execute the request\n\n    ctx.executeQueryAsync( // Success\n    function () {\n      // Set the content type collection\n      var cts = (listName ? lib_1.Web(webUrl).Lists(listName) : lib_1.Web(webUrl)).ContentTypes(); // Find the content type\n\n      cts.query({\n        Filter: "Name eq \'" + ctInfo.Name + "\'"\n      }).execute(function (cts) {\n        // Resolve the request\n        resolve(cts.results[0]);\n      });\n    }, // Error\n    function (sender, args) {\n      // Log\n      console.log("[gd-sprest][Create Content Type] Error adding the content type.", ctInfo.Name); // Reject the request\n\n      reject(args.get_message());\n    });\n  });\n};\n\n//# sourceURL=webpack://gd-sprest/./build/helper/methods/createContentType.js?')},"./build/helper/methods/createDocSet.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.createDocSet = void 0;\n\nvar lib_1 = __webpack_require__(/*! ../../lib */ "./build/lib/index.js");\n\nvar request_1 = __webpack_require__(/*! ./request */ "./build/helper/methods/request.js");\n/**\r\n * Creates a document set item.\r\n * @param name - The name of the document set folder to create.\r\n * @param listName - The name of the document set library.\r\n * @param webUrl - The url of the web containing the document set library.\r\n */\n\n\nexports.createDocSet = function (name, listName, webUrl) {\n  // Return a promise\n  return new Promise(function (resolve, reject) {\n    // Get the document set\'s root folder\n    lib_1.Web(webUrl).Lists(listName).query({\n      Expand: ["ContentTypes", "ParentWeb", "RootFolder"]\n    }).execute(function (list) {\n      // Parse the content types\n      var ctId = "0x0120D520";\n\n      for (var i = 0; i < list.ContentTypes.results.length; i++) {\n        // See if this is the document set content type\n        if (list.ContentTypes.results[i].StringId.indexOf(ctId) == 0) {\n          // Set the content type id\n          ctId = list.ContentTypes.results[i].StringId;\n          break;\n        }\n      } // Create the document set item\n\n\n      request_1.request({\n        method: "POST",\n        url: list.ParentWebUrl + "/_vti_bin/listdata.svc/" + list.Title.replace(/ /g, ""),\n        headers: {\n          Accept: "application/json;odata=verbose",\n          "Content-Type": "application/json;odata=verbose",\n          Slug: list.RootFolder.ServerRelativeUrl + "/" + name + "|" + ctId,\n          "X-Requested-With": "XMLHttpRequest"\n        },\n        data: {\n          Title: name,\n          Path: list.RootFolder.ServerRelativeUrl\n        }\n      }).then(function (response) {\n        // See if the request was successful\n        if (response.d && response.d.Id > 0) {\n          // Get the document set item and resolve the promise\n          lib_1.Web(webUrl).Lists(listName).Items(response.d.Id).execute(resolve);\n        } else {\n          // Reject the promise\n          reject(response["response"]);\n        }\n      });\n    }, reject);\n  });\n};\n\n//# sourceURL=webpack://gd-sprest/./build/helper/methods/createDocSet.js?')},"./build/helper/methods/getCurrentTheme.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.getCurrentTheme = void 0;\n\nvar lib_1 = __webpack_require__(/*! ../../lib */ "./build/lib/index.js");\n/**\r\n * Determines if the user has permissions, based on the permission kind value\r\n */\n\n\nexports.getCurrentTheme = function () {\n  // Return a promise\n  return new Promise(function (resolve, reject) {\n    // Wait for the modern theme information to be loaded\n    waitForModernTheme().then( // Success\n    function () {\n      // Resolve the request with the theme information\n      resolve(window["__themeState__"].theme);\n    }, // Error\n    function () {\n      // Get the current theme info\n      lib_1.Web().Lists("Composed Looks").Items().query({\n        Filter: "DisplayOrder eq 0 or Title eq \'Office\'",\n        OrderBy: ["DisplayOrder"],\n        Select: ["DisplayOrder", "MasterPageUrl", "ThemeUrl", "Title"]\n      }).execute(function (items) {\n        var currentItem = items.results[0];\n        var defaultItem = items.results[1]; // See if the current theme info exists\n\n        if (currentItem && currentItem["ThemeUrl"]) {\n          // Get the theme information\n          getThemeInfo(currentItem["ThemeUrl"].Url).then(resolve, reject);\n        } else if (defaultItem && defaultItem["ThemeUrl"]) {\n          // Get the theme information\n          getThemeInfo(defaultItem["ThemeUrl"].Url).then(resolve, reject);\n        } else {\n          // Unable to determine the theme\n          reject();\n        }\n      }, reject);\n    });\n  });\n}; // Gets the theme information for a color palette\n\n\nvar getThemeInfo = function getThemeInfo(url) {\n  if (url === void 0) {\n    url = "";\n  } // Return a promise\n\n\n  return new Promise(function (resolve, reject) {\n    var themeInfo = {}; // Ensure the url exists\n\n    if (url.length == 0) {\n      resolve(themeInfo);\n      return;\n    } // Get the file item\n\n\n    lib_1.Site().RootWeb().getFileByUrl(url).execute(function (file) {\n      // Read the contents\n      file.content().execute(function (contents) {\n        // Convert the xml\n        var parser = new DOMParser();\n        var xmlDoc = parser.parseFromString(String.fromCharCode.apply(null, new Uint8Array(contents)), "text/xml"); // Get the colors and parse them\n\n        var colors = xmlDoc.getElementsByTagName("s:color");\n\n        for (var i = 0; i < colors.length; i++) {\n          var color = colors[i];\n          var key = color.getAttribute("name");\n          var value = color.getAttribute("value"); // See if the length is > 6 characters\n\n          if (value.length > 6) {\n            // Convert the value\n            value = value.slice(2, 8) + value[0] + value[1];\n          } // Add the color information\n\n\n          themeInfo[key] = "#" + value;\n        } // Resolve the request\n\n\n        resolve(themeInfo);\n      }, reject);\n    }, reject);\n  });\n}; // Waits for the modern theme information to be loaded\n\n\nvar waitForModernTheme = function waitForModernTheme() {\n  // Return a promise\n  return new Promise(function (resolve, reject) {\n    var counter = 0;\n    var maxAttempts = 100; // See if the modern theme exists\n\n    if (window["__themeState__"] != null && window["__themeState__"].theme != null) {\n      // Theme is loaded\n      resolve();\n      return;\n    } // Wait for the theme to be loaded\n\n\n    var loopId = setInterval(function () {\n      // See if the modern theme exists\n      if (window["__themeState__"] != null && window["__themeState__"].theme != null) {\n        // Stop the loop\n        clearInterval(loopId); // Resolve the request\n\n        resolve();\n      } // Else, see if we have hit the max attempts\n      else if (++counter >= maxAttempts) {\n        // Stop the loop\n        clearInterval(loopId); // Reject the request\n\n        reject();\n      }\n    }, 50);\n  });\n};\n\n//# sourceURL=webpack://gd-sprest/./build/helper/methods/getCurrentTheme.js?')},"./build/helper/methods/hasPermissions.js":function(__unused_webpack_module,exports){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.hasPermissions = void 0;\n/**\r\n * Determines if the user has permissions, based on the permission kind value\r\n */\n\nexports.hasPermissions = function (permissionMask, permissions) {\n  if (permissions === void 0) {\n    permissions = [];\n  } // Set the permissions\n\n\n  var requestedPermissions = typeof permissions === "number" ? [permissions] : permissions; // Default the permission flag\n\n  var hasPermissions = true; // See if this user has full permissions\n\n  if ((permissionMask.High & 32767) == 32767 && (permissionMask.Low & 65535) == 65535) {\n    return hasPermissions;\n  } // Parse the requested permissions\n\n\n  for (var i = 0; i < requestedPermissions.length; i++) {\n    var permission = requestedPermissions[i]; // Determine the value to use\n\n    var sequence = permissionMask.Low;\n\n    if (permission >= 32) {\n      // Update the sequence\n      sequence = permissionMask.High;\n      permission -= 32;\n    } // See if the user doesn\'t have permission\n\n\n    if ((sequence & 1 << permission - 1) == 0) {\n      // Set the flag and break from the loop\n      hasPermissions = false;\n      break;\n    }\n  } // Return the result\n\n\n  return hasPermissions;\n};\n\n//# sourceURL=webpack://gd-sprest/./build/helper/methods/hasPermissions.js?')},"./build/helper/methods/index.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nvar __createBinding = this && this.__createBinding || (Object.create ? function (o, m, k, k2) {\n  if (k2 === undefined) k2 = k;\n  Object.defineProperty(o, k2, {\n    enumerable: true,\n    get: function get() {\n      return m[k];\n    }\n  });\n} : function (o, m, k, k2) {\n  if (k2 === undefined) k2 = k;\n  o[k2] = m[k];\n});\n\nvar __exportStar = this && this.__exportStar || function (m, exports) {\n  for (var p in m) {\n    if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);\n  }\n};\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\n\n__exportStar(__webpack_require__(/*! ./addContentEditorWebPart */ "./build/helper/methods/addContentEditorWebPart.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./addPermissionLevel */ "./build/helper/methods/addPermissionLevel.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./addScriptEditorWebPart */ "./build/helper/methods/addScriptEditorWebPart.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./copyPermissionLevel */ "./build/helper/methods/copyPermissionLevel.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./createContentType */ "./build/helper/methods/createContentType.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./createDocSet */ "./build/helper/methods/createDocSet.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./getCurrentTheme */ "./build/helper/methods/getCurrentTheme.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./hasPermissions */ "./build/helper/methods/hasPermissions.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./loadSPCore */ "./build/helper/methods/loadSPCore.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./parse */ "./build/helper/methods/parse.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./request */ "./build/helper/methods/request.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./setContentTypeFields */ "./build/helper/methods/setContentTypeFields.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./setGroupOwner */ "./build/helper/methods/setGroupOwner.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./setWebProperty */ "./build/helper/methods/setWebProperty.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./stringify */ "./build/helper/methods/stringify.js"), exports);\n\n//# sourceURL=webpack://gd-sprest/./build/helper/methods/index.js?')},"./build/helper/methods/loadSPCore.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.loadSPCore = void 0;\n\nvar executor_1 = __webpack_require__(/*! ../executor */ "./build/helper/executor.js");\n/**\r\n * Loads the core SharePoint JavaScript library for JSOM.\r\n */\n\n\nexports.loadSPCore = function (additionalLibs) {\n  // Return a promise\n  return new Promise(function (resolve) {\n    // Define the core libraries to load\n    var libs = ["init", "MicrosoftAjax", "SP.Runtime", "SP", "SP.Init"]; // See if we are loading additional libraries\n\n    if (additionalLibs && additionalLibs.length > 0) {\n      // Append the libs\n      libs = libs.concat(additionalLibs);\n    } // Parse the libraries\n\n\n    var counter = 0;\n    executor_1.Executor(libs, function (libName) {\n      // Return a promise\n      return new Promise(function (resolve) {\n        // See if a script already exists\n        if (document.querySelector("script[title=\'" + libName + "\']") == null) {\n          // Log\n          console.debug("[gd-sprest] Loading the core library: " + libName); // Create the script element to load the library\n\n          var elScript_1 = document.createElement("script");\n          elScript_1.title = libName;\n          elScript_1.src = document.location.origin + "/_layouts/15/" + libName + ".js"; // Wait for the previous library to load\n\n          setTimeout(function () {\n            // Load the script\n            document.head.appendChild(elScript_1); // Resolve the request\n\n            resolve(null);\n          }, 10 * counter++);\n        } else {\n          // Log\n          console.debug("[gd-sprest] Core library already loaded: " + libName);\n        }\n      }).then(function () {\n        // Resolve the request after all the libraries have been requested\n        resolve();\n      });\n    });\n  });\n};\n\n//# sourceURL=webpack://gd-sprest/./build/helper/methods/loadSPCore.js?')},"./build/helper/methods/parse.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.parse = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../../utils */ "./build/utils/index.js");\n/**\r\n * Convert a JSON string to a base object\r\n */\n\n\nexports.parse = function (jsonString) {\n  // Try to parse the string\n  try {\n    var obj = JSON.parse(jsonString); // Create a base object\n\n    var base = new utils_1.Base(obj.props); // Set the properties\n\n    base.response = obj.response;\n    base.status = obj.status;\n    base.targetInfo = obj.targetInfo; // Update the object\n\n    utils_1.Request.updateDataObject(base, false); // Return the base object\n\n    return base;\n  } catch (_a) {}\n\n  return null;\n};\n\n//# sourceURL=webpack://gd-sprest/./build/helper/methods/parse.js?')},"./build/helper/methods/request.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.request = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../../utils */ "./build/utils/index.js");\n/**\r\n * XML HTTP Request\r\n */\n\n\nexports.request = function (props) {\n  // Return a promise\n  return new Promise(function (resolve, reject) {\n    // Execute the request and resolve the promise\n    new utils_1.Base({\n      method: props.method || "GET",\n      url: props.url,\n      requestHeader: props.headers,\n      data: props.data\n    }).execute(resolve, reject);\n  });\n};\n\n//# sourceURL=webpack://gd-sprest/./build/helper/methods/request.js?')},"./build/helper/methods/setContentTypeFields.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.setContentTypeFields = void 0;\n\nvar lib_1 = __webpack_require__(/*! ../../lib */ "./build/lib/index.js");\n/**\r\n * Sets the field links associated with a content type.\r\n * @param ctInfo - The content type information\r\n */\n\n\nexports.setContentTypeFields = function (ctInfo) {\n  // Clears the content type field links\n  var clearLinks = function clearLinks() {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Get the links\n      getLinks().then(function (fieldLinks) {\n        var skipFields = []; // See if we need to remove any fields\n\n        if (fieldLinks.length > 0) {\n          var updateFl = false; // Set the context\n\n          var ctx = ctInfo.webUrl ? new SP.ClientContext(ctInfo.webUrl) : new SP.ClientContext(lib_1.ContextInfo.webServerRelativeUrl); // Get the source\n\n          var src = ctInfo.listName ? ctx.get_web().get_lists().getByTitle(ctInfo.listName) : ctx.get_web(); // Get the content type\n\n          var contentType = src.get_contentTypes().getById(ctInfo.id); // Parse the content type field links\n\n          for (var i = 0; i < fieldLinks.length; i++) {\n            var fieldLink = fieldLinks[i];\n            var removeFl = true;\n\n            var _loop_1 = function _loop_1(j) {\n              var field = ctInfo.fields[j];\n              var fieldName = typeof field === "string" ? field : field.Name || field.FieldInternalName; // See if we are keeping this field\n\n              if (fieldName == fieldLink.Name) {\n                var propUpdateFl_1 = false; // Checks if an update is needed\n\n                var updateField = function updateField(oldValue, newValue) {\n                  // Ensure a value exists\n                  if (newValue == null) {\n                    return;\n                  } // See if an update is needed\n\n\n                  if (oldValue == newValue) {\n                    return;\n                  } // Set the flag\n\n\n                  propUpdateFl_1 = true;\n                }; // Update the properties\n\n\n                updateField(fieldLink.DisplayName, field.DisplayName);\n                updateField(fieldLink.Hidden, field.Hidden);\n                updateField(fieldLink.ReadOnly, field.ReadOnly);\n                updateField(fieldLink.Required, field.Required);\n                updateField(fieldLink.ShowInDisplayForm, field.ShowInDisplayForm); // See if an update to the property is needed\n\n                if (!propUpdateFl_1) {\n                  // Set the flag to not remove this field reference\n                  removeFl = false; // Add the field to skip\n\n                  skipFields.push(fieldLink);\n                }\n\n                return "break";\n              }\n            }; // Parse the fields to add\n\n\n            for (var j = 0; j < ctInfo.fields.length; j++) {\n              var state_1 = _loop_1(j);\n\n              if (state_1 === "break") break;\n            } // See if we are removing the field\n\n\n            if (removeFl) {\n              // Remove the field link\n              contentType.get_fieldLinks().getById(fieldLink.Id).deleteObject(); // Set the flag\n\n              updateFl = true; // Log\n\n              console.log("[gd-sprest][Set Content Type Fields] Removing the field link: " + fieldLink.Name);\n            }\n          } // See if an update is required\n\n\n          if (updateFl) {\n            // Update the content type\n            contentType.update(false); // Execute the request\n\n            ctx.executeQueryAsync( // Success\n            function () {\n              // Log\n              console.log("[gd-sprest][Set Content Type Fields] Removed the field links successfully."); // Resolve the request\n\n              resolve(skipFields);\n            }, // Error\n            function (sender, args) {\n              // Log\n              console.log("[gd-sprest][Set Content Type Fields] Error removing the field links."); // Reject the request\n\n              reject();\n            });\n          } else {\n            // Log\n            console.log("[gd-sprest][Set Content Type Fields] No fields need to be removed."); // Resolve the request\n\n            resolve(skipFields);\n          }\n        } else {\n          // Resolve the request\n          resolve(skipFields);\n        }\n      }, reject);\n    });\n  }; // Creates the field links\n\n\n  var createLinks = function createLinks(skipFields) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Set the context\n      var ctx = ctInfo.webUrl ? new SP.ClientContext(ctInfo.webUrl) : new SP.ClientContext(lib_1.ContextInfo.webServerRelativeUrl); // Get the source\n\n      var src = ctInfo.listName ? ctx.get_web().get_lists().getByTitle(ctInfo.listName) : ctx.get_web();\n\n      var skipField = function skipField(fieldName, fields) {\n        for (var i = 0; i < fields.length; i++) {\n          // See if we are skipping this field\n          if (fields[i].Name == fieldName) {\n            return true;\n          }\n        }\n      }; // Parse the fields to add\n\n\n      var fields = [];\n\n      for (var i = 0; i < ctInfo.fields.length; i++) {\n        var fieldInfo = ctInfo.fields[i];\n        var fieldName = typeof fieldInfo === "string" ? fieldInfo : fieldInfo.Name || fieldInfo.FieldInternalName; // See if we are skipping this field\n\n        if (skipField(fieldName, skipFields)) {\n          continue;\n        } // Load the field\n\n\n        var field = src.get_fields().getByInternalNameOrTitle(fieldName);\n        ctx.load(field); // Log\n\n        console.log("[gd-sprest][Set Content Type Fields] Adding the field link: " + fieldName); // Save a reference to this field\n\n        fields.push({\n          ref: field,\n          info: fieldInfo\n        });\n      } // See if an update is needed\n\n\n      if (fields.length > 0) {\n        // Execute the request\n        ctx.executeQueryAsync(function () {\n          // Get the content type\n          var contentType = src.get_contentTypes().getById(ctInfo.id);\n          ctx.load(contentType); // Parse the fields\n\n          for (var i = 0; i < fields.length; i++) {\n            var field = fields[i];\n            /**\r\n             * The field link set_[property] methods don\'t seem to work. Setting the field information seems to be the only way.\r\n             * The read only property is the only one that doesn\'t seem to work.\r\n             */\n            // See if the field ref has properties to update\n\n            if (typeof field.info !== "string") {\n              // Update the field properties\n              field.info.DisplayName != null ? field.ref.set_title(field.info.DisplayName) : null;\n              field.info.Hidden != null ? field.ref.set_hidden(field.info.Hidden) : null;\n              field.info.ReadOnly != null ? field.ref.set_readOnlyField(field.info.ReadOnly) : null;\n              field.info.Required != null ? field.ref.set_required(field.info.Required) : null;\n              field.info.ShowInDisplayForm != null ? field.ref.setShowInDisplayForm(field.info.ShowInDisplayForm) : null;\n            } // Create the field link\n\n\n            var fieldLink = new SP.FieldLinkCreationInformation();\n            fieldLink.set_field(field.ref); // Add the field link to the content type\n\n            contentType.get_fieldLinks().add(fieldLink);\n          } // Update the content type\n\n\n          contentType.update(false); // Execute the request\n\n          ctx.executeQueryAsync( // Success\n          function () {\n            // Log\n            console.log("[gd-sprest][Set Content Type Fields] Added the field links successfully."); // Resolve the request\n\n            resolve();\n          }, // Error\n          function (sender, args) {\n            // Log\n            console.log("[gd-sprest][Set Content Type Fields] Error adding field references.", args.get_message()); // Reject the request\n\n            reject();\n          });\n        }, function (sender, args) {\n          // Log\n          console.log("[gd-sprest][Set Content Type Fields] Error getting field references.", args.get_message()); // Resolve the request\n\n          resolve();\n        });\n      } else {\n        // Log\n        console.log("[gd-sprest][Set Content Type Fields] No fields need to be added."); // Resolve the request\n\n        resolve();\n      }\n    });\n  }; // Gets the content type field links\n\n\n  var getLinks = function getLinks() {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      var ct = null; // See if list name exists\n\n      if (ctInfo.listName) {\n        // Get the list content type\n        ct = lib_1.Web(ctInfo.webUrl).Lists(ctInfo.listName).ContentTypes(ctInfo.id);\n      } else {\n        // Get the content type\n        ct = lib_1.Web(ctInfo.webUrl).ContentTypes(ctInfo.id);\n      } // Query the fields\n\n\n      ct.query({\n        Expand: ["Fields", "FieldLinks"],\n        Select: ["FieldLinks/DisplayName", "FieldLinks/Id", "FieldLinks/Name", "FieldLinks/Required", "FieldLinks/ReadOnly", "FieldLinks/ShowInDisplayForm", "Fields/InternalName", "Fields/IsDependentLookup"]\n      }).execute(function (ct) {\n        var fieldRefs = []; // Associated lookup fields error when removing from content types\n        // They may get removed when removing the main lookup field\n        // Parse the field links\n\n        for (var i = 0; i < ct.FieldLinks.results.length; i++) {\n          var addField = true;\n          var fieldLink = ct.FieldLinks.results[i]; // Find the field\n\n          for (var j = 0; j < ct.Fields.results.length; j++) {\n            var field = ct.Fields.results[j]; // See if this is the target field\n\n            if (field.InternalName == fieldLink.Name) {\n              // See if this is an associated lookup field\n              if (field.IsDependentLookup) {\n                // Set the flag\n                addField = false;\n              } // Break from the loop\n\n\n              break;\n            }\n          } // Add the field link\n\n\n          if (addField) {\n            fieldRefs.push(fieldLink);\n          }\n        } // Resolve the request\n\n\n        resolve(fieldRefs);\n      }, reject);\n      ct.FieldLinks().query({\n        Select: ["DisplayName", "Id", "Name", "Required", "ReadOnly", "ShowInDisplayForm"]\n      }).execute(function (fieldLinks) {}, reject);\n    });\n  }; // Set the order of the field references\n\n\n  var setOrder = function setOrder() {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Set the context\n      var ctx = ctInfo.webUrl ? new SP.ClientContext(ctInfo.webUrl) : new SP.ClientContext(lib_1.ContextInfo.webServerRelativeUrl); // Get the source\n\n      var src = ctInfo.listName ? ctx.get_web().get_lists().getByTitle(ctInfo.listName) : ctx.get_web(); // Get the content type\n\n      var contentType = src.get_contentTypes().getById(ctInfo.id); // Parse the fields to add\n\n      var fieldNames = [];\n\n      for (var i = 0; i < ctInfo.fields.length; i++) {\n        var fieldInfo = ctInfo.fields[i];\n        var fieldName = typeof fieldInfo === "string" ? fieldInfo : fieldInfo.Name || fieldInfo.FieldInternalName; // Add the field name\n\n        fieldNames.push(fieldName);\n      } // Reorder the content type\n\n\n      contentType.get_fieldLinks().reorder(fieldNames); // Update the content type\n\n      contentType.update(ctInfo.listName ? false : true); // Execute the request\n\n      ctx.executeQueryAsync( // Success\n      function () {\n        // Log\n        console.log("[gd-sprest][Set Content Type Fields] Updated the field order successfully."); // Resolve the request\n\n        resolve();\n      }, // Error\n      function (sender, args) {\n        // Log\n        console.log("[gd-sprest][Set Content Type Fields] Error updating the field order.", args.get_message()); // Reject the request\n\n        reject();\n      });\n    });\n  }; // Return a promise\n\n\n  return new Promise(function (resolve, reject) {\n    // Ensure the SP object exists\n    if (window["SP"]) {\n      // Ensure fields exist\n      if (ctInfo.fields) {\n        // Clear the links\n        clearLinks().then(function (skipFields) {\n          // Create the links\n          createLinks(skipFields).then(function () {\n            // Set the field order\n            setOrder().then(resolve, reject);\n          }, reject);\n        }, reject);\n      } else {\n        // Resolve the promise\n        resolve();\n      }\n    } else {\n      // Resolve the request\n      // This will cause issues in the SPConfig class\n      resolve();\n    }\n  });\n};\n\n//# sourceURL=webpack://gd-sprest/./build/helper/methods/setContentTypeFields.js?')},"./build/helper/methods/setGroupOwner.js":function(__unused_webpack_module,exports){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.setGroupOwner = void 0;\n/**\r\n * Sets the group owner\r\n * This uses JSOM to set a site group owner\'s property to another group. You can only set the owner to a user, using the REST API.\r\n */\n\nexports.setGroupOwner = function (groupName, ownerName, siteUrl) {\n  // Return a promise\n  return new Promise(function (resolve, reject) {\n    // Get the site groups\n    var context = siteUrl ? new SP.ClientContext(siteUrl) : SP.ClientContext.get_current();\n    var siteGroups = context.get_web().get_siteGroups(); // Get the groups\n\n    var group = siteGroups.getByName(groupName);\n    var owner = siteGroups.getByName(ownerName); // Set the owner\n\n    group.set_owner(owner); // Save the changes\n\n    group.update(); // Execute the request\n\n    context.executeQueryAsync(resolve, reject);\n  });\n};\n\n//# sourceURL=webpack://gd-sprest/./build/helper/methods/setGroupOwner.js?')},"./build/helper/methods/setWebProperty.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.setWebProperty = void 0;\n\nvar contextInfo_1 = __webpack_require__(/*! ../../lib/contextInfo */ "./build/lib/contextInfo.js");\n\nvar web_1 = __webpack_require__(/*! ../../lib/web */ "./build/lib/web.js");\n/**\r\n * Adds or updates a key/value pair for a web\'s property bag\r\n * This uses JSOM to set a web\'s property. The REST API doesn\'t support this at the moment.\r\n */\n\n\nexports.setWebProperty = function (key, value, indexed, siteUrl) {\n  var encodedKey = "vti_x005f_indexedpropertykeys";\n  var decodedKey = "vti_indexedpropertykeys"; // Return a promise\n\n  return new Promise(function (resolve, reject) {\n    // Gets the request digest value of the web\n    var getRequestDigest = function getRequestDigest() {\n      // Return a promise\n      return new Promise(function (resolve, reject) {\n        // See if the site url exists\n        if (siteUrl) {\n          contextInfo_1.ContextInfo.getWeb(siteUrl).execute(function (context) {\n            // Resolve the request\n            resolve(context.GetContextWebInformation.FormDigestValue);\n          }, reject);\n        } else {\n          // Resolve the request w/ the default digest value\n          resolve(contextInfo_1.ContextInfo.formDigestValue);\n        }\n      });\n    }; // Get the request digest for the web\n\n\n    getRequestDigest().then(function (requestDigest) {\n      // Convert the key to a base64 string\n      var idxKeyValue = "";\n\n      for (var i = 0; i < key.length; i++) {\n        idxKeyValue += key[i] + "\\x00";\n      }\n\n      idxKeyValue = btoa(idxKeyValue); // See if a value exists\n\n      if (value) {\n        // Add the property\n        web_1.Web(siteUrl, {\n          requestDigest: requestDigest\n        }).AllProperties().add(key, value).execute(function () {\n          // See if we are indexing the property\n          if (indexed) {\n            // Get the properties\n            web_1.Web(siteUrl, {\n              requestDigest: requestDigest\n            }).AllProperties().execute(function (propBag) {\n              // See if the indexed keys doesn\'t contain this value\n              var idxKeys = propBag[encodedKey] || "";\n\n              if (idxKeys.indexOf(idxKeyValue) < 0) {\n                // Append the value\n                web_1.Web(siteUrl, {\n                  requestDigest: requestDigest\n                }).AllProperties().add(decodedKey, idxKeys + idxKeyValue + "|").execute(function () {\n                  // Resolve the request\n                  resolve();\n                }, function () {\n                  // Reject the request\n                  reject("Error setting the indexed keys property.");\n                });\n              } else {\n                // Resolve the request\n                resolve();\n              }\n            });\n          } else {\n            // Resolve the request\n            resolve();\n          }\n        }, function () {\n          // Reject the request\n          reject("Error adding the web property.");\n        });\n      } else {\n        // Remove the property\n        web_1.Web(siteUrl, {\n          requestDigest: requestDigest\n        }).AllProperties().remove(key).execute(function () {\n          // See if we are indexing the property\n          if (indexed) {\n            // Get the properties\n            web_1.Web(siteUrl, {\n              requestDigest: requestDigest\n            }).AllProperties().execute(function (propBag) {\n              // See if the indexed keys contains the value\n              var idxKeys = propBag[encodedKey] || "";\n\n              if (idxKeys.indexOf(idxKeyValue) >= 0) {\n                // Remove the value\n                web_1.Web(siteUrl, {\n                  requestDigest: requestDigest\n                }).AllProperties().add(decodedKey, idxKeys.replace(idxKeyValue + "|", "")).execute(function () {\n                  // Resolve the request\n                  resolve();\n                }, function () {\n                  // Reject the request\n                  reject("Error setting the indexed keys property.");\n                });\n              } else {\n                // Resolve the request\n                resolve();\n              }\n            });\n          } else {\n            // Resolve the request\n            resolve();\n          }\n        }, function () {\n          // Reject the request\n          reject("Error removing the web property.");\n        });\n      }\n    }, function () {\n      // Reject the request\n      reject("Unable to get the context of this web.");\n    });\n  });\n};\n\n//# sourceURL=webpack://gd-sprest/./build/helper/methods/setWebProperty.js?')},"./build/helper/methods/stringify.js":function(__unused_webpack_module,exports){"use strict";eval('\n\nfunction _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.stringify = void 0;\n/**\r\n * Convert an object to a string\r\n */\n\nexports.stringify = function (obj) {\n  // Return the string\n  return JSON.stringify(obj, function (key, value) {\n    // Ensure a key exists\n    if (key) {\n      // See if this is a string or number, and return it\n      var valueType = _typeof(value);\n\n      if (valueType === "string" || valueType === "number") {\n        return value;\n      }\n\n      try {\n        // Try to parse it\n        JSON.parse(value); // Return the value\n\n        return value;\n      } catch (_a) {\n        // Don\'t include this key/value pair\n        return undefined;\n      }\n    } // Return the value\n\n\n    return value;\n  });\n};\n\n//# sourceURL=webpack://gd-sprest/./build/helper/methods/stringify.js?')},"./build/helper/ribbonLink.js":function(__unused_webpack_module,exports){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.RibbonLink = void 0;\n/**\r\n * Ribbon Link\r\n */\n\nexports.RibbonLink = function (props) {\n  // Creates the ribbon link\n  var create = function create() {\n    var link = null; // Default the append flag\n\n    var appendFl = typeof props.appendFl === "boolean" ? props.appendFl : false; // Get the link\n\n    link = _elTopBar.querySelector("#" + props.id);\n\n    if (link == null) {\n      // Create the link\n      link = document.createElement("a");\n      link.className = "ms-promotedActionButton " + (props.className || "");\n      link.href = props.href ? props.href : "javascript:void()";\n      link.innerHTML = "<span class=\'ms-promotedActionButton-text\'>" + props.title + "</span>";\n      link.id = props.id;\n      link.onclick = props.onClick; // Add the link\n\n      appendFl ? _elTopBar.appendChild(link) : _elTopBar.insertBefore(link, _elTopBar.firstChild);\n    } // Return the link\n\n\n    return link;\n  }; // Gets the top bar element\n\n\n  var _elTopBar = null;\n\n  var getTopBar = function getTopBar() {\n    // See if the bar exists\n    if (_elTopBar == null) {\n      // Set the element\n      _elTopBar = document.querySelector("#RibbonContainer-TabRowRight");\n    } // Return the element\n\n\n    return _elTopBar;\n  }; // Return a promise\n\n\n  return new Promise(function (resolve, reject) {\n    // See if the top bar exists\n    if (getTopBar()) {\n      // Create the link\n      var el = create();\n\n      if (el) {\n        // Resolve the promise\n        resolve(el);\n      }\n    } else if (window) {\n      // Wait for the window to be loaded\n      window.addEventListener("load", function () {\n        // See if the top bar exists\n        if (getTopBar()) {\n          // Create the link\n          var el = create();\n\n          if (el) {\n            // Resolve the promise\n            resolve(el);\n          }\n        }\n      });\n    }\n  });\n};\n\n//# sourceURL=webpack://gd-sprest/./build/helper/ribbonLink.js?')},"./build/helper/sbLink.js":function(__unused_webpack_module,exports){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.SuiteBarLink = void 0;\n/**\r\n * Suite Bar Link\r\n */\n\nexports.SuiteBarLink = function (props) {\n  // Creates the ribbon link\n  var create = function create() {\n    // Default the append flag\n    var appendFl = typeof props.appendFl === "boolean" ? props.appendFl : true; // Query for the link, and ensure it exists\n\n    var link = _elTopLinks.querySelector("#" + props.id);\n\n    if (link == null) {\n      // Create a list link\n      link = document.createElement("a");\n      link.className = "ms-core-suiteLink-a " + (props.className || "");\n      link.href = props.href ? props.href : "javascript:void()";\n      link.id = props.id;\n      link.innerHTML = props.title;\n      link.onclick = props.onClick; // Create the suite bar link\n\n      var sbLink = document.createElement("li");\n      sbLink.className = "ms-core-suiteLink";\n      sbLink.appendChild(link); // Append the item to the list\n\n      appendFl ? _elTopLinks.appendChild(sbLink) : _elTopLinks.insertBefore(sbLink, _elTopLinks.firstChild);\n    } // Return the link\n\n\n    return link;\n  }; // Gets the top links element\n\n\n  var _elTopLinks = null;\n\n  var getTopLinks = function getTopLinks() {\n    // See if the bar exists\n    if (_elTopLinks == null) {\n      // Set the element\n      _elTopLinks = document.querySelector("#suiteLinksBox > ul");\n    } // Return the element\n\n\n    return _elTopLinks;\n  }; // Return a promise\n\n\n  return new Promise(function (resolve, reject) {\n    // See if the top links exists\n    if (getTopLinks()) {\n      // Create the link\n      var el = create();\n\n      if (el) {\n        // Resolve the promise\n        resolve(el);\n      }\n    } else if (window) {\n      // Wait for the window to be loaded\n      window.addEventListener("load", function () {\n        // See if the top links exists\n        if (getTopLinks()) {\n          // Create the link\n          var el = create();\n\n          if (el) {\n            // Resolve the promise\n            resolve(el);\n          }\n        }\n      });\n    }\n  });\n};\n\n//# sourceURL=webpack://gd-sprest/./build/helper/sbLink.js?')},"./build/helper/spCfg.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nvar __createBinding = this && this.__createBinding || (Object.create ? function (o, m, k, k2) {\n  if (k2 === undefined) k2 = k;\n  Object.defineProperty(o, k2, {\n    enumerable: true,\n    get: function get() {\n      return m[k];\n    }\n  });\n} : function (o, m, k, k2) {\n  if (k2 === undefined) k2 = k;\n  o[k2] = m[k];\n});\n\nvar __exportStar = this && this.__exportStar || function (m, exports) {\n  for (var p in m) {\n    if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);\n  }\n};\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.SPConfig = void 0;\n\nvar lib_1 = __webpack_require__(/*! ../lib */ "./build/lib/index.js");\n\nvar __1 = __webpack_require__(/*! .. */ "./build/index.js");\n\nvar _1 = __webpack_require__(/*! . */ "./build/helper/index.js");\n\n__exportStar(__webpack_require__(/*! ./spCfgTypes */ "./build/helper/spCfgTypes.js"), exports);\n/**\r\n * SharePoint Configuration\r\n */\n\n\nexports.SPConfig = function (cfg, webUrl) {\n  // The selected configuration type to install\n  var _cfgType; // The request digest\n\n\n  var _requestDigest = null; // The target name to install/uninstall\n\n  var _targetName;\n  /**\r\n   * Logging\r\n   */\n\n\n  var logMessage = function logMessage(message, isError) {\n    if (isError === void 0) {\n      isError = false;\n    } // See if this is an error\n\n\n    if (isError) {\n      // Log the error\n      console.error(message);\n    } else {\n      console.log(message);\n    } // Call the event\n\n\n    cfg.onLogMessage ? cfg.onLogMessage(message, isError) : null;\n  };\n  /**\r\n   * Methods\r\n   */\n  // Method to create the content types\n\n\n  var createContentTypes = function createContentTypes(contentTypes, cfgContentTypes, list) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Ensure fields exist\n      if (cfgContentTypes == null || cfgContentTypes.length == 0) {\n        // Resolve the promise\n        resolve();\n        return;\n      } // Method to add an existing content type\n\n\n      var addParentCT = function addParentCT(cfg, parentInfo) {\n        // Return a promise\n        return new Promise(function (resolve, reject) {\n          // Add the content type\n          var contentTypes = list ? lib_1.Web(webUrl, {\n            disableCache: true,\n            requestDigest: _requestDigest\n          }).Lists(list.Title).ContentTypes() : lib_1.Web(webUrl).ContentTypes();\n          contentTypes.add({\n            Description: cfg.Description,\n            Group: cfg.Group,\n            Name: cfg.Name,\n            Id: {\n              __metadata: {\n                type: "SP.ContentTypeId"\n              },\n              StringValue: parentInfo.Id + (list ? "00" : "") + lib_1.ContextInfo.generateGUID().replace(/-/g, "")\n            }\n          }).execute(resolve, reject);\n        });\n      }; // Method to get the parent content type\n\n\n      var getParentCT = function getParentCT(ctName, url) {\n        // Return a promise\n        return new Promise(function (resolve, reject) {\n          // Get the web containing the parent content type\n          lib_1.Web(url, {\n            disableCache: true\n          }) // Get the content types\n          .ContentTypes() // Filter for the parent name\n          .query({\n            Filter: "Name eq \'" + ctName + "\'"\n          }) // Execute the request\n          .execute(function (cts) {\n            // See if the parent exists\n            if (cts.results[0]) {\n              // Resolve the promise\n              resolve({\n                Id: cts.results[0].Id.StringValue,\n                Url: url\n              });\n            } // Else, ensure this isn\'t the root web\n            else if (url != lib_1.ContextInfo.siteServerRelativeUrl) {\n              // Check the root web for the parent content type\n              getParentCT(ctName, lib_1.ContextInfo.siteServerRelativeUrl).then(resolve, reject);\n            } else {\n              // Reject the promise\n              reject();\n            }\n          }, reject);\n        });\n      }; // Parse the configuration\n\n\n      _1.Executor(cfgContentTypes, function (cfg) {\n        // Return a promise\n        return new Promise(function (resolve, reject) {\n          // See if this content type already exists\n          var ct = isInCollection("Name", cfg.Name, contentTypes.results);\n\n          if (ct) {\n            // Log\n            logMessage("[gd-sprest]" + (list ? "[" + list.Title + " List]" : "") + "[Content Type] The content type \'" + cfg.Name + "\' already exists."); // Update the configuration\n\n            cfg.ContentType = ct; // Resolve the promise and return\n\n            resolve(cfg);\n            return;\n          } // Log\n\n\n          logMessage("[gd-sprest]" + (list ? "[" + list.Title + " List]" : "") + "[Content Type] Creating the \'" + cfg.Name + "\' content type."); // See if the parent name exists\n\n          if (cfg.ParentName) {\n            getParentCT(cfg.ParentName, cfg.ParentWebUrl || webUrl).then( // Success\n            function (parentInfo) {\n              // Add the content type\n              addParentCT(cfg, parentInfo).then( // Success\n              function (ct) {\n                // Log\n                logMessage("[gd-sprest]" + (list ? "[" + list.Title + " List]" : "") + "[Content Type] The content type \'" + cfg.Name + "\' was created successfully."); // Update the configuration\n\n                cfg.ContentType = ct; // Trigger the event\n\n                cfg.onCreated ? cfg.onCreated(ct, list) : null; // Resolve the promise\n\n                resolve(cfg);\n              }, // Error\n              function (error) {\n                // Log\n                logMessage("[gd-sprest]" + (list ? "[" + list.Title + " List]" : "") + "[Content Type] The content type \'" + cfg.Name + "\' failed to be created.", error); // Reject the promise\n\n                reject(error);\n              });\n            }, // Error\n            function () {\n              // Log\n              logMessage("[gd-sprest]" + (list ? "[" + list.Title + " List]" : "") + "[Content Type] The parent content type \'" + cfg.ParentName + "\' was not found."); // Reject the promise\n\n              reject(ct.response);\n            });\n          } else {\n            // Create the content type\n            contentTypes.add({\n              Description: cfg.Description,\n              Group: cfg.Group,\n              Name: cfg.Name,\n              Id: {\n                __metadata: {\n                  type: "SP.ContentTypeId"\n                },\n                StringValue: cfg.Id || "0x0100" + lib_1.ContextInfo.generateGUID().replace(/-/g, "")\n              }\n            }).execute( // Success\n            function (ct) {\n              // Log\n              logMessage("[gd-sprest]" + (list ? "[" + list.Title + " List]" : "") + "[Content Type] The content type \'" + cfg.Name + "\' was created successfully."); // Update the configuration\n\n              cfg.ContentType = ct; // Trigger the event\n\n              cfg.onCreated ? cfg.onCreated(ct, list) : null; // Resolve the promise\n\n              resolve(cfg);\n            }, // Error\n            function (error) {\n              // Log\n              logMessage("[gd-sprest]" + (list ? "[" + list.Title + " List]" : "") + "[Content Type] The content type \'" + cfg.Name + "\' failed to be created.");\n              logMessage("[gd-sprest]" + (list ? "[" + list.Title + " List]" : "") + "[Content Type] Error: " + error.response, true); // Reject the promise\n\n              reject(error.response);\n            });\n          }\n        });\n      }).then(function () {\n        // Parse the configuration\n        _1.Executor(cfgContentTypes, function (cfgContentType) {\n          // Return a promise\n          return new Promise(function (resolve, reject) {\n            var cfgUpdate = {};\n            var updateFl = false; // Ensure the content type exists\n\n            if (cfgContentType.ContentType == null) {\n              // Skip this content type\n              resolve(null);\n              return;\n            } // Log\n\n\n            logMessage("[gd-sprest]" + (list ? "[" + list.Title + " List]" : "") + "[Content Type] Updating the field references for: " + cfgContentType.Name); // Create the field refs\n\n            _1.setContentTypeFields({\n              fields: cfgContentType.FieldRefs,\n              id: cfgContentType.ContentType.Id.StringValue,\n              listName: list ? list.Title : null,\n              webUrl: webUrl\n            }).then(function () {\n              /**\r\n               * See if we need to update the properties\r\n               */\n              // Custom form formatter\n              if (cfgContentType.ClientFormCustomFormatter != null && cfgContentType.ContentType.ClientFormCustomFormatter != cfgContentType.ClientFormCustomFormatter) {\n                // Update the configuration\n                cfgUpdate.ClientFormCustomFormatter = cfgContentType.ClientFormCustomFormatter; // Set the flag\n\n                updateFl = true;\n              } // Description\n\n\n              if (cfgContentType.Description != null && cfgContentType.ContentType.Description != cfgContentType.Description) {\n                // Update the configuration\n                cfgUpdate.Description = cfgContentType.Description; // Set the flag\n\n                updateFl = true;\n              } // Group\n\n\n              if (cfgContentType.Group != null && cfgContentType.ContentType.Group != cfgContentType.Group) {\n                // Update the configuration\n                cfgUpdate.Group = cfgContentType.Group; // Set the flag\n\n                updateFl = true;\n              } // JSLink\n\n\n              if (cfgContentType.JSLink != null && cfgContentType.ContentType.JSLink != cfgContentType.JSLink) {\n                // Update the configuration\n                cfgUpdate.JSLink = cfgContentType.JSLink; // Set the flag\n\n                updateFl = true;\n              } // Name\n\n\n              if (cfgContentType.Name != null && cfgContentType.ContentType.Name != cfgContentType.Name) {\n                // Update the configuration\n                cfgUpdate.Name = cfgContentType.Name; // Set the flag\n\n                updateFl = true;\n              } // See if an update is needed\n\n\n              if (updateFl) {\n                // Log\n                logMessage("[gd-sprest]" + (list ? "[" + list.Title + " List]" : "") + "[Content Type][" + cfgContentType.ContentType.Name + "] Updating the content type."); // Update the content type\n\n                cfgContentType.ContentType.update(cfgUpdate).execute(function () {\n                  // Log\n                  logMessage("[gd-sprest]" + (list ? "[" + list.Title + " List]" : "") + "[Content Type][" + cfgContentType.ContentType.Name + "] Update request completed."); // Trigger the event\n\n                  cfgContentType.onUpdated ? cfgContentType.onUpdated(cfgContentType.ContentType) : null; // Resolve this request\n\n                  resolve(null);\n                }, reject);\n              } else {\n                // Trigger the event\n                cfgContentType.onUpdated ? cfgContentType.onUpdated(cfgContentType.ContentType) : null; // Resolve this request\n\n                resolve(null);\n              }\n            }, reject);\n          });\n        }).then(resolve, reject);\n      }, reject);\n    });\n  }; // Method to create the fields\n\n\n  var createFields = function createFields(fields, cfgFields, list) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      var newFields = []; // Ensure fields exist\n\n      if (cfgFields == null || cfgFields.length == 0) {\n        // Resolve the promise\n        resolve();\n        return;\n      } // Parse the configuration\n\n\n      _1.Executor(cfgFields, function (cfg) {\n        return new Promise(function (resolve) {\n          // See if this field already exists\n          var field = isInCollection("InternalName", cfg.name, fields.results);\n\n          if (field) {\n            // Log\n            logMessage("[gd-sprest][Field] The field \'" + cfg.name + "\' already exists."); // Trigger the event\n\n            cfg.onUpdated ? cfg.onUpdated(field, list) : null; // Resolve the promise\n\n            resolve(null);\n          } else {\n            // Log\n            logMessage("[gd-sprest][Field] Creating the \'" + cfg.name + "\' field."); // See if this is an associated lookup field\n\n            var cfgLookup = cfg;\n\n            if (cfgLookup.type == _1.SPCfgFieldType.Lookup && cfgLookup.fieldRef) {\n              // Get the field reference\n              var fieldRef = isInCollection("InternalName", cfgLookup.fieldRef, fields.results) || isInCollection("InternalName", cfgLookup.fieldRef, newFields);\n\n              if (fieldRef) {\n                // Update the value to be the guid\n                cfgLookup.fieldRef = fieldRef.Id;\n              }\n            } // Compute the schema xml\n\n\n            _1.FieldSchemaXML(cfg, webUrl).then(function (response) {\n              var schemas = typeof response === "string" ? [response] : response; // Parse the fields to add\n\n              for (var i = 0; i < schemas.length; i++) {\n                // Add the field\n                fields.createFieldAsXml(schemas[i]).execute(function (field) {\n                  // See if it was successful\n                  if (field.InternalName) {\n                    // Log\n                    logMessage("[gd-sprest][Field] The field \'" + field.InternalName + "\' was created successfully."); // Save a reference to the field\n\n                    newFields.push(field); // Trigger the event\n\n                    cfg.onCreated ? cfg.onCreated(field, list) : null; // Resolve the promise\n\n                    resolve(null);\n                  } else {\n                    // Log\n                    logMessage("[gd-sprest][Field] The field \'" + cfg.name + "\' failed to be created.");\n                    logMessage("[gd-sprest][Field] Error: " + field.response, true); // Reject the promise\n\n                    reject();\n                  }\n                });\n              }\n            }, reject);\n          }\n        });\n      }).then(resolve);\n    });\n  }; // Method to create the lists\n\n\n  var createLists = function createLists(lists, cfgLists) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Execute code against each list configuration\n      _1.Executor(cfgLists, function (cfgList) {\n        // Return a promise\n        return new Promise(function (resolve) {\n          // See if the target name exists and matches this list\n          if (_cfgType && _targetName) {\n            // Ensure it\'s for this list\n            if (cfgList.ListInformation.Title.toLowerCase() != _targetName) {\n              // Do nothing\n              resolve(null);\n              return;\n            }\n          } // See if this list already exists\n\n\n          var list = isInCollection("Title", cfgList.ListInformation.Title, lists.results);\n\n          if (list) {\n            // Log\n            logMessage("[gd-sprest][List] The list \'" + cfgList.ListInformation.Title + "\' already exists."); // Resolve the promise and do nothing\n\n            resolve(null);\n            return;\n          } // Log\n\n\n          logMessage("[gd-sprest][List] Creating the \'" + cfgList.ListInformation.Title + "\' list."); // Update the list name and remove spaces\n\n          var listInfo = cfgList.ListInformation;\n          var listName = listInfo.Title;\n          listInfo.Title = cfgList.ListUrlName || listName.replace(/ /g, ""); // Add the list\n\n          lists.add(listInfo) // Execute the request\n          .execute(function (list) {\n            cfgList["_list"] = list; // Restore the list name in the configuration\n\n            listInfo.Title = listName; // See if the request was successful\n\n            if (list.Id) {\n              // See if we need to update the list\n              if (list.Title != listName) {\n                // Update the list\n                list.update({\n                  Title: listName\n                }).execute(function () {\n                  // Log\n                  logMessage("[gd-sprest][List] The list \'" + list.Title + "\' was created successfully."); // Resolve the promise\n\n                  resolve(null);\n                });\n              } else {\n                // Log\n                logMessage("[gd-sprest][List] The list \'" + list.Title + "\' was created successfully."); // Resolve the promise\n\n                resolve(null);\n              } // Trigger the event\n\n\n              cfgList.onCreating ? cfgList.onCreating(list) : null;\n            } else {\n              // Log\n              logMessage("[gd-sprest][List] The list \'" + listInfo.Title + "\' failed to be created.");\n              logMessage("[gd-sprest][List] Error: \'" + list.response); // Resolve the promise\n\n              resolve(null);\n            }\n          }, reject);\n        });\n      }).then(function () {\n        // Update the lists\n        updateLists(cfgLists).then(function () {\n          // Parse the lists\n          for (var i = 0; i < cfgLists.length; i++) {\n            var cfgList = cfgLists[i]; // Trigger the event\n\n            cfgList.onCreated ? cfgList.onCreated(cfgList["_list"]) : null;\n          } // Resolve the promise\n\n\n          resolve();\n        }, reject);\n      });\n    });\n  }; // Method to create the user custom actions\n\n\n  var createUserCustomActions = function createUserCustomActions(customActions, cfgCustomActions) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // See if the configuration type exists\n      if (_cfgType) {\n        // Ensure it\'s for this type\n        if (_cfgType != _1.SPCfgType.SiteUserCustomActions || _cfgType != _1.SPCfgType.WebUserCustomActions) {\n          // Resolve the promise\n          resolve();\n          return;\n        }\n      } // Ensure the lists exist\n\n\n      if (cfgCustomActions == null || cfgCustomActions.length == 0) {\n        // Resolve the promise and return it\n        resolve();\n        return;\n      } // Parse the configuration\n\n\n      _1.Executor(cfgCustomActions, function (cfg) {\n        // See if the target name exists\n        if (_cfgType && _targetName) {\n          // Ensure it\'s for this custom action\n          if (cfg.Name.toLowerCase() != _targetName || cfg.Title.toLowerCase() != _targetName) {\n            // Skip this custom action\n            return;\n          }\n        } // See if this custom action already exists\n\n\n        if (isInCollection("Name", cfg.Name, customActions.results)) {\n          // Log\n          logMessage("[gd-sprest][Custom Action] The custom action \'" + cfg.Name + "\' already exists.");\n        } else {\n          // See if rights exist\n          if (cfg.Rights) {\n            // Update the value\n            cfg.Rights = updateBasePermissions(cfg.Rights);\n          } // Add the custom action\n\n\n          customActions.add(cfg).execute(function (ca) {\n            // Ensure it exists\n            if (ca.existsFl) {\n              // Log\n              logMessage("[gd-sprest][Custom Action] The custom action \'" + ca.Name + "\' was created successfully.");\n            } else {\n              // Log\n              logMessage("[gd-sprest][Custom Action] The custom action \'" + ca.Name + "\' failed to be created.");\n              logMessage("[gd-sprest][Custom Action] Error: " + ca.response);\n            }\n          }, reject, true);\n        }\n      }).then(resolve);\n    });\n  }; // Method to create the list views\n\n\n  var createViews = function createViews(list, views, cfgViews) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Ensure the list views exist\n      if (cfgViews == null || cfgViews.length == 0) {\n        // Resolve the promise and return it\n        resolve();\n        return;\n      } // Parse the configuration\n\n\n      _1.Executor(cfgViews, function (cfg) {\n        // See if this view exists\n        var view = isInCollection("Title", cfg.ViewName, views.results);\n\n        if (view) {\n          // Log\n          logMessage("[gd-sprest][View] The view \'" + cfg.ViewName + "\' already exists.");\n        } else {\n          // Add the view\n          views.add({\n            DefaultView: cfg.Default,\n            Hidden: cfg.Hidden,\n            MobileDefaultView: cfg.MobileDefaultView,\n            MobileView: cfg.MobileView,\n            TabularView: cfg.Tabular,\n            Title: cfg.ViewName,\n            RowLimit: cfg.RowLimit,\n            ViewQuery: cfg.ViewQuery\n          }).execute(function (view) {\n            // Ensure it exists\n            if (view.existsFl) {\n              // Log\n              logMessage("[gd-sprest][View] The view \'" + cfg.ViewName + "\' was created successfully."); // Trigger the event\n\n              cfg.onCreated ? cfg.onCreated(view, list) : null;\n            } else {\n              // Log\n              logMessage("[gd-sprest][View] The view \'" + cfg.ViewName + "\' failed to be created.");\n              logMessage("[gd-sprest][View] Error: " + view.response);\n            }\n          }, reject, true);\n        }\n      }).then(function () {\n        // Update the views\n        updateViews(list, views, cfgViews).then(function () {\n          // Resolve the promise\n          resolve();\n        });\n      });\n    });\n  }; // Method to create the web parts\n\n\n  var createWebParts = function createWebParts() {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      var cfgWebParts = cfg.WebPartCfg; // Ensure fields exist\n\n      if (cfgWebParts == null || cfgWebParts.length == 0) {\n        // Resolve the promise\n        resolve();\n        return;\n      } // Log\n\n\n      logMessage("[gd-sprest][WebPart] Creating the web parts."); // Get the web\n\n      lib_1.Web(webUrl, {\n        disableCache: true,\n        requestDigest: _requestDigest\n      }) // Get the web part catalog\n      .getCatalog(__1.SPTypes.ListTemplateType.WebPartCatalog) // Get the root folder\n      .RootFolder() // Expand the files and items\n      .query({\n        Expand: ["Files"]\n      }) // Execute the request\n      .execute(function (folder) {\n        var ctr = 0;\n\n        var _loop_1 = function _loop_1(i) {\n          var cfgWebPart = cfgWebParts[i]; // See if the target name exists\n\n          if (_cfgType && _targetName) {\n            // Ensure it\'s for this list\n            if (cfgWebPart.FileName.toLowerCase() != _targetName) {\n              return "continue";\n            }\n          } // The post execute method\n\n\n          var postExecute = function postExecute() {\n            // Increment the counter\n            if (++ctr >= cfgWebParts.length) {\n              // Resolve the promise\n              resolve();\n            }\n          }; // See if this webpart exists\n\n\n          var file = isInCollection("Name", cfgWebPart.FileName, folder.Files.results);\n\n          if (file.Name) {\n            // Log\n            logMessage("[gd-sprest][WebPart] The webpart \'" + cfgWebPart.FileName + "\' already exists."); // Trigger the event\n\n            cfgWebPart.onUpdated ? cfgWebPart.onUpdated(file) : null; // Execute the post event\n\n            postExecute();\n          } else {\n            // Trim the xml\n            var xml = cfgWebPart.XML.trim(); // Convert the string to an array buffer\n\n            var buffer = new ArrayBuffer(xml.length * 2);\n            var bufferView = new Uint16Array(buffer);\n\n            for (var j = 0; j < xml.length; j++) {\n              bufferView[j] = xml.charCodeAt(j);\n            } // Create the webpart, but execute the requests one at a time\n\n\n            folder.Files.add(cfgWebPart.FileName, true, buffer).execute( // Success\n            function (file) {\n              // See if group exists\n              if (cfgWebPart.Group) {\n                // Set the target to the root web\n                lib_1.Web(lib_1.ContextInfo.siteServerRelativeUrl, {\n                  disableCache: true\n                }) // Get the web part catalog\n                .getCatalog(__1.SPTypes.ListTemplateType.WebPartCatalog) // Get the Items\n                .Items() // Query for this webpart\n                .query({\n                  Filter: "FileLeafRef eq \'" + cfgWebPart.FileName + "\'"\n                }) // Execute the request\n                .execute(function (items) {\n                  // Update the item\n                  items.results[0].update({\n                    Group: cfgWebPart.Group\n                  }).execute(postExecute);\n                });\n              } // Log\n\n\n              logMessage("[gd-sprest][WebPart] The \'" + file.Name + "\' webpart file was uploaded successfully."); // Trigger the event\n\n              cfgWebPart.onCreated ? cfgWebPart.onCreated(file) : null;\n            }, // Error\n            function () {\n              // Log\n              logMessage("[gd-sprest][WebPart] The \'" + file.Name + "\' webpart file upload failed."); // Skip this webpart\n\n              resolve();\n            });\n          }\n        }; // Parse the configuration\n\n\n        for (var i = 0; i < cfgWebParts.length; i++) {\n          _loop_1(i);\n        }\n      }, reject);\n    });\n  }; // Method to see if an object exists in a collection\n\n\n  var isInCollection = function isInCollection(key, value, collection) {\n    // Ensure a value exists\n    if (value) {\n      var valueLower = value ? value.toLowerCase() : ""; // Parse the collection\n\n      for (var i = 0; i < collection.length; i++) {\n        var keyValue = collection[i][key];\n        keyValue = keyValue ? keyValue.toLowerCase() : ""; // See if the item exists\n\n        if (valueLower == keyValue) {\n          // Return true\n          return collection[i];\n        }\n      }\n    } // Not in the collection\n\n\n    return false;\n  }; // Method to remove the content type\n\n\n  var removeContentTypes = function removeContentTypes(contentTypes, cfgContentTypes) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Ensure the content types exist\n      if (cfgContentTypes == null || cfgContentTypes.length == 0) {\n        // Resolve the promise and return it\n        resolve();\n        return;\n      } // Parse the configuration\n\n\n      _1.Executor(cfgContentTypes, function (cfg) {\n        // Get the field\n        var ct = isInCollection("Name", cfg.Name, contentTypes.results);\n\n        if (ct) {\n          // Remove the field\n          ct["delete"]().execute(function () {\n            // Log\n            logMessage("[gd-sprest][Content Type] The content type \'" + ct.Name + "\' was removed.");\n          }, reject, true);\n        }\n      }).then(resolve);\n    });\n  }; // Method to remove the fields\n\n\n  var removeFields = function removeFields(fields, cfgFields) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Ensure the fields exist\n      if (cfgFields == null || cfgFields.length == 0) {\n        // Resolve the promise and return it\n        resolve();\n        return;\n      } // Parse the configuration\n\n\n      _1.Executor(cfgFields, function (cfg) {\n        // Get the field\n        var field = isInCollection("InternalName", cfg.name, fields.results);\n\n        if (field) {\n          // Remove the field\n          field["delete"]().execute(function () {\n            // Log\n            logMessage("[gd-sprest][Field] The field \'" + field.InternalName + "\' was removed.");\n          }, reject, true);\n        }\n      }).then(resolve);\n    });\n  }; // Method to remove the lists\n\n\n  var removeLists = function removeLists(lists, cfgLists) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // See if the configuration type exists\n      if (_cfgType) {\n        // Ensure it\'s for this type\n        if (_cfgType != _1.SPCfgType.Lists) {\n          // Resolve the promise\n          resolve();\n          return;\n        }\n      } // Ensure the lists exist\n\n\n      if (cfgLists == null || cfgLists.length == 0) {\n        // Resolve the promise and return it\n        resolve();\n        return;\n      } // Parse the configuration\n\n\n      _1.Executor(cfgLists, function (cfg) {\n        // See if the target name exists\n        if (_cfgType && _targetName) {\n          // Ensure it\'s for this list\n          if (cfg.ListInformation.Title.toLowerCase() != _targetName) {\n            // Skip this list\n            return;\n          }\n        } // Get the list\n\n\n        var list = isInCollection("Title", cfg.ListInformation.Title, lists.results);\n\n        if (list) {\n          // Remove the list\n          list["delete"]().execute(function () {\n            // Log\n            logMessage("[gd-sprest][List] The list \'" + list.Title + "\' was removed.");\n          }, reject, true);\n        }\n      }).then(resolve);\n    });\n  }; // Method to remove the user custom actions\n\n\n  var removeUserCustomActions = function removeUserCustomActions(customActions, cfgCustomActions) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // See if the configuration type exists\n      if (_cfgType) {\n        // Ensure it\'s for this type\n        if (_cfgType != _1.SPCfgType.SiteUserCustomActions || _cfgType != _1.SPCfgType.WebUserCustomActions) {\n          // Resolve the promise\n          resolve();\n          return;\n        }\n      } // Ensure the custom actions exist\n\n\n      if (cfgCustomActions == null || cfgCustomActions.length == 0) {\n        // Resolve the promise and return it\n        resolve();\n        return;\n      } // Parse the configuration\n\n\n      _1.Executor(cfgCustomActions, function (cfg) {\n        // See if the target name exists\n        if (_cfgType && _targetName) {\n          // Ensure it\'s for this custom action\n          if (cfg.Name.toLowerCase() != _targetName || cfg.Title.toLowerCase() != _targetName) {\n            // Skip this custom action\n            return;\n          }\n        } // Get the custom action\n\n\n        var ca = isInCollection("Name", cfg.Name, customActions.results);\n\n        if (ca) {\n          // Remove the custom action\n          ca["delete"]().execute(function () {\n            // Log\n            logMessage("[gd-sprest][Custom Action] The custom action \'" + ca.Name + "\' was removed.");\n          }, reject, true);\n        }\n      }).then(resolve);\n    });\n  }; // Method to remove the web parts\n\n\n  var removeWebParts = function removeWebParts(site) {\n    var cfgWebParts = cfg.WebPartCfg; // Return a promise\n\n    return new Promise(function (resolve, reject) {\n      // See if the configuration type exists\n      if (_cfgType) {\n        // Ensure it\'s for this type\n        if (_cfgType != _1.SPCfgType.WebParts) {\n          // Resolve the promise\n          resolve();\n          return;\n        }\n      } // Ensure the configuration exists\n\n\n      if (cfgWebParts == null || cfgWebParts.length == 0) {\n        // Resolve the promise and return it\n        resolve();\n        return;\n      } // Log\n\n\n      logMessage("[gd-sprest][WebPart] Removing the web parts."); // Get the webpart gallery from the root web\n\n      site.RootWeb().getCatalog(__1.SPTypes.ListTemplateType.WebPartCatalog) // Get the root folder\n      .RootFolder() // Expand the files\n      .Files() // Execute the request\n      .execute(function (files) {\n        var _loop_2 = function _loop_2(i) {\n          var cfgWebPart = cfgWebParts[i]; // See if the target name exists\n\n          if (_cfgType && _targetName) {\n            // Ensure it\'s for this webpart\n            if (cfgWebPart.FileName.toLowerCase() != _targetName) {\n              return "continue";\n            }\n          } // Get the file\n\n\n          var file = isInCollection("Name", cfgWebPart.FileName, files.results);\n\n          if (file) {\n            // Remove the file\n            file["delete"]().execute(function () {\n              // Log\n              logMessage("[gd-sprest][WebPart] The webpart \'" + file.Name + "\' file was removed.");\n            }, true);\n          }\n        }; // Parse the configuration\n\n\n        for (var i = 0; i < cfgWebParts.length; i++) {\n          _loop_2(i);\n        } // Resolve the promise\n\n\n        resolve();\n      }, reject);\n    });\n  }; // Method to get the web information\n\n\n  var setRequestDigest = function setRequestDigest() {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      if (webUrl) {\n        // Get the web context information\n        lib_1.ContextInfo.getWeb(webUrl).execute(function (webInfo) {\n          _requestDigest = webInfo.GetContextWebInformation.FormDigestValue; // Resolve the promise\n\n          resolve();\n        }, reject);\n      } else {\n        // Resolve the promise\n        resolve();\n      }\n    });\n  }; // Method to update the base permissions\n\n\n  var updateBasePermissions = function updateBasePermissions(values) {\n    var high = values.High;\n    var low = values.Low; // See if this is an array\n\n    for (var i = 0; i < values["length"]; i++) {\n      var value = values[i]; // See if this is the full mask\n\n      if (value == 65) {\n        // Set the values\n        low = 65535;\n        high = 32767; // Break from the loop\n\n        break;\n      } // Else, see if it\'s empty\n      else if (value == 0) {\n        // Clear the values\n        low = 0;\n        high = 0;\n      } // Else, update the base permission\n      else {\n        var bit = value - 1;\n        var bitValue = 1; // Validate the bit\n\n        if (bit < 0) {\n          continue;\n        } // See if it\'s a low permission\n\n\n        if (bit < 32) {\n          // Compute the value\n          bitValue = bitValue << bit; // Set the low value\n\n          low |= bitValue;\n        } // Else, it\'s a high permission\n        else {\n          // Compute the value\n          bitValue = bitValue << bit - 32; // Set the high value\n\n          high |= bitValue;\n        }\n      }\n    } // Return the base permission\n\n\n    return {\n      Low: low.toString(),\n      High: high.toString()\n    };\n  }; // Method to update the lists\n\n\n  var updateLists = function updateLists(cfgLists) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      var request = function request(idx, resolve) {\n        // Get the list configuration\n        var cfgList = cfgLists[idx]; // See if the target name exists\n\n        if (_targetName) {\n          // Ensure it\'s for this list\n          if (cfgList.ListInformation.Title.toLowerCase() != _targetName) {\n            // Update the next list\n            request(idx + 1, resolve);\n            return;\n          }\n        } // Ensure the configuration exists\n\n\n        if (cfgList) {\n          // Get the web\n          lib_1.Web(webUrl, {\n            disableCache: true,\n            requestDigest: _requestDigest\n          }) // Get the list\n          .Lists(cfgList.ListInformation.Title) // Expand the content types, fields and views\n          .query({\n            Expand: ["ContentTypes", "Fields", "UserCustomActions", "Views"]\n          }) // Execute the request\n          .execute(function (list) {\n            // Update the title field\n            updateListTitleField(list, cfgList).then(function () {\n              // Create the fields\n              createFields(list.Fields, cfgList.CustomFields, list).then(function () {\n                // Create the content types\n                createContentTypes(list.ContentTypes, cfgList.ContentTypes, list).then(function () {\n                  // Update the views\n                  createViews(list, list.Views, cfgList.ViewInformation).then(function () {\n                    // Update the views\n                    createUserCustomActions(list.UserCustomActions, cfgList.UserCustomActions).then(function () {\n                      // Trigger the event\n                      cfgList.onUpdated ? cfgList.onUpdated(list) : null; // Update the next list\n\n                      request(idx + 1, resolve);\n                    }, reject);\n                  }, reject);\n                }, reject);\n              }, reject);\n            }, reject);\n          }, reject);\n        } else {\n          // Resolve the promise\n          resolve();\n        }\n      }; // Execute the request\n\n\n      request(0, resolve);\n    });\n  }; // Method to update the list title field\n\n\n  var updateListTitleField = function updateListTitleField(list, cfgList) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Ensure an update is required\n      if (cfgList.TitleFieldDefaultValue || cfgList.TitleFieldDescription || cfgList.TitleFieldDisplayName || cfgList.TitleFieldIndexed || cfgList.TitleFieldRequired || cfgList.TitleFieldUniqueValues) {\n        var values = {}; // See if we are setting a default value\n\n        if (cfgList.TitleFieldDefaultValue) {\n          // Update the values\n          values["DefaultValue"] = cfgList.TitleFieldDefaultValue;\n        } // See if we are setting the description\n\n\n        if (cfgList.TitleFieldDescription) {\n          // Update the values\n          values["Description"] = cfgList.TitleFieldDescription;\n        } // See if the title field is being updated\n\n\n        if (cfgList.TitleFieldDisplayName) {\n          // Update the values\n          values["Title"] = cfgList.TitleFieldDisplayName;\n        } // See if we are indexing the field\n        // Note - To enforce unique values, the field must be indexed\n\n\n        if (cfgList.TitleFieldUniqueValues || cfgList.TitleFieldIndexed) {\n          // Update the values\n          values["Indexed"] = true;\n        } // See if we are requiring a value\n\n\n        if (typeof cfgList.TitleFieldRequired === "boolean") {\n          // Update the values\n          values["Required"] = cfgList.TitleFieldRequired;\n        } // See if we are enforcing unique values\n\n\n        if (cfgList.TitleFieldUniqueValues) {\n          // Update the values\n          values["EnforceUniqueValues"] = true;\n        } // Update the field name\n\n\n        list.Fields.getByInternalNameOrTitle("Title").update(values).execute(function () {\n          // Log\n          cfgList.TitleFieldDefaultValue ? logMessage("[gd-sprest][List] The \'Title\' field\'s default value was updated to \'" + cfgList.TitleFieldDefaultValue + "\'.") : null;\n          cfgList.TitleFieldDescription ? logMessage("[gd-sprest][List] The \'Title\' field\'s description was updated to \'" + cfgList.TitleFieldDescription + "\'.") : null;\n          cfgList.TitleFieldDisplayName ? logMessage("[gd-sprest][List] The \'Title\' field\'s display name was updated to \'" + cfgList.TitleFieldDisplayName + "\'.") : null;\n          cfgList.TitleFieldIndexed ? logMessage("[gd-sprest][List] The \'Title\' field\'s has been indexed.") : null;\n          cfgList.TitleFieldRequired ? logMessage("[gd-sprest][List] The \'Title\' field\'s required flag was updated to \'" + cfgList.TitleFieldRequired + "\'.") : null;\n          cfgList.TitleFieldUniqueValues ? logMessage("[gd-sprest][List] The \'Title\' field\'s unique values flag was updated to \'" + cfgList.TitleFieldUniqueValues + "\'.") : null; // Resolve the promise\n\n          resolve();\n        }, reject);\n      } else {\n        // Resolve the promise\n        resolve();\n      }\n    });\n  }; // Method to update the views\n\n\n  var updateViews = function updateViews(list, views, cfgViews) {\n    // Return a promise\n    return new Promise(function (resolve) {\n      // Parse the configuration\n      _1.Executor(cfgViews, function (cfg) {\n        // Return a promise\n        return new Promise(function (resolve) {\n          // Get the view\n          var view = views.getByTitle(cfg.ViewName); // See if the view fields are defined\n\n          if (cfg.ViewFields && cfg.ViewFields.length > 0) {\n            // Log\n            logMessage("[gd-sprest][View] Updating the view fields for the \'" + cfg.ViewName + "\' view."); // Clear the view fields\n\n            view.ViewFields().removeAllViewFields().execute(true); // Parse the view fields\n\n            for (var i = 0; i < cfg.ViewFields.length; i++) {\n              // Log\n              logMessage("[gd-sprest][View] Add view field \'" + cfg.ViewFields[i] + "\' to the view."); // Add the view field\n\n              view.ViewFields().addViewField(cfg.ViewFields[i]).execute(true);\n            }\n          } // See if we are updating the view properties\n\n\n          if (typeof cfg.Default === "boolean" || typeof cfg.Hidden === "boolean" || typeof cfg.MobileDefaultView === "boolean" || typeof cfg.MobileView === "boolean" || typeof cfg.Tabular === "boolean" || cfg.JSLink || cfg.RowLimit > 0 || cfg.ViewQuery) {\n            var props = {}; // Log\n\n            logMessage("[gd-sprest][View] Updating the view properties for the \'" + cfg.ViewName + "\' view."); // Set the properties\n\n            typeof cfg.Default === "boolean" ? props["DefaultView"] = cfg.Default : null;\n            typeof cfg.Hidden === "boolean" ? props["Hidden"] = cfg.Hidden : null;\n            typeof cfg.MobileDefaultView === "boolean" ? props["MobileDefaultView"] = cfg.MobileDefaultView : null;\n            typeof cfg.MobileView === "boolean" ? props["MobileView"] = cfg.MobileView : null;\n            typeof cfg.Tabular === "boolean" ? props["TabularView"] = cfg.Tabular : null;\n            cfg.CustomFormatter ? props["CustomFormatter"] = cfg.CustomFormatter : null;\n            cfg.JSLink ? props["JSLink"] = cfg.JSLink : null;\n            cfg.RowLimit > 0 ? props["RowLimit"] = cfg.RowLimit : null;\n            cfg.ViewQuery ? props["ViewQuery"] = cfg.ViewQuery : null; // Update the view\n\n            view.update(props).execute(true);\n          } // Wait for the requests to complete\n\n\n          view.done(function () {\n            var args = [];\n\n            for (var _i = 0; _i < arguments.length; _i++) {\n              args[_i] = arguments[_i];\n            } // Log\n\n\n            logMessage("[gd-sprest][View] The updates for the \'" + cfg.ViewName + "\' view has completed."); // Trigger the event\n\n            cfg.onUpdated ? cfg.onUpdated(view, list) : null; // Resolve the promise\n\n            resolve(null);\n          });\n        });\n      }).then(resolve);\n    });\n  }; // Method to uninstall the site components\n\n\n  var uninstallSite = function uninstallSite() {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Ensure we need to complete this request\n      if (cfg.CustomActionCfg != null && cfg.CustomActionCfg.Site != null || cfg.WebPartCfg != null) {\n        // Log\n        logMessage("[gd-sprest][uninstall] Loading the site information..."); // Get the site\n\n        lib_1.Site(webUrl, {\n          disableCache: true,\n          requestDigest: _requestDigest\n        }) // Expand the user custom actions\n        .query({\n          Expand: ["UserCustomActions"]\n        }) // Execute the request\n        .execute(function (site) {\n          // Remove the user custom actions\n          removeUserCustomActions(site.UserCustomActions, cfg.CustomActionCfg ? cfg.CustomActionCfg.Site : []).then(function () {\n            // Remove the webpart\n            removeWebParts(site).then(function () {\n              // Resolve the promise\n              resolve(site);\n            }, reject);\n          });\n        }, reject);\n      } else {\n        // Resolve the promise\n        resolve(null);\n      }\n    });\n  }; // Method to uninstall the web components\n\n\n  var uninstallWeb = function uninstallWeb() {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      var Expand = []; // Log\n\n      logMessage("[gd-sprest][uninstall] Loading the web information..."); // Set the query\n\n      if (cfg.ContentTypes) {\n        Expand.push("ContentTypes");\n      }\n\n      if (cfg.CustomActionCfg) {\n        Expand.push("UserCustomActions");\n      }\n\n      if (cfg.Fields) {\n        Expand.push("Fields");\n      }\n\n      if (cfg.ListCfg) {\n        Expand.push("Lists");\n      } // Query the web\n\n\n      lib_1.Web(webUrl, {\n        disableCache: true,\n        requestDigest: _requestDigest\n      }).query({\n        Expand: Expand\n      }) // Execute the request\n      .execute(function (web) {\n        // Remove the web custom actions\n        removeUserCustomActions(web.UserCustomActions, cfg.CustomActionCfg ? cfg.CustomActionCfg.Web : null).then(function () {\n          // Remove the lists\n          removeLists(web.Lists, cfg.ListCfg).then(function () {\n            // Remove the content types\n            removeContentTypes(web.ContentTypes, cfg.ContentTypes).then(function () {\n              // Remove the fields\n              removeFields(web.Fields, cfg.Fields).then(function () {\n                // Resolve the promise\n                resolve();\n              }, reject);\n            }, reject);\n          }, reject);\n        }, reject);\n      }, reject);\n    });\n  };\n  /**\r\n   * Public Interface\r\n   */\n\n\n  return {\n    // The configuration\n    _configuration: cfg,\n    // Method to get the web url to target\n    getWebUrl: function getWebUrl() {\n      return webUrl;\n    },\n    // Method to install the configuration\n    install: function install() {\n      var initSPLib = function initSPLib() {\n        return new Promise(function (resolve) {\n          // See if the SP lib exists\n          if (window["SP"]) {\n            resolve(null);\n            return;\n          } // Load the core and taxonomy libraries\n\n\n          _1.loadSPCore(["sp.taxonomy"]).then(function () {\n            resolve(null);\n          });\n        });\n      }; // Return a promise\n\n\n      return new Promise(function (resolve, reject) {\n        // Ensure the SP core lib exists\n        initSPLib().then(function () {\n          // Set the request digest\n          setRequestDigest().then(function () {\n            // Log\n            logMessage("[gd-sprest] Installing the web assets..."); // Get the web\n\n            var web = lib_1.Web(webUrl, {\n              disableCache: true,\n              requestDigest: _requestDigest\n            }); // Create the site fields\n\n            var createSiteFields = function createSiteFields() {\n              // Return a promise\n              return new Promise(function (resolve, reject) {\n                // See if we are creating fields\n                if (cfg.Fields && cfg.Fields.length > 0) {\n                  // Log\n                  logMessage("[gd-sprest][Fields] Starting the requests."); // Get the fields\n\n                  web.Fields().execute(function (fields) {\n                    // Create the fields\n                    createFields(fields, cfg.Fields).then(function () {\n                      // Log\n                      logMessage("[gd-sprest][Fields] Completed the requests."); // Resolve the promise\n\n                      resolve(null);\n                    }, reject);\n                  }, reject);\n                } else {\n                  // Resolve the promise\n                  resolve(null);\n                }\n              });\n            }; // Create the site content types\n\n\n            var createSiteContentTypes = function createSiteContentTypes() {\n              // Return a promise\n              return new Promise(function (resolve, reject) {\n                // See if we are creating the content types\n                if (cfg.ContentTypes && cfg.ContentTypes.length > 0) {\n                  // Log\n                  logMessage("[gd-sprest][Content Types] Starting the requests."); // Get the content types\n\n                  web.ContentTypes().execute(function (contentTypes) {\n                    // Create the content types\n                    createContentTypes(contentTypes, cfg.ContentTypes).then(function () {\n                      // Log\n                      logMessage("[gd-sprest][Content Types] Completed the requests."); // Resolve the promise\n\n                      resolve();\n                    }, reject);\n                  }, reject);\n                } else {\n                  // Resolve the promise\n                  resolve();\n                }\n              });\n            }; // Create the site lists\n\n\n            var createSiteLists = function createSiteLists() {\n              // Return a promise\n              return new Promise(function (resolve, reject) {\n                // See if we are creating the lists\n                if (cfg.ListCfg && cfg.ListCfg.length) {\n                  // Log\n                  logMessage("[gd-sprest][Lists] Starting the requests."); // Get the lists\n\n                  web.Lists().execute(function (lists) {\n                    // Create the lists\n                    createLists(lists, cfg.ListCfg).then(function () {\n                      // Log\n                      logMessage("[gd-sprest][Lists] Completed the requests."); // Resolve the promise\n\n                      resolve();\n                    }, reject);\n                  }, reject);\n                } else {\n                  // Resolve the promise\n                  resolve();\n                }\n              });\n            }; // Create the site webparts\n\n\n            var createSiteWebParts = function createSiteWebParts() {\n              // Return a promise\n              return new Promise(function (resolve, reject) {\n                // See if we are creating the webparts\n                if (cfg.WebPartCfg && cfg.WebPartCfg.length > 0) {\n                  // Log\n                  logMessage("[gd-sprest][WebParts] Starting the requests."); // Create the webparts\n\n                  createWebParts().then(function () {\n                    // Log\n                    logMessage("[gd-sprest][WebParts] Completed the requests."); // Resolve the promise\n\n                    resolve();\n                  }, reject);\n                } else {\n                  // Resolve the promise\n                  resolve();\n                }\n              });\n            }; // Create the custom actions\n\n\n            var createSiteCollectionCustomActions = function createSiteCollectionCustomActions() {\n              // Return a promise\n              return new Promise(function (resolve, reject) {\n                // See if we are targeting the site collection\n                if (cfg.CustomActionCfg && cfg.CustomActionCfg.Site) {\n                  // Log\n                  logMessage("[gd-sprest][Site Custom Actions] Starting the requests."); // Get the site\n\n                  lib_1.Site(webUrl, {\n                    disableCache: true,\n                    requestDigest: _requestDigest\n                  }) // Get the user custom actions\n                  .UserCustomActions().execute(function (customActions) {\n                    // Create the user custom actions\n                    createUserCustomActions(customActions, cfg.CustomActionCfg.Site).then(function () {\n                      // Log\n                      logMessage("[gd-sprest][Site Custom Actions] Completed the requests."); // Resolve the promise\n\n                      resolve();\n                    }, reject);\n                  }, reject);\n                } else {\n                  // Resolve the promise\n                  resolve();\n                }\n              });\n            }; // Create the custom actions\n\n\n            var createSiteCustomActions = function createSiteCustomActions() {\n              // Return a promise\n              return new Promise(function (resolve, reject) {\n                // See if we are targeting the web\n                if (cfg.CustomActionCfg && cfg.CustomActionCfg.Web) {\n                  // Log\n                  logMessage("[gd-sprest][Web Custom Actions] Starting the requests."); // Get the user custom actions\n\n                  web.UserCustomActions().execute(function (customActions) {\n                    // Create the user custom actions\n                    createUserCustomActions(customActions, cfg.CustomActionCfg.Web).then(function () {\n                      // Log\n                      logMessage("[gd-sprest][Web Custom Actions] Completed the requests."); // Resolve the promise\n\n                      resolve();\n                    });\n                  }, reject);\n                } else {\n                  // Resolve the promise\n                  resolve();\n                }\n              });\n            }; // Create the site fields\n\n\n            createSiteFields().then(function () {\n              // Create the site content types\n              createSiteContentTypes().then(function () {\n                // Create the site lists\n                createSiteLists().then(function () {\n                  // Create the webparts\n                  createSiteWebParts().then(function () {\n                    // Create the site collection custom actions\n                    createSiteCollectionCustomActions().then(function () {\n                      // Create the site custom actions\n                      createSiteCustomActions().then(function () {\n                        // Log\n                        logMessage("[gd-sprest] The configuration script completed, but some requests may still be running."); // Resolve the request\n\n                        resolve();\n                      }, reject);\n                    }, reject);\n                  }, reject);\n                }, reject);\n              }, reject);\n            }, reject);\n          }, reject);\n        });\n      });\n    },\n    // Method to update the web url to target\n    setWebUrl: function setWebUrl(url) {\n      webUrl = url;\n    },\n    // Method to uninstall the configuration\n    uninstall: function uninstall() {\n      // Return a promise\n      return new Promise(function (resolve, reject) {\n        // Set the request digest\n        setRequestDigest().then(function () {\n          // Uninstall the web components\n          uninstallWeb().then(function () {\n            // Uninstall the site components\n            uninstallSite().then(function () {\n              // Log\n              logMessage("[gd-sprest] The configuration script completed, but some requests may still be running."); // Resolve the promise\n\n              resolve();\n            }, reject);\n          }, reject);\n        });\n      });\n    }\n  };\n};\n\n//# sourceURL=webpack://gd-sprest/./build/helper/spCfg.js?')},"./build/helper/spCfgTypes.js":function(__unused_webpack_module,exports){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.SPCfgType = exports.SPCfgFieldType = void 0;\n/**\r\n * SharePoint Configuration Field Types\r\n */\n\nexports.SPCfgFieldType = {\n  Boolean: 0,\n  Calculated: 1,\n  Choice: 2,\n  Currency: 3,\n  Date: 4,\n  Geolocation: 5,\n  Guid: 6,\n  Image: 7,\n  Lookup: 8,\n  MMS: 9,\n  Note: 10,\n  Number: 11,\n  Text: 12,\n  Url: 13,\n  User: 14\n};\n/**\r\n * SharePoint Configuration Types\r\n * The value determines the order to install the object type.\r\n */\n\nexports.SPCfgType = {\n  Fields: 0,\n  ContentTypes: 1,\n  Lists: 2,\n  SiteUserCustomActions: 3,\n  WebParts: 5,\n  WebUserCustomActions: 4\n};\n\n//# sourceURL=webpack://gd-sprest/./build/helper/spCfgTypes.js?')},"./build/helper/sp/calloutManager.js":function(__unused_webpack_module,exports){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.CalloutManager = void 0;\n/**\r\n * Callout Manager\r\n */\n\nexports.CalloutManager = function () {\n  // Return the callout manager\n  return {\n    /** Closes all callouts on the page. */\n    closeAll: function closeAll() {\n      // Load the library and call the method\n      return window["CalloutManager"].closeAll();\n    },\n\n    /** Returns true if the associated callout is open. */\n    containsOneCalloutOpen: function containsOneCalloutOpen(el) {\n      // Load the library and call the method\n      return window["CalloutManager"].containsOneCalloutOpen(el);\n    },\n\n    /** Returns an callout action entry. */\n    createAction: function createAction(options) {\n      // Create the callout action options entry\n      var caOptions = new window["CalloutActionOptions"](); // Update the options\n\n      for (var key in options) {\n        // Set the option\n        caOptions[key] = options[key];\n      } // Create the action\n\n\n      return new window["CalloutAction"](caOptions);\n    },\n\n    /** Returns an callout action menu entries. */\n    createMenuEntries: function createMenuEntries(entries) {\n      var menuEntries = []; // Parse the action options\n\n      for (var i = 0; i < entries.length; i++) {\n        var entry = entries[i]; // Add the action\n\n        menuEntries.push(new window["CalloutActionMenuEntry"](entry.text, entry.onClickCallback, entry.iconUrl));\n      } // Return the action menu entries\n\n\n      return menuEntries;\n    },\n\n    /** Creates a new callout. */\n    createNew: function createNew(options) {\n      // Load the library and call the method\n      return window["CalloutManager"].createNew(options);\n    },\n\n    /** Checks if the callout id exists, before creating it. */\n    createNewIfNecessary: function createNewIfNecessary(options) {\n      // Load the library and call the method\n      return window["CalloutManager"].createNewIfNecessary(options);\n    },\n\n    /** Performs an action on each callout on the page. */\n    forEach: function forEach(callback) {\n      // Load the library and call the method\n      return window["CalloutManager"].forEach(callback);\n    },\n\n    /** Finds the closest launch point and returns the callout associated with it. */\n    getFromCalloutDescendant: function getFromCalloutDescendant(descendant) {\n      return window["CalloutManager"].getFromCalloutDescendant(descendant);\n    },\n\n    /** Returns the callout from the specified launch point. */\n    getFromLaunchPoint: function getFromLaunchPoint(launchPoint) {\n      return window["CalloutManager"].getFromLaunchPoint(launchPoint);\n    },\n\n    /** Returns the callout for the specified launch point, null if it doesn\'t exist. */\n    getFromLaunchPointIfExists: function getFromLaunchPointIfExists(launchPoint) {\n      return window["CalloutManager"].getFromLaunchPointIfExists(launchPoint);\n    },\n\n    /** Returns true if at least one callout is defined on the page is opened or opening. */\n    isAtLeastOneCalloutOn: function isAtLeastOneCalloutOn() {\n      return window["CalloutManager"].isAtLeastOneCalloutOn();\n    },\n\n    /** Returns true if at least one callout is opened on the page. */\n    isAtLeastOneCalloutOpen: function isAtLeastOneCalloutOpen() {\n      return window["CalloutManager"].isAtLeastOneCalloutOpen();\n    },\n    // Ensures the core library is loaded\n    init: function init() {\n      // Return a promise\n      return new Promise(function (resolve, reject) {\n        // See if the class exists\n        if (window["CalloutManager"]) {\n          resolve();\n        } else {\n          // Wait for the core script to be loaded\n          window["SP"].SOD.executeFunc("callout.js", "Callout", function () {\n            // Resolve the promise\n            resolve();\n          });\n        }\n      });\n    },\n\n    /** Removes the callout and destroys it. */\n    remove: function remove(callout) {\n      // Load the library and call the method\n      return window["CalloutManager"].remove(callout);\n    }\n  };\n}();\n\n//# sourceURL=webpack://gd-sprest/./build/helper/sp/calloutManager.js?')},"./build/helper/sp/index.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.SP = void 0;\n\nvar calloutManager_1 = __webpack_require__(/*! ./calloutManager */ "./build/helper/sp/calloutManager.js");\n\nvar modalDialog_1 = __webpack_require__(/*! ./modalDialog */ "./build/helper/sp/modalDialog.js");\n\nvar notify_1 = __webpack_require__(/*! ./notify */ "./build/helper/sp/notify.js");\n\nvar ribbon_1 = __webpack_require__(/*! ./ribbon */ "./build/helper/sp/ribbon.js");\n\nvar sod_1 = __webpack_require__(/*! ./sod */ "./build/helper/sp/sod.js");\n\nvar status_1 = __webpack_require__(/*! ./status */ "./build/helper/sp/status.js");\n\nexports.SP = {\n  CalloutManager: calloutManager_1.CalloutManager,\n  ModalDialog: modalDialog_1.ModalDialog,\n  Notify: notify_1.Notify,\n  Ribbon: ribbon_1.Ribbon,\n  SOD: sod_1.SOD,\n  Status: status_1.Status\n};\n\n//# sourceURL=webpack://gd-sprest/./build/helper/sp/index.js?')},"./build/helper/sp/modalDialog.js":function(__unused_webpack_module,exports){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.ModalDialog = void 0;\n/**\r\n * Modal Dialog\r\n */\n\nexports.ModalDialog = function () {\n  // Adds the custom methods to the dialog\n  var getDialog = function getDialog(dialog) {\n    // Shows the dialog\n    dialog.show = function () {\n      // Get the modal dialog element\n      var el = dialog.get_dialogElement();\n\n      if (el) {\n        // Show the dialog\n        el.style.display = "";\n      } // Get the iframe element\n\n\n      el = dialog.get_frameElement();\n\n      if (el) {\n        // Show the dialog\n        el.style.display = "";\n      }\n    }; // Updates the title\n\n\n    dialog.setTitle = function (value) {\n      // Get the title element\n      var elDlg = dialog.get_dialogElement();\n      var elTitle = elDlg ? elDlg.querySelector(".ms-dlgLoadingTextDiv .ms-core-pageTitle") : null;\n      elTitle = elTitle || elDlg.querySelector(".ms-dlgTitle .ms-dlgTitleText");\n\n      if (elTitle) {\n        // Update the title\n        elTitle.innerHTML = value;\n      }\n    }; // Updates the sub-title\n\n\n    dialog.setSubTitle = function (value) {\n      // Get the sub-title element\n      var elDlg = dialog.get_dialogElement();\n      var elSubTitle = elDlg ? elDlg.querySelector(".ms-dlgLoadingTextDiv ~ div") : null;\n\n      if (elSubTitle) {\n        // Update the sub-title\n        elSubTitle.innerHTML = value;\n      }\n    }; // Return the dialog\n\n\n    return dialog;\n  }; // Return the modal dialog\n\n\n  return {\n    // Close the dialog\n    commonModalDialogClose: function commonModalDialogClose(dialogResult, returnVal) {\n      // Load the library and call the method\n      exports.ModalDialog.load().then(function () {\n        window["SP"].UI.ModalDialog.commonModalDialogClose(dialogResult, returnVal);\n      });\n    },\n    // Open a dialog\n    commonModalDialogOpen: function commonModalDialogOpen(url, options, callback, args) {\n      // Load the library and call the method\n      exports.ModalDialog.load().then(function () {\n        window["SP"].UI.ModalDialog.commonModalDialogOpen(url, options, callback, args);\n      });\n    },\n    // Method to ensure the core library is loaded\n    load: function load() {\n      // Return a promise\n      return new Promise(function (resolve, reject) {\n        // See if the class exists\n        if (window["SP"] && window["SP"].UI && window["SP"].UI.ModalDialog) {\n          resolve();\n        } else {\n          // Wait for the core script to be loaded\n          window["SP"].SOD.executeFunc("sp.js", "SP.UI.ModalDialog", function () {\n            // Resolve the promise\n            resolve();\n          });\n        }\n      });\n    },\n    // Opens a pop-up page\n    OpenPopUpPage: function OpenPopUpPage(url, callback, width, height) {\n      // Load the library and call the method\n      exports.ModalDialog.load().then(function () {\n        window["SP"].UI.ModalDialog.OpenPopUpPage(url, callback, width, height);\n      });\n    },\n    // Refreshes the page\n    RefreshPage: function RefreshPage(dialogResult) {\n      // Load the library and call the method\n      exports.ModalDialog.load().then(function () {\n        window["SP"].UI.ModalDialog.RefreshPage(dialogResult);\n      });\n    },\n    // Shows a modal dialog\n    showModalDialog: function showModalDialog(options) {\n      // Return a promise\n      return new Promise(function (resolve) {\n        // Load the library and call the method\n        exports.ModalDialog.load().then(function () {\n          // Resolve the promise\n          resolve(getDialog(window["SP"].UI.ModalDialog.showModalDialog(options)));\n        });\n      });\n    },\n    // Shows a pop-up dialog\n    ShowPopupDialog: function ShowPopupDialog(url) {\n      // Load the library and call the method\n      exports.ModalDialog.load().then(function () {\n        window["SP"].UI.ModalDialog.ShowPopupDialog(url);\n      });\n    },\n    // Shows a wait screen\n    showWaitScreenSize: function showWaitScreenSize(title, message, callback, height, width) {\n      // Return a promise\n      return new Promise(function (resolve) {\n        // Load the library and call the method\n        exports.ModalDialog.load().then(function () {\n          // Resolve the promise\n          resolve(getDialog(window["SP"].UI.ModalDialog.showWaitScreenSize(title, message, callback, height, width)));\n        });\n      });\n    },\n    // Shows a wait screen w/ no close button\n    showWaitScreenWithNoClose: function showWaitScreenWithNoClose(title, message, height, width) {\n      // Return a promise\n      return new Promise(function (resolve) {\n        // Load the library and call the method\n        exports.ModalDialog.load().then(function () {\n          // Resolve the promise\n          resolve(getDialog(window["SP"].UI.ModalDialog.showWaitScreenWithNoClose(title, message, height, width)));\n        });\n      });\n    }\n  };\n}();\n\n//# sourceURL=webpack://gd-sprest/./build/helper/sp/modalDialog.js?')},"./build/helper/sp/notify.js":function(__unused_webpack_module,exports){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.Notify = void 0;\n/**\r\n * Notification\r\n */\n\nexports.Notify = {\n  // Adds a notification\n  addNotification: function addNotification(html, sticky) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Load the library and call the method\n      exports.Notify.load().then(function () {\n        resolve(window["SP"].UI.Notify.addNotification(html, sticky));\n      });\n    });\n  },\n  // Method to ensure the core library is loaded\n  load: function load() {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // See if the class exists\n      if (window["SP"] && window["SP"].UI && window["SP"].UI.Notify) {\n        resolve();\n      } else {\n        // Wait for the core script to be loaded\n        window["SP"].SOD.executeFunc("sp.js", "SP.UI.Notify", function () {\n          // Resolve the promise\n          resolve();\n        });\n      }\n    });\n  },\n  // Removes a notification\n  removeNotification: function removeNotification(id) {\n    // Load the library and call the method\n    exports.Notify.load().then(function () {\n      window["SP"].UI.Notify.removeNotification(id);\n    });\n  }\n};\n\n//# sourceURL=webpack://gd-sprest/./build/helper/sp/notify.js?')},"./build/helper/sp/ribbon.js":function(__unused_webpack_module,exports){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.Ribbon = void 0;\n/**\r\n * Ribbon\r\n */\n\nexports.Ribbon = {\n  get exists() {\n    return window["Ribbon"] != null && window["Ribbon"].PageState != null;\n  },\n\n  PageState: {\n    Handlers: {\n      get isApproveEnabled() {\n        return window["Ribbon"] && window["Ribbon"].PageState && window["Ribbon"].PageState.Handlers ? window["Ribbon"].PageState.Handlers.isApproveEnabled : null;\n      },\n\n      get isCancelApprovalEnabled() {\n        return window["Ribbon"] && window["Ribbon"].PageState && window["Ribbon"].PageState.Handlers ? window["Ribbon"].PageState.Handlers.isCancelApprovalEnabled : null;\n      },\n\n      get isCheckinEnabled() {\n        return window["Ribbon"] && window["Ribbon"].PageState && window["Ribbon"].PageState.Handlers ? window["Ribbon"].PageState.Handlers.isCheckinEnabled : null;\n      },\n\n      get isCheckoutEnabled() {\n        return window["Ribbon"] && window["Ribbon"].PageState && window["Ribbon"].PageState.Handlers ? window["Ribbon"].PageState.Handlers.isCheckoutEnabled : null;\n      },\n\n      get isDeleteEnabled() {\n        return window["Ribbon"] && window["Ribbon"].PageState && window["Ribbon"].PageState.Handlers ? window["Ribbon"].PageState.Handlers.isDeleteEnabled : null;\n      },\n\n      get isDiscardcheckoutEnabled() {\n        return window["Ribbon"] && window["Ribbon"].PageState && window["Ribbon"].PageState.Handlers ? window["Ribbon"].PageState.Handlers.isDiscardcheckoutEnabled : null;\n      },\n\n      get isDontSaveAndStopEnabled() {\n        return window["Ribbon"] && window["Ribbon"].PageState && window["Ribbon"].PageState.Handlers ? window["Ribbon"].PageState.Handlers.isDontSaveAndStopEnabled : null;\n      },\n\n      get isEditEnabled() {\n        return window["Ribbon"] && window["Ribbon"].PageState && window["Ribbon"].PageState.Handlers ? window["Ribbon"].PageState.Handlers.isEditEnabled : null;\n      },\n\n      get isInEditMode() {\n        return window["Ribbon"] && window["Ribbon"].PageState && window["Ribbon"].PageState.Handlers ? window["Ribbon"].PageState.Handlers.isInEditMode : null;\n      },\n\n      get isOverrideCheckoutEnabled() {\n        return window["Ribbon"] && window["Ribbon"].PageState && window["Ribbon"].PageState.Handlers ? window["Ribbon"].PageState.Handlers.isOverrideCheckoutEnabled : null;\n      },\n\n      get isPublishEnabled() {\n        return window["Ribbon"] && window["Ribbon"].PageState && window["Ribbon"].PageState.Handlers ? window["Ribbon"].PageState.Handlers.isPublishEnabled : null;\n      },\n\n      get isRejectEnabled() {\n        return window["Ribbon"] && window["Ribbon"].PageState && window["Ribbon"].PageState.Handlers ? window["Ribbon"].PageState.Handlers.isRejectEnabled : null;\n      },\n\n      get isSaveAndStopEditEnabled() {\n        return window["Ribbon"] && window["Ribbon"].PageState && window["Ribbon"].PageState.Handlers ? window["Ribbon"].PageState.Handlers.isSaveAndStopEditEnabled : null;\n      },\n\n      get isSaveEnabled() {\n        return window["Ribbon"] && window["Ribbon"].PageState && window["Ribbon"].PageState.Handlers ? window["Ribbon"].PageState.Handlers.isSaveEnabled : null;\n      },\n\n      get isSubmitForApprovalEnabled() {\n        return window["Ribbon"] && window["Ribbon"].PageState && window["Ribbon"].PageState.Handlers ? window["Ribbon"].PageState.Handlers.isSubmitForApprovalEnabled : null;\n      },\n\n      get isUnpublishEnabled() {\n        return window["Ribbon"] && window["Ribbon"].PageState && window["Ribbon"].PageState.Handlers ? window["Ribbon"].PageState.Handlers.isUnpublishEnabled : null;\n      },\n\n      get onCancelButton() {\n        return window["Ribbon"] && window["Ribbon"].PageState && window["Ribbon"].PageState.Handlers ? window["Ribbon"].PageState.Handlers.onCancelButton : null;\n      },\n\n      get onOkButton() {\n        return window["Ribbon"] && window["Ribbon"].PageState && window["Ribbon"].PageState.Handlers ? window["Ribbon"].PageState.Handlers.onOkButton : null;\n      },\n\n      get showStateChangeDialog() {\n        return window["Ribbon"] && window["Ribbon"].PageState && window["Ribbon"].PageState.Handlers ? window["Ribbon"].PageState.Handlers.showStateChangeDialog : null;\n      }\n\n    }\n  }\n};\n\n//# sourceURL=webpack://gd-sprest/./build/helper/sp/ribbon.js?')},"./build/helper/sp/sod.js":function(__unused_webpack_module,exports){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.SOD = void 0;\n/**\r\n * Script on Demand (SOD)\r\n */\n\nexports.SOD = {\n  // Executes the specified function in the specified file with the optional arguments.\n  execute: function execute(key, functionName) {\n    var args = [];\n\n    for (var _i = 2; _i < arguments.length; _i++) {\n      args[_i - 2] = arguments[_i];\n    }\n\n    window["SP"] ? window["SP"].SOD.execute(key, functionName, args) : null;\n  },\n  // Ensures that the specified file that contains the specified function is loaded and then runs the specified callback function.\n  executeFunc: function executeFunc(key, functionName, fn) {\n    window["SP"] ? window["SP"].SOD.executeFunc(key, functionName, fn) : null;\n  },\n  // Executes the specified function if the specified event has occurred; otherwise, adds the function to the pending job queue.\n  executeOrDelayUntilEventNotified: function executeOrDelayUntilEventNotified(func, eventName) {\n    window["SP"] ? window["SP"].SOD.executeOrDelayUntilEventNotified(func, eventName) : null;\n  },\n  // Executes the specified function if the file containing it is loaded; otherwise, adds it to the pending job queue.\n  executeOrDelayUntilScriptLoaded: function executeOrDelayUntilScriptLoaded(func, depScriptFileName) {\n    window["SP"] ? window["SP"].SOD.executeOrDelayUntilScriptLoaded(func, depScriptFileName) : null;\n  },\n  // Records the event and executes any jobs in the pending job queue that are waiting on the event.\n  notifyEventAndExecuteWaitingJobs: function notifyEventAndExecuteWaitingJobs(eventName) {\n    window["SP"] ? window["SP"].SOD.notifyEventAndExecuteWaitingJobs(eventName) : null;\n  },\n  // Records that the script file is loaded and executes any jobs in the pending job queue that are waiting for the script file to be loaded.\n  notifyScriptLoadedAndExecuteWaitingJobs: function notifyScriptLoadedAndExecuteWaitingJobs(scriptFileName) {\n    window["SP"] ? window["SP"].SOD.notifyScriptLoadedAndExecuteWaitingJobs(scriptFileName) : null;\n  },\n  // Registers the specified file at the specified URL.\n  registerSod: function registerSod(key, url) {\n    window["SP"] ? window["SP"].SOD.registerSod(key, url) : null;\n  },\n  // Registers the specified file as a dependency of another file.\n  registerSodDep: function registerSodDep(key, dep) {\n    window["SP"] ? window["SP"].SOD.registerSodDep(key, dep) : null;\n  }\n};\n\n//# sourceURL=webpack://gd-sprest/./build/helper/sp/sod.js?')},"./build/helper/sp/status.js":function(__unused_webpack_module,exports){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.Status = void 0;\n/**\r\n * Status\r\n */\n\nexports.Status = {\n  // Adds a status\n  addStatus: function addStatus(title, html, prepend) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Load the library\n      exports.Status.load().then(function () {\n        // Add the status and resolve the promise\n        resolve(window["SP"].UI.Status.addStatus(title, html, prepend));\n      });\n    });\n  },\n  // Appends a status\n  appendStatus: function appendStatus(id, title, html) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Load the library\n      exports.Status.load().then(function () {\n        // Add the status and resolve the promise\n        resolve(window["SP"].UI.Status.appendStatus(id, title, html));\n      });\n    });\n  },\n  // Method to ensure the core library is loaded\n  load: function load() {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // See if the class exists\n      if (window["SP"] && window["SP"].UI && window["SP"].UI.Status) {\n        resolve();\n      } else {\n        // Wait for the core script to be loaded\n        window["SP"].SOD.executeFunc("sp.js", "SP.UI.Status", function () {\n          // Resolve the promise\n          resolve();\n        });\n      }\n    });\n  },\n  // Removes all status messages\n  removeAllStatus: function removeAllStatus(hide) {\n    // Load the library and call the method\n    exports.Status.load().then(function () {\n      window["SP"].UI.Status.removeAllStatus(hide);\n    });\n  },\n  // Removes a status\n  removeStatus: function removeStatus(id) {\n    // Load the library and call the method\n    exports.Status.load().then(function () {\n      window["SP"].UI.Status.removeStatus(id);\n    });\n  },\n  // Sets the status color\n  setStatusPriColor: function setStatusPriColor(id, color) {\n    // Load the library and call the method\n    exports.Status.load().then(function () {\n      window["SP"].UI.Status.setStatusPriColor(id, color);\n    });\n  },\n  // Updates the status\n  updateStatus: function updateStatus(id, html) {\n    // Load the library and call the method\n    exports.Status.load().then(function () {\n      window["SP"].UI.Status.updateStatus(id, html);\n    });\n  }\n};\n\n//# sourceURL=webpack://gd-sprest/./build/helper/sp/status.js?')},"./build/helper/taxonomy.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.Taxonomy = void 0;\n\nvar contextInfo_1 = __webpack_require__(/*! ../lib/contextInfo */ "./build/lib/contextInfo.js");\n/**\r\n * Taxonomy Helper Class\r\n */\n\n\nexports.Taxonomy = {\n  /**\r\n   * Method to find a term by id\r\n   */\n  findById: function findById(term, termId) {\n    // See if this is the root node\n    if (term.info && term.info.id == termId) {\n      // Return the root node\n      return term;\n    } // Parse the child nodes\n\n\n    for (var prop in term) {\n      // Skip the info and parent\n      if (prop == "info" || prop == "parent") {\n        continue;\n      } // Find the term by id\n\n\n      var childTerm = exports.Taxonomy.findById(term[prop], termId);\n\n      if (childTerm) {\n        return childTerm;\n      }\n    }\n  },\n\n  /**\r\n   * Method to find a term by name\r\n   */\n  findByName: function findByName(term, termName) {\n    // See if this is the root node\n    if (term.info && term.info.name == termName) {\n      // Return the root node\n      return term;\n    } // Parse the child nodes\n\n\n    for (var prop in term) {\n      // Skip the info and parent\n      if (prop == "info" || prop == "parent") {\n        continue;\n      } // Find the term by id\n\n\n      var childTerm = exports.Taxonomy.findByName(term[prop], termName);\n\n      if (childTerm) {\n        return childTerm;\n      }\n    }\n  },\n\n  /**\r\n   * Method to get the terms\r\n   */\n  getTerms: function getTerms(termSet, termSetTerms) {\n    var terms = []; // Add the root term\n\n    terms.push({\n      description: termSet.get_description(),\n      id: termSet.get_id().toString(),\n      name: termSet.get_name(),\n      path: [],\n      pathAsString: "",\n      props: termSet.get_customProperties()\n    }); // Parse the term sets terms\n\n    var enumerator = termSetTerms.getEnumerator();\n\n    while (enumerator.moveNext()) {\n      var term = enumerator.get_current(); // Create the terms\n\n      terms.push({\n        description: term.get_description(),\n        id: term.get_id().toString(),\n        name: term.get_name(),\n        path: term.get_pathOfTerm().split(";"),\n        pathAsString: term.get_pathOfTerm(),\n        props: term.get_customProperties()\n      });\n    } // Sort the terms\n\n\n    terms = terms.sort(function (a, b) {\n      if (a.pathAsString < b.pathAsString) {\n        return -1;\n      }\n\n      if (a.pathAsString > b.pathAsString) {\n        return 1;\n      }\n\n      return 0;\n    }); // Return the terms\n\n    return terms;\n  },\n\n  /**\r\n   * Method to get the term group\r\n   */\n  getTermGroup: function getTermGroup(groupName) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Load the scripts\n      exports.Taxonomy.loadScripts().then(function () {\n        // Get the taxonomy session\n        var context = SP.ClientContext.get_current();\n        var session = SP.Taxonomy.TaxonomySession.getTaxonomySession(context); // See if we are getting a specific group name\n\n        if (groupName) {\n          // Resolve the promise\n          var termStores_1 = session.get_termStores();\n          context.load(termStores_1, "Include(Groups)");\n          context.executeQueryAsync(function () {\n            // Get the default store\n            var enumerator = termStores_1.getEnumerator();\n            var termStore = enumerator.moveNext() ? enumerator.get_current() : null;\n\n            if (termStore) {\n              // Get the term group\n              var termGroup = termStore.get_groups().getByName(groupName);\n              context.load(termGroup); // Resolve the promise\n\n              resolve({\n                context: context,\n                termGroup: termGroup\n              });\n            } else {\n              // Reject the promise\n              reject("Unable to find the taxonomy store.");\n            }\n          }, function () {\n            var args = [];\n\n            for (var _i = 0; _i < arguments.length; _i++) {\n              args[_i] = arguments[_i];\n            } // Reject the promise\n\n\n            reject(args[1].get_message());\n          });\n        } else {\n          // Get the default site collection group\n          var termStore = session.getDefaultSiteCollectionTermStore();\n          var termGroup = termStore.getSiteCollectionGroup(context.get_site());\n          context.load(termGroup); // Resolve the promise\n\n          resolve({\n            context: context,\n            termGroup: termGroup\n          });\n        }\n      });\n    });\n  },\n\n  /**\r\n   * Method to get the term groups\r\n   */\n  getTermGroups: function getTermGroups() {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Load the scripts\n      exports.Taxonomy.loadScripts().then(function () {\n        // Get the taxonomy session\n        var context = SP.ClientContext.get_current();\n        var session = SP.Taxonomy.TaxonomySession.getTaxonomySession(context); // Resolve the promise\n\n        var termStores = session.get_termStores();\n        context.load(termStores, "Include(Groups)");\n        context.executeQueryAsync(function () {\n          // Get the default store\n          var enumerator = termStores.getEnumerator();\n          var termStore = enumerator.moveNext() ? enumerator.get_current() : null;\n\n          if (termStore) {\n            // Get the term groups\n            var termGroups_1 = termStore.get_groups();\n            context.load(termGroups_1, "Include(Description, Id, Name)"); // Execute the request\n\n            context.executeQueryAsync( // Success\n            function () {\n              var groups = []; // Parse the groups\n\n              var enumerator = termGroups_1.getEnumerator();\n\n              while (enumerator.moveNext()) {\n                var group = enumerator.get_current(); // Add the group information\n\n                groups.push({\n                  description: group.get_description(),\n                  id: group.get_id().toString(),\n                  name: group.get_name()\n                });\n              } // Resolve the promise\n\n\n              resolve(groups);\n            }, function () {\n              var args = [];\n\n              for (var _i = 0; _i < arguments.length; _i++) {\n                args[_i] = arguments[_i];\n              } // Reject the promise\n\n\n              reject(args[1].get_message());\n            });\n          } else {\n            // Reject the promise\n            reject("Unable to find the taxonomy store.");\n          }\n        }, function () {\n          var args = [];\n\n          for (var _i = 0; _i < arguments.length; _i++) {\n            args[_i] = arguments[_i];\n          } // Reject the promise\n\n\n          reject(args[1].get_message());\n        });\n      });\n    });\n  },\n\n  /**\r\n   * Method to get the term sets for a group\r\n   */\n  getTermSets: function getTermSets(groupName) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Get the term gruop\n      exports.Taxonomy.getTermGroup(groupName).then( // Success\n      function (_a) {\n        var context = _a.context,\n            termGroup = _a.termGroup; // Get the term group information\n\n        var termGroupInfo = termGroup.get_termSets();\n        context.load(termGroupInfo, "Include(CustomProperties, Description, Id, Name)"); // Execute the request\n\n        context.executeQueryAsync(function () {\n          var termSets = []; // Parse the term group information\n\n          var enumerator = termGroupInfo.getEnumerator();\n\n          while (enumerator.moveNext()) {\n            var termSet = enumerator.get_current(); // Add the group information\n\n            termSets.push({\n              description: termSet.get_description(),\n              id: termSet.get_id().toString(),\n              name: termSet.get_name(),\n              props: termSet.get_customProperties()\n            });\n          } // Resolve the promise\n\n\n          resolve(termSets);\n        }, function () {\n          var args = [];\n\n          for (var _i = 0; _i < arguments.length; _i++) {\n            args[_i] = arguments[_i];\n          } // Reject the promise\n\n\n          reject(args[1].get_message());\n        });\n      }, // Error\n      function (reason) {\n        // Reject the promise\n        reject(reason);\n      });\n    });\n  },\n\n  /**\r\n   * Method to get the term sets from the default site collection.\r\n   */\n  getTermSetsFromDefaultSC: function getTermSetsFromDefaultSC() {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Load the scripts\n      exports.Taxonomy.loadScripts().then(function () {\n        // Get the taxonomy session\n        var context = SP.ClientContext.get_current();\n        var session = SP.Taxonomy.TaxonomySession.getTaxonomySession(context); // Get the terms sets from the default site collection\n\n        var termStore = session.getDefaultSiteCollectionTermStore();\n        var termGroup = termStore.getSiteCollectionGroup(context.get_site());\n        var termGroupInfo = termGroup.get_termSets();\n        context.load(termGroupInfo, "Include(CustomProperties, Description, Id, Name)"); // Execute the request\n\n        context.executeQueryAsync( // Success\n        function () {\n          var termSets = []; // Parse the term group information\n\n          var enumerator = termGroupInfo.getEnumerator();\n\n          while (enumerator.moveNext()) {\n            var termSet = enumerator.get_current(); // Add the group information\n\n            termSets.push({\n              description: termSet.get_description(),\n              id: termSet.get_id().toString(),\n              name: termSet.get_name(),\n              props: termSet.get_customProperties()\n            });\n          } // Resolve the promise\n\n\n          resolve(termSets);\n        }, function () {\n          var args = [];\n\n          for (var _i = 0; _i < arguments.length; _i++) {\n            args[_i] = arguments[_i];\n          } // Reject the promise\n\n\n          reject(args[1].get_message());\n        });\n      });\n    });\n  },\n\n  /**\r\n   * Method to get the terms by id\r\n   */\n  getTermsById: function getTermsById(termStoreId, termSetId) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Load the scripts\n      exports.Taxonomy.loadScripts().then(function () {\n        // Get the taxonomy session\n        var context = SP.ClientContext.get_current();\n        var session = SP.Taxonomy.TaxonomySession.getTaxonomySession(context); // Get the term set terms\n\n        var termStore = session.get_termStores().getById(termStoreId);\n        var termSet = termStore.getTermSet(termSetId);\n        var terms = termSet.getAllTerms();\n        context.load(termSet);\n        context.load(terms, "Include(CustomProperties, Description, Id, Name, PathOfTerm)"); // Execute the request\n\n        context.executeQueryAsync(function () {\n          // Resolve the promise\n          resolve(exports.Taxonomy.getTerms(termSet, terms));\n        }, function () {\n          var args = [];\n\n          for (var _i = 0; _i < arguments.length; _i++) {\n            args[_i] = arguments[_i];\n          } // Reject the promise\n\n\n          reject(args[1].get_message());\n        });\n      });\n    });\n  },\n\n  /**\r\n   * Method to get the term set by id\r\n   */\n  getTermSetById: function getTermSetById(termStoreId, termSetId) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Get the terms\n      exports.Taxonomy.getTermsById(termStoreId, termSetId).then( // Success\n      function (terms) {\n        // Resolve the promise\n        resolve(exports.Taxonomy.toObject(terms));\n      }, // Error\n      function (reason) {\n        // Reject the promise\n        reject(reason);\n      });\n    });\n  },\n\n  /**\r\n   * Method to get the terms from the default site collection\r\n   */\n  getTermsFromDefaultSC: function getTermsFromDefaultSC(termSetName) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Get the term group\n      exports.Taxonomy.getTermGroup().then( // Success\n      function (_a) {\n        var context = _a.context,\n            termGroup = _a.termGroup; // Get the term set terms\n\n        var termSet = termGroup.get_termSets().getByName(termSetName);\n        var terms = termSet.getAllTerms();\n        context.load(termSet);\n        context.load(terms, "Include(CustomProperties, Description, Id, Name, PathOfTerm)"); // Execute the request\n\n        context.executeQueryAsync(function () {\n          // Resolve the promise\n          resolve(exports.Taxonomy.getTerms(termSet, terms));\n        }, function () {\n          var args = [];\n\n          for (var _i = 0; _i < arguments.length; _i++) {\n            args[_i] = arguments[_i];\n          } // Reject the promise\n\n\n          reject(args[1].get_message());\n        });\n      }, // Error\n      function (reason) {\n        // Reject the promise\n        reject(reason);\n      });\n    });\n  },\n\n  /**\r\n   * Method to get the term set from the default site collection\r\n   */\n  getTermSetFromDefaultSC: function getTermSetFromDefaultSC(termSetName) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Get the terms\n      exports.Taxonomy.getTermsFromDefaultSC(termSetName).then( // Success\n      function (terms) {\n        // Resolve the object\n        resolve(exports.Taxonomy.toObject(terms));\n      }, // Error\n      function (reason) {\n        // Reject the promise\n        reject(reason);\n      });\n    });\n  },\n\n  /**\r\n   * Method to get a terms from a specified group\r\n   */\n  getTermsByGroupName: function getTermsByGroupName(termSetName, groupName) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Get the term group\n      exports.Taxonomy.getTermGroup(groupName).then(function (_a) {\n        var context = _a.context,\n            termGroup = _a.termGroup; // Get the term set terms\n\n        var termSet = termGroup.get_termSets().getByName(termSetName);\n        var terms = termSet.getAllTerms();\n        context.load(termSet);\n        context.load(terms, "Include(CustomProperties, Description, Id, Name, PathOfTerm)"); // Execute the request\n\n        context.executeQueryAsync(function () {\n          // Resolve the promise\n          resolve(exports.Taxonomy.getTerms(termSet, terms));\n        }, function () {\n          var args = [];\n\n          for (var _i = 0; _i < arguments.length; _i++) {\n            args[_i] = arguments[_i];\n          } // Reject the promise\n\n\n          reject(args[1].get_message());\n        });\n      });\n    });\n  },\n\n  /**\r\n   * Method to get the term set from the default site collection\r\n   */\n  getTermSetByGroupName: function getTermSetByGroupName(termSetName, groupName) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Get the terms\n      exports.Taxonomy.getTermsByGroupName(termSetName, groupName).then( // Success\n      function (terms) {\n        // Resolve the object\n        resolve(exports.Taxonomy.toObject(terms));\n      }, // Error\n      function (reason) {\n        // Reject the promise\n        reject(reason);\n      });\n    });\n  },\n\n  /**\r\n   * Method to ensure the taxonomy script references are loaded.\r\n   */\n  loadScripts: function loadScripts() {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Ensure the core script is loaded\n      SP.SOD.executeFunc("sp.js", "SP.Utilities.Utility", function () {\n        // Ensure the taxonomy script is loaded\n        SP.SOD.registerSod("sp.taxonomy.js", contextInfo_1.ContextInfo.webServerRelativeUrl + "/_layouts/15/sp.taxonomy.js");\n        SP.SOD.executeFunc("sp.taxonomy.js", "SP.Taxonomy.TaxonomySession", function () {\n          // Resolve the promise\n          resolve();\n        });\n      }, "sp.js");\n    });\n  },\n\n  /**\r\n   * Method to convert a term to an array of term information\r\n   */\n  toArray: function toArray(term) {\n    var terms = []; // Recursive method to extract the child terms\n\n    var getChildTerms = function getChildTerms(term, terms) {\n      // Parse the properties\n      for (var prop in term) {\n        // Skip the info and parent properties\n        if (prop == "info" || prop == "parent") {\n          continue;\n        } // Add the child term\n\n\n        var childTerm = term[prop];\n        terms.push(childTerm.info); // Add the child terms\n\n        getChildTerms(childTerm, terms);\n      }\n    }; // Ensure the term exists\n\n\n    if (term) {\n      // See if the root node contains term information\n      if (term.info) {\n        // Add the root term\n        terms.push(term.info);\n      } // Get the child terms\n\n\n      getChildTerms(term, terms);\n    } // Return the child terms\n\n\n    return terms;\n  },\n\n  /**\r\n   * Method to convert a term to a field value\r\n   */\n  toFieldValue: function toFieldValue(term) {\n    var termInfo = term ? term["info"] || term : null; // Ensure the term exists\n\n    if (termInfo) {\n      return {\n        __metadata: {\n          "type": "SP.Taxonomy.TaxonomyFieldValue"\n        },\n        Label: termInfo.name,\n        TermGuid: termInfo.id,\n        WssId: -1\n      };\n    } // Return nothing\n\n\n    return null;\n  },\n\n  /**\r\n   * Method to convert a collection of terms to a field value\r\n   */\n  toFieldMultiValue: function toFieldMultiValue(terms) {\n    var results = []; // Ensure terms exist\n\n    if (terms && terms.length > 0) {\n      // Parse the terms\n      for (var i = 0; i < terms.length; i++) {\n        var termInfo = terms[i]["info"] || terms[i]; // Add the term\n\n        results.push(";#" + termInfo.name + "|" + termInfo.id);\n      }\n    } // Return a blank array\n\n\n    return {\n      __metadata: {\n        type: "Collection(SP.Taxonomy.TaxonomyFieldValue)"\n      },\n      results: results\n    };\n  },\n\n  /**\r\n   * Method to convert the terms to an object\r\n   */\n  toObject: function toObject(terms) {\n    var root = {}; // Recursive method to add terms\n\n    var addTerm = function addTerm(node, info, path) {\n      var term = node;\n      var termName = ""; // Loop for each term\n\n      while (path.length > 0) {\n        // Ensure the term exists\n        termName = path[0];\n\n        if (term[termName] == null) {\n          // Create the term\n          term[termName] = {};\n        } // Set the term\n\n\n        var parent_1 = term;\n        term = term[termName]; // Set the parent\n\n        term.parent = parent_1; // Remove the term from the path\n\n        path.splice(0, 1);\n      } // Set the info\n\n\n      term.info = info;\n    }; // Ensure the terms exist\n\n\n    if (terms && terms.length > 0) {\n      // Parse the terms\n      for (var i = 0; i < terms.length; i++) {\n        var term = terms[i]; // See if this is the root term\n\n        if (term.pathAsString == "") {\n          // Set the root information\n          root.info = term;\n        } else {\n          // Add the term\n          addTerm(root, term, term.pathAsString.split(";"));\n        }\n      } // Return the root term\n\n\n      return exports.Taxonomy.findById(root, terms[0].id);\n    } // Return nothing\n\n\n    return null;\n  }\n};\n\n//# sourceURL=webpack://gd-sprest/./build/helper/taxonomy.js?')},"./build/helper/webpart.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.WebPart = void 0;\n\nvar ribbon_1 = __webpack_require__(/*! ./sp/ribbon */ "./build/helper/sp/ribbon.js");\n/**\r\n * Web Part\r\n */\n\n\nvar _WebPart =\n/** @class */\nfunction () {\n  /**\r\n   * Constructor\r\n   * @param props - The webpart properties.\r\n   */\n  function _WebPart(props) {\n    var _this = this;\n\n    this._props = null;\n    this._wp = null;\n    /**\r\n     * Method to add the help link to a script part editor.\r\n     * @wpId - The webpart id.\r\n     */\n\n    this.addHelpLink = function () {\n      // Ensure the help properties exist\n      if (_this._props.helpProps) {\n        // Get the webpart\'s "Snippet"\n        var link = document.querySelector("div[webpartid=\'" + _this._wp.wpId + "\'] a[title=\'Edit Snippet\']");\n\n        if (link) {\n          // Create the help link\n          var helpLink = document.createElement("a");\n          helpLink.href = _this._props.helpProps.url || "#";\n          helpLink.style.paddingLeft = "10px";\n          helpLink.setAttribute("role", "button");\n          helpLink.title = _this._props.helpProps.title || "Help";\n          helpLink.innerHTML = "<span class=\'ms-metadata\'>" + helpLink.title + "</span>";\n          helpLink.target = "_blank"; // Append the link\n\n          link.parentElement.appendChild(helpLink);\n        }\n      }\n    };\n    /**\r\n     * Method to get the webpart id for a specified element\r\n     * @param el - The target element.\r\n     */\n\n\n    this.getWebPartId = function (el) {\n      // Loop until we find the webpart id\n      while (el) {\n        // See if this element contains the webpart id\n        var wpId = el.getAttribute("webpartid");\n\n        if (wpId) {\n          // Return the webpart id\n          return wpId;\n        } // Check the parent\n\n\n        el = el.parentElement;\n      } // Unable to detect\n\n\n      return "";\n    };\n    /**\r\n     * Method to get the webpart information\r\n     */\n\n\n    this.getWebPartInfo = function () {\n      var targetInfo = {\n        cfg: null,\n        el: null,\n        wpId: null\n      }; // Ensure the element id exists\n\n      if (_this._props.elementId) {\n        // Get the webpart elements\n        var elements = document.querySelectorAll("#" + _this._props.elementId);\n\n        for (var i = 0; i < elements.length; i++) {\n          var elWebPart = elements[i]; // See if we have already configured this element\n\n          if (elWebPart.getAttribute("data-isConfigured")) {\n            continue;\n          } // Get the webpart id\n\n\n          var wpId = _this.getWebPartId(elWebPart);\n\n          if (wpId) {\n            // See if the configuration element exists\n            var elCfg = _this._props.cfgElementId ? elWebPart.parentElement.querySelector("#" + _this._props.cfgElementId) : null;\n\n            if (elCfg) {\n              try {\n                // Parse the configuration\n                var cfg = JSON.parse(elCfg.innerText.trim()); // See if the webaprt id exists\n\n                if (cfg.WebPartId) {\n                  // See if it\'s for this webpart\n                  if (cfg.WebPartId == wpId) {\n                    // Set the target information\n                    targetInfo = {\n                      cfg: cfg,\n                      el: elWebPart,\n                      wpId: wpId\n                    };\n                    break;\n                  }\n                } else {\n                  // Set the target information\n                  targetInfo = {\n                    cfg: {\n                      WebPartId: wpId\n                    },\n                    el: elWebPart,\n                    wpId: wpId\n                  };\n                  break;\n                }\n              } catch (ex) {\n                // Set the target information\n                targetInfo = {\n                  cfg: {\n                    WebPartId: wpId\n                  },\n                  el: elWebPart,\n                  wpId: wpId\n                }; // Log\n\n                console.log("[sp-webpart] Error parsing the configuration for element \'" + _this._props.cfgElementId + "\'.");\n              } // Break from the loop\n\n\n              break;\n            } else {\n              // Set the target information\n              targetInfo = {\n                cfg: {\n                  WebPartId: wpId\n                },\n                el: elWebPart,\n                wpId: wpId\n              };\n              break;\n            }\n          }\n        } // Ensure elements were found\n\n\n        if (elements.length == 0) {\n          // Log\n          console.log("[sp-webpart] Error - Unable to find elements with id \'" + _this._props.elementId + "\'.");\n        }\n      } else {\n        // Log\n        console.log("[sp-webpart] The target element id is not defined.");\n      } // Ensure the target element exists\n\n\n      if (targetInfo.el) {\n        // Set the configuration flag\n        targetInfo.el.setAttribute("data-isConfigured", "true");\n      } // Return the target information\n\n\n      return targetInfo;\n    };\n    /**\r\n     * Method to render the webpart\r\n     */\n\n\n    this.render = function () {\n      var element = null; // Get the webpart information\n\n      _this._wp = _this.getWebPartInfo();\n\n      if (_this._wp == null || _this._wp.el == null) {\n        // Log\n        console.log("[sp-webpart] The target webpart element \'" + _this._props.elementId + "\' was not found.");\n        return;\n      } // See if the page is being edited\n\n\n      var returnVal = null;\n\n      if (exports.WebPart.isEditMode()) {\n        // Add the help link\n        _this.addHelpLink(); // Call the render event\n\n\n        if (_this._props.onRenderEdit) {\n          // Execute the render edit event\n          returnVal = _this._props.onRenderEdit(_this._wp);\n        }\n      } else {\n        // See if the configuration is defined, but has no value\n        if (_this._wp.cfg || (_this._props.cfgElementId || "").length == 0) {\n          // Execute the render edit event\n          returnVal = _this._props.onRenderDisplay(_this._wp);\n        } else {\n          // Render a message\n          _this._wp.el.innerHTML = \'<h3>Please edit the page and configure the webpart.</h3>\';\n        }\n      } // See if a promise was returned\n\n\n      if (returnVal && returnVal.then) {\n        // Wait for the request to complete\n        returnVal.then(function () {\n          var args = [];\n\n          for (var _i = 0; _i < arguments.length; _i++) {\n            args[_i] = arguments[_i];\n          } // Execute the post render event\n\n\n          _this._props.onPostRender ? _this._props.onPostRender(_this._wp) : null;\n        });\n      } else {\n        // Execute the post render event\n        _this._props.onPostRender ? _this._props.onPostRender(_this._wp) : null;\n      }\n    }; // Set the properties\n\n\n    this._props = props || {}; // Add a load event\n\n    window.addEventListener("load", function () {\n      // Render the component\n      _this.render();\n    });\n  } // Method to create an instance of the webpart\n\n\n  _WebPart.create = function (props) {\n    // Return an instance of the webpart\n    return new _WebPart(props);\n  }; // Generates the XML for a content editor webpart\n\n\n  _WebPart.generateContentEditorXML = function (props) {\n    return "<?xml version=\\"1.0\\" encoding=\\"utf-8\\"?>\\n<WebPart xmlns=\\"http://schemas.microsoft.com/WebPart/v2\\">\\n    <Assembly>Microsoft.SharePoint, Version=16.0.0.0, Culture=neutral, PublicKeyToken=71e9bce111e9429c</Assembly>\\n    <TypeName>Microsoft.SharePoint.WebPartPages.ContentEditorWebPart</TypeName>\\n    <Title>[[Title]]</Title>\\n    <Description>[[Description]]</Description>\\n    <FrameType>[[FrameType]]</FrameType>\\n    <ContentLink xmlns=\\"http://schemas.microsoft.com/WebPart/v2/ContentEditor\\">[[ContentLink]]</ContentLink>\\n    <Content xmlns=\\"http://schemas.microsoft.com/WebPart/v2/ContentEditor\\"><![CDATA[[[Content]]]]></Content>\\n</WebPart>".replace(/\\r?\\n/g, \'\').replace(/\\[\\[FrameType\\]\\]/g, props.frameType || "Default").replace(/\\[\\[Content\\]\\]/g, props.content || "").replace(/\\[\\[ContentLink\\]\\]/g, props.contentLink || "").replace(/\\[\\[Description\\]\\]/g, props.description || "").replace(/\\[\\[Title\\]\\]/g, props.title || "");\n  }; // Generates the XML for a script editor webpart\n\n\n  _WebPart.generateScriptEditorXML = function (props) {\n    return "<?xml version=\\"1.0\\" encoding=\\"utf-8\\"?>\\n<webParts>\\n    <webPart xmlns=\\"http://schemas.microsoft.com/WebPart/v3\\">\\n        <metaData>\\n            <type name=\\"Microsoft.SharePoint.WebPartPages.ScriptEditorWebPart, Microsoft.SharePoint, Version=15.0.0.0, Culture=neutral, PublicKeyToken=71e9bce111e9429c\\" />\\n            <importErrorMessage>$Resources:core,ImportantErrorMessage;</importErrorMessage>\\n        </metaData>\\n        <data>\\n            <properties>\\n            <property name=\\"Title\\" type=\\"string\\">[[Title]]</property>\\n            <property name=\\"Description\\" type=\\"string\\">[[Description]]</property>\\n            <property name=\\"ChromeType\\" type=\\"chrometype\\">[[ChromeType]]</property>\\n            <property name=\\"Content\\" type=\\"string\\">[[Content]]</property>\\n        </properties>\\n        </data>\\n    </webPart>\\n</webParts>".replace(/\\r?\\n/g, \'\').replace(/\\[\\[ChromeType\\]\\]/g, props.chromeType || "TitleOnly").replace(/\\[\\[Content\\]\\]/g, props.content.replace(/\\</g, \'&lt;\').replace(/\\>/g, \'&gt;\')).replace(/\\[\\[Description\\]\\]/g, props.description || "").replace(/\\[\\[Title\\]\\]/g, props.title || "");\n  };\n  /**\r\n   * Method to detect if a page is being edited\r\n   */\n\n\n  _WebPart.isEditMode = function () {\n    // See if the ribbon page state exists\n    if (ribbon_1.Ribbon.PageState.Handlers.isInEditMode != null) {\n      // Return the mode\n      return ribbon_1.Ribbon.PageState.Handlers.isInEditMode;\n    } // Get the form\n\n\n    var formName = window["MSOWebPartPageFormName"];\n    var form = formName ? document.forms[formName] : null;\n\n    if (form) {\n      // Get the wiki page mode\n      var wikiPageMode = form._wikiPageMode ? form._wikiPageMode.value : null; // Get the webpart page mode\n\n      var wpPageMode = form.MSOLayout_InDesignMode ? form.MSOLayout_InDesignMode.value : null; // Determine if the page is being edited\n\n      return wikiPageMode == "Edit" || wpPageMode == "1";\n    } // Unable to determine\n\n\n    return false;\n  };\n\n  return _WebPart;\n}();\n\nexports.WebPart = _WebPart;\n\n//# sourceURL=webpack://gd-sprest/./build/helper/webpart.js?')},"./build/index.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nvar __createBinding = this && this.__createBinding || (Object.create ? function (o, m, k, k2) {\n  if (k2 === undefined) k2 = k;\n  Object.defineProperty(o, k2, {\n    enumerable: true,\n    get: function get() {\n      return m[k];\n    }\n  });\n} : function (o, m, k, k2) {\n  if (k2 === undefined) k2 = k;\n  o[k2] = m[k];\n});\n\nvar __exportStar = this && this.__exportStar || function (m, exports) {\n  for (var p in m) {\n    if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);\n  }\n};\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.v2 = exports.Helper = void 0; // Type definitions for gd-sprest\n// Project: https://dattabase.com\n// Definitions by: Gunjan Datta <https://github.com/gunjandatta>\n\n/***************************************************************************************************\r\nMIT License\r\n\r\nCopyright (c) 2016 Dattabase, LLC.\r\n\r\nPermission is hereby granted, free of charge, to any person obtaining a copy\r\nof this software and associated documentation files (the "Software"), to deal\r\nin the Software without restriction, including without limitation the rights\r\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\r\ncopies of the Software, and to permit persons to whom the Software is\r\nfurnished to do so, subject to the following conditions:\r\n\r\nThe above copyright notice and this permission notice shall be included in all\r\ncopies or substantial portions of the Software.\r\n\r\nTHE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\r\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\r\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\r\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\r\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\r\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\r\nSOFTWARE.\r\n***************************************************************************************************/\n\nvar Helper = __webpack_require__(/*! ./helper */ "./build/helper/index.js");\n\nexports.Helper = Helper;\n\n__exportStar(__webpack_require__(/*! ./lib */ "./build/lib/index.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./rest */ "./build/rest.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./sptypes */ "./build/sptypes/index.js"), exports);\n\nvar v2 = __webpack_require__(/*! ./v2 */ "./build/v2/index.js");\n\nexports.v2 = v2;\n\n//# sourceURL=webpack://gd-sprest/./build/index.js?')},"./build/lib/apps.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.Apps = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js");\n/**\r\n * Apps\r\n */\n\n\nexports.Apps = function (targetInfo) {\n  var apps = new utils_1.Base(targetInfo); // Default the properties\n\n  apps.targetInfo.defaultToWebFl = true;\n  apps.targetInfo.endpoint = "apps"; // Add the methods\n\n  utils_1.Request.addMethods(apps, {\n    __metadata: {\n      type: "Microsoft.AppServices.AppCollection"\n    }\n  }); // Return the apps\n\n  return apps;\n};\n\n//# sourceURL=webpack://gd-sprest/./build/lib/apps.js?')},"./build/lib/contextInfo.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.ContextInfo = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js");\n/**\r\n * Context Information\r\n */\n\n\nvar _ContextInfo =\n/** @class */\nfunction () {\n  function _ContextInfo() {}\n\n  Object.defineProperty(_ContextInfo, "_contextInfo", {\n    // The current context information\n    get: function get() {\n      // Return the page context by default\n      if (this.window["_spPageContextInfo"]) {\n        return this.window["_spPageContextInfo"];\n      } // See if the SPFx context was set\n\n\n      if (this._spfxPageContext && this._spfxPageContext.legacyPageContext) {\n        return this._spfxPageContext.legacyPageContext;\n      } // Return a default object\n\n\n      return {\n        existsFl: false,\n        isAppWeb: false,\n        isHubSite: false,\n        isSPO: false,\n        siteAbsoluteUrl: "",\n        siteServerRelativeUrl: "",\n        userId: 0,\n        webAbsoluteUrl: "",\n        webServerRelativeUrl: ""\n      };\n    },\n    enumerable: false,\n    configurable: true\n  });\n  ;\n  Object.defineProperty(_ContextInfo, "aadInstanceUrl", {\n    /**\r\n     * Properties\r\n     */\n    get: function get() {\n      return this._contextInfo.aadInstanceUrl;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "aadTenantId", {\n    get: function get() {\n      return this._contextInfo.aadTenantId;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "alertsEnabled", {\n    get: function get() {\n      return this._contextInfo.alertsEnabled;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "allowSilverlightPrompt", {\n    get: function get() {\n      return this._contextInfo.allowSilverlightPrompt == "True" ? true : false;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "canUserCreateMicrosoftForm", {\n    get: function get() {\n      return this._contextInfo.canUserCreateMicrosoftForm;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "canUserCreateVisioDrawing", {\n    get: function get() {\n      return this._contextInfo.canUserCreateVisioDrawing;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "cdnPrefix", {\n    get: function get() {\n      return this._contextInfo.cdnPrefix;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "clientServerTimeDelta", {\n    get: function get() {\n      return this._contextInfo.clientServerTimeDelta;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "CorrelationId", {\n    get: function get() {\n      return this._contextInfo.CorrelationId;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "crossDomainPhotosEnabled", {\n    get: function get() {\n      return this._contextInfo.crossDomainPhotosEnabled;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "currentCultureName", {\n    get: function get() {\n      return this._contextInfo.currentCultureName;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "currentLanguage", {\n    get: function get() {\n      return this._contextInfo.currentLanguage;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "currentUICultureName", {\n    get: function get() {\n      return this._contextInfo.currentUICultureName;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "departmentId", {\n    get: function get() {\n      return this._contextInfo.departmentId;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "DesignPackageId", {\n    get: function get() {\n      return this._contextInfo.DesignPackageId;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "disableAppViews", {\n    get: function get() {\n      return this._contextInfo.disableAppViews;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "disableFlows", {\n    get: function get() {\n      return this._contextInfo.disableFlows;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "document", {\n    get: function get() {\n      return this.window ? this.window.document : null;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "env", {\n    get: function get() {\n      return this._contextInfo.env;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "existsFl", {\n    get: function get() {\n      return this._contextInfo.existsFl == null;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "farmLabel", {\n    get: function get() {\n      return this._contextInfo.farmLabel;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "fid", {\n    get: function get() {\n      return this._contextInfo.fid;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "formDigestTimeoutSeconds", {\n    get: function get() {\n      return this._contextInfo.formDigestTimeoutSeconds;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "formDigestValue", {\n    get: function get() {\n      return this._contextInfo.formDigestValue;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "groupColor", {\n    get: function get() {\n      return this._contextInfo.groupColor;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "groupHasHomepage", {\n    get: function get() {\n      return this._contextInfo.groupHasHomepage;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "groupId", {\n    get: function get() {\n      return this._contextInfo.groupId;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "groupType", {\n    get: function get() {\n      return this._contextInfo.groupType;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "guestsEnabled", {\n    get: function get() {\n      return this._contextInfo.guestsEnabled;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "hasManageWebPermissions", {\n    get: function get() {\n      return this._contextInfo.hasManageWebPermissions;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "hasPendingWebTemplateExtension", {\n    get: function get() {\n      return this._contextInfo.hasPendingWebTemplateExtension;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "hideSyncButtonOnODB", {\n    get: function get() {\n      return this._contextInfo.hideSyncButtonOnODB;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "hubSiteId", {\n    get: function get() {\n      return this._contextInfo.hubSiteId;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "idleSessionSignOutEnabled", {\n    get: function get() {\n      return this._contextInfo.idleSessionSignOutEnabled;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "isAnonymousGuestUser", {\n    get: function get() {\n      return this._contextInfo.isAnonymousGuestUser;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "isAppWeb", {\n    get: function get() {\n      return this._contextInfo.isAppWeb;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "isArchived", {\n    get: function get() {\n      return this._contextInfo.isArchived;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "isEmailAuthenticationGuestUser", {\n    get: function get() {\n      return this._contextInfo.isEmailAuthenticationGuestUser;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "isExternalGuestUser", {\n    get: function get() {\n      return this._contextInfo.isExternalGuestUser;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "isGroupRelatedSite", {\n    get: function get() {\n      return this._contextInfo.isGroupRelatedSite;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "isGroupifyDisabled", {\n    get: function get() {\n      return this._contextInfo.isGroupifyDisabled;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "isGroupifyMenuButtonFeatureOff", {\n    get: function get() {\n      return this._contextInfo.isGroupifyMenuButtonFeatureOff;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "isHubSite", {\n    get: function get() {\n      return this._contextInfo.isHubSite;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "isLocationserviceAvailable", {\n    get: function get() {\n      return this._contextInfo.isLocationserviceAvailable;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "isMultiGeoODBMode", {\n    get: function get() {\n      return this._contextInfo.isMultiGeoODBMode;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "isMultiGeoTenant", {\n    get: function get() {\n      return this._contextInfo.isMultiGeoTenant;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "isNoScriptEnabled", {\n    get: function get() {\n      return this._contextInfo.isNoScriptEnabled;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "isSiteAdmin", {\n    get: function get() {\n      return this._contextInfo.isSiteAdmin;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "isSiteOwner", {\n    get: function get() {\n      return this._contextInfo.isSiteOwner;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "isSPO", {\n    get: function get() {\n      return this._contextInfo.isSPO;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "isShareByLinkEnabled", {\n    get: function get() {\n      return this._contextInfo.isShareByLinkEnabled;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "isTeamsChannelSite", {\n    get: function get() {\n      return this._contextInfo.isTeamsChannelSite;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "isTeamsConnectedSite", {\n    get: function get() {\n      return this._contextInfo.isTeamsConnectedSite;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "isTenantDevSite", {\n    get: function get() {\n      return this._contextInfo.isTenantDevSite;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "isUnauthorizedTenant", {\n    get: function get() {\n      return this._contextInfo.isUnauthorizedTenant;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "isWebWelcomePage", {\n    get: function get() {\n      return this._contextInfo.isWebWelcomePage;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "layoutsUrl", {\n    get: function get() {\n      return this._contextInfo.layoutsUrl;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "listBaseTemplate", {\n    get: function get() {\n      return this._contextInfo.listBaseTemplate;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "listBaseType", {\n    get: function get() {\n      return this._contextInfo.listBaseType;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "listId", {\n    get: function get() {\n      return this._contextInfo.listId;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "listItemCount", {\n    get: function get() {\n      return this._contextInfo.listItemCount;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "listTitle", {\n    get: function get() {\n      return this._contextInfo.listTitle;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "listPermsMask", {\n    get: function get() {\n      return this._contextInfo.listPermsMask;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "listUrl", {\n    get: function get() {\n      return this._contextInfo.listUrl;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "maximumFileSize", {\n    get: function get() {\n      return this._contextInfo.maximumFileSize;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "modernThemingEnabled", {\n    get: function get() {\n      return this._contextInfo.modernThemingEnabled;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "msGraphEndpointUrl", {\n    get: function get() {\n      return this._contextInfo.msGraphEndpointUrl;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "msMruEndpointUrl", {\n    get: function get() {\n      return this._contextInfo.msMruEndpointUrl;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "multiGeoInfo", {\n    get: function get() {\n      return this._contextInfo.multiGeoInfo;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "navigationInfo", {\n    get: function get() {\n      return this._contextInfo.navigationInfo;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "nid", {\n    get: function get() {\n      return this._contextInfo.nid;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "openInClient", {\n    get: function get() {\n      return this._contextInfo.openInClient;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "pageItemId", {\n    get: function get() {\n      return this._contextInfo.pageItemId;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "pageListId", {\n    get: function get() {\n      return this._contextInfo.pageListId;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "pagePermsMask", {\n    get: function get() {\n      return this._contextInfo.pagePermsMask;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "pagePersonalizationScope", {\n    get: function get() {\n      return this._contextInfo.pagePersonalizationScope;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "preferUserTimeZone", {\n    get: function get() {\n      return this._contextInfo.preferUserTimeZone;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "PreviewFeaturesEnabled", {\n    get: function get() {\n      return this._contextInfo.PreviewFeaturesEnabled;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "portalUrl", {\n    get: function get() {\n      return this._contextInfo.portalUrl;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "profileUrl", {\n    get: function get() {\n      return this._contextInfo.profileUrl;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "PublishingFeatureOn", {\n    get: function get() {\n      return this._contextInfo.PublishingFeatureOn;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "RecycleBinItemCount", {\n    get: function get() {\n      return this._contextInfo.RecycleBinItemCount;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "serverRedirectedUrl", {\n    get: function get() {\n      return this._contextInfo.serverRedirectedUrl;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "serverRequestPath", {\n    get: function get() {\n      return this._contextInfo.serverRequestPath;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "serverTime", {\n    get: function get() {\n      return this._contextInfo.serverTime;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "showNGSCDialogForSyncOnODB", {\n    get: function get() {\n      return this._contextInfo.showNGSCDialogForSyncOnODB;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "showNGSCDialogForSyncOnTS", {\n    get: function get() {\n      return this._contextInfo.showNGSCDialogForSyncOnTS;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "siteAbsoluteUrl", {\n    get: function get() {\n      return this._contextInfo.siteAbsoluteUrl;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "siteClassification", {\n    get: function get() {\n      return this._contextInfo.siteClassification;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "siteClientTag", {\n    get: function get() {\n      return this._contextInfo.siteClientTag;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "siteColor", {\n    get: function get() {\n      return this._contextInfo.siteColor;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "siteId", {\n    get: function get() {\n      return this._contextInfo.siteId;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "sitePagesEnabled", {\n    get: function get() {\n      return this._contextInfo.sitePagesEnabled;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "sitePagesFeatureVersion", {\n    get: function get() {\n      return this._contextInfo.sitePagesFeatureVersion;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "siteServerRelativeUrl", {\n    get: function get() {\n      return this._contextInfo.siteServerRelativeUrl;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "siteSubscriptionId", {\n    get: function get() {\n      return this._contextInfo.siteSubscriptionId;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "socialBarEnabled", {\n    get: function get() {\n      return this._contextInfo.socialBarEnabled;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "supportPercentStorePage", {\n    get: function get() {\n      return this._contextInfo.supportPercentStorePage;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "supportPoundStorePath", {\n    get: function get() {\n      return this._contextInfo.supportPoundStorePath;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "systemUserKey", {\n    get: function get() {\n      return this._contextInfo.systemUserKey;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "teamsChannelType", {\n    get: function get() {\n      return this._contextInfo.teamsChannelType;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "tenantAppVersion", {\n    get: function get() {\n      return this._contextInfo.tenantAppVersion;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "tenantDisplayName", {\n    get: function get() {\n      return this._contextInfo.tenantDisplayName;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "theme", {\n    get: function get() {\n      return (this.window && this.window["__themeState__"] ? this.window["__themeState__"].theme : null) || {};\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "themeCacheToken", {\n    get: function get() {\n      return this._contextInfo.themeCacheToken;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "themedCssFolderUrl", {\n    get: function get() {\n      return this._contextInfo.themedCssFolderUrl;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "themedImageFileNames", {\n    get: function get() {\n      return this._contextInfo.themedImageFileNames;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "updateFromDigestPageLoaded", {\n    get: function get() {\n      return this._contextInfo.updateFromDigestPageLoaded;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "userDisplayName", {\n    get: function get() {\n      return this._contextInfo.userDisplayName;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "userEmail", {\n    get: function get() {\n      return this._contextInfo.userEmail;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "userFirstDayOfWeek", {\n    get: function get() {\n      return this._contextInfo.userFirstDayOfWeek;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "userId", {\n    get: function get() {\n      return this._contextInfo.userId;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "userLoginName", {\n    get: function get() {\n      return this._contextInfo.userLoginName;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "userPhotoCdnBaseUrl", {\n    get: function get() {\n      return this._contextInfo.userPhotoCdnBaseUrl;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "userPrincipalName", {\n    get: function get() {\n      return this._contextInfo.userPrincipalName;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "userTime24", {\n    get: function get() {\n      return this._contextInfo.userTime24;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "userTimeZoneData", {\n    get: function get() {\n      return this._contextInfo.userTimeZoneData;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "userVoiceForFeedbackEnabled", {\n    get: function get() {\n      return this._contextInfo.userVoiceForFeedbackEnabled;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "viewId", {\n    get: function get() {\n      return this._contextInfo.viewId;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "viewOnlyExperienceEnabled", {\n    get: function get() {\n      return this._contextInfo.viewOnlyExperienceEnabled;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "webAbsoluteUrl", {\n    get: function get() {\n      return this._contextInfo.webAbsoluteUrl;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "webDescription", {\n    get: function get() {\n      return this._contextInfo.webDescription;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "webDomain", {\n    get: function get() {\n      return this._contextInfo.webDomain;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "webFirstDayOfWeek", {\n    get: function get() {\n      return this._contextInfo.webFirstDayOfWeek;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "webId", {\n    get: function get() {\n      return this._contextInfo.webId;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "webLanguage", {\n    get: function get() {\n      return this._contextInfo.webLanguage;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "webLanguageName", {\n    get: function get() {\n      return this._contextInfo.webLanguageName;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "webLogoUrl", {\n    get: function get() {\n      return this._contextInfo.webLogoUrl;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "webPermMasks", {\n    get: function get() {\n      return this._contextInfo.webPermMasks;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "webServerRelativeUrl", {\n    get: function get() {\n      return this._contextInfo.webServerRelativeUrl;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "webTemplate", {\n    get: function get() {\n      return this._contextInfo.webTemplate;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "webTemplateConfiguration", {\n    get: function get() {\n      return this._contextInfo.webTemplateConfiguration;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "webTime24", {\n    get: function get() {\n      return this._contextInfo.webTime24;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "webTimeZoneData", {\n    get: function get() {\n      return this._contextInfo.webTimeZoneData;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "webTitle", {\n    get: function get() {\n      return this._contextInfo.webTitle;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "webUIVersion", {\n    get: function get() {\n      return this._contextInfo.webUIVersion;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(_ContextInfo, "window", {\n    get: function get() {\n      return typeof window == "undefined" ? {} : window;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  /**\r\n   * Methods\r\n   */\n  // Method to generate a guid\n\n  _ContextInfo.generateGUID = function () {\n    // Set the batch id\n    return \'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx\'.replace(/[xy]/g, function (c) {\n      var r = Math.random() * 16 | 0,\n          v = c == \'x\' ? r : r & 0x3 | 0x8;\n      return v.toString(16);\n    });\n  }; // The page context information from an spfx project\n\n\n  _ContextInfo._spfxPageContext = null; // Method to get the context information for a web\n\n  _ContextInfo.getWeb = function (url) {\n    // Create a new base object\n    return new utils_1.Base({\n      endpoint: "contextinfo",\n      method: "POST",\n      url: url\n    });\n  }; // Method to set the page context information from an SPFX project\n\n\n  _ContextInfo.setPageContext = function (spfxPageContext) {\n    exports.ContextInfo._spfxPageContext = spfxPageContext;\n  };\n\n  return _ContextInfo;\n}();\n\nexports.ContextInfo = _ContextInfo;\n\n//# sourceURL=webpack://gd-sprest/./build/lib/contextInfo.js?')},"./build/lib/graph.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.Graph = void 0;\n\nvar sptypes_1 = __webpack_require__(/*! ../sptypes */ "./build/sptypes/index.js");\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js"); // Default Token\n//export const Token\n\n/**\r\n * Graph\r\n */\n\n\nexports.Graph = function (props) {\n  var graph = new utils_1.Base({\n    accessToken: props && props.accessToken ? props.accessToken : exports.Graph.Token\n  }); // Default the target information\n\n  graph.targetInfo.requestType = (props && props.requestType ? props.requestType : "").toLowerCase() == "post" ? utils_1.RequestType.GraphPost : utils_1.RequestType.GraphGet; // Set the endpoint\n\n  graph.targetInfo.data = props ? props.data : null;\n  graph.targetInfo.endpoint = props && props.cloud ? props.cloud : exports.Graph.Cloud || sptypes_1.SPTypes.CloudEnvironment.Default;\n  graph.targetInfo.endpoint += "/" + (props && props.version ? props.version : exports.Graph.Version || "v1.0"); // See if the url is set\n\n  if (props && props.url) {\n    // Set the endpoint\n    graph.targetInfo.endpoint += "/" + props.url;\n  } else {\n    // Add the default methods\n    utils_1.Request.addMethods(graph, {\n      __metadata: {\n        type: "graph"\n      }\n    });\n  } // Return the graph\n\n\n  return graph;\n}; // Default Values\n\n\nexports.Graph.Cloud = "";\nexports.Graph.Token = "";\nexports.Graph.Version = ""; // Method to get the graph token from a classic page\n\nexports.Graph.getAccessToken = function (resource) {\n  // Set the data \n  var data = {\n    "resource": resource || sptypes_1.SPTypes.CloudEnvironment.Default\n  }; // Get the access token\n\n  return new utils_1.Base({\n    endpoint: "SP.OAuth.Token/Acquire",\n    method: "POST",\n    data: data\n  });\n};\n\n//# sourceURL=webpack://gd-sprest/./build/lib/graph.js?')},"./build/lib/groupService.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.GroupService = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js");\n/**\r\n * Group Service\r\n */\n\n\nexports.GroupService = function (targetInfo) {\n  var svc = new utils_1.Base(targetInfo); // Default the properties\n\n  svc.targetInfo.defaultToWebFl = true;\n  svc.targetInfo.endpoint = "groupservice"; // Add the methods\n\n  utils_1.Request.addMethods(svc, {\n    __metadata: {\n      type: "Microsoft.SharePoint.Portal.GroupService"\n    }\n  }); // Return the group service\n\n  return svc;\n};\n\n//# sourceURL=webpack://gd-sprest/./build/lib/groupService.js?')},"./build/lib/groupSiteManager.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.GroupSiteManager = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js");\n/**\r\n * Group Site Manager\r\n */\n\n\nexports.GroupSiteManager = function (targetInfo) {\n  var mgr = new utils_1.Base(targetInfo); // Default the properties\n\n  mgr.targetInfo.defaultToWebFl = true;\n  mgr.targetInfo.endpoint = "groupsitemanager"; // Add the methods\n\n  utils_1.Request.addMethods(mgr, {\n    __metadata: {\n      type: "Microsoft.SharePoint.Portal.GroupSiteManager"\n    }\n  }); // Return the group site manager\n\n  return mgr;\n};\n\n//# sourceURL=webpack://gd-sprest/./build/lib/groupSiteManager.js?')},"./build/lib/hubSites.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.HubSites = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js");\n/**\r\n * Hub Sites\r\n */\n\n\nexports.HubSites = function (targetInfo) {\n  var hubs = new utils_1.Base(targetInfo); // Default the properties\n\n  hubs.targetInfo.defaultToWebFl = true;\n  hubs.targetInfo.endpoint = "hubsites"; // Add the methods\n\n  utils_1.Request.addMethods(hubs, {\n    __metadata: {\n      type: "SP.HubSite.Collection"\n    }\n  }); // Return the hub sites\n\n  return hubs;\n}; // Static method to see if the current user can create hub sites\n\n\nexports.HubSites.canCreate = function () {\n  // Return the base object\n  return new utils_1.Base({\n    endpoint: "SP.HubSites.CanCreate"\n  });\n};\n\n//# sourceURL=webpack://gd-sprest/./build/lib/hubSites.js?')},"./build/lib/hubSitesUtility.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.HubSitesUtility = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js");\n/**\r\n * Hub Sites Utility\r\n */\n\n\nexports.HubSitesUtility = function (targetInfo) {\n  var utility = new utils_1.Base(targetInfo); // Default the properties\n\n  utility.targetInfo.defaultToWebFl = true;\n  utility.targetInfo.endpoint = "hubsitesutility"; // Add the methods\n\n  utils_1.Request.addMethods(utility, {\n    __metadata: {\n      type: "Microsoft.SharePoint.Portal.SPHubSitesUtility"\n    }\n  }); // Return the hub sites utility\n\n  return utility;\n};\n\n//# sourceURL=webpack://gd-sprest/./build/lib/hubSitesUtility.js?')},"./build/lib/index.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nvar __createBinding = this && this.__createBinding || (Object.create ? function (o, m, k, k2) {\n  if (k2 === undefined) k2 = k;\n  Object.defineProperty(o, k2, {\n    enumerable: true,\n    get: function get() {\n      return m[k];\n    }\n  });\n} : function (o, m, k, k2) {\n  if (k2 === undefined) k2 = k;\n  o[k2] = m[k];\n});\n\nvar __exportStar = this && this.__exportStar || function (m, exports) {\n  for (var p in m) {\n    if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);\n  }\n};\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\n\n__exportStar(__webpack_require__(/*! ./apps */ "./build/lib/apps.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./contextInfo */ "./build/lib/contextInfo.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./graph */ "./build/lib/graph.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./groupService */ "./build/lib/groupService.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./groupSiteManager */ "./build/lib/groupSiteManager.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./hubSites */ "./build/lib/hubSites.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./hubSitesUtility */ "./build/lib/hubSitesUtility.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./list */ "./build/lib/list.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./navigation */ "./build/lib/navigation.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./peopleManager */ "./build/lib/peopleManager.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./peoplePicker */ "./build/lib/peoplePicker.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./profileLoader */ "./build/lib/profileLoader.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./search */ "./build/lib/search.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./site */ "./build/lib/site.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./siteIconManager */ "./build/lib/siteIconManager.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./siteManager */ "./build/lib/siteManager.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./sitePages */ "./build/lib/sitePages.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./socialFeed */ "./build/lib/socialFeed.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./themeManager */ "./build/lib/themeManager.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./userProfile */ "./build/lib/userProfile.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./utility */ "./build/lib/utility.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./web */ "./build/lib/web.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./webTemplateExtensions */ "./build/lib/webTemplateExtensions.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./wfInstanceService */ "./build/lib/wfInstanceService.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./wfSubscriptionService */ "./build/lib/wfSubscriptionService.js"), exports);\n\n//# sourceURL=webpack://gd-sprest/./build/lib/index.js?')},"./build/lib/list.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.List = void 0;\n\nvar sptypes_1 = __webpack_require__(/*! ../sptypes */ "./build/sptypes/index.js");\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js");\n\nvar graph_1 = __webpack_require__(/*! ./graph */ "./build/lib/graph.js");\n\nvar web_1 = __webpack_require__(/*! ./web */ "./build/lib/web.js");\n/**\r\n * List\r\n */\n\n\nexports.List = function (listName, targetInfo) {\n  var list = new utils_1.Base(targetInfo); // Default the properties\n\n  list.targetInfo.defaultToWebFl = true;\n  list.targetInfo.endpoint = "web/lists/getByTitle(\'" + listName.replace(/\\\'/g, "\'\'") + "\')"; // Add the methods\n\n  utils_1.Request.addMethods(list, {\n    __metadata: {\n      type: "SP.List"\n    }\n  }); // Return the list\n\n  return list;\n}; // Static method to get the list by the entity name.\n\n\nexports.List.getByEntityName = function (props) {\n  // Query for the list\n  var query = web_1.Web(props.url, props.targetInfo) // Get the lists\n  .Lists() // Set the query\n  .query({\n    Filter: "EntityTypeName eq \'" + props.name + "\'",\n    Top: 1\n  }); // See if the callback exists\n\n  if (props.callback) {\n    // Execute the request asynchronously\n    query.execute(function (lists) {\n      // Execute the callback method\n      props.callback(lists.results ? lists.results[0] : null);\n    });\n  } else {\n    // Execute the request synchronously and return it\n    var list = query.executeAndWait();\n    return list.results ? list.results[0] : list;\n  }\n}; // Static method to get the list data from the SP.List.getListDataAsStream endpoint\n\n\nexports.List.getDataAsStream = function (listFullUrl, parameters) {\n  if (parameters === void 0) {\n    parameters = {};\n  }\n\n  var params = "?listFullUrl=\'" + listFullUrl + "\'"; // Parse the parameters\n\n  for (var key in parameters) {\n    // Append the parameter\n    params += "&" + key + "=" + parameters[key];\n  } // Return the base object\n\n\n  return new utils_1.Base({\n    endpoint: "SP.List.getListDataAsStream" + params\n  });\n}; // Static method for executing a flow against a list item\n\n\nexports.List.runFlow = function (props) {\n  // Determine the urls\n  var authUrl = "";\n  var flowUrl = "";\n\n  switch (props.cloudEnv) {\n    case sptypes_1.SPTypes.CloudEnvironment.USL4:\n      authUrl = sptypes_1.SPTypes.CloudEnvironment.FlowHigh;\n      flowUrl = sptypes_1.SPTypes.CloudEnvironment.FlowHighAPI;\n      break;\n\n    case sptypes_1.SPTypes.CloudEnvironment.USL5:\n      authUrl = sptypes_1.SPTypes.CloudEnvironment.FlowDoD;\n      flowUrl = sptypes_1.SPTypes.CloudEnvironment.FlowDoDAPI;\n      break;\n    // Default\n\n    default:\n      authUrl = sptypes_1.SPTypes.CloudEnvironment.Flow;\n      flowUrl = sptypes_1.SPTypes.CloudEnvironment.FlowAPI;\n      break;\n  } // Return a promise\n\n\n  return new Promise(function (resolve) {\n    // Gets the graph token\n    var getGraphToken = function getGraphToken() {\n      // Return a promise\n      return new Promise(function (resolveAuth) {\n        // Get the graph token\n        graph_1.Graph.getAccessToken(authUrl).execute(function (auth) {\n          // Resolve the request\n          resolveAuth(auth.access_token);\n        }, // Error\n        function (ex) {\n          // Resolve the request\n          resolve({\n            executed: false,\n            errorDetails: ex.response,\n            errorMessage: "Auth Error: Unable to get the flow token."\n          });\n        });\n      });\n    }; // Gets the flow token\n\n\n    var getFlowToken = function getFlowToken(flowInfo) {\n      // Return a promise\n      return new Promise(function (resolveAuth) {\n        // See if the flow token is provided\n        if (props.token) {\n          // Resolve the request\n          resolveAuth(props.token);\n        } else {\n          // Get the graph token\n          getGraphToken().then(function (token) {\n            // Set the url\n            var authUrl = "" + flowUrl + flowInfo.properties.environment.id + "/users/me/onBehalfOfTokenBundle?api-version=2016-11-01"; // Get the graph token from SharePoint\n\n            new utils_1.Base({\n              endpoint: authUrl,\n              method: "POST",\n              requestType: utils_1.RequestType.GraphPost,\n              requestHeader: {\n                "authorization": "Bearer " + token\n              }\n            }).execute(function (tokenInfo) {\n              // Resolve the request\n              resolveAuth(tokenInfo.audienceToToken["https://" + flowInfo.properties.connectionReferences.shared_sharepointonline.swagger.host] || token);\n            }, // Error\n            function (ex) {\n              // Resolve the request\n              resolve({\n                executed: false,\n                errorDetails: ex.response,\n                errorMessage: "Auth Error: Unable to get the flow token for cloud: " + props.cloudEnv\n              });\n            });\n          });\n        }\n      });\n    }; // Get the flow information\n\n\n    web_1.Web(props.webUrl).Lists(props.list).syncFlowInstance(props.id).execute( // Success\n    function (flow) {\n      // Get the flow information\n      var flowInfo = JSON.parse(flow.SynchronizationData);\n\n      if (flowInfo) {\n        // Get the flow token\n        getFlowToken(flowInfo).then(function (token) {\n          // Trigger the flow\n          new utils_1.Base({\n            accessToken: token,\n            requestType: utils_1.RequestType.GraphPost,\n            endpoint: flowInfo.properties.flowTriggerUri,\n            data: {\n              rows: [{\n                entity: props.data\n              }]\n            }\n          }).execute( // Success\n          function () {\n            // Resolve the request\n            resolve({\n              executed: true,\n              flowToken: token\n            });\n          }, // Error\n          function (ex) {\n            // Resolve the request\n            resolve({\n              executed: false,\n              errorDetails: ex.response,\n              errorMessage: "Error triggering the flow."\n            });\n          });\n        });\n      } else {\n        // Resolve the request\n        resolve({\n          executed: false,\n          errorDetails: "The flow with id " + props.id + " doesn\'t exist.",\n          errorMessage: "Error getting the flow instance."\n        });\n      }\n    }, // Error\n    function (ex) {\n      // Resolve the request\n      resolve({\n        executed: false,\n        errorDetails: ex.response,\n        errorMessage: "Error getting the flow information."\n      });\n    });\n  });\n};\n\n//# sourceURL=webpack://gd-sprest/./build/lib/list.js?')},"./build/lib/navigation.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.Navigation = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js");\n/**\r\n * Navigation\r\n */\n\n\nexports.Navigation = function (url, targetInfo) {\n  var navigation = new utils_1.Base(targetInfo); // Default the properties\n\n  navigation.targetInfo.defaultToWebFl = true;\n  navigation.targetInfo.endpoint = "navigation"; // See if the web url exists\n\n  if (url) {\n    // Set the settings\n    navigation.targetInfo.url = url;\n  } // Add the methods\n\n\n  utils_1.Request.addMethods(navigation, {\n    __metadata: {\n      type: "Microsoft.SharePoint.Navigation.REST.NavigationServiceRest"\n    }\n  }); // Return the navigation\n\n  return navigation;\n};\n\n//# sourceURL=webpack://gd-sprest/./build/lib/navigation.js?')},"./build/lib/peopleManager.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.PeopleManager = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js");\n/**\r\n * People Manager\r\n */\n\n\nexports.PeopleManager = function (targetInfo) {\n  var peopleManager = new utils_1.Base(targetInfo); // Default the properties\n\n  peopleManager.targetInfo.defaultToWebFl = true;\n  peopleManager.targetInfo.endpoint = "sp.userprofiles.peoplemanager"; // Add the methods\n\n  utils_1.Request.addMethods(peopleManager, {\n    __metadata: {\n      type: "SP.UserProfiles.PeopleManager"\n    }\n  }); // Return the people manager\n\n  return peopleManager;\n};\n\n//# sourceURL=webpack://gd-sprest/./build/lib/peopleManager.js?')},"./build/lib/peoplePicker.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.PeoplePicker = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js");\n/**\r\n * People Picker\r\n */\n\n\nexports.PeoplePicker = function (targetInfo) {\n  var peoplePicker = new utils_1.Base(targetInfo); // Default the properties\n\n  peoplePicker.targetInfo.defaultToWebFl = true;\n  peoplePicker.targetInfo.endpoint = "SP.UI.ApplicationPages.ClientPeoplePickerWebServiceInterface";\n  peoplePicker.targetInfo.overrideDefaultRequestToHostFl = true; // Add the methods\n\n  utils_1.Request.addMethods(peoplePicker, {\n    __metadata: {\n      type: "peoplepicker"\n    }\n  }); // Return the people picker\n\n  return peoplePicker;\n};\n\n//# sourceURL=webpack://gd-sprest/./build/lib/peoplePicker.js?')},"./build/lib/profileLoader.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.ProfileLoader = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js");\n/**\r\n * Profile Loader\r\n */\n\n\nexports.ProfileLoader = function (targetInfo) {\n  var profileLoader = new utils_1.Base(targetInfo); // Default the properties\n\n  profileLoader.targetInfo.defaultToWebFl = true;\n  profileLoader.targetInfo.endpoint = "sp.userprofiles.profileloader.getprofileloader";\n  profileLoader.targetInfo.method = "POST"; // Add the methods\n\n  utils_1.Request.addMethods(profileLoader, {\n    __metadata: {\n      type: "SP.UserProfiles.ProfileLoader"\n    }\n  }); // Return the profile loader\n\n  return profileLoader;\n};\n\n//# sourceURL=webpack://gd-sprest/./build/lib/profileLoader.js?')},"./build/lib/search.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.Search = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js");\n/**\r\n * Search\r\n */\n\n\nexports.Search = function (url, targetInfo) {\n  var base = new utils_1.Base(targetInfo);\n  var search = base; // Default the properties\n\n  base.targetInfo.defaultToWebFl = true;\n  base.targetInfo.endpoint = "search"; // See if the web url exists\n\n  if (url) {\n    // Set the settings\n    base.targetInfo.url = url;\n  } // Add the methods\n\n\n  utils_1.Request.addMethods(search, {\n    __metadata: {\n      type: "Microsoft.Office.Server.Search.REST.SearchService"\n    }\n  });\n  /** The search query method */\n\n  search.searchquery = function (settings) {\n    // Execute the request\n    return search.executeMethod("query", {\n      argNames: ["query"],\n      name: "query?[[query]]",\n      requestType: utils_1.RequestType.GetReplace\n    }, exports.Search.getQuery(settings));\n  }; // Return the search\n\n\n  return search;\n}; // Static method to compute the query\n\n\nexports.Search.getQuery = function (parameters) {\n  var query = ""; // Parse the parameters\n\n  for (var key in parameters) {\n    // Append the parameter to the query\n    query += (query == "" ? "" : "&") + key + "=\'" + parameters[key] + "\'";\n  } // Return the query\n\n\n  return [query];\n}; // Static post query method\n\n\nexports.Search.postQuery = function (props) {\n  // Return a promise\n  return new Promise(function (resolve, reject) {\n    var queryProps = props.query; // Set the get all items flag\n\n    var getAllItems = typeof props.getAllItems === "boolean" ? props.getAllItems : false; // Compute the row count\n\n    var rowCount = 500;\n\n    if (typeof queryProps.RowLimit === "number") {\n      // Set the custom limit\n      rowCount = queryProps.RowLimit;\n    } else {\n      // Default to the max size\n      queryProps.RowLimit = rowCount;\n    } // Query the first batch\n\n\n    exports.Search(props.url, props.targetInfo).postquery(queryProps).execute( // Success\n    function (request) {\n      // Updates the table\n      var updateRequest = function updateRequest(searchResult) {\n        // Ensure the results exist\n        if (searchResult) {\n          // Call the event\n          props.onQueryCompleted ? props.onQueryCompleted(searchResult) : null; // Parse rows\n\n          for (var i = 0; i < searchResult.PrimaryQueryResult.RelevantResults.Table.Rows.results.length; i++) {\n            var row = searchResult.PrimaryQueryResult.RelevantResults.Table.Rows.results[i]; // Append the row\n\n            request.postquery.PrimaryQueryResult.RelevantResults.Table.Rows.results.push(row);\n          }\n        }\n      }; // Call the event\n\n\n      props.onQueryCompleted ? props.onQueryCompleted(request.postquery) : null; // See if more results exist\n\n      var results = request.postquery.PrimaryQueryResult.RelevantResults;\n\n      if (getAllItems && results.TotalRows > results.RowCount) {\n        var search = exports.Search(props.url, props.targetInfo);\n        var useBatch = typeof props.useBatch === "boolean" ? props.useBatch : true; // Compute the total # of requests that we need to make\n\n        var totalPages = Math.ceil(results.TotalRows / rowCount); // Loop for the total # of requests\n\n        for (var i = 1; i < totalPages; i++) {\n          // Set the start row\n          queryProps.StartRow = i * rowCount; // See if we are making a batch request\n\n          if (useBatch) {\n            // Create a batch request\n            search.postquery(queryProps).batch( // Success\n            function (batchRequest) {\n              // Update the request\n              updateRequest(batchRequest.postquery);\n            }, // Limit to 100 per request\n            i % 100 == 0);\n          } else {\n            // Create the request\n            search.postquery(queryProps).execute( // Success\n            function (batchRequest) {\n              // Update the request\n              updateRequest(batchRequest.postquery);\n            }, // Wait for the previous request to complete\n            true);\n          }\n        } // See if we are making a batch request\n\n\n        if (useBatch) {\n          // Execute the batch requests\n          search.execute(function () {\n            // Resolve the request\n            resolve(request.postquery);\n          }, reject);\n        } else {\n          // Wait for the requests to complete\n          search.done(function () {\n            // Resolve the request\n            resolve(request.postquery);\n          });\n        }\n      } else {\n        // Resolve the request\n        resolve(request.postquery);\n      }\n    }, // Error\n    reject);\n  });\n};\n\n//# sourceURL=webpack://gd-sprest/./build/lib/search.js?')},"./build/lib/site.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.Site = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js");\n/**\r\n * Site\r\n */\n\n\nexports.Site = function (url, targetInfo) {\n  var site = new utils_1.Base(targetInfo); // Default the properties\n\n  site.targetInfo.defaultToWebFl = true;\n  site.targetInfo.endpoint = "site"; // See if the web url exists\n\n  if (url) {\n    // Set the settings\n    site.targetInfo.url = url;\n  } // Add the methods\n\n\n  utils_1.Request.addMethods(site, {\n    __metadata: {\n      type: "SP.Site"\n    }\n  }); // Return the site\n\n  return site;\n}; // Static method to see if a site exists\n\n\nexports.Site.exists = function (url) {\n  // Return the base object\n  return new utils_1.Base({\n    data: {\n      url: url\n    },\n    defaultToWebFl: true,\n    endpoint: "SP.Site.Exists",\n    method: "POST"\n  });\n}; // Static method to get the app context\n\n\nexports.Site.getAppContext = function (siteUrl) {\n  // Return the base object\n  return new utils_1.Base({\n    data: {\n      siteUrl: siteUrl\n    },\n    defaultToWebFl: true,\n    endpoint: "SP.AppContextSite",\n    method: "POST"\n  });\n}; // Method to get the url by id\n\n\nexports.Site.getUrlById = function (id) {\n  // Return the base object\n  return new utils_1.Base({\n    data: {\n      id: id\n    },\n    defaultToWebFl: true,\n    endpoint: "SP.Site.GetUrlById",\n    method: "POST"\n  });\n};\n\n//# sourceURL=webpack://gd-sprest/./build/lib/site.js?')},"./build/lib/siteIconManager.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.SiteIconManager = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js");\n/**\r\n * Site Icon Manager\r\n */\n\n\nexports.SiteIconManager = function (url, targetInfo) {\n  var siteIconMgr = new utils_1.Base(targetInfo); // Default the properties\n\n  siteIconMgr.targetInfo.defaultToWebFl = true;\n  siteIconMgr.targetInfo.endpoint = "SiteIconManager"; // See if the web url exists\n\n  if (url) {\n    // Set the settings\n    siteIconMgr.targetInfo.url = url;\n  } // Add the methods\n\n\n  utils_1.Request.addMethods(siteIconMgr, {\n    __metadata: {\n      type: "Microsoft.SharePoint.Portal.SiteIconManager"\n    }\n  }); // Return the site\n\n  return siteIconMgr;\n};\n\n//# sourceURL=webpack://gd-sprest/./build/lib/siteIconManager.js?')},"./build/lib/siteManager.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.SiteManager = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js");\n/**\r\n * Site Manager\r\n */\n\n\nexports.SiteManager = function (url, targetInfo) {\n  var siteMgr = new utils_1.Base(targetInfo); // Default the properties\n\n  siteMgr.targetInfo.defaultToWebFl = true;\n  siteMgr.targetInfo.endpoint = "SPSiteManager"; // See if the web url exists\n\n  if (url) {\n    // Set the settings\n    siteMgr.targetInfo.url = url;\n  } // Add the methods\n\n\n  utils_1.Request.addMethods(siteMgr, {\n    __metadata: {\n      type: "Microsoft.SharePoint.Portal.SPSiteManager"\n    }\n  }); // Return the site\n\n  return siteMgr;\n};\n\n//# sourceURL=webpack://gd-sprest/./build/lib/siteManager.js?')},"./build/lib/sitePages.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nvar __assign = this && this.__assign || function () {\n  __assign = Object.assign || function (t) {\n    for (var s, i = 1, n = arguments.length; i < n; i++) {\n      s = arguments[i];\n\n      for (var p in s) {\n        if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];\n      }\n    }\n\n    return t;\n  };\n\n  return __assign.apply(this, arguments);\n};\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.SitePages = void 0;\n\nvar web_1 = __webpack_require__(/*! ./web */ "./build/lib/web.js");\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js");\n\nvar contextInfo_1 = __webpack_require__(/*! ./contextInfo */ "./build/lib/contextInfo.js");\n/**\r\n * Site Pages\r\n */\n\n\nexports.SitePages = function (url, targetInfo) {\n  var sitePages = new utils_1.Base(targetInfo); // Default the properties\n\n  sitePages.targetInfo.defaultToWebFl = true;\n  sitePages.targetInfo.endpoint = "SitePages"; // See if the web url exists\n\n  if (url) {\n    // Set the settings\n    sitePages.targetInfo.url = url;\n  } // Add the methods\n\n\n  utils_1.Request.addMethods(sitePages, {\n    __metadata: {\n      type: "SP.Publishing.SitePageService"\n    }\n  }); // Return the site pages\n\n  return sitePages;\n}; // Static method to convert a modern page type\n\n\nexports.SitePages.convertPage = function (pageUrl, layout, webUrl) {\n  // Return a promise\n  return new Promise(function (resolve, reject) {\n    // Get the page\n    var getPage = function getPage(pageUrl) {\n      // Return a promise\n      return new Promise(function (resolve, reject) {\n        // See if the web url is specified\n        if (webUrl) {\n          // Get the context info\n          contextInfo_1.ContextInfo.getWeb(webUrl).execute(function (context) {\n            // Get the page\n            web_1.Web(webUrl, {\n              requestDigest: context.GetContextWebInformation.FormDigestValue\n            }).Lists("Site Pages").Items().query({\n              Filter: "FileLeafRef eq \'" + pageUrl + "\'"\n            }).execute(function (items) {\n              // Resolve the request\n              resolve(items.results[0]);\n            }, reject);\n          }, reject);\n        } else {\n          // Get the page\n          web_1.Web().Lists("Site Pages").Items().query({\n            Filter: "FileLeafRef eq \'" + pageUrl + "\'"\n          }).execute(function (items) {\n            // Resolve the request\n            resolve(items.results[0]);\n          }, reject);\n        }\n      });\n    }; // Get the page\n\n\n    getPage(pageUrl).then(function (item) {\n      // Update the item\n      item.update({\n        PageLayoutType: layout\n      }).execute(resolve, reject);\n    }, function () {\n      // Log\n      console.error("Unable to get the page: " + pageUrl); // Reject the request\n\n      reject();\n    });\n  });\n}; // Static method to create a modern page\n\n\nexports.SitePages.createPage = function (pageName, pageTitle, pageTemplate, url, targetInfo) {\n  var getContext = function getContext() {\n    return new Promise(function (resolve) {\n      // See if the web url doesn\'t exists\n      if (url == null) {\n        resolve(null);\n      } // Get the context of target web\n\n\n      contextInfo_1.ContextInfo.getWeb(url).execute(function (context) {\n        // Resolve the request\n        resolve(context.GetContextWebInformation.FormDigestValue);\n      });\n    });\n  }; // Return a promise\n\n\n  return new Promise(function (resolve, reject) {\n    // Method called after the updates have completed\n    var onComplete = function onComplete(itemId, fileUrl) {\n      var web = web_1.Web(url, targetInfo);\n      var results = {\n        file: null,\n        item: null,\n        page: null\n      }; // Get the file\n\n      web.getFileByUrl(fileUrl).query({\n        Select: ["*", "ListId"]\n      }).execute(function (file) {\n        // Set the file\n        results.file = file; // Get the list\n\n        web.Lists().getById(file.ListId).Items(itemId).execute(function (item) {\n          // Set the item\n          results.item = item; // Get the page\n\n          exports.SitePages(url, targetInfo).Pages(itemId).execute(function (page) {\n            // Set the page\n            results.page = page; // Resolve the request\n\n            resolve(results);\n          }, reject);\n        }, reject);\n      }, reject);\n    }; // Get the context information\n\n\n    getContext().then(function (requestDigest) {\n      // Set the target information\n      targetInfo = __assign({\n        requestDigest: requestDigest\n      }, targetInfo); // Create the page\n\n      exports.SitePages(url, targetInfo).Pages().createAppPage({\n        Title: pageTitle,\n        PageLayoutType: pageTemplate\n      }).execute(function (page) {\n        // Update the file name\n        web_1.Web(url, targetInfo).Lists("Site Pages").Items(page.Id).update({\n          FileLeafRef: pageName\n        }).execute( // Updated the file name successfully\n        function () {\n          // Update the file url\n          var idx = page.Url.lastIndexOf(\'/\');\n          var fileUrl = page.Url.substring(0, idx + 1) + pageName; // Complete the request\n\n          onComplete(page.Id, fileUrl);\n        }, // Unable to update the file name, but still return the object\n        function () {\n          // Complete the request\n          onComplete(page.Id, page.Url);\n        });\n      }, reject);\n    });\n  });\n};\n\n//# sourceURL=webpack://gd-sprest/./build/lib/sitePages.js?')},"./build/lib/socialFeed.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.SocialFeed = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js");\n/**\r\n * Social Feed\r\n */\n\n\nexports.SocialFeed = function (targetInfo) {\n  var base = new utils_1.Base(targetInfo);\n  var socialFeed = base; // Default the properties\n\n  base.targetInfo.defaultToWebFl = true;\n  base.targetInfo.endpoint = "social.feed"; // Add the methods\n\n  utils_1.Request.addMethods(socialFeed, {\n    __metadata: {\n      type: "SP.Social.SocialRestFeedManager"\n    }\n  }); // Return the social feed\n\n  return socialFeed;\n}; // Method to post to another user\'s feed\n\n\nexports.SocialFeed.postToFeed = function (accountName, creationData) {\n  var postInfo = {\n    ID: null,\n    creationData: creationData\n  }; // Set the post metadata\n\n  postInfo["__metadata"] = {\n    type: "SP.Social.SocialRestPostCreationData"\n  };\n  postInfo.creationData["__metadata"] = {\n    type: "SP.Social.SocialPostCreationData"\n  }; // Execute the request\n\n  return exports.SocialFeed().executeMethod("postToMyFeed", {\n    argNames: ["restCreationData"],\n    name: "actor(item=@v)/feed?@v=\'" + encodeURIComponent(accountName) + "\'",\n    requestType: utils_1.RequestType.PostWithArgsInBody\n  }, [postInfo]);\n}; // Method to post to the current user\'s feed\n\n\nexports.SocialFeed.postToMyFeed = function (creationData) {\n  var postInfo = {\n    ID: null,\n    creationData: creationData\n  }; // Set the post metadata\n\n  postInfo["__metadata"] = {\n    type: "SP.Social.SocialRestPostCreationData"\n  };\n  postInfo.creationData["__metadata"] = {\n    type: "SP.Social.SocialPostCreationData"\n  }; // Execute the request\n\n  return exports.SocialFeed().executeMethod("postToMyFeed", {\n    argNames: ["restCreationData"],\n    name: "my/feed/post",\n    requestType: utils_1.RequestType.PostWithArgsInBody\n  }, [postInfo]);\n};\n\n//# sourceURL=webpack://gd-sprest/./build/lib/socialFeed.js?')},"./build/lib/themeManager.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.ThemeManager = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js");\n/**\r\n * Theme Manager\r\n */\n\n\nexports.ThemeManager = function (targetInfo) {\n  var mgr = new utils_1.Base(targetInfo); // Default the properties\n\n  mgr.targetInfo.defaultToWebFl = true;\n  mgr.targetInfo.endpoint = "thememanager"; // Add the methods\n\n  utils_1.Request.addMethods(mgr, {\n    __metadata: {\n      type: "SP.Utilities.ThemeManager"\n    }\n  }); // Return the theme manager\n\n  return mgr;\n};\n\n//# sourceURL=webpack://gd-sprest/./build/lib/themeManager.js?')},"./build/lib/userProfile.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.UserProfile = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js");\n/**\r\n * User Profile\r\n */\n\n\nexports.UserProfile = function (targetInfo) {\n  var userProfile = new utils_1.Base(targetInfo); // Default the properties\n\n  userProfile.targetInfo.defaultToWebFl = true;\n  userProfile.targetInfo.endpoint = "sp.userprofiles.profileloader.getprofileloader/getUserProfile";\n  userProfile.targetInfo.method = "POST"; // Add the methods\n\n  utils_1.Request.addMethods(userProfile, {\n    __metadata: {\n      type: "SP.UserProfiles.UserProfile"\n    }\n  }); // Return the user profile\n\n  return userProfile;\n};\n\n//# sourceURL=webpack://gd-sprest/./build/lib/userProfile.js?')},"./build/lib/utility.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.Utility = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js");\n/**\r\n * Utility\r\n */\n\n\nexports.Utility = function (url, targetInfo) {\n  var base = new utils_1.Base(targetInfo);\n  var utility = base; // Default the properties\n\n  base.targetInfo.defaultToWebFl = true;\n  base.targetInfo.endpoint = "SP.Utilities.Utility"; // See if the web url exists\n\n  if (url) {\n    // Set the settings\n    base.targetInfo.url = url;\n  } // Add the methods\n\n\n  utils_1.Request.addMethods(utility, {\n    __metadata: {\n      type: "utility"\n    }\n  }); // Method to create a wiki page\n\n  utility.createWikiPage = function (listUrl, content) {\n    if (content === void 0) {\n      content = "";\n    }\n\n    var parameters = {\n      ServerRelativeUrl: listUrl,\n      WikiHtmlContent: content\n    }; // Execute the method\n\n    return utility.executeMethod("createWikiPage", {\n      argNames: ["parameters"],\n      name: "SP.Utilities.Utility.CreateWikiPageInContextWeb",\n      replaceEndpointFl: true,\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    }, [parameters]);\n  }; // Method to send an email\n\n\n  utility.sendEmail = function (properties) {\n    // Parse the email properties\n    for (var _i = 0, _a = ["To", "CC", "BCC"]; _i < _a.length; _i++) {\n      var propName = _a[_i];\n      var propValue = properties[propName]; // Ensure the value exists\n\n      if (propValue) {\n        // See if it\'s a string\n        if (typeof propValue === "string") {\n          // Add the results property\n          properties[propName] = {\n            \'results\': [propValue]\n          };\n        } // Else, assume it\'s an array\n        else {\n          // Add the results property\n          properties[propName] = {\n            \'results\': propValue\n          };\n        }\n      }\n    } // Execute the method\n\n\n    return utility.executeMethod("sendEmail", {\n      argNames: ["properties"],\n      metadataType: "SP.Utilities.EmailProperties",\n      name: "SP.Utilities.Utility.sendEmail",\n      replaceEndpointFl: true,\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    }, [properties]);\n  }; // Return the utility\n\n\n  return utility;\n};\n\n//# sourceURL=webpack://gd-sprest/./build/lib/utility.js?')},"./build/lib/web.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.Web = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js");\n\nexports.Web = function (url, targetInfo) {\n  var web = new utils_1.Base(targetInfo); // Default the properties\n\n  web.targetInfo.defaultToWebFl = true;\n  web.targetInfo.endpoint = "web"; // See if the web url exists\n\n  if (url) {\n    // Set the settings\n    web.targetInfo.url = url;\n  } // Add the methods\n\n\n  utils_1.Request.addMethods(web, {\n    __metadata: {\n      type: "SP.Web"\n    }\n  }); // Return the web\n\n  return web;\n}; // Static method to get a remote web\n\n\nexports.Web.getRemoteWeb = function (requestUrl) {\n  // Return the remote web information\n  return new utils_1.Base({\n    data: {\n      requestUrl: requestUrl\n    },\n    defaultToWebFl: true,\n    endpoint: "SP.RemoteWeb?$expand=Web",\n    method: "POST"\n  });\n}; // Static method to get the url of a web from a page url\n\n\nexports.Web.getWebUrlFromPageUrl = function (pageUrl) {\n  // Return the remote web information\n  return new utils_1.Base({\n    endpoint: "SP.Web.GetWebUrlFromPageUrl(@v)?@v=\'" + pageUrl + "\'",\n    method: "POST"\n  });\n};\n\n//# sourceURL=webpack://gd-sprest/./build/lib/web.js?')},"./build/lib/webTemplateExtensions.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.WebTemplateExtensions = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js");\n/**\r\n * Web Template Extensions\r\n */\n\n\nexports.WebTemplateExtensions = function (url, targetInfo) {\n  var webTemplateExtensions = new utils_1.Base(targetInfo); // Default the properties\n\n  webTemplateExtensions.targetInfo.defaultToWebFl = true;\n  webTemplateExtensions.targetInfo.endpoint = "Microsoft.SharePoint.Utilities.WebTemplateExtensions.SiteScriptUtility"; // See if the web url exists\n\n  if (url) {\n    // Set the settings\n    webTemplateExtensions.targetInfo.url = url;\n  } // Add the methods\n\n\n  utils_1.Request.addMethods(webTemplateExtensions, {\n    __metadata: {\n      type: "webTemplateExtensions"\n    }\n  }); // Return the web template extension utilities\n\n  return webTemplateExtensions;\n};\n\n//# sourceURL=webpack://gd-sprest/./build/lib/webTemplateExtensions.js?')},"./build/lib/wfInstanceService.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.WorkflowInstanceService = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js");\n/**\r\n * Workflow Instance Service\r\n */\n\n\nexports.WorkflowInstanceService = function (url, targetInfo) {\n  var wfs = new utils_1.Base(targetInfo); // Default the properties\n\n  wfs.targetInfo.defaultToWebFl = true;\n  wfs.targetInfo.endpoint = "SP.WorkflowServices.WorkflowInstanceService.Current"; // See if the web url exists\n\n  if (url) {\n    // Set the settings\n    wfs.targetInfo.url = url;\n  } // Add the methods\n\n\n  utils_1.Request.addMethods(wfs, {\n    __metadata: {\n      type: "SP.WorkflowServices.WorkflowInstanceService"\n    }\n  }); // Return the workflow service\n\n  return wfs;\n};\n\n//# sourceURL=webpack://gd-sprest/./build/lib/wfInstanceService.js?')},"./build/lib/wfSubscriptionService.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.WorkflowSubscriptionService = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js");\n/**\r\n * Workflow Subscription Service\r\n */\n\n\nexports.WorkflowSubscriptionService = function (url, targetInfo) {\n  var wfs = new utils_1.Base(targetInfo); // Default the properties\n\n  wfs.targetInfo.defaultToWebFl = true;\n  wfs.targetInfo.endpoint = "SP.WorkflowServices.WorkflowSubscriptionService.Current"; // See if the web url exists\n\n  if (url) {\n    // Set the settings\n    wfs.targetInfo.url = url;\n  } // Add the methods\n\n\n  utils_1.Request.addMethods(wfs, {\n    __metadata: {\n      type: "SP.WorkflowServices.WorkflowSubscriptionService"\n    }\n  }); // Return the workflow service\n\n  return wfs;\n};\n\n//# sourceURL=webpack://gd-sprest/./build/lib/wfSubscriptionService.js?')},"./build/mapper/custom/audit.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.audit = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../../utils */ "./build/utils/index.js");\n/**\r\n * Audit\r\n */\n\n\nexports.audit = {\n  // Queries the collection\n  query: {\n    argNames: ["oData"],\n    requestType: utils_1.RequestType.OData\n  }\n};\n\n//# sourceURL=webpack://gd-sprest/./build/mapper/custom/audit.js?')},"./build/mapper/custom/graph.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.graph = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../../utils */ "./build/utils/index.js");\n/**\r\n * Graph\r\n */\n\n\nexports.graph = {\n  // Me\n  me: {\n    requestType: utils_1.RequestType.GraphGet\n  },\n  // Group\n  group: {\n    argNames: ["id"],\n    name: "groups/[[id]]",\n    requestType: utils_1.RequestType.GraphGetReplace\n  },\n  // Groups\n  groups: {\n    name: "groups",\n    requestType: utils_1.RequestType.GraphGet\n  },\n  // List\n  list: {\n    argNames: ["siteId", "id"],\n    name: "sites/[[siteId]]/lists/[[id]]",\n    requestType: utils_1.RequestType.GraphGetReplace\n  },\n  // Lists\n  lists: {\n    argNames: ["siteId"],\n    name: "sites/[[siteId]]/lists",\n    requestType: utils_1.RequestType.GraphGetReplace\n  },\n  // Queries the collection\n  query: {\n    argNames: ["oData"],\n    requestType: utils_1.RequestType.OData\n  },\n  // Root Site\n  root: {\n    requestType: utils_1.RequestType.GraphGet\n  },\n  // Site\n  site: {\n    argNames: ["id"],\n    name: "sites/[[id]]",\n    requestType: utils_1.RequestType.GraphGetReplace\n  },\n  // Sites\n  sites: {\n    name: "sites",\n    requestType: utils_1.RequestType.GraphGet\n  },\n  // User\n  user: {\n    argNames: ["id"],\n    name: "users/[[id]]",\n    requestType: utils_1.RequestType.GraphGetReplace\n  },\n  // Users\n  users: {\n    requestType: utils_1.RequestType.GraphGet\n  }\n};\n\n//# sourceURL=webpack://gd-sprest/./build/mapper/custom/graph.js?')},"./build/mapper/custom/index.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nvar __createBinding = this && this.__createBinding || (Object.create ? function (o, m, k, k2) {\n  if (k2 === undefined) k2 = k;\n  Object.defineProperty(o, k2, {\n    enumerable: true,\n    get: function get() {\n      return m[k];\n    }\n  });\n} : function (o, m, k, k2) {\n  if (k2 === undefined) k2 = k;\n  o[k2] = m[k];\n});\n\nvar __exportStar = this && this.__exportStar || function (m, exports) {\n  for (var p in m) {\n    if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);\n  }\n};\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\n\n__exportStar(__webpack_require__(/*! ./audit */ "./build/mapper/custom/audit.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./graph */ "./build/mapper/custom/graph.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./old */ "./build/mapper/custom/old.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./peoplePicker */ "./build/mapper/custom/peoplePicker.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./propertyValues */ "./build/mapper/custom/propertyValues.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./utility */ "./build/mapper/custom/utility.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./webTemplateExtensions */ "./build/mapper/custom/webTemplateExtensions.js"), exports);\n\n//# sourceURL=webpack://gd-sprest/./build/mapper/custom/index.js?')},"./build/mapper/custom/old.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.webinfos = exports.webs = exports.web = exports.viewfieldcollection = exports.views = exports.view = exports.versions = exports.usercustomactions = exports.usercustomaction = exports.users = exports.user = exports.tenantappcatalog = exports.tenantapps = exports.tenantapp = exports.sitecollectionappcatalog = exports.site = exports.search = exports.roledefinitions = exports.roledefinition = exports.roleassignments = exports.roleassignment = exports.items = exports.listitem = exports.lists = exports.list = exports.limitedwebpartmanager = exports.sitegroups = exports.group = exports.folders = exports.folder = exports.fileversions = exports.fileversion = exports.files = exports.file = exports.fieldlinks = exports.fields = exports.field = exports.features = exports.eventreceivers = exports.eventreceiver = exports.attachmentfiles = void 0;\n\nvar def_1 = __webpack_require__(/*! ../def */ "./build/mapper/def.js");\n/**\r\n * This is required for dynamic metadata types\r\n */\n\n\nexports.attachmentfiles = def_1.Mapper["SP.Attachment.Collection"];\nexports.eventreceiver = def_1.Mapper["SP.EventReceiverDefinition"];\nexports.eventreceivers = def_1.Mapper["SP.EventReceiverDefinition.Collection"];\nexports.features = def_1.Mapper["SP.Feature.Collection"];\nexports.field = def_1.Mapper["SP.Field"];\nexports.fields = def_1.Mapper["SP.Field.Collection"];\nexports.fieldlinks = def_1.Mapper["SP.FieldLink.Collection"];\nexports.file = def_1.Mapper["SP.File"];\nexports.files = def_1.Mapper["SP.File.Collection"];\nexports.fileversion = def_1.Mapper["SP.FileVersion"];\nexports.fileversions = def_1.Mapper["SP.FileVersion.Collection"];\nexports.folder = def_1.Mapper["SP.Folder"];\nexports.folders = def_1.Mapper["SP.Folder.Collection"];\nexports.group = def_1.Mapper["SP.Group"];\nexports.sitegroups = def_1.Mapper["SP.Directory.Group.Collection"];\nexports.limitedwebpartmanager = def_1.Mapper["SP.WebParts.LimitedWebPartManager"];\nexports.list = def_1.Mapper["SP.List"];\nexports.lists = def_1.Mapper["SP.List.Collection"];\nexports.listitem = def_1.Mapper["SP.ListItem"];\nexports.items = def_1.Mapper["SP.ListItem.Collection"];\nexports.roleassignment = def_1.Mapper["SP.RoleAssignment"];\nexports.roleassignments = def_1.Mapper["SP.RoleAssignment.Collection"];\nexports.roledefinition = def_1.Mapper["SP.RoleDefinition"];\nexports.roledefinitions = def_1.Mapper["SP.RoleDefinition.Collection"];\nexports.search = def_1.Mapper["Microsoft.Office.Server.Search.REST.SearchService"];\nexports.site = def_1.Mapper["SP.Site"];\nexports.sitecollectionappcatalog = def_1.Mapper["Microsoft.SharePoint.Marketplace.CorporateCuratedGallery.SiteCollectionCorporateCatalogAccessor"];\nexports.tenantapp = def_1.Mapper["Microsoft.SharePoint.Marketplace.CorporateCuratedGallery.CorporateCatalogAppMetadata"];\nexports.tenantapps = def_1.Mapper["Microsoft.SharePoint.Marketplace.CorporateCuratedGallery.CorporateCatalogAppMetadata.Collection"];\nexports.tenantappcatalog = def_1.Mapper["Microsoft.SharePoint.Marketplace.CorporateCuratedGallery.TenantCorporateCatalogAccessor"];\nexports.user = def_1.Mapper["SP.User"];\nexports.users = def_1.Mapper["SP.User.Collection"];\nexports.usercustomaction = def_1.Mapper["SP.UserCustomAction"];\nexports.usercustomactions = def_1.Mapper["SP.UserCustomAction.Collection"];\nexports.versions = def_1.Mapper["SP.FileVersion.Collection"];\nexports.view = def_1.Mapper["SP.View"];\nexports.views = def_1.Mapper["SP.View.Collection"];\nexports.viewfieldcollection = def_1.Mapper["SP.ViewFieldCollection"];\nexports.web = def_1.Mapper["SP.Web"];\nexports.webs = def_1.Mapper["SP.Web.Collection"];\nexports.webinfos = def_1.Mapper["SP.WebInformation.Collection"];\n\n//# sourceURL=webpack://gd-sprest/./build/mapper/custom/old.js?')},"./build/mapper/custom/peoplePicker.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.peoplepicker = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../../utils */ "./build/utils/index.js");\n/**\r\n * People Picker\r\n */\n\n\nexports.peoplepicker = {\n  clientPeoplePickerResolveUser: {\n    argNames: ["queryParams"],\n    metadataType: "SP.UI.ApplicationPages.ClientPeoplePickerQueryParameters",\n    name: "SP.UI.ApplicationPages.ClientPeoplePickerWebServiceInterface.ClientPeoplePickerResolveUser",\n    replaceEndpointFl: true,\n    requestType: utils_1.RequestType.PostWithArgsInBody\n  },\n  clientPeoplePickerSearchUser: {\n    argNames: ["queryParams"],\n    metadataType: "SP.UI.ApplicationPages.ClientPeoplePickerQueryParameters",\n    name: "SP.UI.ApplicationPages.ClientPeoplePickerWebServiceInterface.ClientPeoplePickerSearchUser",\n    replaceEndpointFl: true,\n    requestType: utils_1.RequestType.PostWithArgsInBody\n  }\n};\n\n//# sourceURL=webpack://gd-sprest/./build/mapper/custom/peoplePicker.js?')},"./build/mapper/custom/propertyValues.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.propertyvalues = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../../utils */ "./build/utils/index.js");\n/**\r\n * Property Values\r\n */\n\n\nexports.propertyvalues = {\n  // Queries the collection\n  query: {\n    argNames: ["oData"],\n    requestType: utils_1.RequestType.OData\n  }\n};\n\n//# sourceURL=webpack://gd-sprest/./build/mapper/custom/propertyValues.js?')},"./build/mapper/custom/utility.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.utility = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../../utils */ "./build/utils/index.js");\n/**\r\n * Utility\r\n */\n\n\nexports.utility = {\n  createEmailBodyForInvitation: {\n    argNames: ["pageAddress"],\n    name: "SP.Utilities.Utility.CreateEmailBodyForInvitation",\n    replaceEndpointFl: true,\n    requestType: utils_1.RequestType.PostWithArgsInBody\n  },\n  createWikiPage: {\n    argNames: ["parameters"],\n    name: "SP.Utilities.Utility.CreateWikiPageInContextWeb",\n    replaceEndpointFl: true,\n    requestType: utils_1.RequestType.PostWithArgsInBody\n  },\n  getAppLicenseDeploymentId: {\n    name: "SP.Utilities.Utility.GetAppLicenseDeploymentId",\n    replaceEndpointFl: true,\n    requestType: utils_1.RequestType.Get\n  },\n  getAppLicenseInformation: {\n    name: "SP.Utilities.Utility.GetAppLicenseInformation",\n    replaceEndpointFl: true,\n    requestType: utils_1.RequestType.Get\n  },\n  getCurrentUserEmailAddresses: {\n    name: "SP.Utilities.Utility.GetCurrentUserEmailAddresses",\n    replaceEndpointFl: true,\n    requestType: utils_1.RequestType.Get\n  },\n  getLocalizedString: {\n    argNames: ["sourceValue"],\n    name: "SP.Utilities.Utility.GetLocalizedString",\n    replaceEndpointFl: true,\n    requestType: utils_1.RequestType.GetWithArgsValueOnly\n  },\n  getLowerCaseString: {\n    argNames: ["sourceValue", "lcid"],\n    name: "SP.Utilities.Utility.GetLowerCaseString",\n    replaceEndpointFl: true,\n    requestType: utils_1.RequestType.PostWithArgsInBody\n  },\n  importAppLicense: {\n    argNames: ["url"],\n    name: "SP.Utilities.Utility.ImportAppLicense",\n    replaceEndpointFl: true,\n    requestType: utils_1.RequestType.PostWithArgs\n  },\n  isUserLicensedForEntityInContext: {\n    argNames: ["url"],\n    name: "SP.Utilities.Utility.IsUserLicensedForEntityInContext",\n    replaceEndpointFl: true,\n    requestType: utils_1.RequestType.PostWithArgs\n  },\n  localizeWebPartGallery: {\n    argNames: ["url"],\n    name: "SP.Utilities.Utility.LocalizeWebPartGallery",\n    replaceEndpointFl: true,\n    requestType: utils_1.RequestType.PostWithArgs\n  },\n  markDiscussionAsFeatured: {\n    argNames: ["url"],\n    name: "SP.Utilities.Utility.MarkDiscussionAsFeatured",\n    replaceEndpointFl: true,\n    requestType: utils_1.RequestType.PostWithArgs\n  },\n  resolvePrincipal: {\n    name: "SP.Utilities.Utility.ResolvePrincipalInCurrentContext",\n    replaceEndpointFl: true,\n    requestType: utils_1.RequestType.Post\n  },\n  searchPrincipals: {\n    name: "SP.Utilities.Utility.SearchPrincipalsUsingContextWeb",\n    replaceEndpointFl: true,\n    requestType: utils_1.RequestType.Post\n  },\n  sendEmail: {\n    argNames: ["properties"],\n    metadataType: "SP.Utilities.EmailProperties",\n    name: "SP.Utilities.Utility.sendEmail",\n    replaceEndpointFl: true,\n    requestType: utils_1.RequestType.PostWithArgsInBody\n  },\n  unmarkDiscussionAsFeatured: {\n    argNames: ["url"],\n    name: "SP.Utilities.Utility.UnmarkDiscussionAsFeatured",\n    replaceEndpointFl: true,\n    requestType: utils_1.RequestType.PostWithArgsInBody\n  }\n};\n\n//# sourceURL=webpack://gd-sprest/./build/mapper/custom/utility.js?')},"./build/mapper/custom/webTemplateExtensions.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.webtemplateextensions = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../../utils */ "./build/utils/index.js");\n/**\r\n * Web Template Extensions\r\n * https://docs.microsoft.com/en-us/sharepoint/dev/declarative-customization/site-design-rest-api\r\n */\n\n\nexports.webtemplateextensions = {\n  applySiteDesign: {\n    argNames: ["siteDesignId", "webUrl"],\n    appendEndpointFl: true,\n    name: "ApplySiteDesign",\n    requestType: utils_1.RequestType.PostWithArgsInBody\n  },\n  addSiteDesignTaskToCurrentWeb: {\n    argNames: ["siteDesignId"],\n    appendEndpointFl: true,\n    name: "AddSiteDesignTaskToCurrentWeb",\n    requestType: utils_1.RequestType.PostWithArgsInBody\n  },\n  createSiteDesign: {\n    argNames: ["info"],\n    appendEndpointFl: true,\n    name: "CreateSiteDesign",\n    requestType: utils_1.RequestType.PostWithArgsInBody\n  },\n  createSiteScript: {\n    argNames: ["title", "content"],\n    appendEndpointFl: true,\n    name: "CreateSiteScript(@title)?@title=\'[[title]]",\n    requestType: utils_1.RequestType.PostReplaceWithData\n  },\n  deleteSiteDesign: {\n    argNames: ["id"],\n    appendEndpointFl: true,\n    name: "DeleteSiteDesign",\n    requestType: utils_1.RequestType.PostWithArgsInBody\n  },\n  deleteSiteScript: {\n    argNames: ["id"],\n    appendEndpointFl: true,\n    name: "DeleteSiteScript",\n    requestType: utils_1.RequestType.PostWithArgsInBody\n  },\n  getSiteDesigns: {\n    argNames: [],\n    appendEndpointFl: true,\n    name: "GetSiteDesigns",\n    requestType: utils_1.RequestType.Post\n  },\n  getSiteDesignMetadata: {\n    argNames: ["id"],\n    appendEndpointFl: true,\n    name: "GetSiteDesignMetadata",\n    requestType: utils_1.RequestType.PostWithArgsInBody\n  },\n  getSiteScripts: {\n    argNames: [],\n    appendEndpointFl: true,\n    name: "GetSiteScripts",\n    requestType: utils_1.RequestType.Post\n  },\n  getSiteScriptFromWeb: {\n    argNames: ["webUrl", "info"],\n    appendEndpointFl: true,\n    name: "GetSiteScriptFromWeb",\n    requestType: utils_1.RequestType.PostWithArgsInBody\n  },\n  getSiteScriptFromList: {\n    argNames: ["listUrl"],\n    appendEndpointFl: true,\n    name: "GetSiteScriptFromList",\n    requestType: utils_1.RequestType.PostWithArgsInBody\n  },\n  getSiteScriptMetadata: {\n    argNames: ["id"],\n    appendEndpointFl: true,\n    name: "GetSiteScriptMetadata",\n    requestType: utils_1.RequestType.PostWithArgsInBody\n  },\n  getSiteDesignRights: {\n    argNames: ["id"],\n    appendEndpointFl: true,\n    name: "GetSiteDesignRights",\n    requestType: utils_1.RequestType.PostWithArgsInBody\n  },\n  grantSiteDesignRights: {\n    argNames: ["id", "principalNames", "grantedRights"],\n    appendEndpointFl: true,\n    name: "GrantSiteDesignRights",\n    requestType: utils_1.RequestType.PostWithArgsInBody\n  },\n  revokeSiteDesignRights: {\n    argNames: ["id", "principalNames"],\n    appendEndpointFl: true,\n    name: "RevokeSiteDesignRights",\n    requestType: utils_1.RequestType.PostWithArgsInBody\n  },\n  updateSiteDesign: {\n    argNames: ["updateInfo"],\n    appendEndpointFl: true,\n    name: "UpdateSiteDesign",\n    requestType: utils_1.RequestType.PostWithArgsInBody\n  },\n  updateSiteScript: {\n    argNames: ["updateInfo"],\n    appendEndpointFl: true,\n    name: "UpdateSiteScript",\n    requestType: utils_1.RequestType.PostWithArgsInBody\n  }\n};\n\n//# sourceURL=webpack://gd-sprest/./build/mapper/custom/webTemplateExtensions.js?')},"./build/mapper/def.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.Mapper = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js");\n\nexports.Mapper = {\n  "MS.FileServices.File": {\n    copyTo: {\n      argNames: ["target", "overwrite"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    download: {},\n    moveTo: {\n      argNames: ["target", "overwrite"]\n    },\n    upload: {\n      argNames: ["stream"]\n    }\n  },\n  "MS.FileServices.FileSystemItem.Collection": {\n    add: {\n      argNames: ["name", "overwrite", "content"]\n    },\n    getById: {\n      argNames: ["id"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "MS.FileServices.Folder": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    moveTo: {\n      argNames: ["target"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "Microsoft.AppServices.AppCollection": {\n    getAppsFromStore: {\n      argNames: ["addInType", "queryString"]\n    },\n    getByType: {\n      argNames: ["type"]\n    }\n  },\n  "Microsoft.Office.Server.ContentCenter.SPMachineLearningHub": {\n    checkAIBuilderAccess: {\n      argNames: ["environmentName", "isTestEnvironment", "userId"]\n    },\n    getByContentTypeId: {\n      argNames: ["contentTypeId"]\n    },\n    getCDSMetadata: {\n      argNames: ["environmentName", "isTestEnvironment"]\n    },\n    getColumnLLMInfo: {\n      argNames: ["docLibId", "columnId"]\n    },\n    getLibraryLLMInfo: {\n      argNames: ["docLibId"]\n    },\n    getMachineLearningFlags: {\n      argNames: ["docLibId"]\n    },\n    getModelIdForContentType: {\n      argNames: ["contentTypeName"]\n    },\n    getModels: {\n      argNames: ["listId", "modelTypes", "publicationTypes", "includeManagementNotAllowedModels"]\n    },\n    getRetentionLabel: {\n      argNames: ["retentionLabelId"]\n    },\n    getRetentionLabels: {},\n    getSyntexPoweredColumnPrompts: {\n      argNames: ["docLibId"]\n    },\n    invokeDataverseQuery: {},\n    query: {\n      argNames: ["oData"]\n    },\n    setColumnLLMInfo: {\n      argNames: ["docLibId", "columnId", "autofillPrompt", "isEnabled"]\n    },\n    setMachineLearningFlags: {\n      argNames: ["docLibId", "machineLearningFlags"]\n    },\n    setSyntexPoweredColumnPrompts: {\n      argNames: ["docLibId", "syntexPoweredColumnPrompts"]\n    },\n    verifyModelUrls: {\n      argNames: ["urls"]\n    }\n  },\n  "Microsoft.Office.Server.ContentCenter.SPMachineLearningModel": {\n    addModelDependency: {\n      argNames: ["modelId", "updateExisting"]\n    },\n    copy: {\n      argNames: ["copyTo"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    "delete": {},\n    importMeta: {},\n    invokeConnectorQuery: {},\n    removeModelDependency: {\n      argNames: ["modelId"]\n    },\n    rename: {\n      argNames: ["renameTo"]\n    },\n    renameExtractor: {\n      argNames: ["fromExtractorName", "toExtractorName", "toColumnType"]\n    },\n    setAsModelAuthor: {},\n    update: {},\n    updateModelSettings: {\n      argNames: ["ModelSettings"]\n    },\n    updateModelTypeSpecificSettings: {\n      argNames: ["Settings"]\n    }\n  },\n  "Microsoft.Office.Server.ContentCenter.SPMachineLearningModel.Collection": {\n    getByTitle: {\n      argNames: ["title"]\n    },\n    getByUniqueId: {\n      argNames: ["uniqueId"]\n    },\n    getExtractorNames: {\n      argNames: ["packageName"]\n    },\n    "import": {\n      argNames: ["packageName"]\n    },\n    query: {\n      argNames: ["oData"]\n    },\n    setupPrimedLibrary: {\n      argNames: ["primedLibraryName", "packageName", "isTileViewEnabled", "serverRelativeLibraryPath"]\n    },\n    unbindModelFromContentType: {\n      argNames: ["contentTypeId"]\n    }\n  },\n  "Microsoft.Office.Server.ContentCenter.SPMachineLearningPublication": {\n    "delete": {},\n    update: {}\n  },\n  "Microsoft.Office.Server.ContentCenter.SPMachineLearningPublication.Collection": {\n    batchDelete: {\n      argNames: ["publications"]\n    },\n    batchRemove: {\n      argNames: ["modelSiteUrl", "modelWebServerRelativeUrl", "publications"]\n    },\n    batchUnpromote: {\n      argNames: ["promotions"]\n    },\n    checkTenantPublishPermissions: {},\n    getByModelUniqueId: {\n      argNames: ["modelUniqueId"]\n    },\n    getByModelUniqueIdAndPublicationType: {\n      argNames: ["modelUniqueId", "publicationType"]\n    },\n    getByUniqueId: {\n      argNames: ["uniqueId"]\n    },\n    query: {\n      argNames: ["oData"]\n    }\n  },\n  "Microsoft.Office.Server.ContentCenter.SPMachineLearningSample": {\n    update: {}\n  },\n  "Microsoft.Office.Server.ContentCenter.SPMachineLearningSample.Collection": {\n    getByModelId: {\n      argNames: ["modelID"]\n    },\n    getByTitle: {\n      argNames: ["title"]\n    },\n    getByUniqueId: {\n      argNames: ["uniqueId"]\n    },\n    getByUniqueIdWithTokenization: {\n      argNames: ["uniqueId"]\n    },\n    getTemplateByModelId: {\n      argNames: ["modelID"]\n    },\n    query: {\n      argNames: ["oData"]\n    }\n  },\n  "Microsoft.Office.Server.ContentCenter.SPMachineLearningWorkItem.Collection": {\n    getByIdentifier: {\n      argNames: ["identifier"]\n    },\n    query: {\n      argNames: ["oData"]\n    }\n  },\n  "Microsoft.Office.Server.Search.REST.SearchService": {\n    autocompletions: {\n      argNames: ["querytext", "sources", "numberOfCompletions", "cursorPosition"]\n    },\n    "export": {\n      argNames: ["userName", "startTime"]\n    },\n    exportmanualsuggestions: {},\n    exportpopulartenantqueries: {\n      argNames: ["count"]\n    },\n    postquery: {\n      argNames: ["request"],\n      metadataType: "Microsoft.Office.Server.Search.REST.SearchRequest",\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    query: {\n      argNames: ["querytext", "queryTemplate", "enableInterleaving", "sourceId", "rankingModelId", "startRow", "rowLimit", "rowsPerPage", "selectProperties", "culture", "refinementFilters", "refiners", "hiddenConstraints", "sortList", "enableStemming", "trimDuplicates", "timeout", "enableNicknames", "enablePhonetic", "enableFQL", "hitHighlightedProperties", "propertiesToGenerateAcronyms", "bypassResultTypes", "processBestBets", "clientType", "personalizationData", "resultsUrl", "queryTag", "trimDuplicatesIncludeId", "totalRowsExactMinimum", "impressionId", "properties", "enableQueryRules", "summaryLength", "maxSnippetLength", "desiredSnippetLength", "uiLanguage", "blockDedupeMode", "generateBlockRankLog", "enableSorting", "collapseSpecification", "processPersonalFavorites", "enableOrderingHitHighlightedProperty", "hitHighlightedMultivaluePropertyLimit", "queryTemplatePropertiesUrl", "timeZoneId", "useOLSQuery", "OLSQuerySession"]\n    },\n    recordPageClick: {\n      argNames: ["pageInfo", "clickType", "blockType", "clickedResultId", "subResultIndex", "immediacySourceId", "immediacyQueryString", "immediacyTitle", "immediacyUrl"]\n    },\n    resultspageaddress: {},\n    searchcenterurl: {},\n    searchquery: {\n      argNames: ["request"]\n    },\n    suggest: {\n      argNames: ["querytext", "iNumberOfQuerySuggestions", "iNumberOfResultSuggestions", "iNumberOfPopularResultSuggestions", "fPreQuerySuggestions", "fHitHighlighting", "fCapitalizeFirstLetters", "culture", "enableStemming", "showPeopleNameSuggestions", "enableQueryRules", "fPrefixMatchAllTerms", "sourceId", "clientType", "useOLSQuery", "OLSQuerySession", "zeroTermSuggestions"]\n    }\n  },\n  "Microsoft.Office.Server.Search.REST.SearchSetting": {\n    exportSearchReports: {\n      argNames: ["TenantId", "ReportType", "Interval", "StartDate", "EndDate", "SiteCollectionId"]\n    },\n    getpromotedresultqueryrules: {\n      argNames: ["siteCollectionLevel", "offset", "numberOfRules"]\n    },\n    getqueryconfiguration: {\n      argNames: ["callLocalSearchFarmsOnly", "skipGroupObjectIdLookup", "throwOnRemoteApiCheck"]\n    },\n    getxssearchpolicy: {},\n    pingadminendpoint: {},\n    scspartialupdateendpointinfo: {},\n    setxssearchpolicy: {\n      argNames: ["policy"]\n    }\n  },\n  "Microsoft.Online.SharePoint.AppLauncher.AppLauncher": {\n    getData: {\n      argNames: ["suiteVersion", "isMobileRequest", "locale", "onPremVer"]\n    }\n  },\n  "Microsoft.Online.SharePoint.MigrationCenter.Common.MigrationUrlParser": {\n    parseUrl: {}\n  },\n  "Microsoft.Online.SharePoint.MigrationCenter.Common.UrlParseResult": {\n    setDisplayUrl: {\n      argNames: ["displayUrl"]\n    },\n    setSPListInformation: {\n      argNames: ["spListInformationList"]\n    },\n    updateSiteProvisionInformation: {\n      argNames: ["spListInformation", "uri", "originalUrl", "errorCode", "message", "freeSiteStorageBytes", "isCurrentUserSiteAdmin"]\n    }\n  },\n  "Microsoft.Online.SharePoint.MigrationCenter.Service.AgentGroup": {\n    "delete": {}\n  },\n  "Microsoft.Online.SharePoint.MigrationCenter.Service.AgentGroup.Collection": {\n    createByName: {\n      argNames: ["Name"]\n    },\n    deleteByName: {\n      argNames: ["Name"]\n    },\n    getByName: {\n      argNames: ["Name"]\n    },\n    getGroupList: {},\n    query: {\n      argNames: ["oData"]\n    }\n  },\n  "Microsoft.Online.SharePoint.MultiGeo.Service.AllowedDataLocation": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    }\n  },\n  "Microsoft.Online.SharePoint.MultiGeo.Service.AllowedDataLocation.Collection": {\n    getByLocation: {\n      argNames: ["location"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "Microsoft.Online.SharePoint.MultiGeo.Service.CrossFarmGroupMoveJob.Collection": {\n    getByMoveId: {\n      argNames: ["moveId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "Microsoft.Online.SharePoint.MultiGeo.Service.CrossFarmSiteMoveJob": {\n    update: {}\n  },\n  "Microsoft.Online.SharePoint.MultiGeo.Service.CrossFarmSiteMoveJob.Collection": {\n    getByMoveId: {\n      argNames: ["moveId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "Microsoft.Online.SharePoint.MultiGeo.Service.CrossFarmUserMoveJob": {\n    update: {}\n  },\n  "Microsoft.Online.SharePoint.MultiGeo.Service.CrossFarmUserMoveJob.Collection": {\n    getByMoveId: {\n      argNames: ["moveId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "Microsoft.Online.SharePoint.MultiGeo.Service.CrossGeoTenantProperty": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    update: {}\n  },\n  "Microsoft.Online.SharePoint.MultiGeo.Service.CrossGeoTenantProperty.Collection": {\n    getByPropertyNameAndGeoLocation: {\n      argNames: ["propertyName", "geo"]\n    },\n    getChanges: {\n      argNames: ["startTimeInUtc"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "Microsoft.Online.SharePoint.MultiGeo.Service.GeoAdministrator": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    }\n  },\n  "Microsoft.Online.SharePoint.MultiGeo.Service.GeoAdministrator.Collection": {\n    create: {\n      argNames: ["parameters"]\n    },\n    getByLoginName: {\n      argNames: ["loginName"]\n    },\n    getByLoginNameAndType: {\n      argNames: ["loginName", "memberType"]\n    },\n    getByObjectId: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "Microsoft.Online.SharePoint.MultiGeo.Service.GeoExperience": {\n    upgradeAllInstancesToSPOMode: {},\n    upgradeToSPOMode: {}\n  },\n  "Microsoft.Online.SharePoint.MultiGeo.Service.GeoTenantInstanceInformation.Collection": {\n    getByGeoLocation: {\n      argNames: ["geoLocation"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "Microsoft.Online.SharePoint.MultiGeo.Service.GroupMoveJob": {\n    cancel: {},\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    update: {}\n  },\n  "Microsoft.Online.SharePoint.MultiGeo.Service.GroupMoveJob.Collection": {\n    getByGroupName: {\n      argNames: ["groupname"]\n    },\n    getMoveReport: {\n      argNames: ["moveState", "moveDirection", "limit", "startTime", "endTime"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "Microsoft.Online.SharePoint.MultiGeo.Service.MoveSiteROState": {\n    updateSiteROState: {}\n  },\n  "Microsoft.Online.SharePoint.MultiGeo.Service.MoveSiteROState.Collection": {\n    getBySiteId: {\n      argNames: ["siteId"]\n    },\n    query: {\n      argNames: ["oData"]\n    }\n  },\n  "Microsoft.Online.SharePoint.MultiGeo.Service.MultiGeoServicesBeta": {\n    compatibleDB: {\n      argNames: ["dbSchemaVersion", "siteSubscriptionId", "siteUrl"]\n    },\n    dBSchemaCompatibilityCheck: {},\n    geoMoveCompatibilityChecks: {},\n    hasMnALicense: {\n      argNames: ["mnALicenseType"]\n    },\n    mnAGroupMoveValidationResult: {\n      argNames: ["encodedQuery"]\n    },\n    orgRelationGroupManagedPath: {\n      argNames: ["encodedNotificationQuery"]\n    },\n    orgRelationNotification: {\n      argNames: ["encodedNotificationQuery"]\n    },\n    orgRelationVerification: {\n      argNames: ["encodedVerificationQuery"]\n    },\n    restoreWorkflowCount: {\n      argNames: ["isCrossTenant"]\n    },\n    userMnAODMoveValidationResult: {\n      argNames: ["encodedQuery"]\n    },\n    userPersonalSiteId: {\n      argNames: ["userPrincipalName"]\n    },\n    userPersonalSiteLocation: {\n      argNames: ["userPrincipalName"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    xTenantMoveCompatibilityCheck: {\n      argNames: ["targetTenantHostUrl"]\n    }\n  },\n  "Microsoft.Online.SharePoint.MultiGeo.Service.SiteMoveJob": {\n    cancel: {},\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    update: {}\n  },\n  "Microsoft.Online.SharePoint.MultiGeo.Service.SiteMoveJob.Collection": {\n    getByUrl: {\n      argNames: ["url"]\n    },\n    getMoveJobByUrl: {\n      argNames: ["SourceSiteUrl", "SubscriptionId", "SourceDataLocation", "EnableSiteToMoveDatastore"]\n    },\n    getMoveReport: {\n      argNames: ["moveState", "moveDirection", "limit", "startTime", "endTime"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "Microsoft.Online.SharePoint.MultiGeo.Service.StorageQuota": {\n    update: {}\n  },\n  "Microsoft.Online.SharePoint.MultiGeo.Service.StorageQuota.Collection": {\n    getByLocation: {\n      argNames: ["geoLocation"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "Microsoft.Online.SharePoint.MultiGeo.Service.TaxonomyReplicationParameters": {\n    update: {}\n  },\n  "Microsoft.Online.SharePoint.MultiGeo.Service.UnifiedGroup.Collection": {\n    getByAlias: {\n      argNames: ["alias"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "Microsoft.Online.SharePoint.MultiGeo.Service.UserMoveJob": {\n    cancel: {},\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    update: {}\n  },\n  "Microsoft.Online.SharePoint.MultiGeo.Service.UserMoveJob.Collection": {\n    getByDirection: {\n      argNames: ["direction"]\n    },\n    getByMoveId: {\n      argNames: ["odbMoveId"]\n    },\n    getByUpn: {\n      argNames: ["upn"]\n    },\n    getByValidPdl: {\n      argNames: ["validPdl"]\n    },\n    getMoveReport: {\n      argNames: ["moveState", "moveDirection", "limit", "startTime", "endTime"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "Microsoft.Online.SharePoint.Onboarding.RestService.PrioritySiteRename.PrioritySiteRenameJob.Collection": {\n    cancel: {\n      argNames: ["url"]\n    },\n    get: {},\n    query: {\n      argNames: ["oData"]\n    }\n  },\n  "Microsoft.Online.SharePoint.Onboarding.RestService.Service.SiteRenameJob.Collection": {\n    getBySiteUrl: {\n      argNames: ["siteUrl"]\n    },\n    getJobsByParentId: {\n      argNames: ["parentId"]\n    },\n    getJobsByParentIdAndState: {\n      argNames: ["parentId", "state"]\n    },\n    getJobsBySiteUrl: {\n      argNames: ["url"]\n    },\n    getSiteRenameReport: {\n      argNames: ["state"]\n    },\n    query: {\n      argNames: ["oData"]\n    }\n  },\n  "Microsoft.Online.SharePoint.Onboarding.RestService.TenantRename.TenantRenameJob.Collection": {\n    cancel: {},\n    get: {},\n    getWarningMessages: {},\n    query: {\n      argNames: ["oData"]\n    }\n  },\n  "Microsoft.Online.SharePoint.SPLogger.LogExport": {\n    getFiles: {\n      argNames: ["partitionId", "logType"]\n    },\n    getLogTypes: {},\n    getPartitions: {\n      argNames: ["logType"]\n    }\n  },\n  "Microsoft.Online.SharePoint.TenantAdmin.MiddleTier.DDIAdapter": {\n    getList: {\n      argNames: ["schema", "workflow", "stream"]\n    },\n    getObject: {\n      argNames: ["schema", "workflow", "stream"]\n    },\n    multiObjectExecute: {\n      argNames: ["schema", "workflow", "stream"]\n    },\n    newObject: {\n      argNames: ["schema", "workflow", "stream"]\n    },\n    removeObjects: {\n      argNames: ["schema", "workflow", "stream"]\n    },\n    setObject: {\n      argNames: ["schema", "workflow", "stream"]\n    }\n  },\n  "Microsoft.Online.SharePoint.TenantAdministration.HubSiteProperties": {\n    update: {}\n  },\n  "Microsoft.Online.SharePoint.TenantAdministration.Internal.SPOWebAppServicePrincipal": {\n    getIsolatedAppDomainsByAppId: {\n      argNames: ["appIds"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    removeAppsToDelete: {\n      argNames: ["domainsToDelete"]\n    },\n    update: {},\n    updateSpfxClientSecret: {\n      argNames: ["secretValue"]\n    },\n    updateSpfxThirdPartyAppId: {\n      argNames: ["appId"]\n    },\n    updateSpfxThirdPartyIsolatedComponentFields: {\n      argNames: ["catalogItemId", "apAppObjectId", "spObjectId", "appId"]\n    },\n    updateSpfxThirdPartyIsolatedSecret: {\n      argNames: ["servicePrincipalId", "secretValue"]\n    }\n  },\n  "Microsoft.Online.SharePoint.TenantAdministration.Internal.SPOWebAppServicePrincipalPermissionGrant": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    }\n  },\n  "Microsoft.Online.SharePoint.TenantAdministration.Internal.SPOWebAppServicePrincipalPermissionGrant.Collection": {\n    getByObjectId: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "Microsoft.Online.SharePoint.TenantAdministration.Internal.SPOWebAppServicePrincipalPermissionRequest": {\n    approve: {},\n    deny: {}\n  },\n  "Microsoft.Online.SharePoint.TenantAdministration.Internal.SPOWebAppServicePrincipalPermissionRequest.Collection": {\n    approve: {\n      argNames: ["resource", "scope"]\n    },\n    getById: {\n      argNames: ["id"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "Microsoft.Online.SharePoint.TenantAdministration.Office365CommsMessagesServiceProxy": {\n    messageCenterMessages: {\n      argNames: ["messagesFieldsData"]\n    },\n    serviceHealthMessages: {\n      argNames: ["messagesFieldsData"]\n    }\n  },\n  "Microsoft.Online.SharePoint.TenantAdministration.SPDataGovernanceInsightRestApiClient": {\n    createDataAccessGovernanceReport: {\n      argNames: ["reportEntity", "workload", "reportType", "fileSensitivityLabelName", "fileSensitivityLabelGUID", "name", "template", "privacy", "siteSensitivityLabelGUID", "countOfUsersMoreThan"]\n    },\n    exportSPODataAccessGovernanceInsight: {\n      argNames: ["reportId"]\n    },\n    getSPODataAccessGovernanceInsight: {\n      argNames: ["reportEntity", "workLoad"]\n    },\n    getSPODataAccessGovernanceInsightById: {\n      argNames: ["reportId"]\n    },\n    removeDataAccessGovernanceReport: {\n      argNames: ["reportId"]\n    }\n  },\n  "Microsoft.Online.SharePoint.TenantAdministration.SPDataGovernanceSARRestApiClient": {\n    getSPOSiteReview: {\n      argNames: ["reportEntity", "siteReviewtatus", "siteReviewID", "siteID"]\n    },\n    startSPOSiteReview: {\n      argNames: ["detailedSourceReportId", "siteId", "adminComment"]\n    }\n  },\n  "Microsoft.Online.SharePoint.TenantAdministration.SPOContentSecurityPolicyConfiguration": {\n    add: {\n      argNames: ["source"]\n    },\n    remove: {\n      argNames: ["source"]\n    }\n  },\n  "Microsoft.Online.SharePoint.TenantAdministration.SPOGroup": {\n    addAsGroupOwnerAndMember: {\n      argNames: ["groupId", "userId", "userPrincipalName"]\n    },\n    addGroupMember: {\n      argNames: ["groupId", "memberId", "memberPrincipalName"]\n    },\n    addGroupOwner: {\n      argNames: ["groupId", "ownerId", "ownerPrincipalName"]\n    },\n    getGroupInfo: {\n      argNames: ["groupId"]\n    },\n    removeAsGroupOwnerAndMember: {\n      argNames: ["groupId", "userId"]\n    },\n    removeGroupMember: {\n      argNames: ["groupId", "memberId"]\n    },\n    removeGroupOwner: {\n      argNames: ["groupId", "ownerId"]\n    },\n    updateGroupProperties: {\n      argNames: ["groupId", "displayName"]\n    },\n    updateGroupPropertiesBySiteId: {\n      argNames: ["groupId", "siteId", "displayName"]\n    }\n  },\n  "Microsoft.Online.SharePoint.TenantAdministration.SPOMalwareFile": {\n    getMalwareFileStream: {},\n    query: {\n      argNames: ["oData"]\n    }\n  },\n  "Microsoft.Online.SharePoint.TenantAdministration.SPORestrictedContentDiscoverabilityClient": {\n    createRestrictedContentDiscoverabilityReport: {},\n    getAllRestrictedContentDiscoverabilityReports: {},\n    getRestrictContentOrgWideSearchUsageInsightsReportContent: {\n      argNames: ["reportId"]\n    },\n    getSPODataAccessGovernanceInsightById: {\n      argNames: ["reportId"]\n    }\n  },\n  "Microsoft.Online.SharePoint.TenantAdministration.SPOWebAppServicePrincipalPublic": {\n    addCustomSpfx3rdPartyAppPrincipal: {\n      argNames: ["appId", "appUri", "clientSecret"]\n    },\n    getCustomSpfx3rdPartyAppPrincipal: {},\n    removeCustomSpfx3rdPartyAppPrincipal: {},\n    updateCustomSpfx3rdPartyAppPrincipalClientSecret: {\n      argNames: ["clientSecret"]\n    }\n  },\n  "Microsoft.Online.SharePoint.TenantAdministration.SiteProperties": {\n    update: {}\n  },\n  "Microsoft.Online.SharePoint.TenantAdministration.SiteProperties.Collection": {\n    checkSiteIsArchivedById: {\n      argNames: ["siteId"]\n    },\n    getById: {\n      argNames: ["siteId"]\n    },\n    getGroupSiteRelationship: {\n      argNames: ["siteId"]\n    },\n    getLockStateById: {\n      argNames: ["siteId"]\n    },\n    getSiteStateProperties: {\n      argNames: ["siteId"]\n    },\n    getSiteUserGroups: {\n      argNames: ["siteId", "userGroupIds"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "Microsoft.Online.SharePoint.TenantAdministration.Tenant": {\n    acceptSyntexRepositoryTermsOfService: {},\n    activateApplicationBillingPolicy: {\n      argNames: ["billingPolicyId"]\n    },\n    addBrandFontPackage: {\n      argNames: ["creationInformation"]\n    },\n    addHomeSite: {\n      argNames: ["homeSiteUrl", "order", "audiences"]\n    },\n    addRecentAdminAction: {\n      argNames: ["payload"]\n    },\n    addRecentAdminActionReport: {\n      argNames: ["payload"]\n    },\n    addRecentAdminActions: {\n      argNames: ["payload"]\n    },\n    addSPOContainerUserRole: {\n      argNames: ["ContainerId", "loginName", "role"]\n    },\n    addSPORestrictedSearchAllowedList: {\n      argNames: ["siteUrls"]\n    },\n    addTenantAdminListItem: {\n      argNames: ["columnValues", "listName"]\n    },\n    addTenantAdminListView: {\n      argNames: ["parameters"]\n    },\n    addToOrgAssetsLibAndCdnV2: {\n      argNames: ["cdnType", "libUrl", "thumbnailUrl", "orgAssetType", "defaultOriginAdded"]\n    },\n    addToOrgAssetsLibAndCdnWithType: {\n      argNames: ["cdnType", "libUrl", "thumbnailUrl", "orgAssetType"]\n    },\n    addToOrgAssetsLibWithConfig: {\n      argNames: ["cdnType", "libUrl", "thumbnailUrl", "orgAssetType", "defaultOriginAdded", "configParam"]\n    },\n    appHasAdminSitePermission: {},\n    archiveSiteById: {\n      argNames: ["siteId"]\n    },\n    bootstrapPolicyAutomationResources: {},\n    checkTenantIntuneLicense: {},\n    checkTenantLicenses: {\n      argNames: ["licenses"]\n    },\n    clearRestrictedAccessControlOneDrive: {},\n    clearRestrictedAccessControlSharePoint: {},\n    connectSiteToHubSiteById: {\n      argNames: ["siteUrl", "hubSiteId"]\n    },\n    createApplicationBillingPolicyValidation: {\n      argNames: ["applicationId"]\n    },\n    createPolicyDefinition: {\n      argNames: ["policyInputParameters"]\n    },\n    createSite: {\n      argNames: ["siteCreationProperties"]\n    },\n    deletePolicyDefinition: {\n      argNames: ["itemId"]\n    },\n    deleteRecentAdminActionReport: {\n      argNames: ["reportId"]\n    },\n    disableCollaborationInsightsDataCollection: {},\n    downloadRansomwareImpactedAssetsReport: {\n      argNames: ["driveId", "fileName", "reportNameType"]\n    },\n    downloadSPOContainers: {\n      argNames: ["active"]\n    },\n    downloadSharingInsights: {\n      argNames: ["queryMode", "insightsType"]\n    },\n    enableCollaborationInsightsDataCollection: {},\n    ensureBrandCenterFeature: {},\n    exportAdminListToCSV: {\n      argNames: ["viewXml", "listName"]\n    },\n    exportCSVFile: {\n      argNames: ["timeZoneId"]\n    },\n    exportToCSV: {\n      argNames: ["viewXml", "timeZoneId", "columnsInfo", "listName"]\n    },\n    exportUnlicensedOneDriveForBusinessListToCSV: {},\n    getAdminListViews: {},\n    getBillingPolicyIdForApp: {\n      argNames: ["applicationId"]\n    },\n    getBrandCenterConfiguration: {},\n    getBrandFontPackages: {},\n    getCollaborationInsightsData: {},\n    getCollaborationInsightsOverview: {},\n    getFileVersionBatchDeleteJobProgress: {\n      argNames: ["siteUrl"]\n    },\n    getFileVersionBatchDeleteJobProgressForLibrary: {\n      argNames: ["siteUrl", "listParams"]\n    },\n    getFileVersionExpirationReportJobProgress: {\n      argNames: ["siteUrl", "reportUrl"]\n    },\n    getFileVersionExpirationReportJobProgressForLibrary: {\n      argNames: ["siteUrl", "listParams", "reportUrl"]\n    },\n    getFileVersionPolicyForLibrary: {\n      argNames: ["siteUrl", "listParams"]\n    },\n    getFilteredSPListItems: {\n      argNames: ["columnName", "columnValue", "columnType", "listName"]\n    },\n    getHomeSites: {},\n    getHomeSitesDetails: {},\n    getIBVersionForTenant: {},\n    getIdleSessionSignOutForUnmanagedDevices: {},\n    getInsightsSummary: {\n      argNames: ["insightsScenario", "dataFileName", "pageIndex", "modelName", "maxTokenSize", "maxContentSize", "timeoutMS"]\n    },\n    getOneDriveSiteSharingInsights: {\n      argNames: ["queryMode"]\n    },\n    getOrgAssets: {},\n    getPowerAppsEnvironments: {},\n    getRansomwareActivities: {\n      argNames: ["parameters"]\n    },\n    getRansomwareActivitiesOverview: {\n      argNames: ["eventId"]\n    },\n    getRansomwareEvents: {\n      argNames: ["parameters"]\n    },\n    getRansomwareEventsOverview: {},\n    getSPHSiteUrl: {},\n    getSPListItemCount: {\n      argNames: ["listName"]\n    },\n    getSPListRootFolderProperties: {\n      argNames: ["listName"]\n    },\n    getSPOAllWebTemplates: {\n      argNames: ["cultureName", "compatibilityLevel"]\n    },\n    getSPOAppBillingPolicies: {},\n    getSPOContainerByContainerId: {\n      argNames: ["containerId"]\n    },\n    getSPOContainerByContainerSiteUrl: {\n      argNames: ["containerSiteUrl"]\n    },\n    getSPOContainerTypeById: {\n      argNames: ["containerTypeId", "containerTenantType"]\n    },\n    getSPOContainerTypeConfigurationByContainerTypeId: {\n      argNames: ["containerTypeId"]\n    },\n    getSPOContainerTypes: {\n      argNames: ["containerTenantType"]\n    },\n    getSPOContainersByApplicationId: {\n      argNames: ["owningApplicationId", "paged", "pagingToken"]\n    },\n    getSPODeletedContainers: {},\n    getSPORestrictedSearchAllowedList: {},\n    getSPORestrictedSearchMode: {},\n    getSPOSiteCreationSources: {},\n    getSPOSyntexApplications: {},\n    getSPOSyntexConsumingApplications: {\n      argNames: ["owningApplicationId", "applicationId"]\n    },\n    getSPOTenantAllWebTemplates: {},\n    getSPOTenantWebTemplates: {\n      argNames: ["localeId", "compatibilityLevel"]\n    },\n    getSPOWebTemplatesAllowedForArchiving: {},\n    getSharePointSettingData: {},\n    getSharePointSiteSharingInsights: {\n      argNames: ["queryMode"]\n    },\n    getSiteCohortsSummary: {\n      argNames: ["view"]\n    },\n    getSiteAdministrators: {\n      argNames: ["siteId"]\n    },\n    getSiteHealthStatus: {\n      argNames: ["sourceUrl"]\n    },\n    getSitePropertiesByUrl: {\n      argNames: ["url", "includeDetail"]\n    },\n    getSitePropertiesFromSharePointByFilters: {\n      argNames: ["speFilter"]\n    },\n    getSiteSecondaryAdministrators: {\n      argNames: ["secondaryAdministratorsFieldsData"]\n    },\n    getSiteSubscriptionId: {},\n    getSiteThumbnailLogo: {\n      argNames: ["siteUrl"]\n    },\n    getSitesByState: {\n      argNames: ["states"]\n    },\n    getSitesFromSubstrate: {\n      argNames: ["searchText"]\n    },\n    getSortedSPOContainersByApplicationId: {\n      argNames: ["owningApplicationId", "ascending", "paged", "pagingToken"]\n    },\n    getTargetedSitesDetails: {},\n    getTargetedSitesDetailsByUserId: {\n      argNames: ["userId"]\n    },\n    getTenantAllOrCompatibleIBSegments: {\n      argNames: ["segments"]\n    },\n    getTenantSiteCreationSource: {},\n    getTopFilesSharingInsights: {\n      argNames: ["queryMode"]\n    },\n    getTrackViewFeatureAlwaysVisible: {},\n    getVersionPolicyForDocLibsJobProgress: {\n      argNames: ["siteUrl"]\n    },\n    getViewByDisplayName: {\n      argNames: ["viewName", "listName"]\n    },\n    getVivaConnectionsLicense: {},\n    grantHubSiteRightsById: {\n      argNames: ["hubSiteId", "principals", "grantedRights"]\n    },\n    hasValidEducationLicense: {},\n    isSyntexRepositoryTermsOfServiceAccepted: {},\n    newFileVersionBatchDeleteJob: {\n      argNames: ["siteUrl", "batchDeleteParams"]\n    },\n    newFileVersionBatchDeleteJobForLibrary: {\n      argNames: ["siteUrl", "listParams", "batchDeleteParams"]\n    },\n    newFileVersionExpirationReportJob: {\n      argNames: ["siteUrl", "reportUrl"]\n    },\n    newFileVersionExpirationReportJobForLibrary: {\n      argNames: ["siteUrl", "listParams", "reportUrl"]\n    },\n    newSPOContainerType: {\n      argNames: ["containerTypeProperties"]\n    },\n    purgeContainer: {\n      argNames: ["containerId"]\n    },\n    purgeSPODeletedContainerByContainerId: {\n      argNames: ["containerId"]\n    },\n    purgeSPODeletedContainerByContainerSiteUrl: {\n      argNames: ["containerSiteUrl"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    recentAdminActionReportIsAuditEnabled: {},\n    registerHubSite: {\n      argNames: ["siteUrl"]\n    },\n    registerHubSiteWithCreationInformation: {\n      argNames: ["siteUrl", "creationInformation"]\n    },\n    removeContainer: {\n      argNames: ["containerId"]\n    },\n    removeDeletedSite: {\n      argNames: ["siteUrl"]\n    },\n    removeDeletedSitePreferId: {\n      argNames: ["siteUrl", "siteId"]\n    },\n    removeFileVersionBatchDeleteJob: {\n      argNames: ["siteUrl"]\n    },\n    removeFileVersionBatchDeleteJobForLibrary: {\n      argNames: ["siteUrl", "listParams"]\n    },\n    removeFromOrgAssets: {\n      argNames: ["libUrl", "listId"]\n    },\n    removeHomeSite: {\n      argNames: ["homeSiteUrl"]\n    },\n    removeSPHSite: {},\n    removeSPListItem: {\n      argNames: ["listItemId", "listName"]\n    },\n    removeSPOContainerByContainerId: {\n      argNames: ["containerId"]\n    },\n    removeSPOContainerByContainerSiteUrl: {\n      argNames: ["containerSiteUrl"]\n    },\n    removeSPOContainerType: {\n      argNames: ["spDeletedContainerTypeProperties"]\n    },\n    removeSPOContainerUserRole: {\n      argNames: ["ContainerId", "loginName", "role"]\n    },\n    removeSPORestrictedSearchAllowedList: {\n      argNames: ["siteUrls"]\n    },\n    removeSite: {\n      argNames: ["siteUrl"]\n    },\n    removeTargetedSite: {\n      argNames: ["siteId"]\n    },\n    removeTenantAdminListView: {\n      argNames: ["viewId"]\n    },\n    removeVersionPolicyForDocLibsJob: {\n      argNames: ["siteUrl"]\n    },\n    renderActiveContainers: {\n      argNames: ["containerId", "viewXml"]\n    },\n    renderAdminListData: {\n      argNames: ["parameters", "overrideParameters", "listName"]\n    },\n    renderContainersAdminApplicationsData: {},\n    renderContainersAdminListData: {\n      argNames: ["orderByColumnsList", "filterByColumnsList", "pageSize", "pagingToken", "containerStatus", "searchParameters"]\n    },\n    renderDeletedContainers: {\n      argNames: ["viewXml"]\n    },\n    renderFilteredAdminListData: {\n      argNames: ["parameters", "listName"]\n    },\n    renderFilteredAdminListDataByGroupId: {\n      argNames: ["groupId"]\n    },\n    renderIBSegmentListDataAsStream: {\n      argNames: ["parameters", "segments", "CascDelWarnMessage", "CustomAction", "DrillDown", "Field", "FieldInternalName", "Filter", "FilterData", "FilterData1", "FilterData10", "FilterData2", "FilterData3", "FilterData4", "FilterData5", "FilterData6", "FilterData7", "FilterData8", "FilterData9", "FilterField", "FilterField1", "FilterField10", "FilterField2", "FilterField3", "FilterField4", "FilterField5", "FilterField6", "FilterField7", "FilterField8", "FilterField9", "FilterFields", "FilterFields1", "FilterFields10", "FilterFields2", "FilterFields3", "FilterFields4", "FilterFields5", "FilterFields6", "FilterFields7", "FilterFields8", "FilterFields9", "FilterLookupId", "FilterLookupId1", "FilterLookupId10", "FilterLookupId2", "FilterLookupId3", "FilterLookupId4", "FilterLookupId5", "FilterLookupId6", "FilterLookupId7", "FilterLookupId8", "FilterLookupId9", "FilterOp", "FilterOp1", "FilterOp10", "FilterOp2", "FilterOp3", "FilterOp4", "FilterOp5", "FilterOp6", "FilterOp7", "FilterOp8", "FilterOp9", "FilterValue", "FilterValue1", "FilterValue10", "FilterValue2", "FilterValue3", "FilterValue4", "FilterValue5", "FilterValue6", "FilterValue7", "FilterValue8", "FilterValue9", "FilterValues", "FilterValues1", "FilterValues10", "FilterValues2", "FilterValues3", "FilterValues4", "FilterValues5", "FilterValues6", "FilterValues7", "FilterValues8", "FilterValues9", "GroupString", "HasOverrideSelectCommand", "ID", "InplaceFullListSearch", "InplaceSearchQuery", "IsCSR", "IsGroupRender", "IsXslView", "ListViewPageUrl", "OverrideRowLimit", "OverrideScope", "OverrideSelectCommand", "PageFirstRow", "PageLastRow", "QueryParams", "RootFolder", "RootFolderUniqueId", "SortDir", "SortDir1", "SortDir10", "SortDir2", "SortDir3", "SortDir4", "SortDir5", "SortDir6", "SortDir7", "SortDir8", "SortDir9", "SortField", "SortField1", "SortField10", "SortField2", "SortField3", "SortField4", "SortField5", "SortField6", "SortField7", "SortField8", "SortField9", "SortFields", "SortFieldValues", "View", "ViewCount", "ViewId", "ViewPath", "WebPartId", "FilterIBSegmentsBasedOnAndCondition"]\n    },\n    renderIBSegmentListFilterData: {\n      argNames: ["parameters"]\n    },\n    renderOwnershipPolicyStorageSPList: {\n      argNames: ["parameters", "overrideParameters"]\n    },\n    renderPolicyDefinitionList: {\n      argNames: ["parameters", "overrideParameters"]\n    },\n    renderPolicyExecutionsHistory: {\n      argNames: ["parameters", "overrideParameters"]\n    },\n    renderPolicyReport: {\n      argNames: ["parameters", "overrideParameters"]\n    },\n    renderRecentAdminActionReport: {\n      argNames: ["parameters", "overrideParameters"]\n    },\n    renderRecentAdminActions: {\n      argNames: ["parameters", "overrideParameters", "isAutoRefresh"]\n    },\n    reorderTargetedSites: {\n      argNames: ["siteIds"]\n    },\n    restoreContainer: {\n      argNames: ["containerId"]\n    },\n    restoreDeletedSite: {\n      argNames: ["siteUrl"]\n    },\n    restoreDeletedSiteById: {\n      argNames: ["siteId"]\n    },\n    restoreDeletedSitePreferId: {\n      argNames: ["siteUrl", "siteId"]\n    },\n    restoreSPODeletedContainerByContainerId: {\n      argNames: ["containerId"]\n    },\n    restoreSPODeletedContainerByContainerSiteUrl: {\n      argNames: ["containerSiteUrl"]\n    },\n    revokeHubSiteRightsById: {\n      argNames: ["hubSiteId", "principals"]\n    },\n    sendEmail: {\n      argNames: ["siteUrl", "activityEventJson"]\n    },\n    setDefaultView: {\n      argNames: ["viewId", "listName"]\n    },\n    setFileVersionPolicy: {\n      argNames: ["isAutoTrimEnabled", "majorVersionLimit", "expireVersionsAfterDays"]\n    },\n    setFileVersionPolicyForLibrary: {\n      argNames: ["siteUrl", "listParams", "versionPolicyParams"]\n    },\n    setIBSegmentsOnSite: {\n      argNames: ["siteId", "segments", "ibMode"]\n    },\n    setIdleSessionSignOutForUnmanagedDevices: {\n      argNames: ["enabled", "warnAfter", "signOutAfter"]\n    },\n    setJitDlpPolicyData: {\n      argNames: ["markAllFilesAsSensitiveByDefault", "odbSensitivityRefreshWindowInHours", "executionMode"]\n    },\n    setOrgAssetsWithConfig: {\n      argNames: ["libUrl", "thumbnailUrl", "orgAssetType", "configParam"]\n    },\n    setOrgAssetsWithType: {\n      argNames: ["libUrl", "thumbnailUrl", "orgAssetType"]\n    },\n    setSPEmbeddedApplicationPermissions: {\n      argNames: ["spSyntexApplicationProperties"]\n    },\n    setSPHSite: {\n      argNames: ["sphSiteUrl"]\n    },\n    setSPHSiteWithConfiguration: {\n      argNames: ["sphSiteUrl", "configuration"]\n    },\n    setSPOContainerProperties: {\n      argNames: ["spContainerProperties"]\n    },\n    setSPOContainerType: {\n      argNames: ["containerTypeProperties"]\n    },\n    setSPOContainerTypeConfiguration: {\n      argNames: ["spContainerTypeConfigurationProperties"]\n    },\n    setSPORestrictedSearchMode: {\n      argNames: ["mode"]\n    },\n    setSPOSyntexApplicationsClearOverrideSharingCapability: {\n      argNames: ["owningApplicationId"]\n    },\n    setSPOSyntexApplicationsSharingCapability: {\n      argNames: ["owningApplicationId", "sharingCapability", "overrideTenantSharingCapability"]\n    },\n    setSensitivityLabelContainer: {\n      argNames: ["containerId", "sensitivityLabel"]\n    },\n    setSiteAdministrators: {\n      argNames: ["siteAdministratorsFieldsData"]\n    },\n    setSiteSecondaryAdministrators: {\n      argNames: ["secondaryAdministratorsFieldsData"]\n    },\n    setSiteUserGroups: {\n      argNames: ["siteUserGroupsData"]\n    },\n    setSyntexPaygFeatureActivation: {\n      argNames: ["featureName", "activationStatus"]\n    },\n    setTrackViewFeatureAlwaysVisible: {},\n    swapSite: {\n      argNames: ["sourceUrl", "targetUrl", "archiveUrl"]\n    },\n    swapSiteWithSmartGestureOption: {\n      argNames: ["sourceUrl", "targetUrl", "archiveUrl", "includeSmartGestures"]\n    },\n    swapSiteWithSmartGestureOptionForce: {\n      argNames: ["sourceUrl", "targetUrl", "archiveUrl", "includeSmartGestures", "force"]\n    },\n    unarchiveSiteById: {\n      argNames: ["siteId"]\n    },\n    unregisterHubSite: {\n      argNames: ["siteUrl"]\n    },\n    update: {},\n    updateFileSearchVisibility: {\n      argNames: ["url", "isHidden"]\n    },\n    updateGroupSiteProperties: {\n      argNames: ["groupId", "siteId", "updateType", "parameters"]\n    },\n    updatePolicyDefinition: {\n      argNames: ["itemId", "policyInputParameters", "policyOperation"]\n    },\n    updateRansomwareActivity: {\n      argNames: ["listItemId", "columnValues"]\n    },\n    updateRansomwareEvent: {\n      argNames: ["listItemId", "columnValues", "forceResolveActivity"]\n    },\n    updateSPOContainerUserRole: {\n      argNames: ["ContainerId", "loginName", "role"]\n    },\n    updateSiteActivityData: {},\n    updateTargetedSite: {\n      argNames: ["siteUrl", "configurationParam"]\n    },\n    updateTenantAdminListItem: {\n      argNames: ["listItemId", "columnValues", "listName"]\n    },\n    updateTenantAdminListView: {\n      argNames: ["viewId", "viewXml"]\n    },\n    validateHomeSite: {\n      argNames: ["validationActionType", "siteUrl"]\n    },\n    validateMultipleHomeSitesParameterExists: {\n      argNames: ["hasParameters"]\n    }\n  },\n  "Microsoft.Online.SharePoint.TenantAdministration.TenantAdminSettingsService": {\n    getTenantSharingStatus: {},\n    update: {}\n  },\n  "Microsoft.Online.SharePoint.TenantManagement.ExternalUser.Collection": {\n    getById: {\n      argNames: ["uniqueId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "Microsoft.Online.SharePoint.TenantManagement.Office365Tenant": {\n    addPublicCdnOrigin: {\n      argNames: ["origin"]\n    },\n    addSdnProvider: {\n      argNames: ["identifier", "license"]\n    },\n    addTenantCdnOrigin: {\n      argNames: ["cdnType", "originUrl"]\n    },\n    addTenantTheme: {\n      argNames: ["name", "themeJson"]\n    },\n    addToOrgAssetsLibAndCdn: {\n      argNames: ["cdnType", "libUrl", "thumbnailUrl", "orgAssetType", "defaultOriginAdded"]\n    },\n    addToOrgAssetsWithConfig: {\n      argNames: ["cdnType", "libUrl", "thumbnailUrl", "orgAssetType", "defaultOriginAdded", "configParam"]\n    },\n    createTenantCdnDefaultOrigins: {\n      argNames: ["cdnType"]\n    },\n    deleteImportProfilePropertiesJob: {\n      argNames: ["jobId"]\n    },\n    deleteTenantTheme: {\n      argNames: ["name"]\n    },\n    disableSharingForNonOwnersOfSite: {\n      argNames: ["siteUrl"]\n    },\n    getAllTenantThemes: {},\n    getCustomFontsMinorVersion: {\n      argNames: ["libUrl"]\n    },\n    getExternalUsers: {\n      argNames: ["position", "pageSize", "filter", "sortOrder"]\n    },\n    getExternalUsersForSite: {\n      argNames: ["siteUrl", "position", "pageSize", "filter", "sortOrder"]\n    },\n    getExternalUsersWithSortBy: {\n      argNames: ["position", "pageSize", "filter", "sortPropertyName", "sortOrder"]\n    },\n    getHideDefaultThemes: {},\n    getIdleSessionSignOutForUnmanagedDevices: {},\n    getImportProfilePropertyJob: {\n      argNames: ["jobId"]\n    },\n    getImportProfilePropertyJobs: {},\n    getTenantCdnEnabled: {\n      argNames: ["cdnType"]\n    },\n    getTenantCdnOrigins: {\n      argNames: ["cdnType"]\n    },\n    getTenantCdnPolicies: {\n      argNames: ["cdnType"]\n    },\n    getTenantTheme: {\n      argNames: ["name"]\n    },\n    incrementCustomFontsMinorVersion: {\n      argNames: ["libUrl"]\n    },\n    isSharingDisabledForNonOwnersOfSite: {\n      argNames: ["siteUrl"]\n    },\n    logCustomFontsLargeUpload: {\n      argNames: ["numCatalogs", "numFonts", "totalExpectedFiles"]\n    },\n    queueImportProfileProperties: {\n      argNames: ["idType", "sourceDataIdProperty", "propertyMap", "sourceUri"]\n    },\n    removeExternalUsers: {\n      argNames: ["uniqueIds"]\n    },\n    removeFromOrgAssets: {\n      argNames: ["libUrl", "listId"]\n    },\n    removeFromOrgAssetsAndCdn: {\n      argNames: ["remove", "cdnType", "libUrl"]\n    },\n    removePreviousCustomFontUpload: {\n      argNames: ["majVersions", "libUrl"]\n    },\n    removePublicCdnOrigin: {\n      argNames: ["originId"]\n    },\n    removeSdnProvider: {},\n    removeTenantCdnOrigin: {\n      argNames: ["cdnType", "originUrl"]\n    },\n    revokeAllUserSessions: {\n      argNames: ["userName"]\n    },\n    revokeAllUserSessionsByPuid: {\n      argNames: ["puidList"]\n    },\n    setBlockDownloadFileTypePolicyData: {\n      argNames: ["blockDownloadFileTypePolicy", "fileTypeIds", "excludedBlockDownloadGroupIds"]\n    },\n    setBlockDownloadFileTypePolicyExclusionList: {\n      argNames: ["excludedBlockDownloadGroupIds"]\n    },\n    setHideDefaultThemes: {\n      argNames: ["hideDefaultThemes"]\n    },\n    setIdleSessionSignOutForUnmanagedDevices: {\n      argNames: ["enabled", "warnAfter", "signOutAfter"]\n    },\n    setJitDlpPolicyData: {\n      argNames: ["markAllFilesAsSensitiveByDefault", "odbSensitivityRefreshWindowInHours", "executionMode"]\n    },\n    setOrgAssetsLib: {\n      argNames: ["libUrl", "thumbnailUrl", "orgAssetType"]\n    },\n    setOrgAssetsLibWithConfig: {\n      argNames: ["libUrl", "thumbnailUrl", "orgAssetType", "configParam"]\n    },\n    setTenantCdnEnabled: {\n      argNames: ["cdnType", "isEnabled"]\n    },\n    setTenantCdnPolicy: {\n      argNames: ["cdnType", "policy", "policyValue"]\n    },\n    syncAadB2BManagementPolicy: {},\n    updateTenantTheme: {\n      argNames: ["name", "themeJson"]\n    },\n    uploadCustomFontsAndCatalogLib: {\n      argNames: ["customFontFiles", "libUrl"]\n    }\n  },\n  "Microsoft.SharePoint.Administration.FeatureDefinition.Collection": {\n    getFeatureDefinition: {\n      argNames: ["featureDisplayName", "compatibilityLevel"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "Microsoft.SharePoint.Administration.SPAnalyticsUsageService": {\n    logevent: {\n      argNames: ["usageEntry"]\n    }\n  },\n  "Microsoft.SharePoint.Administration.SPAppStateQueryJobDefinition": {\n    performFastRevokeWithClientIds: {}\n  },\n  "Microsoft.SharePoint.Administration.SiteMove.Service.SiteMoveService": {\n    acquireSystemSiteLock: {\n      argNames: ["lockRequestor", "lockType", "leaseDurationInMinutes"]\n    },\n    clearState: {},\n    extendSystemSiteLockExpiration: {\n      argNames: ["leaseDurationInMinute"]\n    },\n    getChecksumForTable: {\n      argNames: ["schemaName", "tableName", "partitionColumnName", "columnsToChecksum"]\n    },\n    getDataReader: {\n      argNames: ["sqlCommandText"]\n    },\n    getDatabaseProperties: {},\n    getMarker: {},\n    getRowCountForTable: {\n      argNames: ["schemaName", "tableName", "partitionColumnName"]\n    },\n    getScalarValue: {\n      argNames: ["sqlCommandText"]\n    },\n    getServiceInfo: {},\n    getSiteProperties: {},\n    getTenantWorkflows: {},\n    isDbReadOnly: {},\n    isSystemSiteLocked: {\n      argNames: ["lockRequestor"]\n    },\n    processStorageMetricsChanges: {},\n    releaseSystemSiteLock: {\n      argNames: ["lockRequestor"]\n    },\n    sourceCleanupAfterMove: {\n      argNames: ["isDeleted"]\n    }\n  },\n  "Microsoft.SharePoint.Administration.SiteMove.Service.SiteRelocationJobManagementService": {\n    enqueueSiteRelocationJob: {\n      argNames: ["siteId", "siteSubscriptionId", "siteUrl", "sourceDatabaseId", "targetDatabaseId", "siteMoveFlags", "preferredStartTimeInUtc"]\n    },\n    getServiceInfo: {}\n  },\n  "Microsoft.SharePoint.AuthPolicy.Events.SPAuthEvent.Collection": {\n    query: {\n      argNames: ["oData"]\n    },\n    roleAssignmentMSGraphNotify: {\n      argNames: ["tenant", "action", "type", "resourcePayload", "id", "containerId"]\n    }\n  },\n  "Microsoft.SharePoint.AuthPolicy.SPTenantIBPolicyComplianceReport.Collection": {\n    getAllReportStates: {},\n    getReportById: {\n      argNames: ["ReportId", "ShowIncompatibleSegmentsPairsAndInvalidSegmentsFilesContents"]\n    },\n    query: {\n      argNames: ["oData"]\n    },\n    removeFinalizedReport: {\n      argNames: ["ReportId"]\n    }\n  },\n  "Microsoft.SharePoint.Client.Search.Administration.DocumentCrawlLog": {\n    getCrawledUrls: {\n      argNames: ["getCountOnly", "maxRows", "queryString", "isLike", "contentSourceID", "errorLevel", "errorID", "startDateTime", "endDateTime"]\n    },\n    getUnsuccesfulCrawledUrls: {\n      argNames: ["displayUrl", "startDateTime", "endDateTime"]\n    }\n  },\n  "Microsoft.SharePoint.Client.Search.Administration.SiteContentProcessingInfoProvider": {\n    getAzureContainerToken: {},\n    notifyContentProcessingStatus: {\n      argNames: ["webId", "listId", "uniqueId", "properties"]\n    },\n    reportContentProcessingStatus: {\n      argNames: ["webId", "listId", "uniqueId", "azureContainerTokenUri", "encryptionKey", "priority", "errorCode", "errorDescription"]\n    }\n  },\n  "Microsoft.SharePoint.Client.Search.Administration.TenantCrawlVersionsInfoProvider": {\n    disableCrawlVersions: {\n      argNames: ["siteId"]\n    },\n    disableCrawlVersionsForTenant: {},\n    enableCrawlVersions: {\n      argNames: ["siteId"]\n    },\n    enableCrawlVersionsForTenant: {},\n    getSiteCrawlVersionStatus: {\n      argNames: ["siteId"]\n    },\n    isCrawlVersionsEnabled: {\n      argNames: ["siteId"]\n    },\n    isCrawlVersionsEnabledForTenant: {}\n  },\n  "Microsoft.SharePoint.Client.Search.Administration.TenantRecycleBinInfoProvider": {\n    disableRecycleBinDiscoverabilityForTenant: {},\n    enableRecycleBinDiscoverabilityForTenant: {},\n    isRecycleBinDiscoverabilityEnabledForTenant: {}\n  },\n  "Microsoft.SharePoint.Client.Search.Analytics.SignalStore": {\n    signals: {\n      argNames: ["signals"]\n    }\n  },\n  "Microsoft.SharePoint.Client.Search.Query.RankingLabeling": {\n    addJudgment: {\n      argNames: ["userQuery", "url", "labelId"]\n    },\n    getJudgementsForQuery: {\n      argNames: ["query"]\n    },\n    normalizeResultUrl: {\n      argNames: ["url"]\n    }\n  },\n  "Microsoft.SharePoint.Client.Search.Query.ReorderingRuleCollection": {\n    add: {\n      argNames: ["matchType", "matchValue", "boost"]\n    },\n    clear: {}\n  },\n  "Microsoft.SharePoint.Client.Search.Query.SortCollection": {\n    add: {\n      argNames: ["strProperty", "direction"]\n    },\n    clear: {}\n  },\n  "Microsoft.SharePoint.Client.Search.Query.StringCollection": {\n    add: {\n      argNames: ["property"]\n    },\n    clear: {}\n  },\n  "Microsoft.SharePoint.ClientSideComponent.HostedApp": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    updateWebPartData: {\n      argNames: ["webPartDataAsJson"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    }\n  },\n  "Microsoft.SharePoint.ClientSideComponent.HostedAppsManager": {\n    add: {\n      argNames: ["webPartDataAsJson", "hostType"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    addEx: {\n      argNames: ["webPartDataAsJson", "hostType"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    getById: {\n      argNames: ["id"]\n    }\n  },\n  "Microsoft.SharePoint.Comments.comment": {\n    "delete": {\n      name: "",\n      requestMethod: "DELETE"\n    },\n    like: {},\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    unlike: {}\n  },\n  "Microsoft.SharePoint.Comments.comment.Collection": {\n    add: {\n      argNames: ["text"],\n      metadataType: "Microsoft.SharePoint.Comments.comment",\n      name: "",\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    deleteAll: {\n      requestType: utils_1.RequestType.Post\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "Microsoft.SharePoint.Convergence.ConsumerPermissions": {\n    grantConsumerSitePermissions: {},\n    revokeConsumerSitePermissions: {}\n  },\n  "Microsoft.SharePoint.Convergence.MigrationCompleteStateApi": {\n    addState: {},\n    isDoclibContributorOwnerEnabled: {}\n  },\n  "Microsoft.SharePoint.Convergence.OdcMetadataCleanedUpApi": {\n    addState: {}\n  },\n  "Microsoft.SharePoint.EmployeeEngagement.VivaConnectionsPage": {\n    getData: {},\n    setSpotlightConfiguration: {\n      argNames: ["configuration"]\n    }\n  },\n  "Microsoft.SharePoint.EmployeeEngagement.VivaHome": {\n    addImage: {\n      argNames: ["fileName", "imageStream"]\n    },\n    addImageFromExternalUrl: {\n      argNames: ["fileName", "externalUrl"]\n    },\n    titleRegion: {\n      argNames: ["vivaHomeTitleRegion"]\n    },\n    updateGoToVCButtonFlag: {\n      argNames: ["isGoBackToConnectionsButtonDisabled"]\n    }\n  },\n  "Microsoft.SharePoint.EmployeeEngagement.VivaResources": {\n    addLink: {\n      argNames: ["newLink"]\n    },\n    getLinks: {},\n    removeLink: {\n      argNames: ["id"]\n    },\n    reorderLink: {\n      argNames: ["linkId", "prevLinkId"]\n    },\n    updateLink: {\n      argNames: ["updatedLink"]\n    },\n    updateLinks: {\n      argNames: ["links"]\n    }\n  },\n  "Microsoft.SharePoint.IR.IRMigration": {\n    deleteAndRecreateIRList: {}\n  },\n  "Microsoft.SharePoint.Insights.SPTenantIBInsightsReportManager": {\n    createReport: {},\n    getAllReportsMetadata: {},\n    getReport: {\n      argNames: ["reportId", "siteType"]\n    },\n    getReportData: {\n      argNames: ["reportId", "siteType", "section"]\n    }\n  },\n  "Microsoft.SharePoint.Internal.ActivityLogger": {\n    feedbackDirect: {\n      argNames: ["Operation", "ListId", "ListItemUniqueId", "AffectedResourceUrl", "ItemType", "json"]\n    },\n    feedbackIndirect: {\n      argNames: ["Operation", "ListId", "ListItemUniqueId", "AffectedResourceUrl", "ItemType", "json"]\n    },\n    logActivity: {\n      argNames: ["Operation", "ListId", "ListItemUniqueId", "AffectedResourceUrl", "ItemType", "ExtraProperties", "AuditCreationTime", "IsOffline"]\n    },\n    logActivityBulk: {\n      argNames: ["Operation", "SiteId", "WebId", "ListId", "Requests"]\n    }\n  },\n  "Microsoft.SharePoint.Marketplace.CorporateCuratedGallery.CardDesigns": {\n    add: {\n      argNames: ["description", "id", "serializedProperties", "showInToolbox", "title"]\n    },\n    update: {\n      argNames: ["description", "id", "serializedProperties", "showInToolbox", "title"]\n    }\n  },\n  "Microsoft.SharePoint.Marketplace.CorporateCuratedGallery.CorporateCatalogAppMetadata": {\n    deploy: {\n      argNames: ["skipFeatureDeployment", "isUpdatingApp"],\n      requestType: utils_1.RequestType.Post\n    },\n    install: {\n      requestType: utils_1.RequestType.Post\n    },\n    remove: {\n      requestType: utils_1.RequestType.Post\n    },\n    retract: {\n      requestType: utils_1.RequestType.Post\n    },\n    uninstall: {\n      requestType: utils_1.RequestType.Post\n    },\n    upgrade: {\n      requestType: utils_1.RequestType.Post\n    }\n  },\n  "Microsoft.SharePoint.Marketplace.CorporateCuratedGallery.CorporateCatalogAppMetadata.Collection": {\n    getById: {\n      argNames: ["id"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "Microsoft.SharePoint.Marketplace.CorporateCuratedGallery.SiteCollectionAppCatalogAllowedItem.Collection": {\n    add: {\n      argNames: ["absolutePath"]\n    },\n    getById: {\n      argNames: ["siteId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    remove: {\n      argNames: ["absolutePath"]\n    },\n    removeById: {\n      argNames: ["siteId"]\n    }\n  },\n  "Microsoft.SharePoint.Marketplace.CorporateCuratedGallery.SiteCollectionCorporateCatalogAccessor": {\n    properties: ["AvailableApps|Microsoft.SharePoint.Marketplace.CorporateCuratedGallery.CorporateCatalogAppMetadata.Collection|/getById(\'[Name]\')|Microsoft.SharePoint.Marketplace.CorporateCuratedGallery.CorporateCatalogAppMetadata"],\n    add: {\n      argNames: ["Url", "Overwrite", "Content"],\n      requestType: utils_1.RequestType.PostWithArgsAndData\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "Microsoft.SharePoint.Marketplace.CorporateCuratedGallery.TeamsPackageDownload": {\n    downloadTeams: {}\n  },\n  "Microsoft.SharePoint.Marketplace.CorporateCuratedGallery.TenantCorporateCatalogAccessor": {\n    properties: ["AvailableApps|Microsoft.SharePoint.Marketplace.CorporateCuratedGallery.CorporateCatalogAppMetadata.Collection|/getById(\'[Name]\')|Microsoft.SharePoint.Marketplace.CorporateCuratedGallery.CorporateCatalogAppMetadata"],\n    add: {\n      argNames: ["Url", "Overwrite", "Content"],\n      requestType: utils_1.RequestType.PostWithArgsAndData\n    },\n    addAndDeployStoreAppById: {\n      argNames: ["CallerId", "CMU", "isUpdatingApp", "Overwrite", "SkipFeatureDeployment", "StoreAssetId"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    addStoreApp: {\n      argNames: ["Url", "Overwrite", "IconUrl", "Publisher", "ShortDescription", "StoreAssetId", "Content"],\n      requestType: utils_1.RequestType.PostWithArgsAndData\n    },\n    appRequests: {\n      argNames: ["AppRequestInfo"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    downloadTeamsSolution: {\n      argNames: ["id"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    downloadTeamsSolutionByUniqueId: {\n      argNames: ["id"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    getAppById: {\n      argNames: ["itemUniqueId"]\n    },\n    isAppUpgradeAvailable: {\n      argNames: ["id"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    sendAppRequestStatusNotificationEmail: {\n      argNames: ["RequestGuid"]\n    },\n    solutionContainsTeamsComponent: {\n      argNames: ["id"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    syncSolutionToTeams: {\n      argNames: ["id"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    syncSolutionToTeamsByUniqueId: {\n      argNames: ["id"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    updateMyRequestStatus: {\n      argNames: ["RequestId", "Status"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    upload: {\n      argNames: ["Url", "Overwrite", "Content"],\n      requestType: utils_1.RequestType.PostWithArgsAndData\n    }\n  },\n  "Microsoft.Online.SharePoint.MigrationCenter.Service.Device": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    update: {}\n  },\n  "Microsoft.Online.SharePoint.MigrationCenter.Service.Device.Collection": {\n    getById: {\n      argNames: ["id"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "Microsoft.Online.SharePoint.MigrationCenter.Service.MigrationCenterDeployStatus": {\n    isChangeDeployed: {\n      argNames: ["changeName"]\n    }\n  },\n  "Microsoft.Online.SharePoint.MigrationCenter.Service.MigrationCenterStorage": {\n    create: {\n      argNames: ["config"]\n    },\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    parseUrl: {\n      argNames: ["destinationUrl", "retrieveAllLists", "retrieveFoldersForAllLists", "forceMySiteDefaultList", "migrationType"]\n    },\n    update: {}\n  },\n  "Microsoft.Online.SharePoint.MigrationCenter.Service.MigrationCenterTeams": {\n    teamChannels: {\n      argNames: ["teamId", "membershipType"]\n    },\n    teamChannelsExperiment: {\n      argNames: ["teamId", "membershipType"]\n    },\n    teams: {\n      argNames: ["startsWith", "limit", "withLogo"]\n    }\n  },\n  "Microsoft.Online.SharePoint.MigrationCenter.Service.MigrationCredential": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    update: {}\n  },\n  "Microsoft.Online.SharePoint.MigrationCenter.Service.MigrationCredential.Collection": {\n    getById: {\n      argNames: ["id"]\n    },\n    getCredentials: {\n      argNames: ["AccountName", "Type"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "Microsoft.Online.SharePoint.MigrationCenter.Service.MigrationProperties": {\n    "delete": {\n      argNames: ["key"]\n    },\n    getProperty: {\n      argNames: ["key"]\n    },\n    setProperty: {\n      argNames: ["key", "value", "throwIfExists"]\n    }\n  },\n  "Microsoft.Online.SharePoint.MigrationCenter.Service.MigrationSPFlight": {\n    isFlightEnabled: {\n      argNames: ["flightName"]\n    }\n  },\n  "Microsoft.Online.SharePoint.MigrationCenter.Service.MigrationTask": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    update: {}\n  },\n  "Microsoft.Online.SharePoint.MigrationCenter.Service.MigrationTask.Collection": {\n    batchCreate: {\n      argNames: ["taskDefinitions", "taskSettings", "mmTaskSettings"]\n    },\n    batchDelete: {\n      argNames: ["taskIdList", "deleteInProgressTask"]\n    },\n    batchUpdate: {\n      argNames: ["tasks"]\n    },\n    createDuplicateTasks: {\n      argNames: ["taskDefinition", "taskSettings", "mmTaskSettings", "taskCount"]\n    },\n    getById: {\n      argNames: ["id"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "Microsoft.Online.SharePoint.MigrationCenter.Service.PerformanceData": {\n    "delete": {}\n  },\n  "Microsoft.Online.SharePoint.MigrationCenter.Service.PerformanceData.Collection": {\n    addPerfDataTest: {\n      argNames: ["Count", "Bottleneck", "MaxDuration", "MaxTaskFiles"]\n    },\n    getById: {\n      argNames: ["id"]\n    },\n    getData: {\n      argNames: ["StartTime", "EndTime", "AgentId", "TimeUnit"]\n    },\n    getPerfDataTest: {\n      argNames: ["StartTime", "EndTime", "AgentId"]\n    },\n    getRawData: {\n      argNames: ["StartTime", "EndTime", "AgentId"]\n    },\n    query: {\n      argNames: ["oData"]\n    }\n  },\n  "Microsoft.Online.SharePoint.MultiGeo.Service.DfDeprecationJob": {\n    "delete": {}\n  },\n  "Microsoft.Online.SharePoint.MultiGeo.Service.DfDeprecationJob.Collection": {\n    getBySiteUrl: {\n      argNames: ["sourceSiteUrl", "targetSiteUrl"]\n    },\n    query: {\n      argNames: ["oData"]\n    }\n  },\n  "Microsoft.Online.SharePoint.TenantAdministration.Internal.SPO3rdPartyAADPermissionGrant.Collection": {\n    add: {\n      argNames: ["servicePrincipalId", "resource", "scope"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    remove: {\n      argNames: ["servicePrincipalId", "resource", "scope"],\n      requestType: utils_1.RequestType.PostWithArgs\n    }\n  },\n  "Microsoft.SharePoint.MultiGeo.SPMultiGeoProxy": {\n    remoteThumbnail: {\n      argNames: ["url"]\n    }\n  },\n  "Microsoft.SharePoint.Navigation.REST.HomeSiteNavigationSettings": {\n    enableGlobalNavigation: {\n      argNames: ["isEnabled"]\n    }\n  },\n  "Microsoft.SharePoint.Navigation.REST.NavigationServiceRest": {\n    properties: ["MenuState|menustate|([Name])|menunode"],\n    currentResourcesNav: {\n      argNames: ["source", "includeVivaResources"]\n    },\n    getPublishingNavigationProviderType: {\n      argNames: ["mapProviderName"]\n    },\n    globalNav: {\n      argNames: ["source", "includeVivaResources"]\n    },\n    globalNavEnabled: {},\n    homeSiteNavigation: {\n      argNames: ["source"]\n    },\n    menuNodeKey: {\n      argNames: ["currentUrl", "mapProviderName"]\n    },\n    menuState: {\n      argNames: ["menuNodeKey", "mapProviderName", "depth", "customProperties"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    saveMenuState: {\n      argNames: ["menuState", "mapProviderName"]\n    },\n    setGlobalNavEnabled: {\n      argNames: ["isEnabled"]\n    }\n  },\n  "Microsoft.SharePoint.OrgNewsSite.OrgNewsSiteApi": {\n    details: {}\n  },\n  "Microsoft.SharePoint.Photos.PhotosMigration": {\n    migratePhotosData: {}\n  },\n  "Microsoft.SharePoint.Portal.GroupService": {\n    getGroupImage: {\n      argNames: ["id", "hash", "color"]\n    },\n    setGroupImage: {\n      argNames: ["imageStream"],\n      requestType: utils_1.RequestType.PostWithArgsAndData\n    },\n    syncGroupProperties: {\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    }\n  },\n  "Microsoft.SharePoint.Portal.GroupSiteManager": {\n    cacheCurrentUserJoinedTeamsResult: {\n      argNames: ["joinedTeams"]\n    },\n    canUserCreateGroup: {},\n    clearCurrentUserTeamsCache: {},\n    create: {\n      argNames: ["groupId"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    createGroup: {\n      argNames: ["displayName", "alias", "isPublic", "ownerPrincipalNames", "description", "creationOptions"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    createGroupEx: {\n      argNames: ["displayName", "alias", "isPublic", "optionalParams"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    createGroupForSite: {\n      argNames: ["displayName", "alias", "isPublic", "optionalParams"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    "delete": {\n      argNames: ["siteUrl"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    ensureTeamForGroup: {\n      argNames: ["siteUrl", "teamTemplate"]\n    },\n    ensureTeamForGroupEx: {\n      argNames: ["siteUrl", "teamTemplate"]\n    },\n    getAllOrgLabels: {\n      argNames: ["pageNumber"]\n    },\n    getCurrentUserJoinedTeams: {\n      argNames: ["getLogoData", "forceCacheUpdate"]\n    },\n    getCurrentUserSharedChannelMemberGroups: {},\n    getCurrentUserTeamConnectedMemberGroups: {},\n    getGroupCreationContext: {},\n    getGroupSiteConversionData: {},\n    getParentGroupForChannel: {\n      argNames: ["siteUrl"]\n    },\n    getSharedChannelSharePointUrl: {\n      argNames: ["tenantId", "groupId"]\n    },\n    getSiteStatus: {\n      argNames: ["groupId"]\n    },\n    getTeamChannelFilesUrl: {\n      argNames: ["teamId", "channelId"]\n    },\n    getTeamChannels: {\n      argNames: ["teamId", "useStagingEndpoint"]\n    },\n    getTeamChannelsDirect: {\n      argNames: ["teamId"]\n    },\n    getTeamChannelsEx: {\n      argNames: ["teamId"]\n    },\n    getTeamChannelsWithSiteUrl: {\n      argNames: ["siteUrl"]\n    },\n    getUserSharedChannelMemberGroups: {\n      argNames: ["userName"]\n    },\n    getUserTeamConnectedMemberGroups: {\n      argNames: ["userName"]\n    },\n    getValidSiteUrlFromAlias: {\n      argNames: ["alias", "managedPath", "isTeamSite"]\n    },\n    hideTeamifyPrompt: {\n      argNames: ["siteUrl"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    isTeamifyPromptHidden: {\n      argNames: ["siteUrl"]\n    },\n    notebook: {\n      argNames: ["groupId"]\n    },\n    pinToTeam: {\n      argNames: ["requestParams"]\n    },\n    recentAndJoinedTeams: {\n      argNames: ["includeRecent", "includeTeams", "includePinned", "existingJoinedTeamsData"]\n    }\n  },\n  "Microsoft.SharePoint.Portal.SPHubSitesUtility": {\n    getHubSites: {\n      requestType: utils_1.RequestType.Post\n    }\n  },\n  "Microsoft.SharePoint.Portal.SPSiteManager": {\n    archiveTeamChannelSite: {\n      argNames: ["siteId", "archive"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    areSegmentsCompatible: {\n      argNames: ["segments"]\n    },\n    canCreateHubJoinedSite: {\n      argNames: ["hubSiteId"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    create: {\n      argNames: ["request"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    "delete": {\n      argNames: ["siteId"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    getCompatibleSegments: {\n      argNames: ["segments"]\n    },\n    getIBSegmentLabels: {\n      argNames: ["IBSegments"],\n      requestType: utils_1.RequestType.GetWithArgsInQS\n    },\n    getTeamChannelSiteOwner: {\n      argNames: ["siteId"],\n      requestType: utils_1.RequestType.GetWithArgsInQS\n    },\n    landingSiteUrlFromName: {\n      argNames: ["siteName"]\n    },\n    restoreTeamsChannelSite: {\n      argNames: ["siteId", "relatedGroupId"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    setIBSegments: {\n      argNames: ["IBSegments"]\n    },\n    setTeamChannelSiteOwner: {\n      argNames: ["siteId", "logonName", "secondaryLogonName"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    siteUrl: {\n      argNames: ["siteId"],\n      requestType: utils_1.RequestType.GetWithArgsInQS\n    },\n    status: {\n      argNames: ["url"],\n      requestType: utils_1.RequestType.GetWithArgsInQS\n    },\n    updateWorkflow2013Endpoint: {\n      argNames: ["workflowServiceAddress", "workflowHostname"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    vivaBackendSiteUrlFromName: {\n      argNames: ["siteName"]\n    }\n  },\n  "Microsoft.SharePoint.Portal.SharePointHomeServiceContextBuilder": {\n    context: {}\n  },\n  "Microsoft.SharePoint.Portal.SiteIconManager": {\n    getSiteLogo: {\n      argNames: ["siteUrl", "target", "type", "hash"]\n    },\n    setSiteLogo: {\n      argNames: ["relativeLogoUrl", "type", "aspect", "focalx", "focaly", "isFocalPatch"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    }\n  },\n  "Microsoft.SharePoint.Portal.SiteLinkingManager": {\n    getSiteLinks: {},\n    linkGroup: {\n      argNames: ["groupId"]\n    },\n    unlinkGroup: {\n      argNames: ["groupId"]\n    }\n  },\n  "Microsoft.SharePoint.Portal.VivaSiteManager": {\n    ensureVivaSite: {}\n  },\n  "Microsoft.SharePoint.QuotaManagement.Consumer.NonQuotaBackfillApi": {\n    backfillNonQuota: {\n      argNames: ["backfillUserfacts"]\n    }\n  },\n  "Microsoft.SharePoint.QuotaManagement.Consumer.NonQuotaMigrationApi": {\n    migrateNonQuota: {}\n  },\n  "Microsoft.SharePoint.QuotaManagement.Consumer.QuotaMigrationApi": {\n    migrateQuota: {\n      argNames: ["IsMaxQuotaCall"]\n    }\n  },\n  "Microsoft.SharePoint.SectionDesignIdeas.SectionDesignIdeasApi": {\n    getSectionDesignIdeas: {\n      argNames: ["title", "subTitle"]\n    },\n    ping: {}\n  },\n  "Microsoft.SharePoint.Sharing.Internal.SharingRestrictions": {\n    update: {}\n  },\n  "Microsoft.SharePoint.TenantCdn.TenantCdnApi": {\n    getCdnUrls: {\n      argNames: ["items"]\n    },\n    isFolderUrlsInTenantCdn: {\n      argNames: ["urls", "cdnType"]\n    }\n  },\n  "Microsoft.SharePoint.Webhooks.Subscription": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    update: {\n      argNames: ["parameters"]\n    }\n  },\n  "Microsoft.SharePoint.Webhooks.Subscription.Collection": {\n    add: {\n      argNames: ["parameters"]\n    },\n    getById: {\n      argNames: ["id"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    remove: {\n      argNames: ["subscriptionId"]\n    }\n  },\n  "OBA.Server.ServerWrapper.Diagnostics": {\n    status: {}\n  },\n  "OBA.Server.ServerWrapper.Reporting": {\n    publishReport: {\n      argNames: ["odataPostBodyStm"]\n    }\n  },\n  "OBA.Server.ServerWrapper.Taskflow": {\n    processTask: {\n      argNames: ["requestBodyStream"]\n    }\n  },\n  "PS.BaseCalendarException": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    }\n  },\n  "PS.Calendar": {\n    copyTo: {\n      argNames: ["name"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.Calendar.Collection": {\n    add: {\n      argNames: ["parameters"]\n    },\n    getByGuid: {\n      argNames: ["uid"]\n    },\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    update: {}\n  },\n  "PS.CalendarException": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.CalendarException.Collection": {\n    add: {\n      argNames: ["parameters"]\n    },\n    getById: {\n      argNames: ["id"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.CustomField": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.CustomField.Collection": {\n    add: {\n      argNames: ["parameters"]\n    },\n    getByAppAlternateId: {\n      argNames: ["objectId"]\n    },\n    getByGuid: {\n      argNames: ["uid"]\n    },\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    update: {}\n  },\n  "PS.DraftAssignment.Collection": {\n    add: {\n      argNames: ["parameters"]\n    },\n    getByGuid: {\n      argNames: ["uid"]\n    },\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.DraftProjectResource.Collection": {\n    add: {\n      argNames: ["parameters"]\n    },\n    addEnterpriseResourceById: {\n      argNames: ["resourceId"]\n    },\n    getByGuid: {\n      argNames: ["uid"]\n    },\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.DraftTask.Collection": {\n    add: {\n      argNames: ["parameters"]\n    },\n    getByGuid: {\n      argNames: ["uid"]\n    },\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.DraftTaskLink.Collection": {\n    add: {\n      argNames: ["parameters"]\n    },\n    getByGuid: {\n      argNames: ["uid"]\n    },\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.EngagementComment.Collection": {\n    add: {\n      argNames: ["comment"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.EnterpriseProjectType": {\n    addDepartment: {\n      argNames: ["departmentValueGuid"]\n    },\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    removeDepartment: {\n      argNames: ["departmentValueGuid"]\n    },\n    updateCreatePDP: {\n      argNames: ["pdp"]\n    }\n  },\n  "PS.EnterpriseProjectType.Collection": {\n    add: {\n      argNames: ["parameters"]\n    },\n    getByGuid: {\n      argNames: ["uid"]\n    },\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    update: {}\n  },\n  "PS.EnterpriseResource": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    forceCheckIn: {},\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    updateClaimsAccount: {\n      argNames: ["newClaimsAccount"]\n    }\n  },\n  "PS.EnterpriseResource.Collection": {\n    add: {\n      argNames: ["parameters"]\n    },\n    getByGuid: {\n      argNames: ["uid"]\n    },\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    update: {}\n  },\n  "PS.EnterpriseResourceCostRate": {\n    rESTfulUpdate: {},\n    restfulDelete: {}\n  },\n  "PS.EnterpriseResourceCostRate.Collection": {\n    add: {\n      argNames: ["parameters"]\n    },\n    getByDateUrl: {\n      argNames: ["effectiveDate"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.EntityLink": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    }\n  },\n  "PS.EntityLink.Collection": {\n    add: {\n      argNames: ["parameters"]\n    },\n    getByGuid: {\n      argNames: ["id"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    update: {}\n  },\n  "PS.Event.Collection": {\n    getById: {\n      argNames: ["objectId"]\n    },\n    getByInt: {\n      argNames: ["id"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.EventHandler": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.EventHandler.Collection": {\n    add: {\n      argNames: ["parameters"]\n    },\n    getByGuid: {\n      argNames: ["uid"]\n    },\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    update: {}\n  },\n  "PS.LookupCost": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    }\n  },\n  "PS.LookupDate": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    }\n  },\n  "PS.LookupDuration": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    }\n  },\n  "PS.LookupEntry": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    }\n  },\n  "PS.LookupEntry.Collection": {\n    add: {\n      argNames: ["parameters"]\n    },\n    getByAppAlternateId: {\n      argNames: ["objectId"]\n    },\n    getByGuid: {\n      argNames: ["uid"]\n    },\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.LookupNumber": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    }\n  },\n  "PS.LookupTable": {\n    addMask: {\n      argNames: ["mask"]\n    },\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    updateMask: {\n      argNames: ["mask", "level"]\n    }\n  },\n  "PS.LookupTable.Collection": {\n    add: {\n      argNames: ["parameters"]\n    },\n    getByAppAlternateId: {\n      argNames: ["objectId"]\n    },\n    getByGuid: {\n      argNames: ["uid"]\n    },\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    update: {}\n  },\n  "PS.LookupText": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.Phase": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.Phase.Collection": {\n    add: {\n      argNames: ["parameters"]\n    },\n    getByGuid: {\n      argNames: ["uid"]\n    },\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    update: {}\n  },\n  "PS.PlanAssignment": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.PlanAssignment.Collection": {\n    add: {\n      argNames: ["parameters"]\n    },\n    getByGuid: {\n      argNames: ["uid"]\n    },\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.PlanAssignmentInterval.Collection": {\n    getById: {\n      argNames: ["id"]\n    },\n    getByStart: {\n      argNames: ["start"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.Project": {\n    getResourcePlanByUrl: {\n      argNames: ["start", "end", "scale"]\n    },\n    leaveProjectStage: {},\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    readyToLeaveProjectStage: {},\n    updateIdeaListItemStatus: {\n      argNames: ["status"]\n    }\n  },\n  "PS.ProjectDetailPage.Collection": {\n    getByGuid: {\n      argNames: ["uid"]\n    },\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.ProjectEngagement": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    getTimephasedByUrl: {\n      argNames: ["start", "end", "scale", "contourType"]\n    }\n  },\n  "PS.ProjectEngagement.Collection": {\n    add: {\n      argNames: ["parameters"]\n    },\n    getByGuid: {\n      argNames: ["uid"]\n    },\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    update: {}\n  },\n  "PS.ProjectEngagementTimephasedPeriod.Collection": {\n    getByStartUrl: {\n      argNames: ["start"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.ProjectServer": {\n    getDeletedPublishedAssignments: {\n      argNames: ["deletedDate"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    stopDelegation: {}\n  },\n  "PS.ProjectWorkflowInstance": {\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    restartWorkflow: {},\n    restartWorkflowSkipToStage: {\n      argNames: ["stageId"]\n    }\n  },\n  "PS.ProjectWorkflowInstance.Collection": {\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.PublishedAssignment.Collection": {\n    getByGuid: {\n      argNames: ["uid"]\n    },\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.PublishedProject": {\n    checkOut: {},\n    createProjectSite: {\n      argNames: ["siteName"]\n    },\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    getResourcePlanByUrl: {\n      argNames: ["start", "end", "scale"]\n    },\n    leaveProjectStage: {},\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    readyToLeaveProjectStage: {},\n    submitToWorkflow: {},\n    unlinkProjectSite: {},\n    updateIdeaListItemStatus: {\n      argNames: ["status"]\n    },\n    updateVisibilityCustomFields: {}\n  },\n  "PS.PublishedProject.Collection": {\n    add: {\n      argNames: ["parameters"]\n    },\n    getByGuid: {\n      argNames: ["uid"]\n    },\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    update: {},\n    validate: {}\n  },\n  "PS.PublishedProjectResource.Collection": {\n    getByGuid: {\n      argNames: ["uid"]\n    },\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.PublishedTask": {\n    addTaskPlanLink: {\n      argNames: ["parameters"]\n    },\n    deleteTaskPlanLink: {},\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.PublishedTask.Collection": {\n    getByGuid: {\n      argNames: ["uid"]\n    },\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.PublishedTaskLink.Collection": {\n    getByGuid: {\n      argNames: ["uid"]\n    },\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.QueueJob": {\n    cancel: {},\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.QueueJob.Collection": {\n    getAll: {},\n    getByGuid: {\n      argNames: ["uid"]\n    },\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.ResourceCalendarException": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    }\n  },\n  "PS.ResourceEngagement": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    getTimephasedByUrl: {\n      argNames: ["start", "end", "scale", "contourType"]\n    }\n  },\n  "PS.ResourceEngagement.Collection": {\n    getByGuid: {\n      argNames: ["uid"]\n    },\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    update: {}\n  },\n  "PS.ResourceEngagementTimephasedPeriod.Collection": {\n    getByStartUrl: {\n      argNames: ["start"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.ResourcePlan": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    forceCheckIn: {},\n    publish: {},\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    update: {}\n  },\n  "PS.ServiceStatus": {\n    stopDelegation: {}\n  },\n  "PS.Stage": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.Stage.Collection": {\n    add: {\n      argNames: ["parameters"]\n    },\n    getByGuid: {\n      argNames: ["uid"]\n    },\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    update: {}\n  },\n  "PS.StageCustomField": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.StageCustomField.Collection": {\n    add: {\n      argNames: ["creationInfo"]\n    },\n    getByGuid: {\n      argNames: ["uid"]\n    },\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.StageDetailPage": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.StageDetailPage.Collection": {\n    add: {\n      argNames: ["parameters"]\n    },\n    getByGuid: {\n      argNames: ["uid"]\n    },\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.StatusAssignment": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    submitStatusUpdates: {\n      argNames: ["comment"]\n    }\n  },\n  "PS.StatusAssignment.Collection": {\n    add: {\n      argNames: ["parameters"]\n    },\n    getByGuid: {\n      argNames: ["uid"]\n    },\n    getById: {\n      argNames: ["objectId"]\n    },\n    getTimePhaseByUrl: {\n      argNames: ["start", "end"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    submitAllStatusUpdates: {\n      argNames: ["comment"]\n    },\n    update: {}\n  },\n  "PS.StatusAssignmentHistoryLine.Collection": {\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.TaskPlanLink": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    }\n  },\n  "PS.TimeSheet": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    recall: {},\n    submit: {\n      argNames: ["comment"]\n    },\n    update: {}\n  },\n  "PS.TimeSheetLine": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    submit: {\n      argNames: ["comment"]\n    }\n  },\n  "PS.TimeSheetLine.Collection": {\n    add: {\n      argNames: ["parameters"]\n    },\n    getByGuid: {\n      argNames: ["uid"]\n    },\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.TimeSheetPeriod": {\n    createTimeSheet: {},\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.TimeSheetPeriod.Collection": {\n    getByGuid: {\n      argNames: ["uid"]\n    },\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.TimeSheetWork": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    }\n  },\n  "PS.TimeSheetWork.Collection": {\n    add: {\n      argNames: ["parameters"]\n    },\n    getById: {\n      argNames: ["objectId"]\n    },\n    getByStartDate: {\n      argNames: ["start"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "PS.WorkflowActivities": {\n    checkInWithJobId: {\n      argNames: ["projId", "jobId", "force"]\n    },\n    createProjectFromListItem: {\n      argNames: ["webId", "listId", "itemId", "eptId"]\n    },\n    enterProjectStage: {\n      argNames: ["projectId", "stageId"]\n    },\n    leaveProjectStage: {\n      argNames: ["projectId"]\n    },\n    publishSummaryWithJobId: {\n      argNames: ["projId", "jobId"]\n    },\n    publishWithJobId: {\n      argNames: ["projectId", "jobId"]\n    },\n    readBooleanProperty: {\n      argNames: ["projectId", "propertyId"]\n    },\n    readCurrencyProperty: {\n      argNames: ["projectId", "propertyId"]\n    },\n    readDateTimeProperty: {\n      argNames: ["projectId", "propertyId"]\n    },\n    readGuidProperty: {\n      argNames: ["projectId", "propertyId"]\n    },\n    readIntegerProperty: {\n      argNames: ["projectId", "propertyId"]\n    },\n    readNumberProperty: {\n      argNames: ["projectId", "propertyId"]\n    },\n    readProjectProperty: {\n      argNames: ["projectId", "propertyId"]\n    },\n    readTextProperty: {\n      argNames: ["projectId", "propertyId"]\n    },\n    readyToLeaveProjectStage: {\n      argNames: ["projectId"]\n    },\n    updateBooleanProperty: {\n      argNames: ["projectId", "propertyId", "value"]\n    },\n    updateCurrencyProperty: {\n      argNames: ["projectId", "propertyId", "value"]\n    },\n    updateDateTimeProperty: {\n      argNames: ["projectId", "propertyId", "value"]\n    },\n    updateGuidProperty: {\n      argNames: ["projectId", "propertyId", "value"]\n    },\n    updateIdeaListItemStatus: {\n      argNames: ["projectId", "status"]\n    },\n    updateIntegerProperty: {\n      argNames: ["projectId", "propertyId", "value"]\n    },\n    updateNumberProperty: {\n      argNames: ["projectId", "propertyId", "value"]\n    },\n    updateProjectStageStatus: {\n      argNames: ["projectId", "stageId", "statusInformation", "stageStatusValue", "append"]\n    },\n    updateTextProperty: {\n      argNames: ["projectId", "propertyId", "value"]\n    }\n  },\n  "PS.WorkflowDesignerField.Collection": {\n    getById: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.Alert": {\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    updateAlert: {}\n  },\n  "SP.Alert.Collection": {\n    add: {\n      argNames: ["alertCreationInformation"],\n      name: "",\n      metadataType: "SP.Alert",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    contains: {\n      argNames: ["idAlert"]\n    },\n    deleteAlert: {\n      argNames: ["idAlert"]\n    },\n    deleteAlertAtIndex: {\n      argNames: ["index"]\n    },\n    getById: {\n      argNames: ["idAlert"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.AppConfiguration": {\n    update: {}\n  },\n  "SP.AppInstance": {\n    cancelAllJobs: {},\n    getAppDatabaseConnectionString: {},\n    getErrorDetails: {},\n    getPreviousAppVersion: {},\n    install: {},\n    recycle: {},\n    restore: {},\n    retryAllJobs: {},\n    uninstall: {},\n    upgrade: {\n      argNames: ["appPackageStream"]\n    }\n  },\n  "SP.Attachment": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    recycleObject: {\n      requestType: utils_1.RequestType.Post\n    }\n  },\n  "SP.Attachment.Collection": {\n    add: {\n      argNames: ["FileName", "Content"],\n      requestType: utils_1.RequestType.PostWithArgsAndData\n    },\n    addUsingPath: {\n      argNames: ["DecodedUrl", "contentStream"],\n      requestType: utils_1.RequestType.PostWithArgsAndData\n    },\n    getByFileName: {\n      argNames: ["fileName"]\n    },\n    getByFileNameAsPath: {\n      argNames: ["DecodedUrl"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.Audit": {\n    update: {\n      metadataType: "",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.BrandCenter": {\n    addSiteTheme: {\n      argNames: ["themeData"]\n    },\n    addTenantTheme: {\n      argNames: ["themeData"]\n    },\n    configuration: {},\n    currentBrandingConfiguration: {},\n    deleteSiteTheme: {\n      argNames: ["themeId"]\n    },\n    ensureBrandColorsListFeature: {},\n    ensureBrandFontsLibraryFeature: {},\n    getFontStream: {\n      argNames: ["fontFileUrl"]\n    },\n    getSiteThemes: {},\n    getTenantThemeById: {\n      argNames: ["id"]\n    },\n    getTenantThemes: {},\n    getTenantThemesXgeoUtil: {},\n    orgAssets: {},\n    orgAssetsWithCacheFlag: {\n      argNames: ["shouldUseCache"]\n    },\n    updateSiteTheme: {\n      argNames: ["themeData"]\n    },\n    updateTenantTheme: {\n      argNames: ["themeData"]\n    },\n    validateTenantThemeName: {\n      argNames: ["name"]\n    }\n  },\n  "SP.BusinessData.AppBdcCatalog": {\n    getConnectionId: {\n      argNames: ["lobSystemName", "lobSystemInstanceName"]\n    },\n    getEntity: {\n      argNames: ["namespace", "name"]\n    },\n    getLobSystemInstanceProperty: {\n      argNames: ["lobSystemName", "lobSystemInstanceName", "propertyName"]\n    },\n    getLobSystemProperty: {\n      argNames: ["lobSystemName", "propertyName"]\n    },\n    getPermissibleConnections: {},\n    setConnectionId: {\n      argNames: ["lobSystemName", "lobSystemInstanceName", "connectionId"]\n    },\n    setLobSystemInstanceProperty: {\n      argNames: ["lobSystemName", "lobSystemInstanceName", "propertyName", "propertyValue"]\n    },\n    setLobSystemProperty: {\n      argNames: ["lobSystemName", "propertyName", "propertyValue"]\n    }\n  },\n  "SP.BusinessData.Entity": {\n    getAssociationView: {\n      argNames: ["associationName"]\n    },\n    getCreatorView: {\n      argNames: ["methodInstanceName"]\n    },\n    getDefaultSpecificFinderView: {},\n    getFilters: {\n      argNames: ["methodInstanceName"]\n    },\n    getFinderView: {\n      argNames: ["methodInstanceName"]\n    },\n    getIdentifierCount: {},\n    getIdentifiers: {},\n    getLobSystem: {},\n    getSpecificFinderView: {\n      argNames: ["specificFinderName"]\n    },\n    getUpdaterView: {\n      argNames: ["updaterName"]\n    }\n  },\n  "SP.BusinessData.EntityIdentifier": {\n    containsLocalizedDisplayName: {},\n    getDefaultDisplayName: {},\n    getLocalizedDisplayName: {}\n  },\n  "SP.BusinessData.EntityView": {\n    getDefaultValues: {},\n    getType: {\n      argNames: ["fieldDotNotation"]\n    },\n    getTypeDescriptor: {\n      argNames: ["fieldDotNotation"]\n    },\n    getXmlSchema: {},\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.BusinessData.Infrastructure.ExternalSubscriptionStore": {\n    indexStore: {}\n  },\n  "SP.BusinessData.LobSystem": {\n    getLobSystemInstances: {}\n  },\n  "SP.BusinessData.Runtime.EntityFieldValueDictionary": {\n    createCollectionInstance: {\n      argNames: ["fieldDotNotation", "size"]\n    },\n    createInstance: {\n      argNames: ["fieldInstanceDotNotation", "fieldDotNotation"]\n    },\n    fromXml: {\n      argNames: ["xml"]\n    },\n    getCollectionSize: {\n      argNames: ["fieldDotNotation"]\n    },\n    toXml: {}\n  },\n  "SP.BusinessData.Runtime.EntityInstance": {\n    createCollectionInstance: {\n      argNames: ["fieldDotNotation", "size"]\n    },\n    createInstance: {\n      argNames: ["fieldInstanceDotNotation", "fieldDotNotation"]\n    },\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    fromXml: {\n      argNames: ["xml"]\n    },\n    getIdentity: {},\n    toXml: {},\n    update: {\n      metadataType: "",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.BusinessData.TypeDescriptor": {\n    containsLocalizedDisplayName: {},\n    getChildTypeDescriptors: {},\n    getDefaultDisplayName: {},\n    getLocalizedDisplayName: {},\n    getParentTypeDescriptor: {},\n    isLeaf: {},\n    isRoot: {}\n  },\n  "SP.CheckedOutFile": {\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    takeOverCheckOut: {}\n  },\n  "SP.CheckedOutFile.Collection": {\n    getByPath: {\n      argNames: ["DecodedUrl"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.ClientWebPart": {\n    render: {\n      argNames: ["properties"]\n    }\n  },\n  "SP.ClientWebPart.Collection": {\n    getById: {\n      argNames: ["id"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.CompliancePolicy.SPPolicyStoreProxy": {\n    bulkUpdateDynamicScopeBindings: {\n      argNames: ["scopesToAdd", "scopesToRemove", "siteId"]\n    },\n    extendReviewItemsRetention: {\n      argNames: ["itemIds", "extensionDate"]\n    },\n    getDynamicScopeBindingBySiteId: {\n      argNames: ["siteId"]\n    },\n    getSiteAdaptivePolicies: {\n      argNames: ["siteId"]\n    },\n    markReviewItemsForDeletion: {\n      argNames: ["itemIds"]\n    },\n    openBinaryStreamForOriginalItem: {\n      argNames: ["itemId"]\n    },\n    removeContainerRetentionPolicy: {\n      argNames: ["siteId"]\n    },\n    removeContainerSettings: {\n      argNames: ["externalId"]\n    },\n    retagReviewItems: {\n      argNames: ["itemIds", "newTag", "newTagIsRecord", "newTagBlockDelete", "newTagIsEventBased"]\n    },\n    retagReviewItemsWithMetas: {\n      argNames: ["itemIds", "newTagName", "newTagMetas"]\n    },\n    retagUnifiedReviewItemsWithMetas: {\n      argNames: ["itemIds", "originalTagName", "newTagName", "newTagMetas"]\n    },\n    setContainerRetentionPolicy: {\n      argNames: ["siteId", "defaultContainerLabel"]\n    },\n    updateContainerSetting: {\n      argNames: ["siteId", "externalId", "settingType", "setting"]\n    },\n    updateSiteAdaptivePolicies: {\n      argNames: ["policiesToAdd", "policiesToRemove", "siteId"]\n    }\n  },\n  "SP.ContentType": {\n    properties: ["FieldLinks|SP.FieldLink.Collection|(\'[Name]\')|SP.FieldLink", "Fields|SP.Field.Collection|/getByInternalNameOrTitle(\'[Name]\')|SP.Field", "WorkflowAssociations|SP.Workflow.WorkflowAssociation.Collection"],\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    reorderFields: {\n      argNames: ["fieldNames"]\n    },\n    update: {\n      argNames: ["properties"],\n      metadataType: "SP.ContentType",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.ContentType.Collection": {\n    add: {\n      argNames: ["parameters"],\n      metadataType: "SP.ContentType",\n      name: "",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    addAvailableContentType: {\n      argNames: ["contentTypeId"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    create: {\n      argNames: ["parameters"],\n      metadataType: "SP.ContentType",\n      name: "",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    getById: {\n      argNames: ["contentTypeId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.ContentType"\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.Directory.DirectorySession": {\n    getGraphUser: {\n      argNames: ["principalName"]\n    },\n    getSharePointDataForUser: {\n      argNames: ["userId"]\n    },\n    group: {\n      argNames: ["groupId", "alias"]\n    },\n    joinGroup: {\n      argNames: ["groupId"]\n    },\n    me: {},\n    user: {\n      argNames: ["id", "principalName"]\n    },\n    validateGroupName: {\n      argNames: ["displayName", "alias"]\n    }\n  },\n  "SP.Directory.Group": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.Directory.Group.Collection": {\n    add: {\n      argNames: ["objectId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    remove: {\n      argNames: ["objectId"]\n    }\n  },\n  "SP.Directory.Provider.DirectoryNotification": {\n    notifyChanges: {\n      argNames: ["directoryObjectChanges"]\n    }\n  },\n  "SP.Directory.Provider.SharePointDirectoryProvider": {\n    createDirectoryObject: {\n      argNames: ["data"]\n    },\n    deleteDirectoryObject: {\n      argNames: ["data"]\n    },\n    getOrCreateUnifiedGroupTenantInstanceId: {\n      argNames: ["groupId", "tenantInstanceId"]\n    },\n    getOrCreateUnifiedGroupWithPreferredDataLocation: {\n      argNames: ["groupId", "preferredDataLocation"]\n    },\n    notifyDataChanges: {\n      argNames: ["data"]\n    },\n    readDirectoryObject: {\n      argNames: ["data"]\n    },\n    readDirectoryObjectBatch: {\n      argNames: ["ids", "objectType"]\n    },\n    updateCache: {\n      argNames: ["data"]\n    },\n    updateDirectoryObject: {\n      argNames: ["data"]\n    }\n  },\n  "SP.Directory.User": {\n    getUserLinks: {\n      argNames: ["linkName", "groupType"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.Directory.User.Collection": {\n    add: {\n      argNames: ["objectId", "principalName"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    remove: {\n      argNames: ["objectId"]\n    }\n  },\n  "SP.DocumentManagement.DocumentId": {\n    resetDocIdByServerRelativePath: {\n      argNames: ["DecodedUrl"]\n    },\n    resetDocIdsInLibrary: {\n      argNames: ["DecodedUrl", "contentTypeId"]\n    },\n    setDocIdSitePrefix: {\n      argNames: ["prefix", "scheduleAssignment", "overwriteExistingIds"]\n    }\n  },\n  "SP.EmployeeEngagement": {\n    configuration: {},\n    dashboardContent: {\n      argNames: ["overrideLanguageCode"]\n    },\n    dashboardOOBContent: {\n      argNames: ["oobContentChoice"]\n    },\n    fullDashboardContent: {\n      argNames: ["canvasAsJson", "includePersonalizationData"]\n    },\n    getDashboardPersonalization: {},\n    getTargetedSitesAsEditor: {},\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    setDashboardPersonalization: {\n      argNames: ["isEnabled"]\n    },\n    vivaConnections: {\n      argNames: ["adminConfiguredUrl"]\n    },\n    vivaHome: {},\n    vivaHomeConfiguration: {\n      argNames: ["shouldByPassCache"]\n    }\n  },\n  "SP.EmployeeExperienceController": {\n    dashboard: {\n      argNames: ["formFactor", "dashboardId", "mySiteUrl"]\n    },\n    getAnnouncementsState: {\n      argNames: ["mySiteUrl"]\n    },\n    saveDashboard: {\n      argNames: ["employeeExperienceDashboardData", "mySiteUrl"]\n    },\n    setAnnouncementState: {\n      argNames: ["announcementStates", "mySiteUrl"]\n    }\n  },\n  "SP.EventReceiverDefinition": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    update: {\n      argNames: ["properties"],\n      metadataType: "SP.EventReceiverDefinition",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.EventReceiverDefinition.Collection": {\n    add: {\n      argNames: ["eventReceiverCreationInformation"],\n      metadataType: "SP.EventReceiverDefinition",\n      name: "",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    getById: {\n      argNames: ["eventReceiverId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.EventReceiverDefinition"\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.Feature.Collection": {\n    add: {\n      argNames: ["featureId", "force", "featdefScope"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    getById: {\n      argNames: ["featureId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.Feature"\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    remove: {\n      argNames: ["featureId", "force"],\n      requestType: utils_1.RequestType.PostWithArgs\n    }\n  },\n  "SP.Field": {\n    add: {\n      argNames: ["parameters"],\n      name: "",\n      metadataType: "SP.Field",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    disableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    enableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    setShowInDisplayForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInEditForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInNewForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    update: {\n      argNames: ["properties"],\n      inheritMetadataType: true,\n      metadataType: "SP.Field",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.Field.Collection": {\n    addDependentLookupField: {\n      argNames: ["displayName", "primaryLookupFieldId", "showField"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    addField: {\n      argNames: ["parameters"],\n      metadataType: "SP.FieldCreationInformation",\n      name: "",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    createFieldAsXml: {\n      argNames: ["schemaXml"],\n      requestType: utils_1.RequestType.PostWithArgsInBody,\n      data: {\n        parameters: {\n          __metadata: {\n            type: "SP.XmlSchemaFieldCreationInformation"\n          },\n          Options: 8,\n          SchemaXml: "[[schemaXml]]"\n        }\n      }\n    },\n    getById: {\n      argNames: ["id"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly,\n      returnType: "SP.Field"\n    },\n    getByInternalNameOrTitle: {\n      argNames: ["strName"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly,\n      returnType: "SP.Field"\n    },\n    getByTitle: {\n      argNames: ["title"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly,\n      returnType: "SP.Field"\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.FieldCalculated": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    disableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    enableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    setShowInDisplayForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInEditForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInNewForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    update: {\n      argNames: ["properties"],\n      inheritMetadataType: true,\n      metadataType: "SP.FieldCalculated",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.FieldChoice": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    disableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    enableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    setShowInDisplayForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInEditForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInNewForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    update: {\n      argNames: ["properties"],\n      inheritMetadataType: true,\n      metadataType: "SP.FieldChoice",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.FieldComputed": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    disableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    enableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    setShowInDisplayForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInEditForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInNewForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    update: {\n      argNames: ["properties"],\n      inheritMetadataType: true,\n      metadataType: "SP.FieldComputed",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.FieldCurrency": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    disableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    enableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    setShowInDisplayForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInEditForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInNewForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    update: {\n      argNames: ["properties"],\n      inheritMetadataType: true,\n      metadataType: "SP.FieldCurrency",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.FieldDateTime": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    disableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    enableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    setShowInDisplayForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInEditForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInNewForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    update: {\n      argNames: ["properties"],\n      inheritMetadataType: true,\n      metadataType: "SP.FieldDateTime",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.FieldGeolocation": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    disableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    enableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    setShowInDisplayForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInEditForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInNewForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    update: {\n      argNames: ["properties"],\n      inheritMetadataType: true,\n      metadataType: "SP.FieldGeolocation",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.FieldGuid": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    disableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    enableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    setShowInDisplayForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInEditForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInNewForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    update: {\n      argNames: ["properties"],\n      inheritMetadataType: true,\n      metadataType: "SP.FieldGuid",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.FieldLink": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    update: {\n      argNames: ["properties"],\n      inheritMetadataType: true,\n      metadataType: "SP.FieldLink",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.FieldLink.Collection": {\n    add: {\n      argNames: ["parameters"],\n      metadataType: "SP.FieldLink",\n      name: "",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    getById: {\n      argNames: ["id"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.FieldLink"\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    reorder: {\n      argNames: ["internalNames"]\n    }\n  },\n  "SP.FieldLocation": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    disableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    enableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    setShowInDisplayForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInEditForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInNewForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    update: {\n      argNames: ["properties"],\n      inheritMetadataType: true,\n      metadataType: "SP.FieldLocation",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.FieldLookup": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    disableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    enableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    setShowInDisplayForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInEditForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInNewForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    update: {\n      argNames: ["properties"],\n      inheritMetadataType: true,\n      metadataType: "SP.FieldLookup",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.FieldMultiChoice": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    disableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    enableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    setShowInDisplayForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInEditForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInNewForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    update: {\n      argNames: ["properties"],\n      inheritMetadataType: true,\n      metadataType: "SP.FieldMultiChoice",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.FieldMultiLineText": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    disableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    enableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    setShowInDisplayForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInEditForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInNewForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    update: {\n      argNames: ["properties"],\n      inheritMetadataType: true,\n      metadataType: "SP.FieldMultiLineText",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.FieldNumber": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    disableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    enableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    setShowInDisplayForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInEditForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInNewForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    update: {\n      argNames: ["properties"],\n      inheritMetadataType: true,\n      metadataType: "SP.FieldNumber",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.FieldRatingScale": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    disableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    enableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    setShowInDisplayForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInEditForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInNewForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    update: {\n      argNames: ["properties"],\n      inheritMetadataType: true,\n      metadataType: "SP.FieldRatingScale",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.FieldText": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    disableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    enableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    setShowInDisplayForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInEditForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInNewForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    update: {\n      argNames: ["properties"],\n      inheritMetadataType: true,\n      metadataType: "SP.FieldText",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.FieldThumbnail": {\n    update: {\n      argNames: ["properties"],\n      inheritMetadataType: true,\n      metadataType: "SP.FieldThumbnail",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.FieldUrl": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    disableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    enableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    setShowInDisplayForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInEditForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInNewForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    update: {\n      argNames: ["properties"],\n      inheritMetadataType: true,\n      metadataType: "SP.FieldUrl",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.FieldUser": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    disableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    enableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    setShowInDisplayForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInEditForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInNewForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    update: {\n      argNames: ["properties"],\n      inheritMetadataType: true,\n      metadataType: "SP.FieldUser",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.File": {\n    properties: ["Author|SP.User", "CheckedOutByUser|SP.User", "EffectiveInformationRightsManagementSettings", "InformationRightsManagementSettings", "ListItemAllFields", "LockedByUser|SP.User", "ModifiedBy|SP.User", "Properties", "VersionEvents", "Versions|SP.FileVersion.Collection"],\n    addClientActivities: {\n      argNames: ["activitiesStream"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    addFileScannerWorkItem: {\n      argNames: ["dispatchType", "jobType", "jobSubType"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    approve: {\n      argNames: ["comment"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    cancelUpload: {\n      argNames: ["uploadId"],\n      name: "cancelupload(guid\'[[uploadId]]\')",\n      requestType: utils_1.RequestType.PostReplace\n    },\n    changeContentStorageSchema: {\n      argNames: ["desiredSchema"]\n    },\n    checkAccessAndPostViewAuditEvent: {},\n    checkIn: {\n      argNames: ["comment", "checkInType"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    checkOut: {\n      requestType: utils_1.RequestType.Post\n    },\n    content: {\n      name: "$value",\n      requestType: utils_1.RequestType.GetBuffer\n    },\n    continueUpload: {\n      argNames: ["uploadId", "fileOffset", "stream"],\n      name: "continueUpload(uploadId=guid\'[[uploadId]]\', fileOffset=[[fileOffset]])",\n      requestType: utils_1.RequestType.PostReplaceWithData\n    },\n    copyTo: {\n      argNames: ["strNewUrl", "bOverWrite"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    copyToUsingPath: {\n      argNames: ["DecodedUrl", "bOverWrite"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    deleteWithParameters: {\n      argNames: ["parameters"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    deny: {\n      argNames: ["comment"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    executeCobaltRequest: {\n      argNames: ["inputStream"],\n      requestType: utils_1.RequestType.PostWithArgsAndData\n    },\n    finishUpload: {\n      argNames: ["uploadId", "fileOffset", "stream"],\n      name: "finishUpload(uploadId=guid\'[[uploadId]]\', fileOffset=[[fileOffset]])",\n      requestType: utils_1.RequestType.PostReplaceWithData\n    },\n    finishUploadWithChecksum: {\n      argNames: ["uploadId", "fileOffset", "checksum", "stream", "sandboxId"],\n      name: "finishUploadWithChecksum(uploadId=guid\'[[uploadId]]\', fileOffset=[[fileOffset]], checksum=[[checksum]])",\n      requestType: utils_1.RequestType.PostReplaceWithData\n    },\n    getFileUserValue: {\n      argNames: ["key"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    getImagePreviewUri: {\n      argNames: ["width", "height", "clientType"],\n      requestType: utils_1.RequestType.GetWithArgs\n    },\n    getImagePreviewUrl: {\n      argNames: ["width", "height", "clientType"],\n      requestType: utils_1.RequestType.GetWithArgs\n    },\n    getLimitedWebPartManager: {\n      argNames: ["scope"],\n      name: "getLimitedWebPartManager(scope=[[scope]])",\n      requestType: utils_1.RequestType.GetReplace,\n      returnType: "SP.WebParts.LimitedWebPartManager"\n    },\n    getMediaServiceMetadata: {},\n    getPreAuthorizedAccessUrl: {\n      argNames: ["expirationHours"],\n      requestType: utils_1.RequestType.GetWithArgs\n    },\n    getPreAuthorizedAccessUrl2: {\n      argNames: ["expirationHours", "expirationMinuites"],\n      requestType: utils_1.RequestType.GetWithArgs\n    },\n    getUploadStatus: {\n      argNames: ["uploadId"],\n      requestType: utils_1.RequestType.GetWithArgs\n    },\n    getWOPIFrameUrl: {\n      argNames: ["action"],\n      requestType: utils_1.RequestType.PostWithArgsInQS\n    },\n    moveTo: {\n      argNames: ["newUrl", "flags"],\n      name: "moveTo(newUrl=\'[[newUrl]]\', flags=[[flags]])",\n      requestType: utils_1.RequestType.PostReplace\n    },\n    moveToUsingPath: {\n      argNames: ["DecodedUrl", "moveOperations"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    openBinaryStream: {\n      requestType: utils_1.RequestType.GetBuffer\n    },\n    openBinaryStreamWithOptions: {\n      argNames: ["openOptions"],\n      requestType: utils_1.RequestType.GetBuffer\n    },\n    publish: {\n      argNames: ["comment"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    recycle: {\n      requestType: utils_1.RequestType.Post\n    },\n    recycleWithETag: {\n      argNames: ["etagMatch"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    recycleWithParameters: {\n      argNames: ["parameters"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    saveBinaryStream: {\n      argNames: ["file"],\n      requestType: utils_1.RequestType.PostWithArgsAndData\n    },\n    setExpirationDate: {\n      argNames: ["expirationDate"]\n    },\n    setFileUserValue: {\n      argNames: ["key", "value"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    setMediaServiceMetadata: {\n      argNames: ["parameters"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    startUpload: {\n      argNames: ["uploadId", "stream"],\n      name: "startupload(uploadId=guid\'[[uploadId]]\')",\n      requestType: utils_1.RequestType.PostReplaceWithData\n    },\n    startUploadFile: {\n      argNames: ["uploadId", "stream"],\n      name: "startUploadFile(uploadId=guid\'[[uploadId]]\')",\n      requestType: utils_1.RequestType.PostReplaceWithData\n    },\n    unPublish: {\n      argNames: ["comment"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    undoCheckOut: {\n      requestType: utils_1.RequestType.Post\n    },\n    update: {\n      argNames: ["properties"],\n      metadataType: "SP.File",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    updateVirusInfo: {\n      argNames: ["virusStatus", "virusMessage", "etagToCheck"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    upload: {\n      argNames: ["uploadId", "stream"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    uploadWithChecksum: {\n      argNames: ["uploadId", "checksum", "stream"],\n      requestType: utils_1.RequestType.PostWithArgs\n    }\n  },\n  "SP.File.Collection": {\n    add: {\n      argNames: ["Url", "Overwrite", "Content"],\n      requestType: utils_1.RequestType.PostWithArgsAndData\n    },\n    addStub: {\n      argNames: ["urlOfFile"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    addStubUsingPath: {\n      argNames: ["DecodedUrl", "AutoCheckoutOnInvalidData", "EnsureUniqueFileName", "Overwrite", "XorHash"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    addTemplateFile: {\n      argNames: ["urlOfFile", "templateFileType"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    addUsingPath: {\n      argNames: ["DecodedUrl", "AutoCheckoutOnInvalidData", "EnsureUniqueFileName", "Overwrite", "XorHash", "contentStream"],\n      requestType: utils_1.RequestType.PostWithArgsAndData\n    },\n    getByPathOrAddStub: {\n      argNames: ["DecodedUrl"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getByUrl: {\n      argNames: ["url"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.File"\n    },\n    getByUrlOrAddStub: {\n      argNames: ["urlOfFile"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.FileVersion": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    openBinaryStream: {\n      requestType: utils_1.RequestType.GetBuffer\n    },\n    openBinaryStreamWithOptions: {\n      argNames: ["openOptions"],\n      requestType: utils_1.RequestType.GetBuffer\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    setExpirationDate: {\n      argNames: ["expirationDate"]\n    }\n  },\n  "SP.FileVersion.Collection": {\n    deleteAll: {\n      requestType: utils_1.RequestType.Post\n    },\n    deleteByID: {\n      argNames: ["vid"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    deleteByLabel: {\n      argNames: ["versionlabel"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    getById: {\n      argNames: ["versionid"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.Version"\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    recycleByID: {\n      argNames: ["vid"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    recycleByLabel: {\n      argNames: ["versionlabel"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    restoreByLabel: {\n      argNames: ["versionlabel"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    }\n  },\n  "SP.Folder": {\n    properties: ["Files|SP.File.Collection|/getByUrl(\'[Name]\')|SP.File", "Folders|SP.Folder.Collection|/getByUrl(\'[Name]\')|SP.Folder", "ListItemAllFields", "ParentFolder|SP.Folder", "Properties", "StorageMetrics"],\n    addSubFolder: {\n      argNames: ["leafName", "updateParams"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    addSubFolderUsingPath: {\n      argNames: ["DecodedUrl"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    deleteWithParameters: {\n      argNames: ["parameters"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    getChanges: {\n      argNames: ["query"],\n      metadataType: "SP.ChangeQuery",\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    getListItemChanges: {\n      argNames: ["query"],\n      metadataType: "SP.ChangeQuery",\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    moveTo: {\n      argNames: ["newUrl"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    moveToUsingPath: {\n      argNames: ["DecodedUrl"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    recycle: {\n      requestType: utils_1.RequestType.Post\n    },\n    recycleWithParameters: {\n      argNames: ["parameters"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    startDelete: {},\n    update: {\n      argNames: ["properties"],\n      metadataType: "SP.Folder",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    }\n  },\n  "SP.Folder.Collection": {\n    properties: ["Files|SP.File.Collection|/getByUrl(\'[Name]\')|SP.File", "Folders|SP.Folder.Collection|/getByUrl(\'[Name]\')|SP.Folder", "ListItemAllFields", "ParentFolder", "StorageMetrics"],\n    add: {\n      argNames: ["url"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    addUsingPath: {\n      argNames: ["DecodedUrl", "EnsureUniqueFileName", "Overwrite"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    addWithOverwrite: {\n      argNames: ["url", "overwrite"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    getByPath: {\n      argNames: ["DecodedUrl"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getByUrl: {\n      argNames: ["url"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.Folder"\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.FolderColoring": {\n    createFolder: {\n      argNames: ["DecodedUrl", "EnsureUniqueFileName", "Overwrite", "coloringInformation"]\n    },\n    renameFolder: {\n      argNames: ["DecodedUrl", "newName", "coloringInformation"]\n    },\n    stampColor: {\n      argNames: ["DecodedUrl", "coloringInformation"]\n    }\n  },\n  "SP.FontPackage": {\n    apply: {},\n    "delete": {},\n    fontStream: {\n      argNames: ["fontFamily"]\n    },\n    update: {}\n  },\n  "SP.FontPackage.Collection": {\n    getById: {\n      argNames: ["id"]\n    },\n    getByTitle: {\n      argNames: ["title"]\n    },\n    query: {\n      argNames: ["oData"]\n    }\n  },\n  "SP.Form.Collection": {\n    getById: {\n      argNames: ["id"]\n    },\n    getByPageType: {\n      argNames: ["formType"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.Group": {\n    properties: ["Users|SP.User.Collection|/getById([Name])|SP.User"],\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    setUserAsOwner: {\n      argNames: ["ownerId"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    update: {\n      metadataType: "SP.Group",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.Group.Collection": {\n    add: {\n      argNames: ["parameters"],\n      metadataType: "SP.Group",\n      name: "",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    getById: {\n      argNames: ["id"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.Group"\n    },\n    getByName: {\n      argNames: ["name"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.Group"\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    removeById: {\n      argNames: ["id"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    removeByLoginName: {\n      argNames: ["loginName"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    }\n  },\n  "SP.HubSite": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    }\n  },\n  "SP.HubSite.Collection": {\n    getById: {\n      argNames: ["hubSiteId"],\n      requestType: utils_1.RequestType.GetWithArgsInQS\n    },\n    getConnectedHubs: {\n      argNames: ["hubSiteId", "option"],\n      requestType: utils_1.RequestType.GetWithArgsInQS\n    },\n    getSiteUrlByHubSiteId: {\n      argNames: ["hubSiteId"],\n      requestType: utils_1.RequestType.GetWithArgsInQS\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.InformationRightsManagementFileSettings": {\n    reset: {},\n    update: {\n      metadataType: "",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.InformationRightsManagementSettings": {\n    reset: {},\n    update: {\n      metadataType: "",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.KnowledgeHub": {\n    addOrUpdateSiteReference: {\n      argNames: ["knowledgeHubSiteUrl"]\n    },\n    siteReference: {}\n  },\n  "SP.LanguageCollection": {\n    getById: {\n      argNames: ["id"]\n    }\n  },\n  "SP.List": {\n    properties: ["BrowserFileHandling", "ContentTypes|SP.ContentType.Collection|(\'[Name]\')|SP.ContentType", "CreatablesInfo", "DefaultDisplayFormUrl", "DefaultEditFormUrl", "DefaultNewFormUrl", "DefaultView|SP.View", "DescriptionResource", "EffectiveBasePermissions", "EventReceivers|SP.EventReceiverDefinition.Collection|(\'[Name]\')|SP.EventReceiverDefinition", "Fields|SP.Field.Collection|/getByInternalNameOrTitle(\'[Name]\')|SP.Field", "FirstUniqueAncestorSecurableObject", "Forms|SP.Form.Collection|(\'[Name]\')|SP.Form", "InformationRightsManagementSettings", "Items|SP.ListItem.Collection|([Name])|SP.ListItem", "ParentWeb", "RoleAssignments|SP.RoleAssignment.Collection|([Name])|SP.RoleAssignment", "RootFolder|SP.Folder|/getByUrl(\'[Name]\')|SP.File", "Subscriptions", "TitleResource", "UserCustomActions|SP.UserCustomAction.Collection|(\'[Name]\')|SP.UserCustomAction", "Views|SP.View.Collection|(\'[Name]\')|SP.View", "WorkflowAssociations"],\n    addItem: {\n      name: "Items",\n      argNames: ["parameters"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    addItemUsingPath: {\n      argNames: ["parameters"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    addValidateUpdateItem: {\n      argNames: ["listItemCreateInfo", "formValues", "bNewDocumentUpdate", "checkInComment", "datesInUTC", "numberInInvariantCulture"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    addValidateUpdateItemUsingPath: {\n      argNames: ["listItemCreateInfo", "formValues", "bNewDocumentUpdate", "checkInComment", "datesInUTC", "numberInInvariantCulture"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    breakRoleInheritance: {\n      argNames: ["copyRoleAssignments", "clearSubscopes"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    bulkValidateUpdateListItems: {\n      argNames: ["itemIds", "formValues", "bNewDocumentUpdate", "checkInComment", "folderPath"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    cAAECreateTempFolder: {},\n    cAAERenameFileInTemporaryFolder: {\n      argNames: ["Path", "UpdatedName"]\n    },\n    cancelDeleteFileVersions: {},\n    clearBusinessAppMigrationInteractiveData: {},\n    copyTemplateAndGetMetadata: {\n      argNames: ["Id"]\n    },\n    createDocumentAndGetEditLink: {\n      argNames: ["fileName", "folderPath", "documentTemplateType", "templateUrl"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    createDocumentFromCAAETemplate: {\n      argNames: ["ContentTypeName", "documentGenerationInfo"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    createDocumentFromCAAETemplateV2: {\n      argNames: ["Id", "documentGenerationInfo"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    createDocumentFromContentAssemblyTemplate: {\n      argNames: ["TemplateUrl", "documentGenerationInfo"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    createDocumentWithDefaultName: {\n      argNames: ["folderPath", "extension"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    createHVCSItemApprovalRequest: {\n      argNames: ["createItemRequestPayload"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    createMappedView: {\n      argNames: ["appViewCreationInfo", "visualizationTarget"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    createRule: {\n      argNames: ["condition", "outcome", "title", "triggerType", "emailField", "actionType", "ruleTemplateId"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    createRuleEx: {\n      argNames: ["condition", "title", "triggerType", "action", "ruleTemplateId"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    createSmartTemplateContentTypeAndAddToList: {\n      argNames: ["Name", "Description"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    createSmartTemplateContentTypeAndAddToListV2: {\n      argNames: ["Name", "Description", "TemplatePath", "Status"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    deleteRule: {\n      argNames: ["ruleId", "triggerType"],\n      requestType: utils_1.RequestType.Delete\n    },\n    enableQueryableColumns: {},\n    enqueueAsyncActionTaskById: {\n      argNames: ["id", "parameters"]\n    },\n    ensureSignoffStatusField: {},\n    getAllRules: {},\n    getAsyncActionConfig: {\n      argNames: ["id"]\n    },\n    getAsyncActionTaskIds: {},\n    getBloomFilter: {\n      argNames: ["startItemId"]\n    },\n    getBloomFilterWithCustomFields: {\n      argNames: ["listItemStartingID", "internalFieldNames"]\n    },\n    getBusinessAppMigrationInteractiveData: {},\n    getBusinessAppOperationStatus: {},\n    getCAAETemplateMetadata: {\n      argNames: ["Name", "Published"]\n    },\n    getCAAETemplateMetadataV2: {\n      argNames: ["Id"]\n    },\n    getChanges: {\n      argNames: ["query"],\n      metadataType: "SP.ChangeQuery",\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    getCheckedOutFiles: {},\n    getContentAssemblyDocumentFieldValues: {\n      argNames: ["DocumentUrl"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getContentAssemblyTemplateFields: {\n      argNames: ["TemplateUrl"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getItemById: {\n      argNames: ["id"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.ListItem"\n    },\n    getItemByStringId: {\n      argNames: ["sId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getItemByUniqueId: {\n      argNames: ["uniqueId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getItems: {\n      argNames: ["viewXML"],\n      requestType: utils_1.RequestType.PostWithArgsInBody,\n      data: {\n        query: {\n          __metadata: {\n            type: "SP.CamlQuery"\n          },\n          ViewXml: "[[viewXML]]"\n        }\n      }\n    },\n    getItemsByQuery: {\n      argNames: ["camlQuery"],\n      name: "getItems",\n      requestType: utils_1.RequestType.PostWithArgsInBody,\n      data: {\n        query: {\n          __metadata: {\n            type: "SP.CamlQuery"\n          },\n          ViewXml: "<View>[[camlQuery]]</View>"\n        }\n      }\n    },\n    getListItemChangesSinceToken: {\n      argNames: ["query"],\n      metadataType: "SP.ChangeLogItemQuery",\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    getLookupFieldChoices: {\n      argNames: ["targetFieldName", "pagingInfo"]\n    },\n    getMappedApp: {\n      argNames: ["appId", "visualizationAppTarget"]\n    },\n    getMappedApps: {\n      argNames: ["visualizationAppTarget"]\n    },\n    getProgressForDeleteFileVersions: {},\n    getProgressForFileVersionExpirationReport: {\n      argNames: ["reportFileUrl"]\n    },\n    getRelatedFields: {},\n    getSpecialFolderUrl: {\n      argNames: ["type", "bForceCreate", "existingFolderGuid"]\n    },\n    getUserEffectivePermissions: {\n      argNames: ["userName"],\n      name: "getUserEffectivePermissions(@user)?@user=\'[[userName]]\'",\n      requestType: utils_1.RequestType.GetReplace\n    },\n    getView: {\n      argNames: ["viewGuid"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.View"\n    },\n    isSyntexAIFeaturesFlightEnabled: {},\n    lockSmartTemplate: {\n      argNames: ["Id"]\n    },\n    mapFieldsToColumnsForModernTemlate: {\n      argNames: ["payload"]\n    },\n    mapFieldsToColumnsForModernTemplate: {\n      argNames: ["templatePayload"]\n    },\n    parseDocumentTemplate: {\n      argNames: ["Name"]\n    },\n    publishMappedView: {\n      argNames: ["appId", "visualizationTarget"]\n    },\n    publishSnippet: {\n      argNames: ["publishSnippetPayload"]\n    },\n    publishTemplateV2: {\n      argNames: ["payload"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    recycle: {\n      requestType: utils_1.RequestType.Post\n    },\n    refreshLockSmartTemplate: {\n      argNames: ["Id", "LockId"]\n    },\n    renderExtendedListFormData: {\n      argNames: ["itemId", "formId", "mode", "options", "cutoffVersion"]\n    },\n    renderListContextMenuData: {\n      argNames: ["CascDelWarnMessage", "CustomAction", "Field", "ID", "InplaceFullListSearch", "InplaceSearchQuery", "IsCSR", "IsXslView", "ItemId", "ListViewPageUrl", "OverrideScope", "RootFolder", "View", "ViewCount"]\n    },\n    renderListData: {\n      argNames: ["viewXml"],\n      name: "renderListData(@v)?@v=\'<View>[[viewXml]]</View>\'",\n      requestType: utils_1.RequestType.PostReplace\n    },\n    renderListDataAsStream: {\n      argNames: ["parameters", "CascDelWarnMessage", "CustomAction", "DrillDown", "Field", "FieldInternalName", "Filter", "FilterData", "FilterData1", "FilterData10", "FilterData2", "FilterData3", "FilterData4", "FilterData5", "FilterData6", "FilterData7", "FilterData8", "FilterData9", "FilterField", "FilterField1", "FilterField10", "FilterField2", "FilterField3", "FilterField4", "FilterField5", "FilterField6", "FilterField7", "FilterField8", "FilterField9", "FilterFields", "FilterFields1", "FilterFields10", "FilterFields2", "FilterFields3", "FilterFields4", "FilterFields5", "FilterFields6", "FilterFields7", "FilterFields8", "FilterFields9", "FilterLookupId", "FilterLookupId1", "FilterLookupId10", "FilterLookupId2", "FilterLookupId3", "FilterLookupId4", "FilterLookupId5", "FilterLookupId6", "FilterLookupId7", "FilterLookupId8", "FilterLookupId9", "FilterOp", "FilterOp1", "FilterOp10", "FilterOp2", "FilterOp3", "FilterOp4", "FilterOp5", "FilterOp6", "FilterOp7", "FilterOp8", "FilterOp9", "FilterValue", "FilterValue1", "FilterValue10", "FilterValue2", "FilterValue3", "FilterValue4", "FilterValue5", "FilterValue6", "FilterValue7", "FilterValue8", "FilterValue9", "FilterValues", "FilterValues1", "FilterValues10", "FilterValues2", "FilterValues3", "FilterValues4", "FilterValues5", "FilterValues6", "FilterValues7", "FilterValues8", "FilterValues9", "GroupString", "HasOverrideSelectCommand", "ID", "InplaceFullListSearch", "InplaceSearchQuery", "IsCSR", "IsGroupRender", "IsXslView", "ListViewPageUrl", "OverrideRowLimit", "OverrideScope", "OverrideSelectCommand", "PageFirstRow", "PageLastRow", "QueryParams", "RootFolder", "RootFolderUniqueId", "SortDir", "SortDir1", "SortDir10", "SortDir2", "SortDir3", "SortDir4", "SortDir5", "SortDir6", "SortDir7", "SortDir8", "SortDir9", "SortField", "SortField1", "SortField10", "SortField2", "SortField3", "SortField4", "SortField5", "SortField6", "SortField7", "SortField8", "SortField9", "SortFields", "SortFieldValues", "View", "ViewCount", "ViewId", "ViewPath", "WebPartId"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    renderListFilterData: {\n      argNames: ["ExcludeFieldFilteringHtml", "FieldInternalName", "OverrideScope", "ProcessQStringToCAML", "ViewId", "ViewXml"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    renderListFormData: {\n      argNames: ["itemId", "formId", "mode"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    reserveListItemId: {\n      requestType: utils_1.RequestType.Post\n    },\n    resetRoleInheritance: {\n      requestType: utils_1.RequestType.Post\n    },\n    saveAsNewView: {\n      argNames: ["oldName", "newName", "privateView", "uri"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    saveAsTemplate: {\n      argNames: ["strFileName", "strName", "strDescription", "bSaveData"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    searchLookupFieldChoices: {\n      argNames: ["targetFieldName", "beginsWithSearchString", "pagingInfo"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    setContentAssemblyTemplateReadOnly: {\n      argNames: ["Id"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    setExemptFromBlockDownloadOfNonViewableFiles: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    setItemsOrder: {\n      argNames: ["itemIds", "lowerOrderItemId", "higherOrderItemId"]\n    },\n    setListCustomOrderFlag: {\n      argNames: ["value"]\n    },\n    startDeleteFileVersions: {\n      argNames: ["deleteOlderThanDays"]\n    },\n    startDeleteFileVersionsByMode: {\n      argNames: ["batchDeleteParameters"]\n    },\n    startFileVersionExpirationReport: {\n      argNames: ["reportFileUrl"]\n    },\n    startRecycle: {},\n    syncFlowCallbackUrl: {\n      argNames: ["flowId"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    syncFlowInstance: {\n      argNames: ["flowID"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    syncFlowInstances: {\n      argNames: ["retrieveGroupFlows"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    syncFlowTemplates: {\n      argNames: ["category"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    unlockSmartTemplate: {\n      argNames: ["Id", "LockId"]\n    },\n    unpublishMappedView: {\n      argNames: ["appId", "visualizationTarget"],\n      requestType: utils_1.RequestType.Post\n    },\n    unsetContentAssemblyTemplateReadOnly: {\n      argNames: ["Id"]\n    },\n    update: {\n      argNames: ["properties"],\n      metadataType: "SP.List",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    updateCAAETemplate: {\n      argNames: ["Name", "updateTemplateInfo"]\n    },\n    updateCAAETemplateV2: {\n      argNames: ["Id", "updateTemplateInfo"]\n    },\n    updateContentAssemblyDocument: {\n      argNames: ["TemplateUrl", "contentAssemblyFormAnswers"]\n    },\n    updateFormProcessingModelRetentionLabel: {\n      argNames: ["retentionLabel"]\n    },\n    updateFormProcessingModelSettings: {\n      argNames: ["retentionLabel", "linkedList"]\n    },\n    updateRule: {\n      argNames: ["ruleId", "condition", "outcome", "title", "emailField", "status", "actionType", "triggerType"]\n    },\n    updateRuleEx: {\n      argNames: ["ruleId", "condition", "title", "status", "action", "triggerType"]\n    },\n    validateAppName: {\n      argNames: ["appDisplayName"]\n    }\n  },\n  "SP.List.Collection": {\n    add: {\n      argNames: ["parameters"],\n      metadataType: "SP.List",\n      name: "",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    ensureClientRenderedSitePagesLibrary: {\n      requestType: utils_1.RequestType.Post\n    },\n    ensureEventsList: {\n      requestType: utils_1.RequestType.Post\n    },\n    ensureSiteAssetsLibrary: {\n      requestType: utils_1.RequestType.Post\n    },\n    ensureSitePagesLibrary: {\n      requestType: utils_1.RequestType.Post\n    },\n    getById: {\n      argNames: ["id"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.List"\n    },\n    getByTitle: {\n      argNames: ["title"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.List"\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.ListItem": {\n    properties: ["AttachmentFiles|SP.Attachment.Collection|(\'[Name]\')|SP.Attachment", "Comments|Microsoft.SharePoint.Comments.comment.Collection|(\'[Name]\')|Microsoft.SharePoint.Comments.comment", "ContentType|SP.ContentType", "FieldValuesAsHtml", "FieldValuesAsText", "FieldValuesForEdit", "File|SP.File", "FirstUniqueAncestorSecurableObject", "Folder|SP.Folder", "GetDlpPolicyTip", "ParentList", "Properties", "RoleAssignments|SP.RoleAssignment.Collection|roleassignments|([Name])|SP.RoleAssignment", "Versions|SP.ListItemVersion.Collection|([Name])|SP.ListItemVersion"],\n    addThumbnailFieldData: {\n      argNames: ["imageStream", "imageName", "fieldInternalName", "lockId"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    archive: {\n      requestType: utils_1.RequestType.Post\n    },\n    attachImage: {\n      argNames: ["imageStream", "imageName", "fieldInternalName"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    breakRoleInheritance: {\n      argNames: ["copyRoleAssignments", "clearSubscopes"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    deleteWithParameters: {\n      argNames: ["parameters"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    getChanges: {\n      argNames: ["query"],\n      metadataType: "SP.ChangeQuery",\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    getComments: {\n      returnType: "Microsoft.SharePoint.Comments.comment.Collection"\n    },\n    getUserEffectivePermissions: {\n      argNames: ["userName"],\n      name: "getUserEffectivePermissions(@user)?@user=\'[[userName]]\'",\n      requestType: utils_1.RequestType.GetReplace\n    },\n    getVersions: {\n      argNames: ["getVersionsParams"]\n    },\n    getWOPIFrameUrl: {\n      argNames: ["action"],\n      requestType: utils_1.RequestType.PostWithArgsInQS\n    },\n    mediaServiceUpdate: {\n      argNames: ["parameters"]\n    },\n    mediaServiceUpdateV2: {\n      argNames: ["parameters", "eventFiringEnabled"]\n    },\n    overridePolicyTip: {\n      argNames: ["userAction", "justification"]\n    },\n    parseAndSetFieldValue: {\n      argNames: ["fieldName", "value"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    recycle: {\n      requestType: utils_1.RequestType.Post\n    },\n    recycleWithParameters: {\n      argNames: ["parameters"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    resetRoleInheritance: {\n      requestType: utils_1.RequestType.Post\n    },\n    setCommentsDisabled: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    setComplianceTag: {\n      argNames: ["complianceTag", "isTagPolicyHold", "isTagPolicyRecord", "isEventBasedTag", "isTagSuperLock", "isUnlockedAsDefault"]\n    },\n    setComplianceTagWithExplicitMetasUpdate: {\n      argNames: ["complianceTag", "complianceFlags", "complianceTagWrittenTime", "userEmailAddress"]\n    },\n    setComplianceTagWithHold: {\n      argNames: ["complianceTag"]\n    },\n    setComplianceTagWithMetaInfo: {\n      argNames: ["complianceTag", "blockDelete", "blockEdit", "complianceTagWrittenTime", "userEmailAddress", "isTagSuperLock", "isRecordUnlockedAsDefault"]\n    },\n    setComplianceTagWithNoHold: {\n      argNames: ["complianceTag"]\n    },\n    setComplianceTagWithRecord: {\n      argNames: ["complianceTag"]\n    },\n    systemUpdate: {},\n    unarchive: {},\n    update: {\n      argNames: ["properties"],\n      inheritMetadataType: true,\n      metadataType: function metadataType(obj) {\n        return obj.parent && obj.parent["ListItemEntityTypeFullName"] || "SP.ListItem";\n      },\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    updateEx: {\n      argNames: ["parameters"]\n    },\n    updateOverwriteVersion: {},\n    validateUpdateFetchListItem: {\n      argNames: ["formValues", "bNewDocumentUpdate", "checkInComment", "datesInUTC", "numberInInvariantCulture", "View", "RootFolder"]\n    },\n    validateUpdateFetchListItemInFolder: {\n      argNames: ["formValues", "bNewDocumentUpdate", "checkInComment", "datesInUTC", "numberInInvariantCulture", "rootFolder"]\n    },\n    validateUpdateListItem: {\n      argNames: ["formValues", "bNewDocumentUpdate", "checkInComment", "datesInUTC", "numberInInvariantCulture", "sharedLockId"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    }\n  },\n  "SP.ListItem.Collection": {\n    add: {\n      argNames: ["parameters"],\n      metadataType: function metadataType(obj) {\n        return obj.parent && obj.parent["ListItemEntityTypeFullName"] || "SP.ListItem";\n      },\n      name: "",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    getById: {\n      argNames: ["itemId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.ListItem"\n    },\n    getByStringId: {\n      argNames: ["sId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.ListItemVersion": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    recycle: {}\n  },\n  "SP.ListItemVersion.Collection": {\n    deleteAll: {\n      requestType: utils_1.RequestType.Post\n    },\n    getById: {\n      argNames: ["versionId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    recycleAll: {},\n    restoreByID: {\n      argNames: ["versionId"]\n    }\n  },\n  "SP.ListTemplate": {\n    getGlobalSchemaXml: {}\n  },\n  "SP.ListTemplate.Collection": {\n    getByName: {\n      argNames: ["name"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.MicroService.MicroServiceManager": {\n    addMicroserviceWorkItem: {\n      argNames: ["payLoad", "minutes", "properties"]\n    },\n    deleteMicroserviceWorkItem: {\n      argNames: ["workItemId"]\n    },\n    deleteMicroserviceWorkItemByContentDbId: {\n      argNames: ["contentDatabaseId", "siteId", "workItemId"]\n    },\n    getServiceInternalUrls: {\n      argNames: ["service"]\n    },\n    getServiceUrls: {\n      argNames: ["service"]\n    }\n  },\n  "SP.Microfeed.MicrofeedAttachmentStore": {\n    deletePreProcessedAttachment: {\n      argNames: ["attachmentUri"]\n    },\n    getImage: {\n      argNames: ["imageUrl", "key", "iv"]\n    },\n    preProcessAttachment: {\n      argNames: ["link"]\n    },\n    putFile: {\n      argNames: ["originalFileName", "fileData"]\n    },\n    putImage: {\n      argNames: ["imageData"]\n    }\n  },\n  "SP.Microfeed.MicrofeedData": {\n    addAttachment: {\n      argNames: ["name", "bytes"]\n    },\n    systemUpdate: {},\n    update: {\n      metadataType: "",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.Microfeed.MicrofeedData.Collection": {\n    deleteAll: {},\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.Microfeed.MicrofeedManager": {\n    addUserToPostPeopleList: {\n      argNames: ["postIdentifier", "UserLoginName"]\n    },\n    clearUnreadMentionsCount: {},\n    deleteById: {\n      argNames: ["postIdentifier"]\n    },\n    deleteUserFromPostPeopleList: {\n      argNames: ["postIdentifier", "UserLoginName"]\n    },\n    getMyCategoricalFeed: {\n      argNames: ["feedOptions"]\n    },\n    getMyConsolidatedFeed: {\n      argNames: ["feedOptions"]\n    },\n    getMyPublishedFeed: {\n      argNames: ["feedOptions", "typeOfPubFeed", "ShowPublicView"]\n    },\n    getPublishedFeed: {\n      argNames: ["feedOwner", "feedOptions", "typeOfPubFeed"]\n    },\n    getThread: {\n      argNames: ["postIdentifier"]\n    },\n    getUnreadMentionsCount: {},\n    like: {\n      argNames: ["postIdentifier"]\n    },\n    lockThreadById: {\n      argNames: ["threadIdentifier"]\n    },\n    post: {\n      argNames: ["postOptions"]\n    },\n    postReply: {\n      argNames: ["postIdentifier", "postReplyOptions"]\n    },\n    repopulateLMT: {\n      argNames: ["timeStamp", "secureHash"]\n    },\n    unLike: {\n      argNames: ["postIdentifier"]\n    },\n    unLockThreadById: {\n      argNames: ["threadIdentifier"]\n    },\n    unsubscribeFromEMail: {\n      argNames: ["postIdentifier"]\n    }\n  },\n  "SP.Microfeed.MicrofeedPostDefinitionManager": {\n    deleteMicrofeedPostDefinition: {\n      argNames: ["postDefinition"]\n    },\n    getMicrofeedPostDefinition: {\n      argNames: ["definitionName"]\n    },\n    getMicrofeedPostDefinitions: {},\n    newMicrofeedPostDefinition: {\n      argNames: ["definitionName"]\n    },\n    updateMicrofeedPostDefinition: {\n      argNames: ["postDefinition"]\n    }\n  },\n  "SP.Microfeed.MicrofeedStore": {\n    addData: {\n      argNames: ["name", "data"]\n    },\n    addDataAsStream: {\n      argNames: ["name", "data"]\n    },\n    executePendingOperations: {},\n    getItem: {\n      argNames: ["storeIdentifier"]\n    },\n    getSocialProperties: {\n      argNames: ["accountName"]\n    },\n    incrementUnreadAtMentionCount: {\n      argNames: ["accountName"]\n    },\n    newItem: {\n      argNames: ["storeIdentifier"]\n    },\n    query: {\n      argNames: ["storeIdentifier", "query"]\n    },\n    setPostLikeStatus: {\n      argNames: ["accountName", "postId", "like"]\n    }\n  },\n  "SP.MultilingualSettings": {\n    query: {\n      argNames: ["oData"]\n    },\n    setFollowLangPreference: {\n      argNames: ["request"]\n    },\n    setNotificationRecipients: {\n      argNames: ["request"]\n    }\n  },\n  "SP.Navigation": {\n    properties: ["QuickLaunch|SP.NavigationNode.Collection|/../getNodeById([Name])|SP.NavigationNode", "TopNavigationBar|SP.NavigationNode.Collection|/../getNodeById([Name])|SP.NavigationNode"],\n    getNodeById: {\n      argNames: ["id"],\n      returnType: "SP.NavigationNode"\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.NavigationNode": {\n    properties: ["Children|SP.NavigationNode.Collection|/../getNodeById([Name])|SP.NavigationNode"],\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    update: {\n      metadataType: "SP.NavigationNode",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.NavigationNode.Collection": {\n    add: {\n      argNames: ["properties"],\n      metadataType: "SP.NavigationNode",\n      name: "",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    getById: {\n      argNames: ["id"]\n    },\n    getByIndex: {\n      argNames: ["index"]\n    },\n    moveAfter: {\n      argNames: ["nodeId", "previousNodeId"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.OAuth.Authentication": {\n    getRenewalUrl: {\n      argNames: ["redirectUrl"]\n    }\n  },\n  "SP.OAuth.NativeClient": {\n    authenticate: {}\n  },\n  "SP.OAuth.Token": {\n    acquire: {\n      argNames: ["resource", "tokenType"]\n    }\n  },\n  "SP.ObjectSharingInformation": {\n    getSharedWithUsers: {},\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.OrganizationNews": {\n    sitesReference: {}\n  },\n  "SP.PropertyValues.Collection": {\n    add: {\n      argNames: ["key", "value"],\n      name: "_vti_bin/client.svc/ProcessQuery",\n      replaceEndpointFl: true,\n      requestType: utils_1.RequestType.PostReplace,\n      data: "<Request xmlns=\\"http://schemas.microsoft.com/sharepoint/clientquery/2009\\" SchemaVersion=\\"15.0.0.0\\" LibraryVersion=\\"16.0.0.0\\" ApplicationName=\\"Javascript Library\\">\\n                <Actions>\\n                    <ObjectPath Id=\\"1\\" ObjectPathId=\\"0\\" />\\n                    <ObjectPath Id=\\"3\\" ObjectPathId=\\"2\\" />\\n                    <ObjectPath Id=\\"5\\" ObjectPathId=\\"4\\" />\\n                    <Method Name=\\"SetFieldValue\\" Id=\\"6\\" ObjectPathId=\\"4\\">\\n                    <Parameters>\\n                        <Parameter Type=\\"String\\">[[key]]</Parameter>\\n                        <Parameter Type=\\"String\\">[[value]]</Parameter>\\n                    </Parameters>\\n                    </Method>\\n                    <Query Id=\\"7\\" ObjectPathId=\\"2\\">\\n                        <Query SelectAllProperties=\\"true\\">\\n                            <Properties />\\n                        </Query>\\n                    </Query>\\n                    <Method Name=\\"Update\\" Id=\\"8\\" ObjectPathId=\\"2\\" />\\n                </Actions>\\n                <ObjectPaths>\\n                    <StaticProperty Id=\\"0\\" TypeId=\\"{3747adcd-a3c3-41b9-bfab-4a64dd2f1e0a}\\" Name=\\"Current\\" />\\n                    <Property Id=\\"2\\" ParentId=\\"0\\" Name=\\"Web\\" />\\n                    <Property Id=\\"4\\" ParentId=\\"2\\" Name=\\"AllProperties\\" />\\n                </ObjectPaths>\\n            </Request>"\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    remove: {\n      argNames: ["key"],\n      name: "_vti_bin/client.svc/ProcessQuery",\n      replaceEndpointFl: true,\n      requestType: utils_1.RequestType.PostReplace,\n      data: "<Request xmlns=\\"http://schemas.microsoft.com/sharepoint/clientquery/2009\\" SchemaVersion=\\"15.0.0.0\\" LibraryVersion=\\"16.0.0.0\\" ApplicationName=\\"Javascript Library\\">\\n                <Actions>\\n                    <ObjectPath Id=\\"1\\" ObjectPathId=\\"0\\" />\\n                    <ObjectPath Id=\\"3\\" ObjectPathId=\\"2\\" />\\n                    <ObjectPath Id=\\"5\\" ObjectPathId=\\"4\\" />\\n                    <Method Name=\\"SetFieldValue\\" Id=\\"6\\" ObjectPathId=\\"4\\">\\n                    <Parameters>\\n                        <Parameter Type=\\"String\\">[[key]]</Parameter>\\n                        <Parameter Type=\\"Null\\" />\\n                    </Parameters>\\n                    </Method>\\n                    <Query Id=\\"7\\" ObjectPathId=\\"2\\">\\n                        <Query SelectAllProperties=\\"true\\">\\n                            <Properties />\\n                        </Query>\\n                    </Query>\\n                    <Method Name=\\"Update\\" Id=\\"8\\" ObjectPathId=\\"2\\" />\\n                </Actions>\\n                <ObjectPaths>\\n                    <StaticProperty Id=\\"0\\" TypeId=\\"{3747adcd-a3c3-41b9-bfab-4a64dd2f1e0a}\\" Name=\\"Current\\" />\\n                    <Property Id=\\"2\\" ParentId=\\"0\\" Name=\\"Web\\" />\\n                    <Property Id=\\"4\\" ParentId=\\"2\\" Name=\\"AllProperties\\" />\\n                </ObjectPaths>\\n            </Request>"\n    }\n  },\n  "SP.Publishing.AnnouncementsController": {\n    active: {},\n    channel: {},\n    flwPropertyFilteringMapping: {\n      argNames: ["flwPropertyFilteringMapping"]\n    }\n  },\n  "SP.Publishing.CampaignPublication": {\n    approveApprovalRequest: {\n      argNames: ["comment"]\n    },\n    boostNews: {\n      argNames: ["SitePageBoost"]\n    },\n    cancelApprovalRequest: {},\n    cancelSchedulePublication: {},\n    checkOut: {},\n    checkoutPage: {},\n    copy: {},\n    copyTo: {\n      argNames: ["copyToParams"]\n    },\n    copyWithConfiguration: {\n      argNames: ["sitePageFlags", "isNews", "creationMode"]\n    },\n    createApprovalRequest: {\n      argNames: ["creationInfo"]\n    },\n    createNewsCopy: {},\n    "delete": {},\n    deletePublication: {},\n    demoteFromNews: {},\n    discardPage: {},\n    extendSessionCoAuth: {},\n    getDependencyMetadata: {},\n    getHighlightsInfo: {},\n    getLatestVersionsInDescendingOrder: {\n      argNames: ["numVersions"]\n    },\n    getPrePublishValidationStatus: {},\n    getPublishingStatus: {},\n    getVersion: {\n      argNames: ["versionId"]\n    },\n    loadMailDraft: {\n      argNames: ["requestParam"]\n    },\n    move: {\n      argNames: ["pageMoveParams"]\n    },\n    promoteToNews: {},\n    publish: {},\n    publishCoAuth: {\n      argNames: ["pageStream"]\n    },\n    publishPublication: {},\n    rejectApprovalRequest: {\n      argNames: ["comment"]\n    },\n    resetEndpoint: {\n      argNames: ["requestParam"]\n    },\n    sPSiteValidator: {\n      argNames: ["siteUrl"]\n    },\n    saveDraft: {\n      argNames: ["sitePage"]\n    },\n    saveMailDraft: {\n      argNames: ["requestParam"]\n    },\n    savePage: {\n      argNames: ["pageStream"]\n    },\n    savePageAsDraft: {\n      argNames: ["pageStream"]\n    },\n    savePageAsTemplate: {},\n    savePageAsTemplate2: {\n      argNames: ["creationMode"]\n    },\n    savePageAsTemplateOnMySite: {},\n    savePageCoAuth: {\n      argNames: ["pageStream"]\n    },\n    saveStreams: {\n      argNames: ["contentStream"]\n    },\n    schedulePublication: {\n      argNames: ["publishStartDate"]\n    },\n    schedulePublish: {\n      argNames: ["sitePage"]\n    },\n    sendTestEmail: {\n      argNames: ["transpileContent"]\n    },\n    sendTestTeamsMessage: {\n      argNames: ["audienceId", "transpileContent"]\n    },\n    sharePagePreviewByEmail: {\n      argNames: ["message", "recipientEmails"]\n    },\n    startCoAuth: {\n      argNames: ["paramsStream"]\n    },\n    startExclusiveAuthoring: {},\n    syncApprovalRequest: {},\n    tryProcessSourcePageAfterPageMovePublish: {},\n    tryProcessSourcePageBeforePageMovePublish: {},\n    update: {},\n    updateSharePointPublishingStatus: {\n      argNames: ["destinationSiteId", "destinationUrl", "sharePointPublishingStatus", "sharePointPublishingErrorCode"]\n    }\n  },\n  "SP.Publishing.CommunicationSite": {\n    create: {\n      argNames: ["request"]\n    },\n    enable: {\n      argNames: ["designPackageId"]\n    },\n    status: {\n      argNames: ["url"]\n    }\n  },\n  "SP.Publishing.EmbedService": {\n    embedData: {\n      argNames: ["url", "version"]\n    }\n  },\n  "SP.Publishing.FeedVideoPage": {\n    boostNews: {\n      argNames: ["SitePageBoost"]\n    },\n    checkOut: {},\n    checkoutPage: {},\n    copy: {\n      requestType: utils_1.RequestType.Post\n    },\n    copyTo: {\n      argNames: ["copyToParams"]\n    },\n    copyWithConfiguration: {\n      argNames: ["sitePageFlags", "isNews", "creationMode"],\n      requestType: utils_1.RequestType.Post\n    },\n    createNewsCopy: {},\n    demoteFromNews: {\n      requestType: utils_1.RequestType.Post\n    },\n    discardPage: {},\n    extendSessionCoAuth: {},\n    getDependencyMetadata: {},\n    getHighlightsInfo: {},\n    getLatestVersionsInDescendingOrder: {\n      argNames: ["numVersions"]\n    },\n    getVersion: {\n      argNames: ["versionId"]\n    },\n    move: {\n      argNames: ["pageMoveParams"]\n    },\n    promoteToNews: {\n      requestType: utils_1.RequestType.Post\n    },\n    publish: {},\n    publishCoAuth: {\n      argNames: ["pageStream"]\n    },\n    saveDraft: {\n      argNames: ["sitePage"]\n    },\n    savePage: {\n      argNames: ["pageStream"]\n    },\n    savePageAsDraft: {\n      argNames: ["pageStream"]\n    },\n    savePageAsTemplate: {},\n    savePageAsTemplate2: {\n      argNames: ["creationMode"]\n    },\n    savePageAsTemplateOnMySite: {},\n    savePageCoAuth: {\n      argNames: ["pageStream"]\n    },\n    saveStreams: {\n      argNames: ["contentStream"]\n    },\n    schedulePublish: {\n      argNames: ["sitePage"]\n    },\n    sharePagePreviewByEmail: {\n      argNames: ["message", "recipientEmails"]\n    },\n    startCoAuth: {\n      argNames: ["paramsStream"]\n    },\n    startExclusiveAuthoring: {},\n    tryProcessSourcePageAfterPageMovePublish: {},\n    tryProcessSourcePageBeforePageMovePublish: {},\n    update: {}\n  },\n  "SP.Publishing.FeedVideoPage.Collection": {\n    isContentTypeAvailable: {},\n    query: {\n      argNames: ["oData"]\n    }\n  },\n  "SP.Publishing.Navigation.PortalNavigationCacheWrapper": {\n    disable: {},\n    enable: {},\n    refresh: {}\n  },\n  "SP.Publishing.Navigation.StructuralNavigationCacheWrapper": {\n    setSiteState: {\n      argNames: ["state"]\n    },\n    setWebState: {\n      argNames: ["state"]\n    },\n    siteState: {},\n    webState: {}\n  },\n  "SP.Publishing.PageDiagnosticsController": {\n    byPage: {\n      argNames: ["pageRelativeFilePath"]\n    },\n    save: {\n      argNames: ["pageDiagnosticsResult"]\n    }\n  },\n  "SP.Publishing.PointPublishingPost": {\n    addImageFromUrl: {\n      argNames: ["fromImageUrl"]\n    },\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.Publishing.PointPublishingPost.Collection": {\n    getById: {\n      argNames: ["id", "publishedOnly"]\n    },\n    getByName: {\n      argNames: ["name", "publishedOnly"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.Publishing.PointPublishingPostServiceManager": {\n    addBannerImageFromUrl: {\n      argNames: ["fromImageUrl"]\n    },\n    deleteMagazine: {},\n    getDocProps: {\n      argNames: ["docUrls"]\n    },\n    getPostsQuery: {\n      argNames: ["top", "itemIdBoundary", "directionAscending", "publishedOnly", "draftsOnly"]\n    },\n    getTopAuthors: {\n      argNames: ["count"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    queryGroupNames: {\n      argNames: ["query"]\n    },\n    setMagazineProperties: {\n      argNames: ["title", "description", "bannerImageUrl", "bannerColor", "bannerPattern"]\n    }\n  },\n  "SP.Publishing.PointPublishingSiteManager": {\n    create: {\n      argNames: ["siteInfo"]\n    },\n    getSiteStatus: {\n      argNames: ["siteInfo"]\n    }\n  },\n  "SP.Publishing.PointPublishingTenantManager": {\n    isBlogEnabled: {}\n  },\n  "SP.Publishing.PointPublishingUser": {\n    deleteUserFromContainerGroup: {}\n  },\n  "SP.Publishing.PointPublishingUser.Collection": {\n    addOrUpdateUser: {\n      argNames: ["loginName", "isOwner"]\n    },\n    getById: {\n      argNames: ["userId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.Publishing.PortalLaunch.PortalLaunchWavesManager": {\n    get: {\n      argNames: ["siteUrl"]\n    },\n    getAll: {},\n    remove: {\n      argNames: ["siteUrl"]\n    },\n    saveWaveSetup: {\n      argNames: ["portalLaunchSetup"]\n    }\n  },\n  "SP.Publishing.RepostPage": {\n    boostNews: {\n      argNames: ["SitePageBoost"]\n    },\n    checkOut: {},\n    checkoutPage: {},\n    copy: {\n      requestType: utils_1.RequestType.Post\n    },\n    copyTo: {\n      argNames: ["copyToParams"]\n    },\n    copyWithConfiguration: {\n      argNames: ["sitePageFlags", "isNews", "creationMode"],\n      requestType: utils_1.RequestType.Post\n    },\n    createNewsCopy: {},\n    demoteFromNews: {\n      requestType: utils_1.RequestType.Post\n    },\n    discardPage: {},\n    extendSessionCoAuth: {},\n    getDependencyMetadata: {},\n    getHighlightsInfo: {},\n    getLatestVersionsInDescendingOrder: {\n      argNames: ["numVersions"]\n    },\n    getVersion: {\n      argNames: ["versionId"]\n    },\n    move: {\n      argNames: ["pageMoveParams"]\n    },\n    promoteToNews: {\n      requestType: utils_1.RequestType.Post\n    },\n    publish: {},\n    publishCoAuth: {\n      argNames: ["pageStream"]\n    },\n    saveDraft: {\n      argNames: ["sitePage"]\n    },\n    savePage: {\n      argNames: ["pageStream"]\n    },\n    savePageAsDraft: {\n      argNames: ["pageStream"]\n    },\n    savePageAsTemplate: {},\n    savePageAsTemplate2: {\n      argNames: ["creationMode"]\n    },\n    savePageAsTemplateOnMySite: {},\n    savePageCoAuth: {\n      argNames: ["pageStream"]\n    },\n    saveStreams: {\n      argNames: ["contentStream"]\n    },\n    schedulePublish: {\n      argNames: ["sitePage"]\n    },\n    sharePagePreviewByEmail: {\n      argNames: ["message", "recipientEmails"]\n    },\n    startCoAuth: {\n      argNames: ["paramsStream"]\n    },\n    startExclusiveAuthoring: {},\n    tryProcessSourcePageAfterPageMovePublish: {},\n    tryProcessSourcePageBeforePageMovePublish: {},\n    update: {\n      metadataType: "",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.Publishing.RepostPage.Collection": {\n    isContentTypeAvailable: {},\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.Publishing.RichSharing": {\n    sharePageByEmail: {\n      argNames: ["url", "message", "recipientEmails", "pageContent", "subject", "ccEmails", "bccEmails"]\n    },\n    shareSiteByEmail: {\n      argNames: ["CustomDescription", "CustomTitle", "Message", "Url", "recipientEmails"]\n    }\n  },\n  "SP.Publishing.Search": {\n    newest: {\n      argNames: ["startItemIndex", "itemLimit"]\n    },\n    popular: {\n      argNames: ["startItemIndex", "itemLimit"]\n    },\n    query: {\n      argNames: ["queryText", "startItemIndex", "itemLimit", "culture"]\n    },\n    queryChannels: {\n      argNames: ["queryText", "startItemIndex", "itemLimit", "culture"]\n    },\n    related: {\n      argNames: ["videoId", "startItemIndex", "itemLimit"]\n    }\n  },\n  "SP.Publishing.SharePointHomeServiceManager": {\n    getAcronymsAndColors: {\n      argNames: ["labels"]\n    }\n  },\n  "SP.Publishing.SitePage": {\n    boostNews: {\n      argNames: ["SitePageBoost"]\n    },\n    checkOut: {\n      requestType: utils_1.RequestType.Post\n    },\n    checkoutPage: {\n      requestType: utils_1.RequestType.Post\n    },\n    copy: {\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    copyTo: {\n      argNames: ["copyToParams"]\n    },\n    copyWithConfiguration: {\n      argNames: ["sitePageFlags", "isNews", "creationMode"],\n      requestType: utils_1.RequestType.Post\n    },\n    createNewsCopy: {\n      requestType: utils_1.RequestType.Post\n    },\n    demoteFromNews: {\n      requestType: utils_1.RequestType.Post\n    },\n    discardPage: {\n      requestType: utils_1.RequestType.Post\n    },\n    extendSessionCoAuth: {},\n    getDependencyMetadata: {},\n    getHighlightsInfo: {},\n    getLatestVersionsInDescendingOrder: {\n      argNames: ["numVersions"]\n    },\n    getVersion: {\n      argNames: ["versionId"]\n    },\n    move: {\n      argNames: ["pageMoveParams"]\n    },\n    promoteToNews: {\n      requestType: utils_1.RequestType.Post\n    },\n    publish: {\n      requestType: utils_1.RequestType.Post\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    publishCoAuth: {\n      argNames: ["pageStream"]\n    },\n    saveDraft: {\n      argNames: ["sitePage"],\n      requestType: utils_1.RequestType.Post\n    },\n    savePage: {\n      argNames: ["pageStream"],\n      requestType: utils_1.RequestType.Post\n    },\n    savePageAsDraft: {\n      argNames: ["pageStream"],\n      requestType: utils_1.RequestType.Post\n    },\n    savePageAsTemplate: {\n      requestType: utils_1.RequestType.Post\n    },\n    savePageAsTemplate2: {\n      argNames: ["creationMode"]\n    },\n    savePageAsTemplateOnMySite: {},\n    savePageCoAuth: {\n      argNames: ["pageStream"]\n    },\n    saveStreams: {\n      argNames: ["contentStream"]\n    },\n    schedulePublish: {\n      argNames: ["sitePage"],\n      requestType: utils_1.RequestType.Post\n    },\n    sharePagePreviewByEmail: {\n      argNames: ["message", "recipientEmails"],\n      requestType: utils_1.RequestType.Post\n    },\n    startCoAuth: {\n      argNames: ["paramsStream"],\n      requestType: utils_1.RequestType.Post\n    },\n    startExclusiveAuthoring: {},\n    tryProcessSourcePageAfterPageMovePublish: {},\n    tryProcessSourcePageBeforePageMovePublish: {},\n    update: {\n      argNames: ["properties"],\n      metadataType: "",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.Publishing.SitePage.Collection": {\n    copyToStatus: {\n      argNames: ["workItemId"]\n    },\n    createAppPage: {\n      argNames: ["webPartDataAsJson"],\n      metadataType: "SP.Publishing.SitePage",\n      name: "",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    ensureTitleResource: {},\n    feed: {\n      argNames: ["promotedState", "published", "metadataFilter", "languageOverride"]\n    },\n    feedTargeted: {\n      argNames: ["promotedState", "published", "metadataFilter", "languageOverride"]\n    },\n    getById: {\n      argNames: ["id"]\n    },\n    getByUniqueId: {\n      argNames: ["uniqueId"]\n    },\n    getByUrl: {\n      argNames: ["url"]\n    },\n    getPageColumnState: {\n      argNames: ["url"]\n    },\n    getTranslations: {\n      argNames: ["sourceItemId"]\n    },\n    isSitePage: {\n      argNames: ["url"]\n    },\n    pageMoveStatus: {\n      argNames: ["workItemId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    setMultilingual: {\n      argNames: ["enabled"]\n    },\n    setScheduling: {\n      argNames: ["enabled"]\n    },\n    templates: {},\n    updateAppPage: {\n      argNames: ["pageId", "webPartDataAsJson", "title", "includeInNavigation"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    updateFullPageApp: {\n      argNames: ["serverRelativeUrl", "webPartDataAsJson"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    }\n  },\n  "SP.Publishing.SitePage3D": {\n    boostNews: {\n      argNames: ["SitePageBoost"]\n    },\n    checkOut: {},\n    checkoutPage: {},\n    copy: {\n      requestType: utils_1.RequestType.Post\n    },\n    copyTo: {\n      argNames: ["copyToParams"]\n    },\n    copyWithConfiguration: {\n      argNames: ["sitePageFlags", "isNews", "creationMode"]\n    },\n    createNewsCopy: {},\n    demoteFromNews: {\n      requestType: utils_1.RequestType.Post\n    },\n    discardPage: {},\n    extendSessionCoAuth: {},\n    getDependencyMetadata: {},\n    getHighlightsInfo: {},\n    getLatestVersionsInDescendingOrder: {\n      argNames: ["numVersions"]\n    },\n    getVersion: {\n      argNames: ["versionId"]\n    },\n    move: {\n      argNames: ["pageMoveParams"]\n    },\n    promoteToNews: {\n      requestType: utils_1.RequestType.Post\n    },\n    publish: {},\n    publishCoAuth: {\n      argNames: ["pageStream"]\n    },\n    saveDraft: {\n      argNames: ["sitePage"]\n    },\n    savePage: {\n      argNames: ["pageStream"]\n    },\n    savePageAsDraft: {\n      argNames: ["pageStream"]\n    },\n    savePageAsTemplate: {},\n    savePageAsTemplate2: {\n      argNames: ["creationMode"]\n    },\n    savePageAsTemplateOnMySite: {},\n    savePageCoAuth: {\n      argNames: ["pageStream"]\n    },\n    saveStreams: {\n      argNames: ["contentStream"]\n    },\n    schedulePublish: {\n      argNames: ["sitePage"]\n    },\n    sharePagePreviewByEmail: {\n      argNames: ["message", "recipientEmails"]\n    },\n    startCoAuth: {\n      argNames: ["paramsStream"]\n    },\n    startExclusiveAuthoring: {},\n    tryProcessSourcePageAfterPageMovePublish: {},\n    tryProcessSourcePageBeforePageMovePublish: {},\n    update: {}\n  },\n  "SP.Publishing.SitePage3D.Collection": {\n    activate: {},\n    query: {\n      argNames: ["oData"]\n    }\n  },\n  "SP.Publishing.SitePageMetadata.Collection": {\n    getById: {\n      argNames: ["id"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.Publishing.SitePageService": {\n    properties: ["CommunicationSite|SP.Publishing.CommunicationSite", "Pages|SP.Publishing.SitePage.Collection|(\'[Name]\')|SP.Publishing.SitePage"],\n    addImage: {\n      argNames: ["pageName", "imageFileName", "imageStream", "pageId"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    addImageFromExternalUrl: {\n      argNames: ["pageName", "imageFileName", "externalUrl", "subFolderName", "pageId"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    canCreatePage: {},\n    canCreatePromotedPage: {},\n    enableAnnouncements: {},\n    enableCategories: {},\n    pagesInLib: {\n      argNames: ["id"]\n    },\n    pagesInLibByName: {\n      argNames: ["name"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    update: {\n      metadataType: "",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.Publishing.SpotlightChannel": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    update: {\n      metadataType: "",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.Publishing.SpotlightChannel.Collection": {\n    getById: {\n      argNames: ["id"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.Publishing.SpotlightVideo": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    update: {\n      metadataType: "",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.Publishing.SpotlightVideo.Collection": {\n    getById: {\n      argNames: ["id"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.Publishing.SubtitleFile.Collection": {\n    add: {\n      argNames: ["language", "extension", "stream"]\n    },\n    getSubtitleFile: {\n      argNames: ["name"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    remove: {\n      argNames: ["name"]\n    }\n  },\n  "SP.Publishing.TopicSitePage": {\n    boostNews: {\n      argNames: ["SitePageBoost"]\n    },\n    checkOut: {},\n    checkoutPage: {},\n    copy: {\n      requestType: utils_1.RequestType.Post\n    },\n    copyTo: {\n      argNames: ["copyToParams"]\n    },\n    copyWithConfiguration: {\n      argNames: ["sitePageFlags", "isNews", "creationMode"]\n    },\n    createNewsCopy: {},\n    demoteFromNews: {\n      requestType: utils_1.RequestType.Post\n    },\n    discardPage: {},\n    extendSessionCoAuth: {},\n    getDependencyMetadata: {},\n    getHighlightsInfo: {},\n    getLatestVersionsInDescendingOrder: {\n      argNames: ["numVersions"]\n    },\n    getVersion: {\n      argNames: ["versionId"]\n    },\n    move: {\n      argNames: ["pageMoveParams"]\n    },\n    promoteToNews: {\n      requestType: utils_1.RequestType.Post\n    },\n    publish: {},\n    publishCoAuth: {\n      argNames: ["pageStream"]\n    },\n    saveDraft: {\n      argNames: ["sitePage"]\n    },\n    savePage: {\n      argNames: ["pageStream"]\n    },\n    savePageAsDraft: {\n      argNames: ["pageStream"]\n    },\n    savePageAsTemplate: {},\n    savePageAsTemplate2: {\n      argNames: ["creationMode"]\n    },\n    savePageAsTemplateOnMySite: {},\n    savePageCoAuth: {\n      argNames: ["pageStream"]\n    },\n    saveStreams: {\n      argNames: ["contentStream"]\n    },\n    schedulePublish: {\n      argNames: ["sitePage"]\n    },\n    sharePagePreviewByEmail: {\n      argNames: ["message", "recipientEmails"]\n    },\n    startCoAuth: {\n      argNames: ["paramsStream"]\n    },\n    startExclusiveAuthoring: {},\n    tryProcessSourcePageAfterPageMovePublish: {},\n    tryProcessSourcePageBeforePageMovePublish: {},\n    update: {}\n  },\n  "SP.Publishing.TopicSitePage.Collection": {\n    getByEntityId: {\n      argNames: ["entityId"]\n    },\n    getPagesByEntityId: {\n      argNames: ["entityId"]\n    },\n    getByEntityIdAndCulture: {\n      argNames: ["id", "culture"]\n    },\n    isContentTypeAvailable: {},\n    query: {\n      argNames: ["oData"]\n    }\n  },\n  "SP.Publishing.VideoChannel": {\n    getAllVideos: {\n      argNames: ["skip", "limit"]\n    },\n    getChannelPageUrl: {\n      argNames: ["viewMode"]\n    },\n    getMyVideos: {\n      argNames: ["skip", "limit"]\n    },\n    getPermissionGroup: {\n      argNames: ["permission"]\n    },\n    getVideoCount: {},\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    update: {\n      metadataType: "",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.Publishing.VideoChannel.Collection": {\n    getById: {\n      argNames: ["id"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.Publishing.VideoItem": {\n    customThumbnail: {},\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    getFile: {},\n    getPlaybackMetadata: {\n      argNames: ["sdnConfiguration"]\n    },\n    getPlaybackUrl: {\n      argNames: ["videoFormat"]\n    },\n    getStreamingKeyAccessToken: {},\n    getVideoDetailedViewCount: {},\n    getVideoEmbedCode: {\n      argNames: ["width", "height", "autoplay", "showInfo", "makeResponsive"]\n    },\n    getVideoViewProgressCount: {},\n    incrementVideoViewProgressCount: {\n      argNames: ["percentageViewed"]\n    },\n    incrementViewCount: {\n      argNames: ["viewOrigin"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    setPeopleInMedia: {\n      argNames: ["loginNames"]\n    },\n    setVideoOwner: {\n      argNames: ["id"]\n    },\n    subtitles: {},\n    thumbnailStream: {\n      argNames: ["preferredWidth"]\n    },\n    thumbnails: {\n      argNames: ["preferredWidth"]\n    },\n    update: {\n      metadataType: "",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    uploadCustomThumbnail: {\n      argNames: ["fileExtension", "customVideoThumbnail"]\n    }\n  },\n  "SP.Publishing.VideoItem.Collection": {\n    getById: {\n      argNames: ["id"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.Publishing.VideoPermissionGroup": {\n    hasCurrentUser: {},\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.Publishing.VideoServiceManager": {\n    getChannels: {\n      argNames: ["startIndex", "limit"]\n    },\n    getPermissionGroup: {\n      argNames: ["permission"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.Publishing.VideoThumbnail.Collection": {\n    getByIndex: {\n      argNames: ["choice"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.PushNotificationSubscriber": {\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    update: {\n      metadataType: "",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.PushNotificationSubscriber.Collection": {\n    getByStoreId: {\n      argNames: ["id"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.RecycleBinItem": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    moveToSecondStage: {},\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    restore: {}\n  },\n  "SP.RecycleBinItem.Collection": {\n    deleteAll: {},\n    deleteAllSecondStageItems: {},\n    deleteByIds: {\n      argNames: ["ids"]\n    },\n    getById: {\n      argNames: ["id"]\n    },\n    moveAllToSecondStage: {},\n    moveToSecondStageByIds: {\n      argNames: ["ids"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    restoreAll: {},\n    restoreByIds: {\n      argNames: ["ids", "bRenameExistingItems"]\n    }\n  },\n  "SP.RegionalSettings": {\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    update: {\n      metadataType: "",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.RelatedField.Collection": {\n    getByFieldId: {\n      argNames: ["fieldId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.RemoteWeb": {\n    getFileByServerRelativePath: {\n      argNames: ["serverRelatvieFilePath"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.File"\n    },\n    getFileByServerRelativeUrl: {\n      argNames: ["serverRelativeFileUrl"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.File"\n    },\n    getFileByUrl: {\n      argNames: ["fileUrl"],\n      name: "getFileByUrl(@url)?@url=\'[[fileUrl]]\'",\n      requestType: utils_1.RequestType.GetReplace,\n      returnType: "SP.File"\n    },\n    getFolderByServerRelativeUrl: {\n      argNames: ["serverRelativeUrl"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.Folder"\n    },\n    getGroupById: {\n      argNames: ["groupId"]\n    },\n    getListById: {\n      argNames: ["listGuid"]\n    },\n    getListByServerRelativeUrl: {\n      argNames: ["serverRelativeUrl"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.RequestContext": {\n    getRemoteContext: {},\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.RoleAssignment": {\n    properties: ["Member", "RoleDefinitionBindings|SP.RoleDefinition.Collection"],\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    update: {\n      argNames: ["properties"],\n      metadataType: "SP.RoleAssignment",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.RoleAssignment.Collection": {\n    addRoleAssignment: {\n      argNames: ["principalId", "roleDefId"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    getByPrincipalId: {\n      argNames: ["principalId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.RoleAssignment"\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    removeRoleAssignment: {\n      argNames: ["principalId", "roleDefId"],\n      requestType: utils_1.RequestType.PostWithArgs\n    }\n  },\n  "SP.RoleDefinition": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    update: {\n      argNames: ["properties"],\n      metadataType: "SP.RoleDefinition",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.RoleDefinition.Collection": {\n    add: {\n      argNames: ["properties"],\n      metadataType: "SP.RoleDefinition",\n      name: "",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    getById: {\n      argNames: ["id"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.RoleDefinition"\n    },\n    getByName: {\n      argNames: ["name"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.RoleDefinition"\n    },\n    getByType: {\n      argNames: ["roleType"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.RoleDefinition"\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    recreateMissingDefaultRoleDefinitions: {},\n    removeAll: {}\n  },\n  "SP.ScriptSafeDomain": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    }\n  },\n  "SP.ScriptSafeDomain.Collection": {\n    create: {\n      argNames: ["parameters"]\n    },\n    getByDomainName: {\n      argNames: ["domainName"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.SecurableObject": {\n    breakRoleInheritance: {\n      argNames: ["copyRoleAssignments", "clearSubscopes"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    resetRoleInheritance: {}\n  },\n  "SP.Site": {\n    properties: ["EventReceivers|SP.EventReceiverDefinition.Collection|(\'[Name]\')|SP.EventReceiverDefinition", "Features|SP.Feature.Collection|(\'[Name]\')|SP.Feature", "Owner|SP.User", "RootWeb|SP.Web", "UserCustomActions|SP.UserCustomAction.Collection|(\'[Name]\')|SP.UserCustomAction"],\n    cancelDeleteFileVersions: {\n      requestType: utils_1.RequestType.Post\n    },\n    cancelSetVersionPolicyForDocLibs: {\n      requestType: utils_1.RequestType.Post\n    },\n    createCopyJob: {\n      argNames: ["exportObjectUris", "destinationUri", "options"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    createCopyJobs: {\n      argNames: ["exportObjectUris", "destinationUri", "options"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    createMigrationIngestionJob: {\n      argNames: ["gWebId", "azureContainerSourceUri", "azureContainerManifestUri", "azureQueueReportUri", "ingestionTaskKey"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    createMigrationJob: {\n      argNames: ["gWebId", "azureContainerSourceUri", "azureContainerManifestUri", "azureQueueReportUri"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    createMigrationJobEncrypted: {\n      argNames: ["gWebId", "azureContainerSourceUri", "azureContainerManifestUri", "azureQueueReportUri", "options"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    createPreviewSPSite: {\n      argNames: ["upgrade", "sendemail"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    createSPAsyncReadJob: {\n      argNames: ["url", "readOptions", "encryptionOption", "azureContainerManifestUri", "azureQueueReportUri"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    createSPAsyncReadJobWithMultiUrl: {\n      argNames: ["urls", "readOptions", "encryptionOption", "azureContainerManifestUri", "azureQueueReportUri"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    deleteMigrationJob: {\n      argNames: ["id"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    enqueueApplySensitivityLabelWork: {\n      argNames: ["workItemInformation"]\n    },\n    extendUpgradeReminderDate: {\n      requestType: utils_1.RequestType.Post\n    },\n    getBlockDownloadPolicyForFilesData: {},\n    getBringYourOwnKeyRecoveryKeyMode: {},\n    getBringYourOwnKeySiteStatus: {},\n    getBringYourOwnKeyTenantStatus: {},\n    getCatalog: {\n      argNames: ["typeCatalog"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    getChanges: {\n      argNames: ["query"],\n      metadataType: "SP.ChangeQuery",\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    getCopyJobProgress: {\n      argNames: ["copyJobInfo"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getHubSiteJoinApprovalCorrelationId: {},\n    getMigrationJobStatus: {\n      argNames: ["id"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getMigrationStatus: {},\n    getProgressForDeleteFileVersions: {},\n    getProgressForExpireFileVersionsBySchedule: {\n      argNames: ["scheduleFilePath"]\n    },\n    getProgressForFileVersionExpirationReport: {\n      argNames: ["reportFileUrl"]\n    },\n    getProgressForSetVersionPolicyForDocLibs: {},\n    getRecycleBinItems: {\n      argNames: ["pagingInfo", "rowLimit", "isAscending", "orderBy", "itemState"],\n      requestType: utils_1.RequestType.GetWithArgsInBody\n    },\n    getWebPath: {\n      argNames: ["siteId", "webId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getWebTemplates: {\n      argNames: ["LCID", "overrideCompatLevel"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    invalidate: {\n      requestType: utils_1.RequestType.Post\n    },\n    joinHubSite: {\n      argNames: ["hubSiteId", "approvalToken", "approvalCorrelationId"],\n      requestType: utils_1.RequestType.GetWithArgsInBody\n    },\n    multiGeoCopyJob: {\n      argNames: ["jobId", "userId", "binaryPayload"]\n    },\n    needsUpgradeByType: {\n      argNames: ["versionUpgrade", "recursive"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    needsUpgradeByTypeFromSpoShell: {\n      argNames: ["versionUpgrade", "recursive"]\n    },\n    onHubSiteJoinRequestApproved: {\n      argNames: ["joiningSiteId"]\n    },\n    onHubSiteJoinRequestCanceled: {\n      argNames: ["approvalCorrelationId"]\n    },\n    onHubSiteJoinRequestStarted: {\n      argNames: ["approvalCorrelationId"]\n    },\n    onboardTenantForBringYourOwnKey: {\n      argNames: ["keyInfo"]\n    },\n    openWeb: {\n      argNames: ["strUrl"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    openWebById: {\n      argNames: ["gWebId"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    openWebUsingPath: {\n      argNames: ["path"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    processStorageMetricsChanges: {},\n    provisionMigrationContainers: {},\n    provisionMigrationQueue: {},\n    provisionTemporaryAzureContainer: {},\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    recoverTenantForBringYourOwnKey: {\n      argNames: ["keyInfo"]\n    },\n    registerHubSite: {\n      argNames: ["creationInformation"],\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    rollTenantBringYourOwnKey: {\n      argNames: ["keyType", "keyVaultInfo"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    runHealthCheck: {\n      argNames: ["ruleId", "bRepair", "bRunAlways"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    runUpgradeSiteSession: {\n      argNames: ["versionUpgrade", "queueOnly", "sendEmail"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    setBlockDownloadPolicyForFiles: {\n      argNames: ["blockDownloadPolicyFileTypeIds"]\n    },\n    setIsContributorOwnerEnabledPropertyForDefaultDocLib: {\n      argNames: ["propertyValue", "forceDocLibActivation", "deleteIfDocLibAlreadyExists"]\n    },\n    startDeleteFileVersions: {\n      argNames: ["deleteOlderThanDays"]\n    },\n    startDeleteFileVersionsByMode: {\n      argNames: ["batchDeleteParameters"]\n    },\n    startExpireFileVersionsBySchedule: {\n      argNames: ["scheduleFilePath"]\n    },\n    startFileVersionExpirationReport: {\n      argNames: ["reportFileUrl"]\n    },\n    startSetVersionPolicyForDocLibs: {\n      argNames: ["enableAutoTrim", "majorVersionLimit", "majorWithMinorVersionsLimit", "expireAfterDays"]\n    },\n    unregisterHubSite: {},\n    update: {\n      argNames: ["properties"],\n      metadataType: "SP.Site",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    updateClientObjectModelUseRemoteAPIsPermissionSetting: {\n      argNames: ["requireUseRemoteAPIs"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    updateInactiveSiteProperties: {\n      argNames: ["operation", "executionId"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    validateHubSiteJoinApprovalToken: {\n      argNames: ["joiningSiteId", "approvalToken"],\n      requestType: utils_1.RequestType.PostWithArgs\n    }\n  },\n  "SP.SiteVersionPolicyManager": {\n    inheritTenantSettings: {},\n    query: {\n      argNames: ["oData"]\n    },\n    setAutoExpiration: {},\n    setExpireAfter: {\n      argNames: ["majorVersionLimit", "expireAfterDays"]\n    },\n    setNoExpiration: {\n      argNames: ["majorVersionLimit"]\n    }\n  },\n  "SP.Social.SocialFeedManager": {\n    createFileAttachment: {\n      argNames: ["name", "description", "fileData"],\n      requestType: utils_1.RequestType.PostWithArgsAndData\n    },\n    createImageAttachment: {\n      argNames: ["name", "description", "imageData"],\n      requestType: utils_1.RequestType.PostWithArgsAndData\n    },\n    createPost: {\n      argNames: ["targetId", "creationData"],\n      requestType: utils_1.RequestType.PostWithArgsAndData\n    },\n    deletePost: {\n      argNames: ["postId"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    getAllLikers: {\n      argNames: ["postId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getFeed: {\n      argNames: ["type", "options"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getFeedFor: {\n      argNames: ["actorId", "options"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getFullThread: {\n      argNames: ["threadId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getMentions: {\n      argNames: ["clearUnreadMentions", "options"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getPreview: {\n      argNames: ["itemUrl"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getPreviewImage: {\n      argNames: ["url", "key", "iv"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getUnreadMentionCount: {\n      requestType: utils_1.RequestType.Get\n    },\n    likePost: {\n      argNames: ["postId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    lockThread: {\n      argNames: ["threadId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    suppressThreadNotifications: {\n      argNames: ["threadId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    unlikePost: {\n      argNames: ["postId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    unlockThread: {\n      argNames: ["threadId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    }\n  },\n  "SP.Social.SocialFollowingManager": {\n    follow: {\n      argNames: ["actor"],\n      name: "follow(actor=@v)?@v=\'[[actor]]\'",\n      requestType: utils_1.RequestType.PostReplace\n    },\n    getFollowed: {\n      argNames: ["types"],\n      requestType: utils_1.RequestType.GetWithArgs\n    },\n    getFollowedCount: {\n      argNames: ["types"],\n      requestType: utils_1.RequestType.GetWithArgs\n    },\n    getFollowers: {\n      requestType: utils_1.RequestType.Get\n    },\n    getSuggestions: {\n      requestType: utils_1.RequestType.Get\n    },\n    isFollowed: {\n      argNames: ["actor"],\n      requestType: utils_1.RequestType.GetWithArgs\n    },\n    mergeFollowedSites: {\n      argNames: ["followedSites"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    stopFollowing: {\n      argNames: ["actor"],\n      requestType: utils_1.RequestType.GetWithArgs\n    }\n  },\n  "SP.Social.SocialRestActor": {\n    feed: {\n      argNames: ["MaxThreadCount", "NewerThan", "OlderThan", "SortOrder"],\n      requestType: utils_1.RequestType.GetWithArgs\n    },\n    likes: {\n      argNames: ["MaxThreadCount", "NewerThan", "OlderThan", "SortOrder"],\n      requestType: utils_1.RequestType.GetWithArgs\n    },\n    mentionFeed: {\n      argNames: ["MaxThreadCount", "NewerThan", "OlderThan", "SortOrder"],\n      requestType: utils_1.RequestType.GetWithArgs\n    },\n    news: {\n      argNames: ["MaxThreadCount", "NewerThan", "OlderThan", "SortOrder"],\n      requestType: utils_1.RequestType.GetWithArgs\n    },\n    organizationFeed: {\n      argNames: ["MaxThreadCount", "NewerThan", "OlderThan", "SortOrder"],\n      requestType: utils_1.RequestType.GetWithArgs\n    },\n    timelineFeed: {\n      argNames: ["MaxThreadCount", "NewerThan", "OlderThan", "SortOrder"],\n      requestType: utils_1.RequestType.GetWithArgs\n    },\n    unreadMentionCount: {\n      requestType: utils_1.RequestType.Get\n    }\n  },\n  "SP.Social.SocialRestFeed": {\n    clearUnReadMentionCount: {\n      argNames: ["MaxThreadCount", "NewerThan", "OlderThan", "SortOrder"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    post: {\n      argNames: ["restCreationData"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    }\n  },\n  "SP.Social.SocialRestFeedManager": {\n    actor: {\n      argNames: ["item"],\n      name: "actor(item=@v)?@v=\'[[item]]\'",\n      requestType: utils_1.RequestType.GetReplace,\n      returnType: "SP.Social.SocialRestActor"\n    },\n    my: {\n      requestType: utils_1.RequestType.Get,\n      returnType: "SP.Social.SocialRestActor"\n    },\n    post: {\n      argNames: ["ID"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    }\n  },\n  "SP.Social.SocialRestFollowingManager": {\n    follow: {\n      argNames: ["AccountName", "ActorType", "ContentUri", "Id", "TagGuid"],\n      name: "follow(AccountName=@v, ActorType=\'[[ActorType]]\', ContentUri=\'[[ContentUri]]\', Id=\'[[Id]]\', TagGuid=\'[[TagGuid]]\')?@v=\'[[AccountName]]\'",\n      requestType: utils_1.RequestType.GetReplace\n    },\n    followed: {\n      argNames: ["types", "count"],\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    followedCount: {\n      argNames: ["types"],\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    followers: {\n      requestType: utils_1.RequestType.Get\n    },\n    isFollowed: {\n      argNames: ["AccountName", "ActorType", "ContentUri", "Id", "TagGuid"],\n      name: "isFollowed(AccountName=@v, ActorType=\'[[ActorType]]\', ContentUri=\'[[ContentUri]]\', Id=\'[[Id]]\', TagGuid=\'[[TagGuid]]\')?@v=\'[[AccountName]]\'",\n      requestType: utils_1.RequestType.GetReplace\n    },\n    mergeFollowedSites: {\n      argNames: ["followedSites"],\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    my: {\n      requestType: utils_1.RequestType.Get\n    },\n    stopFollowing: {\n      argNames: ["AccountName", "ActorType", "ContentUri", "Id", "TagGuid"],\n      name: "stopFollowing(AccountName=@v, ActorType=\'[[ActorType]]\', ContentUri=\'[[ContentUri]]\', Id=\'[[Id]]\', TagGuid=\'[[TagGuid]]\')?@v=\'[[AccountName]]\'",\n      requestType: utils_1.RequestType.GetReplace\n    },\n    suggestions: {\n      requestType: utils_1.RequestType.Get\n    }\n  },\n  "SP.Social.SocialRestThread": {\n    "delete": {\n      argNames: ["ID"],\n      requestType: utils_1.RequestType.Delete\n    },\n    like: {\n      argNames: ["ID"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    likers: {\n      argNames: ["ID"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    lock: {\n      argNames: ["ID"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    reply: {\n      argNames: ["restCreationData"],\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    unLike: {\n      argNames: ["ID"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    unLock: {\n      argNames: ["ID"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    }\n  },\n  "SP.SPAppLicenseManager": {\n    checkLicense: {\n      argNames: ["productId"]\n    }\n  },\n  "SP.SPHSite": {\n    addHomeSite: {\n      argNames: ["siteUrl", "order", "audiences", "vivaConnectionsDefaultStart", "isInDraftMode"]\n    },\n    details: {\n      argNames: ["includeVivaSites"]\n    },\n    setSPHSite: {\n      argNames: ["siteUrl", "vivaConnectionsDefaultStart", "isInDraftMode"]\n    }\n  },\n  "SP.Taxonomy.TaxonomyField": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    disableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    enableIndex: {\n      requestType: utils_1.RequestType.Post\n    },\n    setShowInDisplayForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInEditForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    setShowInNewForm: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    update: {\n      argNames: ["properties"],\n      inheritMetadataType: true,\n      metadataType: "SP.Field",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.TenantSettings": {\n    clearCorporateCatalog: {},\n    getDataAccessGovernanceReportConfig: {},\n    query: {\n      argNames: ["oData"]\n    },\n    setCorporateCatalog: {\n      argNames: ["url"]\n    }\n  },\n  "SP.ThemeInfo": {\n    getThemeFontByName: {\n      argNames: ["name", "lcid"]\n    },\n    getThemeShadeByName: {\n      argNames: ["name"]\n    }\n  },\n  "SP.TimeZone": {\n    localTimeToUTC: {\n      argNames: ["date"]\n    },\n    setId: {\n      argNames: ["id"]\n    },\n    uTCToLocalTime: {\n      argNames: ["date"]\n    }\n  },\n  "SP.TimeZone.Collection": {\n    getById: {\n      argNames: ["id"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.Translation.SyncTranslator": {\n    translate: {\n      argNames: ["inputFile", "outputFile"]\n    },\n    translateStream: {\n      argNames: ["inputFile", "fileExtension"]\n    }\n  },\n  "SP.Translation.TranslationJob": {\n    translateFile: {\n      argNames: ["inputFile", "outputFile"]\n    },\n    translateFolder: {\n      argNames: ["inputFolder", "outputFolder", "recursion"]\n    },\n    translateLibrary: {\n      argNames: ["inputLibrary", "outputLibrary"]\n    }\n  },\n  "SP.Translation.TranslationJobStatus": {\n    getAllItems: {}\n  },\n  "SP.TranslationStatusCollection": {\n    create: {\n      argNames: ["request"]\n    },\n    set: {\n      argNames: ["request"]\n    },\n    updateTranslationLanguages: {}\n  },\n  "SP.User": {\n    properties: ["Groups|SP.Group.Collection|([Name])|SP.Group"],\n    expire: {\n      requestType: utils_1.RequestType.Post\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    update: {\n      argNames: ["properties"],\n      metadataType: "SP.User",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.User.Collection": {\n    add: {\n      argNames: ["properties"],\n      metadataType: "SP.User",\n      name: "",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    addUserById: {\n      argNames: ["id"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    getByEmail: {\n      argNames: ["emailAddress"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.User"\n    },\n    getById: {\n      argNames: ["id"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.User"\n    },\n    getByLoginName: {\n      argNames: ["loginName"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.User"\n    },\n    getByObjectId: {\n      argNames: ["objectId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.User"\n    },\n    getByPuid: {\n      argNames: ["puid"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.User"\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    removeById: {\n      argNames: ["id"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    removeByLoginName: {\n      argNames: ["loginName"],\n      name: "removeByLoginName(@v)?@v=\'[[loginName]]\'",\n      requestType: utils_1.RequestType.PostReplace\n    }\n  },\n  "SP.UserCustomAction": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    update: {\n      argNames: ["properties"],\n      metadataType: "SP.UserCustomAction",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.UserCustomAction.Collection": {\n    add: {\n      argNames: ["properties"],\n      metadataType: "SP.UserCustomAction",\n      name: "",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    clear: {\n      requestType: utils_1.RequestType.Post\n    },\n    getById: {\n      argNames: ["id"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.UserCustomAction"\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.UserExperienceState": {\n    setFlag: {\n      argNames: ["flag", "reset"]\n    }\n  },\n  "SP.UserProfiles.FollowedContent": {\n    findAndUpdateFollowedGroup: {\n      argNames: ["groupId"]\n    },\n    findAndUpdateFollowedItem: {\n      argNames: ["url"]\n    },\n    followItem: {\n      argNames: ["item"]\n    },\n    getFollowedStatus: {\n      argNames: ["url"]\n    },\n    getGroups: {\n      argNames: ["rowLimit"]\n    },\n    getItem: {\n      argNames: ["url"]\n    },\n    getItems: {\n      argNames: ["options", "subtype"]\n    },\n    hasGroupMembershipChangedAndSyncChanges: {},\n    isFollowed: {\n      argNames: ["url"]\n    },\n    refreshFollowedItem: {\n      argNames: ["item"]\n    },\n    setItemPinState: {\n      argNames: ["uri", "groupId", "pinState"]\n    },\n    stopFollowing: {\n      argNames: ["url"]\n    },\n    updateFollowedGroupForUser: {\n      argNames: ["contextUri", "groupId", "loginName"]\n    }\n  },\n  "SP.UserProfiles.PeopleManager": {\n    amIFollowedBy: {\n      argNames: ["accountName"],\n      name: "amIFollowedBy(@v)?@v=\'[[accountName]]\'",\n      requestType: utils_1.RequestType.GetReplace\n    },\n    amIFollowing: {\n      argNames: ["accountName"],\n      name: "amIFollowing(@v)?@v=\'[[accountName]]\'",\n      requestType: utils_1.RequestType.GetReplace\n    },\n    follow: {\n      argNames: ["accountName"],\n      name: "follow(@v)?@v=\'[[accountName]]\'",\n      requestType: utils_1.RequestType.PostReplace\n    },\n    followTag: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    getDefaultDocumentLibrary: {\n      argNames: ["accountName", "createSiteIfNotExists", "siteCreationPriority"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getFollowedTags: {\n      argNames: ["cTagsToFetch"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getFollowersFor: {\n      argNames: ["accountName"],\n      name: "getFollowersFor(@v)?@v=\'[[accountName]]\'",\n      requestType: utils_1.RequestType.GetReplace\n    },\n    getMyFollowers: {\n      requestType: utils_1.RequestType.Get\n    },\n    getMyProperties: {\n      requestType: utils_1.RequestType.Get\n    },\n    getMySuggestions: {\n      requestType: utils_1.RequestType.Get\n    },\n    getPeopleFollowedBy: {\n      argNames: ["accountName"],\n      name: "getPeopleFollowedBy(@v)?@v=\'[[accountName]]\'",\n      requestType: utils_1.RequestType.GetReplace\n    },\n    getPeopleFollowedByMe: {\n      requestType: utils_1.RequestType.Get\n    },\n    getPropertiesFor: {\n      argNames: ["accountName"],\n      name: "getPropertiesFor(@v)?@v=\'[[accountName]]\'",\n      requestType: utils_1.RequestType.GetReplace\n    },\n    getSPUserInformation: {\n      argNames: ["accountName", "siteId"],\n      name: "getSPUserInformation(accountName=@v, siteId=\'[[siteId]]\')?@v=\'[[accountName]]\'",\n      requestType: utils_1.RequestType.GetReplace\n    },\n    getUserOneDriveQuotaMax: {\n      argNames: ["accountName"],\n      name: "getUserOneDriveQuotaMax(accountName=@v)?@v=\'[[accountName]]\'",\n      requestType: utils_1.RequestType.GetReplace\n    },\n    getUserProfileProperties: {\n      argNames: ["accountName"],\n      name: "getUserProfileProperties(@v)?@v=\'[[accountName]]\'",\n      requestType: utils_1.RequestType.GetReplace\n    },\n    getUserProfilePropertyFor: {\n      argNames: ["accountName", "propertyName"],\n      name: "getUserProfilePropertyFor(accountName=@v, propertyName=\'[[propertyName]]\')?@v=\'[[accountName]]\'",\n      requestType: utils_1.RequestType.GetReplace\n    },\n    hardDeleteUserProfile: {\n      argNames: ["accountName", "userId"],\n      name: "hardDeleteUserProfile(accountName=@v, userId=\'[[userId]]\')?@v=\'[[accountName]]\'",\n      requestType: utils_1.RequestType.GetReplace\n    },\n    hideSuggestion: {\n      argNames: ["accountName"],\n      name: "hideSuggestion(@v)?@v=\'[[accountName]]\'",\n      requestType: utils_1.RequestType.PostReplace\n    },\n    removeSPUserInformation: {\n      argNames: ["accountName", "siteId", "redactName"],\n      name: "removeSPUserInformation(accountName=@v, siteId=\'[[siteId]]\', redactName=\'[[redactName]]\')?@v=\'[[accountName]]\'",\n      requestType: utils_1.RequestType.GetReplace\n    },\n    resetUserOneDriveQuotaToDefault: {\n      argNames: ["accountName"],\n      name: "resetUserOneDriveQuotaToDefault(@v)?@v=\'[[accountName]]\'",\n      requestType: utils_1.RequestType.PostReplace\n    },\n    setMultiValuedProfileProperty: {\n      argNames: ["accountName", "propertyName", "propertyValues"],\n      name: "setMultiValuedProfileProperty(accountName=@v, propertyName=\'[[propertyName]]\', propertyValues=\'[[propertyValues]]\')?@v=\'[[accountName]]\'",\n      requestType: utils_1.RequestType.GetReplace\n    },\n    setMyProfilePicture: {\n      argNames: ["picture"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    setSingleValueProfileProperty: {\n      argNames: ["accountName", "propertyName", "propertyValue"],\n      name: "setSingleValueProfileProperty(accountName=@v, propertyName=\'[[propertyName]]\', propertyValue=\'[[propertyValue]]\')?@v=\'[[accountName]]\'",\n      requestType: utils_1.RequestType.GetReplace\n    },\n    setUserOneDriveQuota: {\n      argNames: ["accountName", "newQuota", "newQuotaWarning"],\n      name: "setUserOneDriveQuota(accountName=@v, newQuota=\'[[newQuota]]\', newQuotaWarning=\'[[newQuotaWarning]]\')?@v=\'[[accountName]]\'",\n      requestType: utils_1.RequestType.GetReplace\n    },\n    stopFollowing: {\n      argNames: ["accountName"],\n      name: "stopFollowing(@v)?@v=\'[[accountName]]\'",\n      requestType: utils_1.RequestType.PostWithArgsInQSAsVar\n    },\n    stopFollowingTag: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    }\n  },\n  "SP.UserProfiles.PersonalCache": {\n    deleteCacheItemsAsync: {\n      argNames: ["cacheItems"],\n      requestType: utils_1.RequestType.Delete\n    },\n    deleteCacheItemsAsync2: {\n      argNames: ["cacheItems", "mySiteUrl"],\n      requestType: utils_1.RequestType.Delete\n    },\n    dispose: {},\n    loadUserProfile: {\n      argNames: ["email"]\n    },\n    readCache: {\n      argNames: ["folderPath"]\n    },\n    readCache2: {\n      argNames: ["folderPath", "mySiteUrl"]\n    },\n    readCacheOrCreate: {\n      argNames: ["folderPath", "requiredCacheKeys", "createIfMissing"]\n    },\n    readCacheOrCreate2: {\n      argNames: ["folderPath", "requiredCacheKeys", "createIfMissing", "mySiteUrl"]\n    },\n    readCacheOrCreateOrderById: {\n      argNames: ["folderPath", "requiredCacheKeys", "createIfMissing"]\n    },\n    readCacheOrCreateOrderById2: {\n      argNames: ["folderPath", "requiredCacheKeys", "createIfMissing", "mySiteUrl"]\n    },\n    writeCache: {\n      argNames: ["cacheItems"]\n    },\n    writeCache2: {\n      argNames: ["cacheItems", "mySiteUrl"]\n    }\n  },\n  "SP.UserProfiles.ProfileImageStore": {\n    saveUploadedFile: {\n      argNames: ["profileType", "fileNamePrefix", "isFeedAttachment", "clientFilePath", "fileSize", "fileStream"]\n    }\n  },\n  "SP.UserProfiles.ProfileLoader": {\n    createPersonalSiteEnqueueBulk: {\n      argNames: ["emailIDs"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    getUserProfile: {\n      requestType: utils_1.RequestType.Post,\n      returnType: "SP.UserProfiles.UserProfile"\n    }\n  },\n  "SP.UserProfiles.UserProfile": {\n    properties: ["PersonalSite|site"],\n    createPersonalSite: {\n      argNames: ["lcid"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    createPersonalSiteEnque: {\n      argNames: ["isInteractive"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    createPersonalSiteFromWorkItem: {\n      argNames: ["workItemType"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    setMySiteFirstRunExperience: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    shareAllSocialData: {\n      argNames: ["shareAll"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    }\n  },\n  "SP.UserProfiles.UserProfilePropertiesForUser": {\n    getPropertyNames: {}\n  },\n  "SP.UserResource": {\n    getResourceEntries: {},\n    getValueForUICulture: {\n      argNames: ["cultureName"]\n    },\n    setValueForUICulture: {\n      argNames: ["cultureName", "value"]\n    }\n  },\n  "SP.UserSolution.Collection": {\n    add: {\n      argNames: ["solutionGalleryItemId"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.Utilities.ThemeManager": {\n    addTenantTheme: {\n      argNames: ["name", "themeJson"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    applyTheme: {\n      argNames: ["name", "themeJson"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    deleteTenantTheme: {\n      argNames: ["name"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    getAvailableThemes: {},\n    getHideDefaultThemes: {},\n    getTenantTheme: {\n      argNames: ["name"],\n      requestType: utils_1.RequestType.GetWithArgsInBody\n    },\n    getTenantThemingOptions: {},\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    setHideDefaultThemes: {\n      argNames: ["hideDefaultThemes"]\n    },\n    updateTenantTheme: {\n      argNames: ["name", "themeJson"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    }\n  },\n  "SP.View": {\n    properties: ["ViewFields|SP.ViewFieldCollection"],\n    addToSpotlight: {\n      argNames: ["itemId", "folderPath", "afterItemId"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    removeFromSpotlight: {\n      argNames: ["itemId", "folderPath"]\n    },\n    renderAsHtml: {},\n    setViewXml: {\n      argNames: ["viewXml"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    update: {\n      argNames: ["properties"],\n      metadataType: "SP.View",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.View.Collection": {\n    add: {\n      argNames: ["properties"],\n      metadataType: "SP.View",\n      name: "",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    getById: {\n      argNames: ["guidId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.View"\n    },\n    getByTitle: {\n      argNames: ["strTitle"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.View"\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.ViewFieldCollection": {\n    addViewField: {\n      argNames: ["strField"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    moveViewFieldTo: {\n      argNames: ["field", "index"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    removeAllViewFields: {\n      requestType: utils_1.RequestType.Post\n    },\n    removeViewField: {\n      argNames: ["strField"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    }\n  },\n  "SP.Web": {\n    properties: ["AllProperties|SP.PropertyValues.Collection", "AppTiles", "AssociatedMemberGroup|SP.Group", "AssociatedOwnerGroup|SP.Group", "AssociatedVisitorGroup|SP.Group", "Author|SP.User", "AvailableContentTypes|SP.ContentType.Collection", "AvailableFields|SP.Field.Collection", "ClientWebParts", "ContentTypes|SP.ContentType.Collection|(\'[Name]\')|SP.ContentType", "CurrentUser|SP.User", "DataLeakagePreventionStatusInfo", "DescriptionResource", "EffectiveBasePermissions", "EventReceivers|SP.EventReceiverDefinition.Collection|(\'[Name]\')|SP.EventReceiverDefinition", "Features|SP.Feature.Collection|(\'[Name]\')|SP.Feature", "Fields|SP.Field.Collection|/getByInternalNameOrTitle(\'[Name]\')|SP.Field", "FirstUniqueAncestorSecurableObject", "Folders|SP.Folder.Collection|/getByUrl(\'[Name]\')|SP.Folder", "Lists|SP.List.Collection|/getByTitle(\'[Name]\')|SP.List", "ListTemplates|SP.ListTemplate.Collection|(\'[Name]\')|SP.ListTemplate", "Navigation|SP.Navigation", "ParentWeb", "PushNotificationSubscribers", "RecycleBin", "RegionalSettings", "RoleAssignments|SP.RoleAssignment.Collection|([Name])|SP.RoleAssignment", "RoleDefinitions|SP.RoleDefinition.Collection|/getByName(\'[Name]\')|SP.RoleDefinition", "RootFolder|SP.Folder|/getByUrl(\'[Name]\')|SP.File", "SiteCollectionAppCatalog|sitecollectionappcatalog", "SiteGroups|SP.Group.Collection|/getByName(\'[Name]\')|SP.Group", "SiteUserInfoList", "SiteUsers|SP.User.Collection|/getById([Name])|SP.User", "TenantAppCatalog|tenantappcatalog", "ThemeInfo", "TitleResource", "UserCustomActions|SP.UserCustomAction.Collection|(\'[Name]\')|SP.UserCustomAction", "WebInfos|SP.WebInformation.Collection", "Webs|SP.Web.Collection", "WorkflowAssociations", "WorkflowTemplates"],\n    addCrossFarmMessage: {\n      argNames: ["messagePayloadBase64"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    addPlaceholderUser: {\n      argNames: ["listId", "placeholderText"]\n    },\n    addSupportedUILanguage: {\n      argNames: ["lcid"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    addinPermissions: {\n      argNames: ["addins"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    applyTheme: {\n      argNames: ["colorPaletteUrl", "fontSchemeUrl", "backgroundImageUrl", "shareGenerated"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    applyWebTemplate: {\n      argNames: ["webTemplate"],\n      requestType: utils_1.RequestType.PostWithArgsInQSAsVar\n    },\n    availableAddins: {\n      argNames: ["serverRelativeUrls", "urls"]\n    },\n    breakRoleInheritance: {\n      argNames: ["copyRoleAssignments", "clearSubscopes"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    consentToPowerPlatform: {},\n    createDefaultAssociatedGroups: {\n      argNames: ["userLogin", "userLogin2", "groupNameSeed"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    createGroupBasedEnvironment: {},\n    createSitePage: {\n      argNames: ["pageMetaData"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    defaultDocumentLibrary: {},\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    doesPushNotificationSubscriberExist: {\n      argNames: ["deviceAppInstanceId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    doesUserHavePermissions: {\n      argNames: ["high", "low"],\n      requestType: utils_1.RequestType.GetWithArgsInQSAsVar\n    },\n    ensureEduClassSetup: {\n      argNames: ["byPassForAutomation"]\n    },\n    ensureSolutioningEnvironment: {\n      argNames: ["ensureCdsInstance"]\n    },\n    ensureTenantAppCatalog: {\n      argNames: ["callerId"]\n    },\n    ensureUser: {\n      argNames: ["logonName"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    ensureUserByObjectId: {\n      argNames: ["objectId", "tenantId", "principalType"]\n    },\n    executeRemoteLOB: {\n      argNames: ["inputStream"],\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    getACSServicePrincipals: {\n      argNames: ["appIds"]\n    },\n    getAdaptiveCardExtensions: {\n      argNames: ["includeErrors", "project"]\n    },\n    getAddinPrincipalsHavingPermissionsInSites: {\n      argNames: ["serverRelativeUrls", "urls"]\n    },\n    getAddinUninstallJobDetail: {\n      argNames: ["jobId", "serverRelativeUrl", "url"]\n    },\n    getAllClientSideComponents: {\n      argNames: ["languages"],\n      requestType: utils_1.RequestType.Post\n    },\n    getAppBdcCatalog: {\n      requestType: utils_1.RequestType.Post\n    },\n    getAppBdcCatalogForAppInstance: {\n      argNames: ["appInstanceId"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    getAppInstanceById: {\n      argNames: ["appInstanceId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getAppInstancesByProductId: {\n      argNames: ["productId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getAvailableWebTemplates: {\n      argNames: ["lcid", "doIncludeCrossLanguage"],\n      requestType: utils_1.RequestType.GetWithArgs\n    },\n    getCatalog: {\n      argNames: ["typeCatalog"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.List"\n    },\n    getChanges: {\n      argNames: ["query"],\n      metadataType: "SP.ChangeQuery",\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    getClientSideComponents: {\n      argNames: ["components", "project"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getClientSideComponentsByComponentType: {\n      argNames: ["componentTypesString", "supportedHostTypeValue", "includeErrors", "project", "includeManifestActivatedTime"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getClientSideComponentsById: {\n      argNames: ["componentIds", "project"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getClientSideWebParts: {\n      argNames: ["includeErrors", "project"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getCustomListTemplates: {},\n    getEntity: {\n      argNames: ["namespace", "name"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    getFileByGuestUrl: {\n      argNames: ["guestUrl"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.File"\n    },\n    getFileByGuestUrlEnsureAccess: {\n      argNames: ["guestUrl", "ensureAccess"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.File"\n    },\n    getFileByGuestUrlExtended: {\n      argNames: ["guestUrl", "requestSettings"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.File"\n    },\n    getFileById: {\n      argNames: ["uniqueId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.File"\n    },\n    getFileByLinkingUrl: {\n      argNames: ["linkingUrl"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.File"\n    },\n    getFileByServerRelativePath: {\n      argNames: ["DecodedUrl"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.File"\n    },\n    getFileByServerRelativeUrl: {\n      argNames: ["serverRelativeUrl"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.File"\n    },\n    getFileByStreamFrameUrl: {\n      argNames: ["streamFrameUrl"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.File"\n    },\n    getFileByUrl: {\n      argNames: ["fileUrl"],\n      name: "getFileByUrl(@url)?@url=\'[[fileUrl]]\'",\n      requestType: utils_1.RequestType.GetReplace,\n      returnType: "SP.File"\n    },\n    getFileByWOPIFrameUrl: {\n      argNames: ["wopiFrameUrl"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.File"\n    },\n    getFolderByGuestUrl: {\n      argNames: ["guestUrl", "ensureAccess"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.Folder"\n    },\n    getFolderByGuestUrlExtended: {\n      argNames: ["guestUrl", "requestSettings"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.Folder"\n    },\n    getFolderById: {\n      argNames: ["uniqueId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.Folder"\n    },\n    getFolderByServerRelativePath: {\n      argNames: ["DecodedUrl"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.Folder"\n    },\n    getFolderByServerRelativeUrl: {\n      argNames: ["serverRelativeUrl"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.Folder"\n    },\n    getGroupBasedEnvironment: {},\n    getList: {\n      argNames: ["strUrl"],\n      name: "getList(@l)?@l=\'[[strUrl]]\'",\n      requestType: utils_1.RequestType.GetReplace,\n      returnType: "SP.List"\n    },\n    getListByTitle: {\n      argNames: ["title"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.List"\n    },\n    getListItem: {\n      argNames: ["strUrl"],\n      name: "getListItem(@l)?@l=\'[[strUrl]]\'",\n      requestType: utils_1.RequestType.GetReplace\n    },\n    getListItemByResourceId: {\n      argNames: ["resourceId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getListItemUsingPath: {\n      argNames: ["DecodedUrl"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getListOperation: {\n      argNames: ["listId", "operationId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getListUsingPath: {\n      argNames: ["DecodedUrl"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getLists: {\n      argNames: ["getListsParams"]\n    },\n    getNewsList: {\n      argNames: ["allowCreate"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getPushNotificationSubscriber: {\n      argNames: ["deviceAppInstanceId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getPushNotificationSubscribersByArgs: {\n      argNames: ["customArgs"],\n      requestType: utils_1.RequestType.GetWithArgsInQSAsVar\n    },\n    getPushNotificationSubscribersByUser: {\n      argNames: ["userName"],\n      requestType: utils_1.RequestType.GetWithArgsInQSAsVar\n    },\n    getRecycleBinItems: {\n      argNames: ["pagingInfo", "rowLimit", "isAscending", "orderBy", "itemState"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getRecycleBinItemsByQueryInfo: {\n      argNames: ["IsAscending", "ItemState", "OrderBy", "PagingInfo", "RowLimit", "ShowOnlyMyItems"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getRegionalDateTimeSchema: {},\n    getSPAppContextAsStream: {},\n    getSharingLinkData: {\n      argNames: ["linkUrl"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getSitePageCopyToStatus: {\n      argNames: ["workItemId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getSitePageMoveStatus: {\n      argNames: ["workItemId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getSiteUserIncludingDeletedByPuid: {\n      argNames: ["puid"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getStorageEntity: {\n      argNames: ["key"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getSubwebsFilteredForCurrentUser: {\n      argNames: ["nWebTemplateFilter", "nConfigurationFilter"],\n      requestType: utils_1.RequestType.GetWithArgs,\n      returnType: "SP.WebInformation.Collection"\n    },\n    getUserById: {\n      argNames: ["userId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "SP.User"\n    },\n    getUserEffectivePermissions: {\n      argNames: ["userName"],\n      name: "getUserEffectivePermissions(@user)?@user=\'[[userName]]\'",\n      requestType: utils_1.RequestType.GetReplace\n    },\n    getViewFromPath: {\n      argNames: ["DecodedUrl"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    getViewFromUrl: {\n      argNames: ["listUrl"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    hubSiteData: {\n      argNames: ["forceRefresh"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    hubSiteDataAsStream: {\n      argNames: ["forceRefresh"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    incrementSiteClientTag: {},\n    listPowerPlatformUserDetails: {},\n    loadAndInstallApp: {\n      argNames: ["appPackageStream"],\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    loadAndInstallAppInSpecifiedLocale: {\n      argNames: ["appPackageStream", "installationLocaleLCID"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    loadApp: {\n      argNames: ["appPackageStream", "installationLocaleLCID"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    mapToIcon: {\n      argNames: ["fileName", "progId", "size"],\n      requestType: utils_1.RequestType.GetWithArgs\n    },\n    markReplyUrisUpdated: {},\n    pageContextCore: {},\n    pageContextInfo: {\n      argNames: ["includeODBSettings", "emitNavigationInfo"],\n      requestType: utils_1.RequestType.GetWithArgs\n    },\n    parseDateTime: {\n      argNames: ["value", "displayFormat", "calendarType"],\n      requestType: utils_1.RequestType.GetWithArgs\n    },\n    processExternalNotification: {\n      argNames: ["stream"],\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    registerPushNotificationSubscriber: {\n      argNames: ["deviceAppInstanceId", "serviceToken"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    removeStorageEntity: {\n      argNames: ["key"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    removeSupportedUILanguage: {\n      argNames: ["lcid"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    resetRoleInheritance: {\n      requestType: utils_1.RequestType.Post\n    },\n    setAccessRequestSiteDescriptionAndUpdate: {\n      argNames: ["description"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    setChromeOptions: {\n      argNames: ["headerLayout", "headerEmphasis", "megaMenuEnabled", "footerEnabled", "footerLayout", "footerEmphasis", "hideTitleInHeader", "logoAlignment", "horizontalQuickLaunch"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    setDefaultNewPageTemplateId: {\n      argNames: ["defaultNewPageTemplateId"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    setGlobalNavSettings: {\n      argNames: ["title", "source"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    setStorageEntity: {\n      argNames: ["key", "value", "description", "comments"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    setUseAccessRequestDefaultAndUpdate: {\n      argNames: ["useAccessRequestDefault"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    syncFlowInstances: {\n      argNames: ["targetWebUrl"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    syncFlowTemplates: {\n      argNames: ["category"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    syncHubSiteTheme: {\n      requestType: utils_1.RequestType.Post\n    },\n    syncTeamsComponent: {\n      argNames: ["teamsComponent"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    uninstallAddins: {\n      argNames: ["uninstallAddins"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    unregisterPushNotificationSubscriber: {\n      argNames: ["deviceAppInstanceId"],\n      requestType: utils_1.RequestType.PostWithArgsValueOnly\n    },\n    unsetIsEduClassProvisionPending: {},\n    update: {\n      argNames: ["properties"],\n      metadataType: "SP.Web",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    uploadImage: {\n      argNames: ["listTitle", "imageName", "contentStream", "listId", "itemId", "fieldId", "overwrite"],\n      name: "uploadImage(listTitle=guid\'[[listTitle]]\', imageName=[[imageName]], listId=[[listId]], itemId=[[itemId]], fieldId=[[fieldId]], overwrite=[[overwrite]])",\n      requestType: utils_1.RequestType.PostReplaceWithData\n    }\n  },\n  "SP.Web.Collection": {\n    add: {\n      argNames: ["parameters"],\n      metadataType: "SP.WebCreationInformation",\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.WebInformation.Collection": {\n    add: {\n      argNames: ["parameters"],\n      metadataType: "SP.WebInfoCreationInformation",\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    getById: {\n      argNames: ["id"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.WebParts.LimitedWebPartManager": {\n    properties: ["WebParts|SP.WebParts.WebPartDefinition.Collection|/([Name])|SP.WebParts.WebPartDefinition"],\n    exportWebPart: {\n      argNames: ["webPartId"],\n      requestType: utils_1.RequestType.GetWithArgsValueOnly\n    },\n    importWebPart: {\n      argNames: ["webPartXml"],\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.WebParts.WebPartDefinition": {\n    closeWebPart: {},\n    deleteWebPart: {},\n    moveWebPartTo: {\n      argNames: ["zoneID", "zoneIndex"]\n    },\n    openWebPart: {\n      requestType: utils_1.RequestType.Get\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    saveWebPartChanges: {}\n  },\n  "SP.WebParts.WebPartDefinition.Collection": {\n    getByControlId: {\n      argNames: ["controlId"]\n    },\n    getById: {\n      argNames: ["id"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.WebTemplate.Collection": {\n    getByName: {\n      argNames: ["name"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.WorkManagement.OM.BaseSession": {\n    addAttributeToTask: {\n      argNames: ["taskKey", "attribute"]\n    },\n    beginCacheRefresh: {},\n    beginExchangeSync: {},\n    createPersonalTaskAndPromoteToProviderTask: {\n      argNames: ["taskName", "description", "localizedStartDate", "localizedDueDate", "completed", "pinned", "locationKey"]\n    },\n    createTask: {\n      argNames: ["taskName", "description", "localizedStartDate", "localizedDueDate", "completed", "pinned", "locationKey", "editUrl"]\n    },\n    deleteTask: {\n      argNames: ["taskKey"]\n    },\n    getCalloutInfo: {\n      argNames: ["taskKey"]\n    },\n    getRefreshHealthInfo: {},\n    getRefreshHistory: {\n      argNames: ["since"]\n    },\n    getRefreshStatus: {\n      argNames: ["refreshId"]\n    },\n    isExchangeJobPending: {},\n    pinTask: {\n      argNames: ["taskKey"]\n    },\n    promotePersonalTaskToProviderTaskInLocation: {\n      argNames: ["taskKey", "locationId"]\n    },\n    readAllNonTaskData: {},\n    refreshSingleTask: {\n      argNames: ["taskKey"]\n    },\n    removeAttributeFromTask: {\n      argNames: ["taskKey", "attribute"]\n    },\n    removePinOnTask: {\n      argNames: ["taskKey"]\n    },\n    updateTaskWithLocalizedValue: {\n      argNames: ["taskKey", "field", "value"]\n    }\n  },\n  "SP.WorkManagement.OM.LocationOrientedSortableSession": {\n    addAttributeToTask: {\n      argNames: ["taskKey", "attribute"]\n    },\n    beginCacheRefresh: {},\n    beginExchangeSync: {},\n    createPersonalTaskAndPromoteToProviderTask: {\n      argNames: ["taskName", "description", "localizedStartDate", "localizedDueDate", "completed", "pinned", "locationKey"]\n    },\n    createTask: {\n      argNames: ["taskName", "description", "localizedStartDate", "localizedDueDate", "completed", "pinned", "locationKey", "editUrl"]\n    },\n    deleteTask: {\n      argNames: ["taskKey"]\n    },\n    getCalloutInfo: {\n      argNames: ["taskKey"]\n    },\n    getRefreshHealthInfo: {},\n    getRefreshHistory: {\n      argNames: ["since"]\n    },\n    getRefreshStatus: {\n      argNames: ["refreshId"]\n    },\n    isExchangeJobPending: {},\n    movePersonalTaskToLocation: {\n      argNames: ["taskKey", "newLocationKey"]\n    },\n    pinTask: {\n      argNames: ["taskKey"]\n    },\n    promotePersonalTaskToProviderTaskInLocation: {\n      argNames: ["taskKey", "locationId"]\n    },\n    readAllNonTaskData: {},\n    refreshSingleTask: {\n      argNames: ["taskKey"]\n    },\n    removeAttributeFromTask: {\n      argNames: ["taskKey", "attribute"]\n    },\n    removePinOnTask: {\n      argNames: ["taskKey"]\n    },\n    updateTaskWithLocalizedValue: {\n      argNames: ["taskKey", "field", "value"]\n    }\n  },\n  "SP.WorkManagement.OM.LocationOrientedUserOrderedSession": {\n    addAttributeToTask: {\n      argNames: ["taskKey", "attribute"]\n    },\n    beginCacheRefresh: {},\n    beginExchangeSync: {},\n    createPersonalTaskAndPromoteToProviderTask: {\n      argNames: ["taskName", "description", "localizedStartDate", "localizedDueDate", "completed", "pinned", "locationKey"]\n    },\n    createTask: {\n      argNames: ["taskName", "description", "localizedStartDate", "localizedDueDate", "completed", "pinned", "locationKey", "editUrl"]\n    },\n    deleteTask: {\n      argNames: ["taskKey"]\n    },\n    getCalloutInfo: {\n      argNames: ["taskKey"]\n    },\n    getRefreshHealthInfo: {},\n    getRefreshHistory: {\n      argNames: ["since"]\n    },\n    getRefreshStatus: {\n      argNames: ["refreshId"]\n    },\n    isExchangeJobPending: {},\n    movePersonalTaskToLocation: {\n      argNames: ["taskKey", "newLocationKey"]\n    },\n    pinTask: {\n      argNames: ["taskKey"]\n    },\n    promotePersonalTaskToProviderTaskInLocation: {\n      argNames: ["taskKey", "locationId"]\n    },\n    readAllNonTaskData: {},\n    refreshSingleTask: {\n      argNames: ["taskKey"]\n    },\n    removeAttributeFromTask: {\n      argNames: ["taskKey", "attribute"]\n    },\n    removePinOnTask: {\n      argNames: ["taskKey"]\n    },\n    reorderTask: {\n      argNames: ["taskKey", "newAfterTaskKey"]\n    },\n    updateTaskWithLocalizedValue: {\n      argNames: ["taskKey", "field", "value"]\n    }\n  },\n  "SP.WorkManagement.OM.SortableSession": {\n    addAttributeToTask: {\n      argNames: ["taskKey", "attribute"]\n    },\n    beginCacheRefresh: {},\n    beginExchangeSync: {},\n    createPersonalTaskAndPromoteToProviderTask: {\n      argNames: ["taskName", "description", "localizedStartDate", "localizedDueDate", "completed", "pinned", "locationKey"]\n    },\n    createTask: {\n      argNames: ["taskName", "description", "localizedStartDate", "localizedDueDate", "completed", "pinned", "locationKey", "editUrl"]\n    },\n    deleteTask: {\n      argNames: ["taskKey"]\n    },\n    getCalloutInfo: {\n      argNames: ["taskKey"]\n    },\n    getRefreshHealthInfo: {},\n    getRefreshHistory: {\n      argNames: ["since"]\n    },\n    getRefreshStatus: {\n      argNames: ["refreshId"]\n    },\n    isExchangeJobPending: {},\n    pinTask: {\n      argNames: ["taskKey"]\n    },\n    promotePersonalTaskToProviderTaskInLocation: {\n      argNames: ["taskKey", "locationId"]\n    },\n    readAllNonTaskData: {},\n    refreshSingleTask: {\n      argNames: ["taskKey"]\n    },\n    removeAttributeFromTask: {\n      argNames: ["taskKey", "attribute"]\n    },\n    removePinOnTask: {\n      argNames: ["taskKey"]\n    },\n    updateTaskWithLocalizedValue: {\n      argNames: ["taskKey", "field", "value"]\n    }\n  },\n  "SP.WorkManagement.OM.SortableSessionManager": {\n    createLocationOrientedSession: {},\n    createSession: {}\n  },\n  "SP.WorkManagement.OM.UserOrderedSession": {\n    addAttributeToTask: {\n      argNames: ["taskKey", "attribute"]\n    },\n    beginCacheRefresh: {},\n    beginExchangeSync: {},\n    createPersonalTaskAndPromoteToProviderTask: {\n      argNames: ["taskName", "description", "localizedStartDate", "localizedDueDate", "completed", "pinned", "locationKey"]\n    },\n    createTask: {\n      argNames: ["taskName", "description", "localizedStartDate", "localizedDueDate", "completed", "pinned", "locationKey", "editUrl"]\n    },\n    deleteTask: {\n      argNames: ["taskKey"]\n    },\n    getCalloutInfo: {\n      argNames: ["taskKey"]\n    },\n    getRefreshHealthInfo: {},\n    getRefreshHistory: {\n      argNames: ["since"]\n    },\n    getRefreshStatus: {\n      argNames: ["refreshId"]\n    },\n    isExchangeJobPending: {},\n    pinTask: {\n      argNames: ["taskKey"]\n    },\n    promotePersonalTaskToProviderTaskInLocation: {\n      argNames: ["taskKey", "locationId"]\n    },\n    readAllNonTaskData: {},\n    refreshSingleTask: {\n      argNames: ["taskKey"]\n    },\n    removeAttributeFromTask: {\n      argNames: ["taskKey", "attribute"]\n    },\n    removePinOnTask: {\n      argNames: ["taskKey"]\n    },\n    reorderTask: {\n      argNames: ["taskKey", "newAfterTaskKey"]\n    },\n    updateTaskWithLocalizedValue: {\n      argNames: ["taskKey", "field", "value"]\n    }\n  },\n  "SP.WorkManagement.OM.UserOrderedSessionManager": {\n    createLocationOrientedSession: {},\n    createSession: {}\n  },\n  "SP.WorkManagement.OM.UserSettingsManager": {\n    getAllLocations: {},\n    getExchangeSyncInfo: {},\n    getImportantLocations: {},\n    getLocations: {\n      argNames: ["locationsId"]\n    },\n    getPersistedProperties: {},\n    getUserSettings: {},\n    isExchangeJobPending: {},\n    optIntoExchangeSync: {},\n    optOutOfExchangeSync: {}\n  },\n  "SP.Workflow.SPWorkflowTask": {\n    addThumbnailFieldData: {\n      argNames: ["imageStream", "imageName", "fieldInternalName", "lockId"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    archive: {},\n    attachImage: {\n      argNames: ["imageStream", "imageName", "fieldInternalName"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    breakRoleInheritance: {\n      argNames: ["copyRoleAssignments", "clearSubscopes"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    deleteWithParameters: {\n      argNames: ["parameters"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    getChanges: {\n      argNames: ["query"],\n      metadataType: "SP.ChangeQuery",\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    getComments: {\n      returnType: "Microsoft.SharePoint.Comments.comment.Collection"\n    },\n    getUserEffectivePermissions: {\n      argNames: ["userName"],\n      name: "getUserEffectivePermissions(@user)?@user=\'[[userName]]\'",\n      requestType: utils_1.RequestType.GetReplace\n    },\n    getVersions: {\n      argNames: ["getVersionsParams"]\n    },\n    getWOPIFrameUrl: {\n      argNames: ["action"],\n      requestType: utils_1.RequestType.PostWithArgsInQS\n    },\n    mediaServiceUpdate: {\n      argNames: ["parameters"]\n    },\n    mediaServiceUpdateV2: {\n      argNames: ["parameters", "eventFiringEnabled"]\n    },\n    overridePolicyTip: {\n      argNames: ["userAction", "justification"]\n    },\n    parseAndSetFieldValue: {\n      argNames: ["fieldName", "value"]\n    },\n    recycle: {\n      requestType: utils_1.RequestType.Post\n    },\n    recycleWithParameters: {\n      argNames: ["parameters"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    resetRoleInheritance: {},\n    setCommentsDisabled: {\n      argNames: ["value"],\n      requestType: utils_1.RequestType.PostWithArgsInBody\n    },\n    setComplianceTag: {\n      argNames: ["complianceTag", "isTagPolicyHold", "isTagPolicyRecord", "isEventBasedTag", "isTagSuperLock", "isUnlockedAsDefault"]\n    },\n    setComplianceTagWithExplicitMetasUpdate: {\n      argNames: ["complianceTag", "complianceFlags", "complianceTagWrittenTime", "userEmailAddress"]\n    },\n    setComplianceTagWithHold: {\n      argNames: ["complianceTag"]\n    },\n    setComplianceTagWithMetaInfo: {\n      argNames: ["complianceTag", "blockDelete", "blockEdit", "complianceTagWrittenTime", "userEmailAddress", "isTagSuperLock", "isRecordUnlockedAsDefault"]\n    },\n    setComplianceTagWithNoHold: {\n      argNames: ["complianceTag"]\n    },\n    setComplianceTagWithRecord: {\n      argNames: ["complianceTag"]\n    },\n    systemUpdate: {},\n    unarchive: {},\n    update: {\n      argNames: ["properties"],\n      metadataType: "SP.Workflow.SPWorkflowTask",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    updateEx: {\n      argNames: ["parameters"]\n    },\n    updateOverwriteVersion: {},\n    validateUpdateFetchListItem: {\n      argNames: ["formValues", "bNewDocumentUpdate", "checkInComment", "datesInUTC", "numberInInvariantCulture", "View", "RootFolder"]\n    },\n    validateUpdateFetchListItemInFolder: {\n      argNames: ["formValues", "bNewDocumentUpdate", "checkInComment", "datesInUTC", "numberInInvariantCulture", "rootFolder"]\n    },\n    validateUpdateListItem: {\n      argNames: ["formValues", "bNewDocumentUpdate", "checkInComment", "datesInUTC", "numberInInvariantCulture", "sharedLockId"]\n    }\n  },\n  "SP.Workflow.WorkflowAssociation": {\n    "delete": {\n      requestType: utils_1.RequestType.Delete\n    },\n    update: {\n      metadataType: "",\n      name: "",\n      requestMethod: "MERGE",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  "SP.Workflow.WorkflowAssociation.Collection": {\n    add: {\n      argNames: ["parameters"]\n    },\n    getById: {\n      argNames: ["associationId"]\n    },\n    getByName: {\n      argNames: ["name"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.Workflow.WorkflowTemplate.Collection": {\n    getById: {\n      argNames: ["templateId"]\n    },\n    getByName: {\n      argNames: ["name"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.WorkflowServices.InteropService": {\n    cancelWorkflow: {\n      argNames: ["instanceId"]\n    },\n    disableEvents: {\n      argNames: ["listId", "itemGuid"]\n    },\n    enableEvents: {\n      argNames: ["listId", "itemGuid"]\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    startWorkflow: {\n      argNames: ["associationName", "correlationId", "listId", "itemGuid", "workflowParameters"]\n    }\n  },\n  "SP.WorkflowServices.WorkflowDefinition": {\n    setProperty: {\n      argNames: ["propertyName", "value"]\n    }\n  },\n  "SP.WorkflowServices.WorkflowDefinition.Collection": {\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    sort: {}\n  },\n  "SP.WorkflowServices.WorkflowDeploymentService": {\n    deleteCollateral: {\n      argNames: ["workflowDefinitionId", "leafFileName"]\n    },\n    deleteDefinition: {\n      argNames: ["definitionId"]\n    },\n    deprecateDefinition: {\n      argNames: ["definitionId"]\n    },\n    enumerateDefinitions: {\n      argNames: ["publishedOnly"]\n    },\n    enumerateIntegratedApps: {},\n    getActivitySignatures: {\n      argNames: ["lastChanged"]\n    },\n    getCollateralUri: {\n      argNames: ["workflowDefinitionId", "leafFileName"]\n    },\n    getDefinition: {\n      argNames: ["definitionId"]\n    },\n    isIntegratedApp: {},\n    packageDefinition: {\n      argNames: ["definitionId", "packageDefaultFilename", "packageTitle", "packageDescription"]\n    },\n    publishDefinition: {\n      argNames: ["definitionId"]\n    },\n    saveCollateral: {\n      argNames: ["workflowDefinitionId", "leafFileName", "fileContent"]\n    },\n    validateActivity: {\n      argNames: ["activityXaml"]\n    }\n  },\n  "SP.WorkflowServices.WorkflowInstanceService": {\n    enumerateInstancesForListItem: {\n      argNames: ["listId", "itemId"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    enumerateInstancesForListItemWithOffset: {\n      argNames: ["listId", "itemId", "offset"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    enumerateInstancesForSite: {\n      requestType: utils_1.RequestType.Post\n    },\n    enumerateInstancesForSiteWithOffset: {\n      argNames: ["offset"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    getInstance: {\n      argNames: ["instanceId"],\n      requestType: utils_1.RequestType.Get\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    startWorkflowOnListItemBySubscriptionId: {\n      argNames: ["subscriptionId", "itemId", "payload"],\n      requestType: utils_1.RequestType.PostWithArgs\n    }\n  },\n  "SP.WorkflowServices.WorkflowMessagingService": {\n    publishEvent: {\n      argNames: ["eventSourceId", "eventName", "payload"]\n    }\n  },\n  "SP.WorkflowServices.WorkflowServicesManager": {\n    getWorkflowDeploymentService: {},\n    getWorkflowInstanceService: {},\n    getWorkflowInteropService: {},\n    getWorkflowSubscriptionService: {},\n    isIntegratedApp: {},\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  "SP.WorkflowServices.WorkflowSubscription": {\n    getExternalVariable: {\n      argNames: ["name"]\n    },\n    setExternalVariable: {\n      argNames: ["name", "value"]\n    },\n    setProperty: {\n      argNames: ["name", "value"]\n    }\n  },\n  "SP.WorkflowServices.WorkflowSubscription.Collection": {\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    sort: {}\n  },\n  "SP.WorkflowServices.WorkflowSubscriptionService": {\n    deleteSubscription: {\n      argNames: ["subscriptionId"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    enumerateSubscriptions: {\n      requestType: utils_1.RequestType.Post\n    },\n    enumerateSubscriptionsByDefinition: {\n      argNames: ["definitionId"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    enumerateSubscriptionsByEventSource: {\n      argNames: ["eventSourceId"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    enumerateSubscriptionsByList: {\n      argNames: ["listId"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    enumerateSubscriptionsByListAndParentContentType: {\n      argNames: ["listId", "parentContentTypeId", "includeNoContentTypeSpecified"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    enumerateSubscriptionsByListWithContentType: {\n      argNames: ["listId", "includeContentTypeSpecified"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    getSubscription: {\n      argNames: ["subscriptionId"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    registerInterestInHostWebList: {\n      argNames: ["listId", "eventName"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    registerInterestInList: {\n      argNames: ["listId", "eventName"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    unregisterInterestInHostWebList: {\n      argNames: ["listId", "eventName"],\n      requestType: utils_1.RequestType.PostWithArgs\n    },\n    unregisterInterestInList: {\n      argNames: ["listId", "eventName"],\n      requestType: utils_1.RequestType.PostWithArgs\n    }\n  }\n};\n\n//# sourceURL=webpack://gd-sprest/./build/mapper/def.js?')},"./build/mapper/index.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.Mapper_Custom = exports.MapperV2 = exports.Mapper = void 0; // TO DO:\n// 1) Merge mapper files into mapper.ts file\n// 2) Reference the main mapper.ts during the merge, so the reference is still there\n// 3) Update the method adder helper function to reference the mapper using the raw metadata type, use old code as a backup\n// 4) Test if mapper files is still needed (Lots of testing here...), but references will ensure library won\'t break\n// 5) Remove the mapper files and code used as a backup\n\nvar Mapper_Custom = __webpack_require__(/*! ./custom */ "./build/mapper/custom/index.js");\n\nexports.Mapper_Custom = Mapper_Custom;\n\nvar def_1 = __webpack_require__(/*! ./def */ "./build/mapper/def.js");\n\nObject.defineProperty(exports, "Mapper", ({\n  enumerable: true,\n  get: function get() {\n    return def_1.Mapper;\n  }\n}));\n\nvar v2_1 = __webpack_require__(/*! ./v2 */ "./build/mapper/v2.js");\n\nObject.defineProperty(exports, "MapperV2", ({\n  enumerable: true,\n  get: function get() {\n    return v2_1.MapperV2;\n  }\n}));\n\n//# sourceURL=webpack://gd-sprest/./build/mapper/index.js?')},"./build/mapper/v2.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.MapperV2 = void 0;\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js");\n/**\r\n * Mapper for the v2 api.\r\n */\n\n\nexports.MapperV2 = {\n  activity: {\n    driveItem: {\n      requestType: utils_1.RequestType.Get,\n      returnType: "driveItem"\n    },\n    listItem: {\n      requestType: utils_1.RequestType.Get,\n      returnType: "listItem"\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  column: {\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    update: {\n      args: ["values"],\n      name: "",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  columns: {\n    add: {\n      args: ["values"],\n      name: "",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  contentType: {\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    update: {\n      args: ["values"],\n      name: "",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    }\n  },\n  contentTypes: {\n    add: {\n      args: ["values"],\n      name: "",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  drive: {\n    activities: {\n      requestType: utils_1.RequestType.Get\n    },\n    activitycounts: {\n      requestType: utils_1.RequestType.Get\n    },\n    apps: {\n      requestType: utils_1.RequestType.Get\n    },\n    commands: {\n      requestType: utils_1.RequestType.Get\n    },\n    items: {\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "items"\n    },\n    list: {\n      requestType: utils_1.RequestType.Get,\n      returnType: "list"\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    special: {\n      requestType: utils_1.RequestType.Get\n    },\n    trackedItems: {\n      requestType: utils_1.RequestType.Get\n    }\n  },\n  item: {\n    permissions: {\n      requestType: utils_1.RequestType.Get\n    },\n    children: {\n      requestType: utils_1.RequestType.Get\n    },\n    subscriptions: {\n      requestType: utils_1.RequestType.Get\n    },\n    thumbnails: {\n      requestType: utils_1.RequestType.Get\n    },\n    activities: {\n      requestType: utils_1.RequestType.Get\n    },\n    analytics: {\n      requestType: utils_1.RequestType.Get\n    },\n    commands: {\n      requestType: utils_1.RequestType.Get\n    },\n    featureStatus: {\n      requestType: utils_1.RequestType.Get\n    },\n    listItem: {\n      requestType: utils_1.RequestType.Get\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    update: {\n      args: ["values"],\n      name: "",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    versions: {\n      requestType: utils_1.RequestType.Get\n    }\n  },\n  list: {\n    activities: {\n      requestType: utils_1.RequestType.Get\n    },\n    columns: {\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "columns"\n    },\n    contentTypes: {\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "contentTypes"\n    },\n    drive: {\n      requestType: utils_1.RequestType.Get,\n      returnType: "drive"\n    },\n    items: {\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "listItems"\n    },\n    update: {\n      args: ["values"],\n      name: "",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  listItem: {\n    activities: {\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "activities"\n    },\n    analytics: {\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "analytics"\n    },\n    driveItem: {\n      requestType: utils_1.RequestType.Get,\n      returnType: "driveItem"\n    },\n    fields: {\n      appendRequest: true,\n      requestType: utils_1.RequestType.Get,\n      name: "([base.id])/fields",\n      returnType: "fields"\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    permissions: {\n      requestType: utils_1.RequestType.Get\n    },\n    update: {\n      args: ["values"],\n      name: "",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    versions: {\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "versions"\n    }\n  },\n  listItems: {\n    add: {\n      args: ["values"],\n      name: "",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  listItemVersion: {\n    fields: {\n      requestType: utils_1.RequestType.Get\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  lists: {\n    add: {\n      args: ["values"],\n      name: "",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    analytics: {\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "analytics"\n    },\n    columns: {\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "columns"\n    },\n    contentTypes: {\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "contentTypes"\n    },\n    drive: {\n      requestType: utils_1.RequestType.Get,\n      returnType: "drive"\n    },\n    getById: {\n      argNames: ["id"],\n      name: "[[id]]",\n      requestType: utils_1.RequestType.GetReplace,\n      returnType: "list"\n    },\n    getByTitle: {\n      argNames: ["title"],\n      name: "[[title]]",\n      requestType: utils_1.RequestType.GetReplace,\n      returnType: "list"\n    },\n    items: {\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "listItems"\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  permission: {\n    update: {\n      args: ["values"],\n      name: "",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  permissions: {\n    add: {\n      args: ["values"],\n      name: "",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    }\n  },\n  sites: {\n    add: {\n      args: ["values"],\n      name: "",\n      requestType: utils_1.RequestType.PostBodyNoArgs\n    },\n    analytics: {\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "analytics"\n    },\n    columns: {\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "columns"\n    },\n    contentTypes: {\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "contentTypes"\n    },\n    drive: {\n      requestType: utils_1.RequestType.Get,\n      returnType: "drive"\n    },\n    drives: {\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "drives"\n    },\n    items: {\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "listItems"\n    },\n    lists: {\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "lists"\n    },\n    query: {\n      argNames: ["oData"],\n      requestType: utils_1.RequestType.OData\n    },\n    permissions: {\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "permissions"\n    },\n    sites: {\n      requestType: utils_1.RequestType.GetWithArgsValueOnly,\n      returnType: "sites"\n    }\n  }\n};\n\n//# sourceURL=webpack://gd-sprest/./build/mapper/v2.js?')},"./build/rest.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.$REST = void 0;\n\nvar Helper = __webpack_require__(/*! ./helper */ "./build/helper/index.js");\n\nvar Lib = __webpack_require__(/*! ./lib */ "./build/lib/index.js");\n\nvar LibV2 = __webpack_require__(/*! ./v2 */ "./build/v2/index.js");\n\nvar sptypes_1 = __webpack_require__(/*! ./sptypes */ "./build/sptypes/index.js");\n/**\r\n * SharePoint REST Library\r\n */\n\n\nexports.$REST = {\n  __ver: 8.33,\n  AppContext: function AppContext(siteUrl) {\n    return Lib.Site.getAppContext(siteUrl);\n  },\n  Apps: Lib.Apps,\n  ContextInfo: Lib.ContextInfo,\n  DefaultRequestToHostFl: false,\n  drive: LibV2.drive,\n  drives: LibV2.drives,\n  GetWebUrlFromPageUrl: Lib.Web.getWebUrlFromPageUrl,\n  Graph: Lib.Graph,\n  GroupService: Lib.GroupService,\n  GroupSiteManager: Lib.GroupSiteManager,\n  Helper: Helper,\n  HubSites: Lib.HubSites,\n  HubSitesUtility: Lib.HubSitesUtility,\n  List: Lib.List,\n  ListByEntityName: Lib.List.getByEntityName,\n  ListDataAsStream: Lib.List.getDataAsStream,\n  Navigation: Lib.Navigation,\n  PeopleManager: Lib.PeopleManager,\n  PeoplePicker: Lib.PeoplePicker,\n  ProfileLoader: Lib.ProfileLoader,\n  RemoteWeb: function RemoteWeb(requestUrl) {\n    return Lib.Web.getRemoteWeb(requestUrl);\n  },\n  Search: Lib.Search,\n  Site: Lib.Site,\n  SiteIconManager: Lib.SiteIconManager,\n  SiteManager: Lib.SiteManager,\n  SitePages: Lib.SitePages,\n  SiteExists: function SiteExists(url) {\n    return Lib.Site.exists(url);\n  },\n  sites: LibV2.sites,\n  SiteUrl: function SiteUrl(id) {\n    return Lib.Site.getUrlById(id);\n  },\n  SPTypes: sptypes_1.SPTypes,\n  SocialFeed: Lib.SocialFeed,\n  ThemeManager: Lib.ThemeManager,\n  UserProfile: Lib.UserProfile,\n  Utility: Lib.Utility,\n  Web: Lib.Web,\n  WebTemplateExtensions: Lib.WebTemplateExtensions,\n  WorkflowInstanceService: Lib.WorkflowInstanceService,\n  WorkflowSubscriptionService: Lib.WorkflowSubscriptionService\n}; // See if the library doesn\'t exist, or is an older version\n\nvar global = Lib.ContextInfo.window.$REST;\n\nif (global == null || global.__ver == null || global.__ver < exports.$REST.__ver) {\n  // Set the global variable\n  Lib.ContextInfo.window.$REST = exports.$REST; // Ensure the SP lib exists\n\n  if (Lib.ContextInfo.window.SP) {\n    // If MDS is turned on in a SP2013 environment, it may throw an error\n    try {\n      // Alert other scripts this library is loaded\n      Lib.ContextInfo.window.SP.SOD.notifyScriptLoadedAndExecuteWaitingJobs("gd-sprest");\n      Lib.ContextInfo.window.SP.SOD.notifyScriptLoadedAndExecuteWaitingJobs("gd-sprest.js");\n    } catch (_a) {\n      // Log\n      console.error("[gd-sprest] Error notifying scripts using the SP SOD library.");\n    }\n  }\n}\n\n//# sourceURL=webpack://gd-sprest/./build/rest.js?')},"./build/sptypes/index.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.SPTypes = void 0;\n\nvar SPTypes = __webpack_require__(/*! ./sptypes */ "./build/sptypes/sptypes.js");\n\nexports.SPTypes = SPTypes;\n\n//# sourceURL=webpack://gd-sprest/./build/sptypes/index.js?')},"./build/sptypes/sptypes.js":function(__unused_webpack_module,exports){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.WebTemplateType = exports.ViewType = exports.UserCustomActionRegistrationType = exports.URLZones = exports.UrlFormatType = exports.StatusPriColor = exports.RoleType = exports.ReorderingRuleMatchType = exports.RenderListDataOptions = exports.RelationshipDeleteBehaviorType = exports.PropertyPaneType = exports.PrincipalTypes = exports.PrincipalSources = exports.PersonalSiteCapabilities = exports.PersonalizationScope = exports.PageType = exports.ModalDialogResult = exports.LocaleLCIDType = exports.ListTemplateType = exports.ListExperienceOptions = exports.FriendlyDateFormat = exports.FormDisplayMode = exports.FileTemplateType = exports.FileLevelType = exports.FieldUserSelectionType = exports.FieldType = exports.FieldResultType = exports.FieldNumberType = exports.FieldNoteType = exports.FieldIndexStatus = exports.EnvironmentType = exports.EventReceiverType = exports.EventReceiverSynchronizationType = exports.DraftVisibilityType = exports.DisplayMode = exports.DateFormat = exports.ControlMode = exports.CloudEnvironment = exports.ClientTemplatesUtility = exports.ClientSidePageLayout = exports.ChoiceFormatType = exports.CheckOutType = exports.CheckInType = exports.CalendarTypes = exports.BasePermissionTypes = void 0;\n/**\r\n * Base Permission Types\r\n */\n\nexports.BasePermissionTypes = {\n  EmptyMask: 0,\n  ViewListItems: 1,\n  AddListItems: 2,\n  EditListItems: 3,\n  DeleteListItems: 4,\n  ApproveItems: 5,\n  OpenItems: 6,\n  ViewVersions: 7,\n  DeleteVersions: 8,\n  CancelCheckout: 9,\n  ManagePersonalViews: 10,\n  ManageLists: 12,\n  ViewFormPages: 13,\n  AnonymousSearchAccessList: 14,\n  Open: 17,\n  ViewPages: 18,\n  AddAndCustomizePages: 19,\n  ApplyThemeAndBorder: 20,\n  ApplyStyleSheets: 21,\n  ViewUsageData: 22,\n  CreateSSCSite: 23,\n  ManageSubwebs: 24,\n  CreateGroups: 25,\n  ManagePermissions: 26,\n  BrowseDirectories: 27,\n  BrowseUserInfo: 28,\n  AddDelPrivateWebParts: 29,\n  UpdatePersonalWebParts: 30,\n  ManageWeb: 31,\n  AnonymousSearchAccessWebLists: 32,\n  UseClientIntegration: 37,\n  UseRemoteAPIs: 38,\n  ManageAlerts: 39,\n  CreateAlerts: 40,\n  EditMyUserInfo: 41,\n  EnumeratePermissions: 63,\n  FullMask: 65\n};\n/**\r\n * Calendar Types\r\n */\n\nexports.CalendarTypes = {\n  Gregorian: 1,\n  JapaneseEmperorEra: 3,\n  TaiwanCalendar: 4,\n  KoreanTangunEra: 5,\n  Hijri: 6,\n  Thai: 7,\n  HebrewLunar: 8,\n  GregorianMiddleEastFrench: 9,\n  GregorianArabic: 10,\n  GregorianTransliteratedEnglish: 11,\n  GregorianTransliteratedFrench: 12,\n  KoreanandJapaneseLunar: 14,\n  ChineseLunar: 15,\n  SakaEra: 16\n};\n/**\r\n * Check In Types\r\n */\n\nexports.CheckInType = {\n  MinorCheckIn: 0,\n  MajorCheckIn: 1,\n  OverwriteCheckIn: 2\n};\n/**\r\n * Check Out Types\r\n */\n\nexports.CheckOutType = {\n  Online: 0,\n  Offline: 1,\n  None: 2\n};\n/**\r\n * Choice Format Types\r\n */\n\nexports.ChoiceFormatType = {\n  Dropdown: 0,\n  RadioButtons: 1\n};\n/**\r\n * Client Side Templates\r\n */\n\nexports.ClientSidePageLayout = {\n  Article: "Article",\n  HeaderlessSearchResults: "HeaderlessSearchResults",\n  Home: "Home",\n  RepostPage: "RepostPage",\n  SingleWebPartAppPage: "SingleWebPartAppPage",\n  Spaces: "Spaces",\n  Topic: "Topic"\n};\n/**\r\n * Client Template Utility\r\n */\n\nexports.ClientTemplatesUtility = {\n  UserLookupDelimitString: ";#",\n  UserMultiValueDelimitString: ",#"\n};\n/**\r\n * Cloud Environments\r\n */\n\nexports.CloudEnvironment = {\n  China: "https://microsoftgraph.chinacloudapi.cn",\n  Default: "https://graph.microsoft.com",\n  Flow: "https://service.flow.microsoft.com/",\n  FlowAPI: "https://api.flow.microsoft.com/",\n  FlowChina: "https://service.powerautomate.cn/",\n  FlowChinaAPI: "https://api.powerautomate.cn/",\n  FlowDoD: "https://service.flow.appsplatform.us/",\n  FlowDoDAPI: "https://api.flow.appsplatform.us/",\n  FlowGov: "https://gov.service.flow.microsoft.us/",\n  FlowGovAPI: "https://gov.api.flow.microsoft.us/",\n  FlowHigh: "https://high.service.flow.microsoft.us/",\n  FlowHighAPI: "https://high.api.flow.microsoft.us/",\n  FlowUSNat: "https://service.flow.eaglex.ic.gov/",\n  FlowUSNatAPI: "https://api.flow.eaglex.ic.gov/",\n  FlowUSSec: "https://service.flow.microsoft.scloud/",\n  FlowUSSecAPI: "https://api.flow.microsoft.scloud/",\n  Office: "https://substrate.office.com",\n  USL4: "https://graph.microsoft.us",\n  USL5: "https://dod-graph.microsoft.us"\n};\n/**\r\n * Control Modes\r\n */\n\nexports.ControlMode = {\n  Invalid: 0,\n  Display: 1,\n  Edit: 2,\n  New: 3,\n  View: 4\n};\n/**\r\n * Date Format\r\n */\n\nexports.DateFormat = {\n  DateOnly: 0,\n  DateTime: 1\n};\n/**\r\n * Display Mode\r\n */\n\nexports.DisplayMode = {\n  Edit: 2,\n  Read: 1\n};\n/**\r\n * Draft Visibility Types\r\n */\n\nexports.DraftVisibilityType = {\n  Reader: 0,\n  Author: 1,\n  Approver: 2\n};\n/**\r\n * Event Receiver Synchronization Types\r\n */\n\nexports.EventReceiverSynchronizationType = {\n  Synchronization: 1,\n  Asynchronous: 2\n};\n/**\r\n * Event Receiver Types\r\n */\n\nexports.EventReceiverType = {\n  ItemAdding: 1,\n  ItemUpdating: 2,\n  ItemDeleting: 3,\n  ItemCheckingIn: 4,\n  ItemCheckingOut: 5,\n  ItemUncheckingOut: 6,\n  ItemAttachmentAdding: 7,\n  ItemAttachmentDeleting: 8,\n  ItemFileMoving: 9,\n  ItemVersionDeleting: 11,\n  FieldAdding: 101,\n  FieldUpdating: 102,\n  FieldDeleting: 103,\n  ListAdding: 104,\n  ListDeleting: 105,\n  SiteDeleting: 201,\n  WebDeleting: 202,\n  WebMoving: 203,\n  WebAdding: 204,\n  GroupAdding: 301,\n  GroupUpdating: 302,\n  GroupDeleting: 303,\n  GroupUserAdding: 304,\n  GroupUserDeleting: 305,\n  RoleDefinitionAdding: 306,\n  RoleDefinitionUpdating: 307,\n  RoleDefinitionDeleting: 308,\n  RoleAssignmentAdding: 309,\n  RoleAssignmentDeleting: 310,\n  InheritanceBreaking: 311,\n  InheritanceResetting: 312,\n  WorkflowStarting: 501,\n  ItemAdded: 10001,\n  ItemUpdated: 10002,\n  ItemDeleted: 10003,\n  ItemCheckedIn: 10004,\n  ItemCheckedOut: 10005,\n  ItemUncheckedOut: 10006,\n  ItemAttachmentAdded: 10007,\n  ItemAttachmentDeleted: 10008,\n  ItemFileMoved: 10009,\n  ItemFileConverted: 10010,\n  ItemVersionDeleted: 10011,\n  FieldAdded: 10101,\n  FieldUpdated: 10102,\n  FieldDeleted: 10103,\n  ListAdded: 10104,\n  ListDeleted: 10105,\n  SiteDeleted: 10201,\n  WebDeleted: 10202,\n  WebMoved: 10203,\n  WebProvisioned: 10204,\n  GroupAdded: 10301,\n  GroupUpdated: 10302,\n  GroupDeleted: 10303,\n  GroupUserAdded: 10304,\n  GroupUserDeleted: 10305,\n  RoleDefinitionAdded: 10306,\n  RoleDefinitionUpdated: 10307,\n  RoleDefinitionDeleted: 10308,\n  RoleAssignmentAdded: 10309,\n  RoleAssignmentDeleted: 10310,\n  InheritanceBroken: 10311,\n  InheritanceReset: 10312,\n  WorkflowStarted: 10501,\n  WorkflowPostponed: 10502,\n  WorkflowCompleted: 10503,\n  EntityInstanceAdded: 10601,\n  EntityInstanceUpdated: 10602,\n  EntityInstanceDeleted: 10603,\n  AppInstalled: 10701,\n  AppUpgraded: 10702,\n  AppUninstalling: 10703,\n  EmailReceived: 20000,\n  ContextEvent: 32766\n};\n/**\r\n * Environment Type\r\n */\n\nexports.EnvironmentType = {\n  ClassicSharePoint: 3,\n  Local: 1,\n  SharePoint: 2,\n  Test: 0\n};\n/**\r\n * Field Index Status\r\n */\n\nexports.FieldIndexStatus = {\n  None: 0,\n  Indexed: 1,\n  Enabling: 2,\n  Disabling: 3\n};\n/**\r\n * Field Note Types\r\n */\n\nexports.FieldNoteType = {\n  /** Enhance Rich Text */\n  EnhancedRichText: 0,\n\n  /** Rich Text */\n  RichText: 1,\n\n  /** Text Only */\n  TextOnly: 2\n};\n/**\r\n * Field Number Type\r\n */\n\nexports.FieldNumberType = {\n  /** Decimal */\n  Decimal: 0,\n\n  /** Integer */\n  Integer: 1,\n\n  /** Percentage */\n  Percentage: 2\n};\n/**\r\n * Field Result Types\r\n */\n\nexports.FieldResultType = {\n  /** Boolean */\n  Boolean: "Boolean",\n\n  /** Currency */\n  Currency: "Currency",\n\n  /** Date Only */\n  DateOnly: "DateOnly",\n\n  /** Date & Time */\n  DateTime: "DateTime",\n\n  /** Number */\n  Number: "Number",\n\n  /** Text */\n  Text: "Text"\n};\n/**\r\n * Field Types\r\n */\n\nexports.FieldType = {\n  AllDayEvent: 29,\n  Attachments: 19,\n  Boolean: 8,\n  Calculated: 17,\n  Choice: 6,\n  Computed: 12,\n  ContentTypeId: 25,\n  Counter: 5,\n  CrossProjectLink: 22,\n  Currency: 10,\n  DateTime: 4,\n  Error: 24,\n  File: 18,\n  Geolocation: 31,\n  GridChoice: 16,\n  Guid: 14,\n  Image: 34,\n  Integer: 1,\n  Invalid: 0,\n  Lookup: 7,\n  MaxItems: 31,\n  ModStat: 23,\n  MultiChoice: 15,\n  Note: 3,\n  Number: 9,\n  PageSeparator: 26,\n  Recurrence: 21,\n  Text: 2,\n  ThreadIndex: 27,\n  Threading: 13,\n  URL: 11,\n  User: 20,\n  WorkflowEventType: 30,\n  WorkflowStatus: 28\n};\n/**\r\n * Field User Selection Types\r\n */\n\nexports.FieldUserSelectionType = {\n  PeopleOnly: 0,\n  PeopleAndGroups: 1\n};\n/**\r\n * File Level\r\n */\n\nexports.FileLevelType = {\n  Published: 1,\n  Draft: 2,\n  Checkout: 3\n};\n/**\r\n * File Template Types\r\n*/\n\nexports.FileTemplateType = {\n  StandardPage: 0,\n  WikiPage: 1,\n  FormPage: 2\n};\n/**\r\n * Form Display Mode\r\n */\n\nexports.FormDisplayMode = {\n  Display: 4,\n  Edit: 6,\n  New: 8\n};\n/**\r\n * Friendly Date Format\r\n */\n\nexports.FriendlyDateFormat = {\n  Unspecified: 0,\n  Disabled: 1,\n  Relative: 2\n};\n/**\r\n * List Experience Types\r\n */\n\nexports.ListExperienceOptions = {\n  Auto: 0,\n  NewExperience: 1,\n  ClassicExperience: 2\n};\n/**\r\n * List Template Types\r\n*/\n\nexports.ListTemplateType = {\n  AccessApp: 3100,\n  AccessRequest: 160,\n  AdminTasks: 1200,\n  Agenda: 201,\n  AlchemyApprovalWorkflow: 3102,\n  AlchemyMobileForm: 3101,\n  Announcements: 104,\n  AppCatalog: 336,\n  AppDataCatalog: 125,\n  AssetLibrary: 851,\n  CallTrack: 404,\n  Categories: 303,\n  Circulation: 405,\n  ClientSideAssets: 334,\n  ClientSideComponentManifests: 331,\n  Comments: 302,\n  Contacts: 105,\n  CustomGrid: 120,\n  DataConnectionLibrary: 130,\n  DataSources: 110,\n  Decision: 204,\n  DesignCatalog: 124,\n  DeveloperSiteDraftApps: 1230,\n  DiscussionBoard: 108,\n  DocumentLibrary: 101,\n  Events: 106,\n  ExternalList: 600,\n  Facility: 402,\n  GanttTasks: 150,\n  GenericList: 100,\n  HealthReports: 1221,\n  HealthRules: 1220,\n  HelpLibrary: 151,\n  Holidays: 421,\n  HomePageLibrary: 212,\n  IMEDic: 499,\n  IssueTracking: 1100,\n  KPIStatusList: 432,\n  LanguagesAndTranslatorsList: 1301,\n  Links: 103,\n  ListTemplateCatalog: 114,\n  MaintenanceLogs: 175,\n  MasterPageCatalog: 116,\n  MeetingObjective: 207,\n  Meetings: 200,\n  MeetingUser: 202,\n  MicroFeed: 544,\n  MySiteDocumentLibrary: 700,\n  NoCodePublic: 122,\n  NoCodeWorkflows: 117,\n  PageLibrary: 850,\n  PerformancePointContentList: 450,\n  PerformancePointDataSourceLibrary: 460,\n  PerformancePointDataConnectionsLibrary: 470,\n  PerformancePointDashboardsLibrary: 480,\n  PersonalDocumentLibrary: 2002,\n  PictureLibrary: 109,\n  Posts: 301,\n  PrivateDocumentLibrary: 2003,\n  RecordLibrary: 1302,\n  ReportLibrary: 433,\n  SharingLinks: 3300,\n  SolutionCatalog: 121,\n  Survey: 102,\n  Tasks: 107,\n  TasksWithTimelineAndHierarchy: 171,\n  TenantAppCatalog: 330,\n  TenantWideExtensions: 337,\n  TextBox: 210,\n  ThemeCatalog: 123,\n  ThingsToBring: 211,\n  Timecard: 420,\n  TranslationManagementLibrary: 1300,\n  UserInformation: 112,\n  VisioProcessDiagramMetricLibrary: 505,\n  VisioProcessDiagramUSUnitsLibrary: 506,\n  WebPageLibrary: 119,\n  WebPartCatalog: 113,\n  WebTemplateCatalog: 111,\n  Whereabouts: 403,\n  WorkflowHistory: 140,\n  WorkflowProcess: 118,\n  XMLForm: 115\n};\n/**\r\n * Locale LCID Types\r\n */\n\nexports.LocaleLCIDType = {\n  Afrikaans: 1078,\n  Albanian: 1052,\n  ArabicAlgeria: 5121,\n  ArabicBahrain: 15361,\n  ArabicEgypt: 3073,\n  ArabicIraq: 2049,\n  ArabicJordan: 11265,\n  ArabicLebanon: 12289,\n  ArabicLibya: 4097,\n  ArabicMorocco: 6145,\n  ArabicOman: 8193,\n  ArabicQatar: 16385,\n  ArabicSaudiArabia: 1025,\n  ArabicSyria: 10241,\n  ArabicTunisia: 7169,\n  ArabicUAE: 14337,\n  ArabicYemen: 9217,\n  Armenian: 1067,\n  AzeriCyrillic: 2092,\n  AzeriLatin: 1068,\n  Basque: 1069,\n  Belarusian: 1059,\n  Bulgarian: 1026,\n  Catalan: 1027,\n  ChineseHongKongSAR: 3076,\n  ChineseMacaoSAR: 5124,\n  ChinesePRC: 2052,\n  ChineseSingapore: 4100,\n  ChineseTaiwan: 1028,\n  CroatianCroatia: 1050,\n  Czech: 1029,\n  Danish: 1030,\n  Divehi: 1125,\n  DutchBelgium: 2067,\n  DutchNetherlands: 1043,\n  EnglishAustralia: 3081,\n  EnglishBelize: 10249,\n  EnglishCanada: 4105,\n  EnglishCaribbean: 9225,\n  EnglishIreland: 6153,\n  EnglishJamaica: 8201,\n  EnglishNewZealand: 5129,\n  EnglishPhilippines: 13321,\n  EnglishSouthAfrica: 7177,\n  EnglishTrinidad: 11273,\n  EnglishUnitedKingdom: 2057,\n  EnglishUnitedStates: 1033,\n  EnglishZimbabwe: 12297,\n  Estonian: 1061,\n  Faeroese: 1080,\n  Finnish: 1035,\n  FrenchBelgium: 2060,\n  FrenchCanada: 3084,\n  FrenchFrance: 1036,\n  FrenchLuxembourg: 5132,\n  FrenchMonaco: 6156,\n  FrenchSwitzerland: 4108,\n  Galician: 1110,\n  Georgian: 1079,\n  GermanAustria: 3079,\n  GermanGermany: 1031,\n  GermanLiechtenstein: 5127,\n  GermanLuxembourg: 4103,\n  GermanSwitzerland: 2055,\n  Greek: 1032,\n  Gujarati: 1095,\n  HebrewIsrael: 1037,\n  HindiIndia: 1081,\n  Hungarian: 1038,\n  Icelandic: 1039,\n  Indonesian: 1057,\n  ItalianItaly: 1040,\n  ItalianSwitzerland: 2064,\n  Japanese: 1041,\n  Kannada: 1099,\n  Kazakh: 1087,\n  Konkani: 1111,\n  Korean: 1042,\n  KyrgyzCyrillic: 1088,\n  Latvian: 1062,\n  Lithuanian: 1063,\n  MacedonianFYROM: 1071,\n  Malay: 1086,\n  MalayBruneiDarussalam: 2110,\n  Marathi: 1102,\n  MongolianCyrillic: 1104,\n  NorwegianBokmal: 1044,\n  NorwegianNynorsk: 2068,\n  PersianIran: 1065,\n  Polish: 1045,\n  PortugueseBrazil: 1046,\n  PortuguesePortugal: 2070,\n  Punjabi: 1094,\n  Romanian: 1048,\n  Russian: 1049,\n  Sanskrit: 1103,\n  SerbianCyrillic: 3098,\n  SerbianLatin: 2074,\n  Slovak: 1051,\n  Slovenian: 1060,\n  SpanishArgentina: 11274,\n  SpanishBolivia: 16394,\n  SpanishChile: 13322,\n  SpanishColombia: 9226,\n  SpanishCostaRica: 5130,\n  SpanishDominicanRepublic: 7178,\n  SpanishEcuador: 12298,\n  SpanishElSalvador: 17418,\n  SpanishGuatemala: 4106,\n  SpanishHonduras: 18442,\n  SpanishMexico: 2058,\n  SpanishNicaragua: 19466,\n  SpanishPanama: 6154,\n  SpanishParaguay: 15370,\n  SpanishPeru: 10250,\n  SpanishPuertoRico: 20490,\n  SpanishSpain: 3082,\n  SpanishUruguay: 14346,\n  SpanishVenezuela: 8202,\n  Swahili: 1089,\n  Swedish: 1053,\n  SwedishFinland: 2077,\n  Syriac: 1114,\n  Tamil: 1097,\n  Tatar: 1092,\n  Telugu: 1098,\n  ThaiThailand: 1054,\n  Turkish: 1055,\n  Ukrainian: 1058,\n  UrduPakistan: 1056,\n  UzbekCyrillic: 2115,\n  UzbekLatin: 1091,\n  Vietnamese: 1066\n};\n/**\r\n * Modal Dialog Results\r\n */\n\nexports.ModalDialogResult = {\n  Invalid: -1,\n  Cancel: 0,\n  OK: 1\n};\n/**\r\n * Page Types\r\n */\n\nexports.PageType = {\n  DefaultView: 0,\n  DialogView: 2,\n  DisplayForm: 4,\n  DisplayFormDialog: 5,\n  EditForm: 6,\n  EditFormDialog: 7,\n  Invalid: -1,\n  NewForm: 8,\n  NewFormDialog: 9,\n  NormalView: 1,\n  Page_MAXITEMS: 11,\n  SolutionForm: 10,\n  View: 3\n};\n/**\r\n * Personalization Scope\r\n */\n\nexports.PersonalizationScope = {\n  Shared: 1,\n  User: 0\n};\n/**\r\n * Personal Site Capabilities\r\n */\n\nexports.PersonalSiteCapabilities = {\n  Education: 16,\n  Guest: 32,\n  MyTasksDashboard: 8,\n  None: 0,\n  Profile: 1,\n  Social: 2,\n  Storage: 4\n};\n/**\r\n * Principal Sources\r\n */\n\nexports.PrincipalSources = {\n  All: 15,\n  MembershipProvider: 4,\n  None: 0,\n  RoleProvider: 8,\n  UserInfoList: 1,\n  Windows: 2\n};\n/**\r\n * Principal Types\r\n */\n\nexports.PrincipalTypes = {\n  All: 15,\n  DistributionList: 2,\n  None: 0,\n  SecurityGroup: 4,\n  SharePointGroup: 8,\n  User: 1\n};\n/**\r\n * Property Pane Types\r\n */\n\nexports.PropertyPaneType = {\n  Button: 11,\n  CheckBox: 2,\n  ChoiceGroup: 10,\n  Custom: 1,\n  Dropdown: 6,\n  DynamicField: 14,\n  DynamicFieldSet: 16,\n  DynamicTextField: 15,\n  Heading: 9,\n  HorizontalRule: 12,\n  IconPicker: 19,\n  Label: 7,\n  Link: 13,\n  Slider: 8,\n  SpinButton: 17,\n  TextField: 3,\n  ThumbnailPicker: 18,\n  Toggle: 5\n};\n/**\r\n * Relationship Delete Behavior Types\r\n */\n\nexports.RelationshipDeleteBehaviorType = {\n  None: 0,\n  Cascade: 1,\n  Restrict: 2\n};\n/**\r\n * Render List Data Options\r\n */\n\nexports.RenderListDataOptions = {\n  None: 0,\n  ContextInfo: 1,\n  ListData: 2,\n  ListSchema: 4,\n  MenuView: 8,\n  ListContentType: 16,\n  FileSystemItemId: 32,\n  ClientFormSchema: 64,\n  QuickLaunch: 128,\n  Spotlight: 256,\n  Visualization: 512,\n  ViewMetadata: 1024,\n  DisableAutoHyperlink: 2048,\n  EnableMediaTAUrls: 4096,\n  ParentInfo: 8192,\n  PageContextInfo: 16384,\n  ClientSideComponentManifest: 32768\n};\n/**\r\n * Reordering Rule Match Types\r\n */\n\nexports.ReorderingRuleMatchType = {\n  ContentTypeIs: 5,\n  FileExtensionMatches: 6,\n  ManualCondition: 8,\n  ResultContainsKeyword: 0,\n  ResultHasTag: 7,\n  TitleContainsKeyword: 1,\n  TitleMatchesKeyword: 2,\n  UrlExactlyMatches: 4,\n  UrlStartsWith: 3\n};\n/**\r\n * Role Types\r\n */\n\nexports.RoleType = {\n  Administrator: 5,\n  Contributor: 3,\n  Editor: 6,\n  Guest: 1,\n  None: 0,\n  Reader: 2,\n  WebDesigner: 4\n};\n/**\r\n * Status Pri Color\r\n */\n\nexports.StatusPriColor = {\n  Blue: "blue",\n  Green: "green",\n  Red: "red",\n  Yellow: "yellow"\n};\n/**\r\n * URL Format Types\r\n */\n\nexports.UrlFormatType = {\n  Hyperlink: 0,\n  Image: 1\n};\n/**\r\n * URL Zones\r\n */\n\nexports.URLZones = {\n  Default: 0,\n  Intranet: 1,\n  Internet: 2,\n  Custom: 3,\n  Extranet: 4\n};\n/**\r\n * User Custom Action Registration Types\r\n */\n\nexports.UserCustomActionRegistrationType = {\n  None: 0,\n  List: 1,\n  ContentType: 2,\n  ProgId: 3,\n  FileType: 4\n};\n/**\r\n * View Types\r\n */\n\nexports.ViewType = {\n  Calendar: 524288,\n  Chart: 131072,\n  Gantt: 67108864,\n  Grid: 2048,\n  Html: 1,\n  Recurrence: 8193\n};\n/**\r\n * Web Template Types\r\n */\n\nexports.WebTemplateType = {\n  AcademicLibrary: "DOCMARKETPLACESITE",\n  App: "APP",\n  AppCatalog: "APPCATALOG",\n  BasicSearch: "SRCHCENTERLITE",\n  Blog: "BLOG",\n  BusinessIntelligenceCenter: "BICenterSite",\n  CentralAdmin: "CENTRALADMIN",\n  Community: "COMMUNITY",\n  CommunityPortal: "COMMUNITYPORTAL",\n  Dev: "DEV",\n  DocumentCenter: "BDR",\n  eDiscoveryCenter: "EDISC",\n  EnterpriseSearch: "SRCHCEN",\n  EnterpriseWiki: "ENTERWIKI",\n  Global: "GLOBAL",\n  GroupWorkSite: "SGS",\n  Meetings: "MEETINGS",\n  MeetingWorkspace: "MPS",\n  PerformancePoint: "PPMASite",\n  ProductCatalog: "PRODUCTCATALOG",\n  Profiles: "PROFILES",\n  ProjectSite: "PROJECTSITE",\n  Publishing: "BLANKINTERNET",\n  PublishingSite: "CMSPUBLISHING",\n  RecordsCenter: "OFFILE",\n  SharedServicesAdminSite: "OSRV",\n  Site: "STS",\n  TeamCollaborationSite: "TEAM",\n  TenantAdmin: "TENANTADMIN",\n  Wiki: "WIKI"\n};\n\n//# sourceURL=webpack://gd-sprest/./build/sptypes/sptypes.js?')},"./build/utils/base.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.Base = void 0;\n\nvar _1 = __webpack_require__(/*! . */ "./build/utils/index.js");\n/*********************************************************************************************************************************/\n// Base\n// This is the base class for all objects.\n\n/*********************************************************************************************************************************/\n\n\nvar Base =\n/** @class */\nfunction () {\n  /**\r\n   * Constructor\r\n   * @param targetInfo - The target information.\r\n   */\n  function Base(targetInfo) {\n    // Default the properties\n    this.targetInfo = Object.create(targetInfo || {});\n    this.responses = [];\n    this.requestType = 0;\n    this.waitFlags = [];\n  } // Method to update the object functions, based on the type\n\n\n  Base.prototype.addMethods = function (data, context) {\n    return _1.Request.addMethods(this, data, context);\n  }; // Method to execute this request as a batch request\n\n\n  Base.prototype.batch = function () {\n    var args = [];\n\n    for (var _i = 0; _i < arguments.length; _i++) {\n      args[_i] = arguments[_i];\n    }\n\n    return _1.Batch.execute(this, args);\n  }; // Method to wait for the requests to complete\n\n\n  Base.prototype.done = function (resolve) {\n    return _1.Helper.done(this, resolve);\n  }; // Method to execute the request\n\n\n  Base.prototype.execute = function () {\n    var args = [];\n\n    for (var _i = 0; _i < arguments.length; _i++) {\n      args[_i] = arguments[_i];\n    }\n\n    return _1.Request.execute(this, args);\n  }; // Method to execute a method\n\n\n  Base.prototype.executeMethod = function (methodName, methodConfig, args) {\n    return _1.Helper.executeMethod(this, methodName, methodConfig, args);\n  }; // Method to execute the request synchronously\n\n\n  Base.prototype.executeAndWait = function () {\n    return _1.Request.executeRequest(this, false);\n  }; // Method to return a collection\n\n\n  Base.prototype.getCollection = function (method, args) {\n    return _1.Helper.getCollection(this, method, args);\n  }; // Method to get the request information\n\n\n  Base.prototype.getInfo = function () {\n    return _1.Helper.getRequestInfo(this);\n  }; // Method to get the next set of results\n\n\n  Base.prototype.getNextSetOfResults = function () {\n    return _1.Helper.getNextSetOfResults(this);\n  }; // Method to return a property of the base object\n\n\n  Base.prototype.getProperty = function (propertyName, requestType) {\n    return _1.Helper.getProperty(this, propertyName, requestType);\n  }; // Method to stringify the object\n\n\n  Base.prototype.stringify = function () {\n    return _1.Helper.stringify(this);\n  }; // Method to update the metadata uri\n\n\n  Base.prototype.updateMetadataUri = function (metadata, targetInfo) {\n    return _1.Helper.updateMetadataUri(this, metadata, targetInfo);\n  }; // Method to update the request digest value\n\n\n  Base.prototype.updateRequestDigest = function (value) {\n    this.targetInfo.requestDigest = value;\n  }; // Method to wait for the parent requests to complete\n\n\n  Base.prototype.waitForRequestsToComplete = function (callback, requestIdx) {\n    _1.Request.waitForRequestsToComplete(this, callback, requestIdx);\n  };\n\n  return Base;\n}();\n\nexports.Base = Base;\n\n//# sourceURL=webpack://gd-sprest/./build/utils/base.js?')},"./build/utils/batch.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nfunction _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.Batch = void 0;\n\nvar lib_1 = __webpack_require__(/*! ../lib */ "./build/lib/index.js");\n\nvar _1 = __webpack_require__(/*! . */ "./build/utils/index.js");\n/**\r\n * Batch Requests\r\n */\n\n\nvar Batch =\n/** @class */\nfunction () {\n  function Batch() {} // Method to execute a batch request\n\n\n  Batch.execute = function (base, args) {\n    var createFl = false;\n    var callback = null; // Parse the arguments\n\n    for (var i = 0; i < args.length; i++) {\n      var arg = args[i]; // Check the type\n\n      switch (_typeof(arg)) {\n        case "boolean":\n          // Set the create flag\n          createFl = arg;\n          break;\n\n        case "function":\n          // Set the callback method\n          callback = arg;\n          break;\n      }\n    } // Set the base\n\n\n    base.base = base.base ? base.base : base; // See if we are creating a new request\n\n    if (createFl || base.base.batchRequests == null) {\n      // Ensure the batch requests exist\n      base.base.batchRequests = base.base.batchRequests || []; // Create the request\n\n      base.base.batchRequests.push([{\n        callback: callback,\n        changesetId: lib_1.ContextInfo.generateGUID(),\n        targetInfo: new _1.TargetInfo(base.targetInfo)\n      }]);\n    } else {\n      // Batch requests are limited to 100 per execution, so we need to add a check for this\n      if (base.base.batchRequests[base.base.batchRequests.length - 1].length == 100) {\n        // Create a new request\n        base.base.batchRequests.push([{\n          callback: callback,\n          changesetId: lib_1.ContextInfo.generateGUID(),\n          targetInfo: new _1.TargetInfo(base.targetInfo)\n        }]);\n      } else {\n        // Append the request\n        base.base.batchRequests[base.base.batchRequests.length - 1].push({\n          callback: callback,\n          changesetId: lib_1.ContextInfo.generateGUID(),\n          targetInfo: new _1.TargetInfo(base.targetInfo)\n        });\n      }\n    } // Return this object\n\n\n    return base;\n  }; // Method to generate a batch request\n\n\n  Batch.getTargetInfo = function (url, requests, requestDigest) {\n    var batchId = "batch_" + lib_1.ContextInfo.generateGUID();\n    var batchRequests = []; // Create the batch request\n\n    batchRequests.push(Batch.createBatch(batchId, requests)); // End the batch request\n\n    batchRequests.push("--" + batchId + "--"); // Return the target information\n\n    return new _1.TargetInfo({\n      url: url,\n      endpoint: "$batch",\n      method: "POST",\n      data: batchRequests.join("\\r\\n"),\n      requestDigest: requestDigest,\n      requestHeader: {\n        "Content-Type": \'multipart/mixed; boundary="\' + batchId + \'"\'\n      }\n    });\n  }; // Method to generate a batch request\n\n\n  Batch.createBatch = function (batchId, requests) {\n    var batch = []; // Parse the requests\n\n    for (var i = 0; i < requests.length; i++) {\n      var request_1 = requests[i]; // Create the batch request\n\n      batch.push("--" + batchId); // Determine if the batch requires a change set\n\n      var requiresChangeset = request_1 && request_1.targetInfo.requestMethod != "GET";\n\n      if (requiresChangeset) {\n        // Create a change set\n        batch.push("Content-Type: multipart/mixed; boundary=" + request_1.changesetId);\n        batch.push("");\n        batch.push("--" + request_1.changesetId);\n        batch.push("Content-Type: application/http");\n        batch.push("Content-Transfer-Encoding: binary");\n        batch.push("");\n        batch.push(request_1.targetInfo.requestMethod + " " + request_1.targetInfo.requestUrl + " HTTP/1.1");\n        batch.push("Content-Type: application/json;odata=verbose"); // See if we are making a delete/update\n\n        if (request_1.targetInfo.requestMethod == "DELETE" || request_1.targetInfo.requestMethod == "MERGE") {\n          // Append the header for deleting/updating\n          batch.push("IF-MATCH: *");\n        }\n\n        batch.push("");\n        request_1.targetInfo.requestData ? batch.push(request_1.targetInfo.requestData) : null;\n        batch.push("");\n        batch.push("--" + request_1.changesetId + "--");\n      } else {\n        // Create a change set\n        batch.push("Content-Type: application/http");\n        batch.push("Content-Transfer-Encoding: binary");\n        batch.push("");\n        batch.push("GET " + request_1.targetInfo.requestUrl + " HTTP/1.1");\n        batch.push("Accept: application/json;odata=verbose");\n        batch.push("");\n        request_1.targetInfo.requestData ? batch.push(request_1.targetInfo.requestData) : null;\n        batch.push("");\n      }\n    } // Add the change set information to the batch\n\n\n    var batchRequest = batch.join("\\r\\n");\n    var request = [];\n    request.push("Content-Type: multipart/mixed; boundary=" + batchId);\n    request.push("Content-Length: " + batchRequest.length);\n    request.push("");\n    request.push(batchRequest);\n    request.push(""); // Return the batch request\n\n    return request.join("\\r\\n");\n  }; // Process the batch request callbacks\n\n\n  Batch.processCallbacks = function (batchRequests) {\n    if (batchRequests === void 0) {\n      batchRequests = [];\n    } // Parse the requests\n\n\n    for (var i = 0; i < batchRequests.length; i++) {\n      var batchRequest = batchRequests[i]; // See if a callback exists\n\n      if (batchRequest.callback) {\n        // Execute the callback\n        batchRequest.callback(batchRequest.response, batchRequest.targetInfo);\n      }\n    }\n  };\n\n  return Batch;\n}();\n\nexports.Batch = Batch;\n\n//# sourceURL=webpack://gd-sprest/./build/utils/batch.js?')},"./build/utils/helper.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.Helper = void 0;\n\nvar lib_1 = __webpack_require__(/*! ../lib */ "./build/lib/index.js");\n\nvar _1 = __webpack_require__(/*! . */ "./build/utils/index.js");\n\nvar xhrRequest_1 = __webpack_require__(/*! ./xhrRequest */ "./build/utils/xhrRequest.js");\n/**\r\n * Request Helper\r\n */\n\n\nexports.Helper = {\n  // Method to add the base references\n  addBaseMethods: function addBaseMethods(base, obj) {\n    // Add the base references\n    obj["addMethods"] = base.addMethods;\n    obj["base"] = base.base;\n    obj["done"] = base.done;\n    obj["execute"] = base.execute;\n    obj["executeAndWait"] = base.executeAndWait;\n    obj["executeMethod"] = base.executeMethod;\n    obj["existsFl"] = true;\n    obj["getCollection"] = base.getCollection;\n    obj["getProperty"] = base.getProperty;\n    obj["parent"] = base;\n    obj["targetInfo"] = base.targetInfo;\n    obj["updateMetadataUri"] = base.updateMetadataUri;\n    obj["waitForRequestsToComplete"] = base.waitForRequestsToComplete;\n  },\n  // Method to wait for all requests to complete, before resolving the request\n  done: function done(base, resolve) {\n    // Ensure the base is set\n    base.base = base.base ? base.base : base; // Ensure the response index is set\n\n    base.responseIndex = base.responseIndex >= 0 ? base.responseIndex : 0; // Wait for the responses to execute\n\n    _1.Request.waitForRequestsToComplete(base, function () {\n      var responses = base.base.responses; // Clear the responses\n\n      base.base.responses = []; // Clear the wait flags\n\n      base.base.waitFlags = []; // Resolve the request\n\n      resolve ? resolve.apply(base, responses) : null;\n    });\n  },\n  // Method to execute a method\n  executeMethod: function executeMethod(base, methodName, methodConfig, args) {\n    var _a, _b;\n\n    var targetInfo = null; // See if the metadata is defined for the base object\n\n    var metadata = base["d"] ? base["d"].__metadata : base["__metadata"];\n\n    if (metadata && metadata.uri) {\n      // Create the target information and use the url defined for the base object\n      targetInfo = {\n        requestDigest: (_a = base.targetInfo) === null || _a === void 0 ? void 0 : _a.requestDigest,\n        url: metadata.uri\n      }; // See if we are inheriting the metadata type\n\n      if (methodConfig.inheritMetadataType && metadata.type) {\n        // Copy the metadata type\n        methodConfig.metadataType = metadata.type;\n      } // Update the metadata uri\n\n\n      exports.Helper.updateMetadataUri(base, metadata, targetInfo);\n    } else {\n      // Copy the target information\n      targetInfo = Object.create(base.targetInfo);\n    } // See if the method config name has a base reference\n\n\n    var startIdx = (_b = methodConfig === null || methodConfig === void 0 ? void 0 : methodConfig.name) === null || _b === void 0 ? void 0 : _b.indexOf("[base.");\n\n    if (startIdx >= 0) {\n      // Get the property name\n      var endIdx = methodConfig.name.indexOf("]", startIdx);\n      var propName = methodConfig.name.substring(startIdx, endIdx).split(".")[1]; // Get the property value\n\n      var propValue = base[propName] || ""; // Update the method name\n\n      methodConfig.name = methodConfig.name.replace("[base." + propName + "]", propValue);\n    } // Get the method information\n\n\n    var methodInfo = new _1.MethodInfo(methodName, methodConfig, args); // Update the target information\n\n    targetInfo.bufferFl = methodConfig.requestType == _1.RequestType.GetBuffer;\n    targetInfo.data = methodInfo.body;\n    targetInfo.defaultToWebFl = typeof targetInfo.defaultToWebFl === "undefined" && base.base ? base.base.targetInfo.defaultToWebFl : targetInfo.defaultToWebFl;\n    targetInfo.method = methodInfo.requestMethod;\n    targetInfo.requestDigest = typeof targetInfo.requestDigest === "undefined" && base.base && base.base.targetInfo.requestDigest ? base.base.targetInfo.requestDigest : targetInfo.requestDigest;\n    targetInfo.requestType = methodConfig.requestType; // See if we are appending the endpoint\n\n    if (methodInfo.appendEndpointFl) {\n      // Append to the endpoint\n      targetInfo.endpoint += "." + methodInfo.url;\n    } // Else, see if we are replacing the endpoint\n    else if (methodInfo.replaceEndpointFl) {\n      // Replace the endpoint\n      targetInfo.endpoint = methodInfo.url;\n    } // Else, ensure the method url exists\n    else if (methodInfo.url && methodInfo.url.length > 0) {\n      // Ensure the end point exists\n      targetInfo.endpoint = targetInfo.endpoint ? targetInfo.endpoint : ""; // See if the endpoint exists, and the method is not a query string\n\n      if (targetInfo.endpoint && methodInfo.url && methodInfo.url.indexOf("?") != 0) {\n        // Add a "/" separator to the url\n        targetInfo.endpoint += methodInfo.appendRequest ? "" : "/";\n      } // See if we already have a qs defined and appending another qs\n\n\n      if (methodInfo.url.indexOf("?") == 0 && targetInfo.endpoint.indexOf(\'?\') > 0) {\n        // Append the url\n        targetInfo.endpoint += \'&\' + methodInfo.url.substring(1);\n      } else {\n        // Append the url\n        targetInfo.endpoint += methodInfo.url;\n      }\n    } // Create a new object\n\n\n    var obj = new _1.Base(targetInfo); // Set the properties\n\n    obj.base = base.base ? base.base : base;\n    obj.getAllItemsFl = methodInfo.getAllItemsFl;\n    obj.parent = base;\n    obj.requestType = methodConfig.requestType; // Ensure the return type exists\n\n    if (methodConfig.returnType) {\n      // Add the methods\n      _1.Request.addMethods(obj, {\n        __metadata: {\n          type: methodConfig.returnType\n        }\n      });\n    } // Return the object\n\n\n    return obj;\n  },\n  // Method to return a collection\n  getCollection: function getCollection(base, method, args) {\n    // Copy the target information\n    var targetInfo = Object.create(base.targetInfo); // Clear the target information properties from any previous requests\n\n    targetInfo.data = null;\n    targetInfo.method = null; // See if the metadata is defined for the base object\n\n    var metadata = base["d"] ? base["d"].__metadata : base["__metadata"];\n\n    if (metadata && metadata.uri) {\n      // Update the url of the target information\n      targetInfo.url = metadata.uri; // Update the metadata uri\n\n      exports.Helper.updateMetadataUri(base, metadata, targetInfo); // Set the endpoint\n\n      targetInfo.endpoint = method;\n    } else {\n      // Append the method to the endpoint\n      targetInfo.endpoint += "/" + method;\n    } // Update the callback\n\n\n    targetInfo.callback = args && typeof args[0] === "function" ? args[0] : null; // Create a new object\n\n    var obj = new _1.Base(targetInfo); // Set the properties\n\n    obj.base = base.base ? base.base : base;\n    obj.parent = base; // Return the object\n\n    return obj;\n  },\n  // Method to get the next set of results\n  getNextSetOfResults: function getNextSetOfResults(base) {\n    // Create the target information to query the next set of results\n    var targetInfo = Object.create(base.targetInfo);\n    targetInfo.endpoint = "";\n    targetInfo.url = base["d"].__next; // Create a new object\n\n    var obj = new _1.Base(targetInfo); // Set the properties\n\n    obj.base = base.base ? base.base : base;\n    obj.parent = base; // Return the object\n\n    return obj;\n  },\n  // Method to return a property of the base object\n  getProperty: function getProperty(base, propertyName, requestType) {\n    // Copy the target information\n    var targetInfo = Object.create(base.targetInfo); // See if this is a graph request\n\n    if (requestType.indexOf("graph") == 0) {\n      // Default the request type\n      targetInfo.requestType = _1.RequestType.GraphGet;\n    } // Clear the target information properties from any previous requests\n\n\n    targetInfo.data = null;\n    targetInfo.method = null; // See if the metadata is defined for the base object\n\n    var metadata = base["d"] ? base["d"].__metadata : base["__metadata"];\n\n    if (metadata && metadata.uri) {\n      // Update the url of the target information\n      targetInfo.url = metadata.uri; // Update the metadata uri\n\n      exports.Helper.updateMetadataUri(base, metadata, targetInfo); // Set the endpoint\n\n      targetInfo.endpoint = propertyName;\n    } else {\n      // Append the property name to the endpoint\n      targetInfo.endpoint += "/" + propertyName;\n    } // Create a new object\n\n\n    var obj = new _1.Base(targetInfo); // Set the properties\n\n    obj.base = base.base ? base.base : base;\n    obj.parent = base; // Add the methods\n\n    requestType ? _1.Request.addMethods(obj, {\n      __metadata: {\n        type: requestType\n      }\n    }) : null; // Return the object\n\n    return obj;\n  },\n  // Method to get the request information\n  getRequestInfo: function getRequestInfo(base) {\n    // Create the request, but don\'t execute it\n    var xhr = new xhrRequest_1.XHRRequest(true, new _1.TargetInfo(base.targetInfo), null, false); // Return the request information\n\n    return xhr.requestInfo;\n  },\n  // Method to stringify the object\n  stringify: function stringify(base) {\n    // Stringify the object\n    return JSON.stringify({\n      response: base.response,\n      status: base.status,\n      targetInfo: {\n        accessToken: base.targetInfo.accessToken,\n        bufferFl: base.targetInfo.bufferFl,\n        defaultToWebFl: base.targetInfo.defaultToWebFl,\n        endpoint: base.targetInfo.endpoint,\n        method: base.targetInfo.method,\n        overrideDefaultRequestToHostFl: base.targetInfo.overrideDefaultRequestToHostFl,\n        requestDigest: base.targetInfo.requestDigest,\n        requestHeader: base.targetInfo.requestHeader,\n        requestInfo: base.targetInfo.requestInfo,\n        requestType: base.targetInfo.requestType,\n        url: base.targetInfo.url\n      }\n    });\n  },\n  // Method to update a collection object\n  updateDataCollection: function updateDataCollection(obj, results) {\n    // Ensure the base is a collection\n    if (results) {\n      // Save the results\n      obj["results"] = obj["results"] ? obj["results"].concat(results) : results; // See if only one object exists\n\n      if (obj["results"].length > 0) {\n        var results_2 = obj["results"]; // See if this is a v2 request, and set the default object type\n\n        var objType = null;\n\n        if (obj["@odata.context"]) {\n          // Get the object type\n          var metadataType = obj["@odata.context"] || objType;\n          var values = metadataType.split("_api/v2.0/$metadata#");\n\n          if (values.length > 1) {\n            objType = values[1];\n          } else {\n            values = metadataType.split("/");\n            objType = values[values.length - 1].split("?")[0];\n          } // Ensure its not the collection type\n\n\n          objType = objType.replace(/s$/, \'\');\n        } // Parse the results\n\n\n        for (var _i = 0, results_1 = results_2; _i < results_1.length; _i++) {\n          var result = results_1[_i]; // Add the base methods\n\n          exports.Helper.addBaseMethods(obj, result); // Update the metadata\n\n          exports.Helper.updateMetadata(obj, result); // Add the methods\n\n          _1.Request.addMethods(result, result, objType);\n        }\n      }\n    }\n  },\n  // Method to update the expanded collection property\n  updateExpandedCollection: function updateExpandedCollection(base, results) {\n    // Parse the results\n    for (var i = 0; i < results.length; i++) {\n      var result = results[i]; // See if this property was expanded\n\n      if (result["__metadata"]) {\n        // Add the base methods\n        exports.Helper.addBaseMethods(base, result); // Update the metadata\n\n        exports.Helper.updateMetadata(result, result); // Add the methods\n\n        _1.Request.addMethods(result, result);\n      }\n    }\n  },\n  // Method to update the expanded properties\n  updateExpandedProperties: function updateExpandedProperties(base) {\n    // Ensure this is an OData request\n    if (base["results"] == null || base.requestType != _1.RequestType.OData) {\n      return;\n    } // Parse the results\n\n\n    for (var i = 0; i < base["results"].length; i++) {\n      var result = base["results"][i]; // Parse the properties\n\n      for (var key in result) {\n        // Skip the parent property\n        if (key == "parent") {\n          continue;\n        } // Ensure the property exists\n\n\n        var prop = result[key];\n\n        if (prop) {\n          // See if this is a collection\n          if (prop["results"] && prop["results"].length > 0) {\n            // Update the expanded collection\n            exports.Helper.updateExpandedCollection(base, prop.results);\n          } // Else, see if this property was expanded\n          else if (prop["__metadata"]) {\n            // Add the base methods\n            exports.Helper.addBaseMethods(result, prop); // Update the metadata\n\n            exports.Helper.updateMetadata(result, prop); // Add the methods\n\n            _1.Request.addMethods(prop, prop);\n          }\n        }\n      }\n    }\n  },\n  // Method to update the metadata\n  updateMetadata: function updateMetadata(base, data) {\n    // See if this is the app web\n    if (lib_1.ContextInfo.isAppWeb) {\n      // Get the url information\n      var hostUrl = lib_1.ContextInfo.webAbsoluteUrl.toLowerCase();\n      var requestUrl = data && data.__metadata && data.__metadata.uri ? data.__metadata.uri.toLowerCase() : null;\n      var targetUrl = base.targetInfo && base.targetInfo.url ? base.targetInfo.url.toLowerCase() : null; // Ensure the urls exist\n\n      if (hostUrl == null || requestUrl == null || targetUrl == null) {\n        return;\n      } // See if we need to make an update\n\n\n      if (targetUrl.indexOf(hostUrl) == 0) {\n        return;\n      } // Update the metadata uri\n\n\n      data.__metadata.uri = requestUrl.replace(hostUrl, targetUrl);\n    } // See if this is a webpart definition\n\n\n    if (data.__metadata && /SP.WebParts.WebPartDefinition/.test(data.__metadata.type)) {\n      // Update the metadata uri\n      data.__metadata.uri = data.__metadata.uri.replace(/SP.WebParts.WebPartDefinition/, base.targetInfo.endpoint + "/getById(\'") + "\')";\n    }\n  },\n  // Method to update the metadata uri\n  updateMetadataUri: function updateMetadataUri(base, metadata, targetInfo) {\n    // See if this is a field\n    if (/^SP.Field/.test(metadata.type) || /^SP\\..*Field$/.test(metadata.type)) {\n      // Fix the url reference\n      targetInfo.url = targetInfo.url.replace(/AvailableFields/, "fields");\n    } // Else, see if this is an event receiver\n    else if (/SP.EventReceiverDefinition/.test(metadata.type)) {\n      // Fix the url reference\n      targetInfo.url = targetInfo.url.replace(/\\/EventReceiver\\//, "/EventReceivers/");\n    } // Else, see if this is a tenant app\n    else if (/Microsoft.SharePoint.Marketplace.CorporateCuratedGallery.CorporateCatalogAppMetadata/.test(targetInfo.url)) {\n      // Fix the url reference\n      targetInfo.url = targetInfo.url.split("Microsoft.SharePoint.Marketplace.CorporateCuratedGallery.CorporateCatalogAppMetadata")[0] + "web/tenantappcatalog/availableapps/getbyid(\'" + base["ID"] + "\')";\n    }\n  },\n  // Method to update the search results\n  updateSearchResults: function updateSearchResults(base) {\n    // See if this contains search results\n    var results = base["postquery"];\n\n    if (results == null || results.PrimaryQueryResult == null) {\n      return;\n    } // Clear the results\n\n\n    base["postquery"].results = base["postquery"].results || []; // Parse the results\n\n    for (var i = 0; i < results.PrimaryQueryResult.RelevantResults.RowCount; i++) {\n      var data = {};\n      var result = results.PrimaryQueryResult.RelevantResults.Table.Rows.results[i]; // Parse the cells\n\n      for (var j = 0; j < result.Cells.results.length; j++) {\n        var cell = result.Cells.results[j]; // Add the key/value\n\n        data[cell.Key] = cell.Value;\n      } // Append the result\n\n\n      base["postquery"].results.push(data);\n    }\n  }\n};\n\n//# sourceURL=webpack://gd-sprest/./build/utils/helper.js?')},"./build/utils/index.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nvar __createBinding = this && this.__createBinding || (Object.create ? function (o, m, k, k2) {\n  if (k2 === undefined) k2 = k;\n  Object.defineProperty(o, k2, {\n    enumerable: true,\n    get: function get() {\n      return m[k];\n    }\n  });\n} : function (o, m, k, k2) {\n  if (k2 === undefined) k2 = k;\n  o[k2] = m[k];\n});\n\nvar __exportStar = this && this.__exportStar || function (m, exports) {\n  for (var p in m) {\n    if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);\n  }\n};\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\n\n__exportStar(__webpack_require__(/*! ./requestType */ "./build/utils/requestType.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./helper */ "./build/utils/helper.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./base */ "./build/utils/base.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./batch */ "./build/utils/batch.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./methodInfo */ "./build/utils/methodInfo.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./oData */ "./build/utils/oData.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./request */ "./build/utils/request.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./targetInfo */ "./build/utils/targetInfo.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./xhrRequest */ "./build/utils/xhrRequest.js"), exports);\n\n//# sourceURL=webpack://gd-sprest/./build/utils/index.js?')},"./build/utils/methodInfo.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nfunction _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.MethodInfo = void 0;\n\nvar _1 = __webpack_require__(/*! . */ "./build/utils/index.js");\n/*********************************************************************************************************************************/\n// Method Information\n// This class will create the method information for the request.\n\n/*********************************************************************************************************************************/\n\n\nvar MethodInfo =\n/** @class */\nfunction () {\n  /*********************************************************************************************************************************/\n  // Constructor\n\n  /*********************************************************************************************************************************/\n  function MethodInfo(methodName, methodInfo, args) {\n    // Default the properties\n    this.methodInfo = methodInfo;\n    this.methodInfo.argValues = args;\n    this.methodInfo.name = typeof this.methodInfo.name === "string" ? this.methodInfo.name : methodName; // Generate the parameters\n\n    this.generateParams(); // Generate the url\n\n    this.methodUrl = this.generateUrl();\n  }\n\n  Object.defineProperty(MethodInfo.prototype, "appendEndpointFl", {\n    /*********************************************************************************************************************************/\n    // Public Properties\n\n    /*********************************************************************************************************************************/\n    // Flag to determine if this method replaces the endpoint\n    get: function get() {\n      return this.methodInfo.appendEndpointFl ? true : false;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(MethodInfo.prototype, "appendRequest", {\n    // Flag to determine if we are appending the name or adding a "/" to the request url\n    get: function get() {\n      return this.methodInfo.appendRequest;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(MethodInfo.prototype, "body", {\n    // The data passed through the body of the request\n    get: function get() {\n      return this.methodData;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(MethodInfo.prototype, "getAllItemsFl", {\n    // Flag to determine if we are getting all items\n    get: function get() {\n      return this.methodInfo.getAllItemsFl;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(MethodInfo.prototype, "replaceEndpointFl", {\n    // Flag to determine if this method replaces the endpoint\n    get: function get() {\n      return this.methodInfo.replaceEndpointFl ? true : false;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(MethodInfo.prototype, "requestMethod", {\n    // The request method\n    get: function get() {\n      // Return the request method if it exists\n      if (typeof this.methodInfo.requestMethod === "string") {\n        return this.methodInfo.requestMethod;\n      } // Determine the request method, based on the request type\n\n\n      switch (this.methodInfo.requestType) {\n        case _1.RequestType.Delete:\n        case _1.RequestType.Post:\n        case _1.RequestType.PostBodyNoArgs:\n        case _1.RequestType.PostReplace:\n        case _1.RequestType.PostReplaceWithData:\n        case _1.RequestType.PostWithArgs:\n        case _1.RequestType.PostWithArgsAndData:\n        case _1.RequestType.PostWithArgsInBody:\n        case _1.RequestType.PostWithArgsInQS:\n        case _1.RequestType.PostWithArgsInQSAsVar:\n        case _1.RequestType.PostWithArgsValueOnly:\n          return "POST";\n\n        default:\n          return "GET";\n      }\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(MethodInfo.prototype, "url", {\n    // The url of the method and parameters\n    get: function get() {\n      return this.methodUrl;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(MethodInfo.prototype, "passDataInBody", {\n    /*********************************************************************************************************************************/\n    // Private Variables\n\n    /*********************************************************************************************************************************/\n    get: function get() {\n      return this.methodInfo.requestType == _1.RequestType.GetWithArgsInBody || this.methodInfo.requestType == _1.RequestType.PostBodyNoArgs || this.methodInfo.requestType == _1.RequestType.PostWithArgsInBody;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(MethodInfo.prototype, "passDataInQS", {\n    get: function get() {\n      return this.methodInfo.requestType == _1.RequestType.GetWithArgsInQS || this.methodInfo.requestType == _1.RequestType.PostWithArgsInQS;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(MethodInfo.prototype, "passDataInQSAsVar", {\n    get: function get() {\n      return this.methodInfo.requestType == _1.RequestType.GetWithArgsInQSAsVar || this.methodInfo.requestType == _1.RequestType.PostWithArgsInQSAsVar;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(MethodInfo.prototype, "isTemplate", {\n    get: function get() {\n      return this.methodInfo.data ? true : false;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(MethodInfo.prototype, "replace", {\n    get: function get() {\n      return this.methodInfo.requestType == _1.RequestType.GetReplace || this.methodInfo.requestType == _1.RequestType.GraphGetReplace || this.methodInfo.requestType == _1.RequestType.PostReplace || this.methodInfo.requestType == _1.RequestType.GraphPostReplace;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  /*********************************************************************************************************************************/\n  // Private Methods\n\n  /*********************************************************************************************************************************/\n  // Method to generate the method input parameters\n\n  MethodInfo.prototype.generateParams = function () {\n    var maxArgNames = 0;\n    var params = {}; // Ensure values exist\n\n    if (this.methodInfo.argValues == null) {\n      return;\n    } // See if the argument names exist\n\n\n    if (this.methodInfo.argNames) {\n      // Set the max arguments\n      maxArgNames = this.methodInfo.argNames.length - (this.methodInfo.requestType == _1.RequestType.PostWithArgsAndData || this.methodInfo.requestType == _1.RequestType.PostReplaceWithData ? 1 : 0); // Parse the argument names\n\n      for (var i = 0; i < maxArgNames && i < this.methodInfo.argValues.length; i++) {\n        var name_1 = this.methodInfo.argNames[i];\n        var value = this.methodInfo.argValues[i]; // Copy the parameter value\n\n        switch (_typeof(this.methodInfo.argValues[i])) {\n          case "boolean":\n            params[name_1] = this.methodInfo.argValues[i] ? "true" : "false";\n            break;\n\n          case "number":\n            params[name_1] = this.methodInfo.argValues[i];\n            break;\n          //case "string":\n          //params[name] = this.isTemplate || this.replace ? value : "\'" + value + "\'";\n          //break;\n\n          default:\n            params[name_1] = value;\n            break;\n        }\n      }\n    } // See if the method has parameters\n\n\n    var isEmpty = true;\n\n    for (var k in params) {\n      isEmpty = false;\n      break;\n    }\n\n    this.methodParams = isEmpty ? null : params; // See if method parameters exist\n\n    if (this.methodParams) {\n      // See if a template is defined for the method data\n      if (this.isTemplate) {\n        // Ensure the object is a string\n        if (typeof this.methodInfo.data !== "string") {\n          // Stringify the object\n          this.methodInfo.data = JSON.stringify(this.methodInfo.data);\n        } // Parse the arguments\n\n\n        for (var key in this.methodParams) {\n          // Replace the argument in the template\n          this.methodInfo.data = this.methodInfo.data.replace("[[" + key + "]]", this.methodParams[key].replace(/"/g, \'\\\\"\').replace(/\\n/g, ""));\n        } // For custom format properties of fields, we need to escape \\\\quot;\n        // Replace \\\\&quot; with \\\\\\\\&quot;\n\n\n        var methodData = this.methodInfo.data.replace(/\\\\&quot;/g, \'\\\\\\\\&quot;\'); // Set the method data\n\n        this.methodData = typeof methodData === "string" ? methodData : JSON.parse(methodData);\n      }\n    } // See if argument values exist\n\n\n    if (this.methodInfo.argValues && this.methodInfo.argValues.length > 0) {\n      // See if argument names exist\n      if (this.methodInfo.argNames == null || this.methodInfo.requestType == _1.RequestType.PostBodyNoArgs) {\n        // Set the method data to first argument value\n        this.methodData = this.methodInfo.argValues[0];\n      } // Else, see if we are passing arguments outside of the parameters\n      else if (this.methodInfo.argValues.length > maxArgNames) {\n        // Set the method data to the next available argument value\n        this.methodData = this.methodInfo.argValues[maxArgNames];\n      }\n    } // See if the metadata type exists\n\n\n    if (this.methodInfo.metadataType) {\n      // See if parameters exist\n      if (this.methodInfo.argNames && this.methodInfo.requestType != _1.RequestType.PostBodyNoArgs && typeof (this.methodData || this.methodParams)[this.methodInfo.argNames[0]] !== "string") {\n        // Append the metadata to the first parameter, if it doesn\'t exist\n        (this.methodData || this.methodParams)[this.methodInfo.argNames[0]]["__metadata"] = (this.methodData || this.methodParams)[this.methodInfo.argNames[0]]["__metadata"] || {\n          "type": this.methodInfo.metadataType\n        };\n      } else {\n        // Append the metadata to the parameters, if it doesn\'t exist\n        (this.methodData || this.methodParams)["__metadata"] = (this.methodData || this.methodParams)["__metadata"] || {\n          "type": this.methodInfo.metadataType\n        };\n      }\n    }\n  }; // Method to generate the method and parameters as a url\n\n\n  MethodInfo.prototype.generateUrl = function () {\n    var url = this.methodInfo.name; // See if we are deleting the object\n\n    if (this.methodInfo.requestType == _1.RequestType.Delete) {\n      // Default the value\n      url = "deleteObject";\n    } // See if we are passing the data in the body\n\n\n    if (this.passDataInBody) {\n      var data = this.methodData || this.methodParams; // Stringify the data to be passed in the body\n\n      this.methodData = data ? JSON.stringify(data) : null;\n    } // See if we are passing the data in the query string as a variable\n\n\n    if (this.passDataInQSAsVar) {\n      var data = this.methodParams || this.methodData; // Append the parameters in the query string\n\n      url += "(@v)?@v=" + (typeof data === "string" ? "\'" + encodeURIComponent(data) + "\'" : JSON.stringify(data));\n    } // See if we are replacing the arguments\n\n\n    if (this.replace) {\n      // Parse the arguments\n      for (var key in this.methodParams) {\n        // Replace the argument in the url\n        url = url.replace("[[" + key + "]]", encodeURIComponent(this.methodParams[key]));\n      }\n    } // Else, see if this is an odata request\n    else if (this.methodInfo.requestType == _1.RequestType.OData) {\n      var oData = new _1.OData(this.methodParams["oData"]); // Update the url\n\n      url = "?" + oData.QueryString; // Set the get all items Flag\n\n      this.methodInfo.getAllItemsFl = oData.GetAllItems;\n    } // Else, see if we are not passing the data in the body or query string as a variable\n    else if (!this.passDataInBody && !this.passDataInQSAsVar) {\n      var params = ""; // Ensure data exists\n\n      var data = this.methodParams || this.methodData;\n\n      if (data) {\n        // Ensure the data is an object\n        data = data && _typeof(data) === "object" ? data : {\n          value: data\n        }; // Parse the parameters\n\n        for (var name_2 in data) {\n          var value = data[name_2];\n          value = typeof value === "string" ? "\'" + value.replace(/\'/g, "\'\'") + "\'" : value;\n\n          switch (this.methodInfo.requestType) {\n            // Append the value only\n            case _1.RequestType.GetWithArgsValueOnly:\n            case _1.RequestType.PostWithArgsValueOnly:\n              params += value + ", ";\n              break;\n            // Append the parameter and value\n\n            default:\n              params += name_2 + "=" + value + ", ";\n              break;\n          }\n        }\n      } // See if we are passing data in the query string\n\n\n      if (this.passDataInQS) {\n        // Set the url\n        url += params.length > 0 ? "?" + params.replace(/, $/, "&") : "";\n      } else {\n        // Set the url\n        url += params.length > 0 ? "(" + params.replace(/, $/, "") + ")" : "";\n      }\n    } // Return the url\n\n\n    return url;\n  };\n\n  return MethodInfo;\n}();\n\nexports.MethodInfo = MethodInfo;\n\n//# sourceURL=webpack://gd-sprest/./build/utils/methodInfo.js?')},"./build/utils/oData.js":function(__unused_webpack_module,exports){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.OData = void 0;\n/**\r\n * OData\r\n */\n\nvar OData =\n/** @class */\nfunction () {\n  /*********************************************************************************************************************************/\n  // Constructor\n\n  /*********************************************************************************************************************************/\n  // The class constructor\n  function OData(oData) {\n    // Default the Variables\n    this._custom = oData && oData.Custom ? oData.Custom : null;\n    this._expand = oData && oData.Expand ? oData.Expand : [];\n    this._filter = oData && oData.Filter ? oData.Filter : null;\n    this._getAllItems = oData && oData.GetAllItems ? oData.GetAllItems : false;\n    this._orderBy = oData && oData.OrderBy ? oData.OrderBy : [];\n    this._select = oData && oData.Select ? oData.Select : [];\n    this._skip = oData && oData.Skip ? oData.Skip : null;\n    this._top = oData && oData.Top ? oData.Top : null;\n  }\n\n  Object.defineProperty(OData.prototype, "Custom", {\n    /*********************************************************************************************************************************/\n    // Properties\n\n    /*********************************************************************************************************************************/\n    // Custom\n    get: function get() {\n      return this._custom;\n    },\n    set: function set(value) {\n      this._custom = value;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(OData.prototype, "Expand", {\n    // Expand\n    get: function get() {\n      return this._expand;\n    },\n    set: function set(value) {\n      this._expand = value;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(OData.prototype, "Filter", {\n    // Filter\n    get: function get() {\n      return this._filter;\n    },\n    set: function set(value) {\n      this._filter = value;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(OData.prototype, "GetAllItems", {\n    // Flag to get all items\n    get: function get() {\n      return this._getAllItems;\n    },\n    set: function set(value) {\n      this._getAllItems = value;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(OData.prototype, "OrderBy", {\n    // Order By\n    get: function get() {\n      return this._orderBy;\n    },\n    set: function set(value) {\n      this._orderBy = value;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(OData.prototype, "QueryString", {\n    // Query String\n    get: function get() {\n      var qs = "";\n      var values = []; // Get the query string values for the properties\n\n      values.push(this.getQSValue("$select", this._select));\n      values.push(this.getQSValue("$orderby", this._orderBy));\n      this._top ? values.push("$top=" + this._top) : null;\n      this._skip ? values.push("$skip=" + this._skip) : null;\n      this._filter ? values.push("$filter=" + this._filter) : null;\n      values.push(this.getQSValue("$expand", this._expand));\n      this._custom ? values.push(this._custom) : null; // Parse the values\n\n      for (var _i = 0, values_1 = values; _i < values_1.length; _i++) {\n        var value = values_1[_i]; // Ensure a value exists\n\n        if (value && value != "") {\n          // Append the query string value\n          qs += (qs == "" ? "" : "&") + value;\n        }\n      } // Return the query string\n\n\n      return qs;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(OData.prototype, "Select", {\n    // Select\n    get: function get() {\n      return this._select;\n    },\n    set: function set(value) {\n      this._select = value;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(OData.prototype, "Skip", {\n    // Skip\n    get: function get() {\n      return this._skip;\n    },\n    set: function set(value) {\n      this._skip = value;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(OData.prototype, "Top", {\n    // Top\n    get: function get() {\n      return this._top;\n    },\n    set: function set(value) {\n      this._top = value;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  /*********************************************************************************************************************************/\n  // Methods\n\n  /*********************************************************************************************************************************/\n  // Method to convert the array of strings to a query string value.\n\n  OData.prototype.getQSValue = function (qsKey, keys) {\n    // Return the query string\n    return keys.length > 0 ? qsKey + "=" + keys.join(",") : "";\n  };\n\n  return OData;\n}();\n\nexports.OData = OData;\n\n//# sourceURL=webpack://gd-sprest/./build/utils/oData.js?')},"./build/utils/request.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nfunction _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }\n\nvar __assign = this && this.__assign || function () {\n  __assign = Object.assign || function (t) {\n    for (var s, i = 1, n = arguments.length; i < n; i++) {\n      s = arguments[i];\n\n      for (var p in s) {\n        if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];\n      }\n    }\n\n    return t;\n  };\n\n  return __assign.apply(this, arguments);\n};\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.Request = void 0;\n\nvar base_1 = __webpack_require__(/*! ./base */ "./build/utils/base.js");\n\nvar batch_1 = __webpack_require__(/*! ./batch */ "./build/utils/batch.js");\n\nvar lib_1 = __webpack_require__(/*! ../lib */ "./build/lib/index.js");\n\nvar executor_1 = __webpack_require__(/*! ../helper/executor */ "./build/helper/executor.js");\n\nvar helper_1 = __webpack_require__(/*! ./helper */ "./build/utils/helper.js");\n\nvar mapper_1 = __webpack_require__(/*! ../mapper */ "./build/mapper/index.js");\n\nvar requestType_1 = __webpack_require__(/*! ./requestType */ "./build/utils/requestType.js");\n\nvar targetInfo_1 = __webpack_require__(/*! ./targetInfo */ "./build/utils/targetInfo.js");\n\nvar xhrRequest_1 = __webpack_require__(/*! ./xhrRequest */ "./build/utils/xhrRequest.js");\n/**\r\n * Request\r\n */\n\n\nexports.Request = {\n  // Method to add the methods to base object\n  addMethods: function addMethods(base, data, resultsObjType) {\n    var _a, _b;\n\n    var obj = base;\n    var isCollection = data.results && data.results.length > 0;\n    var methods = null; // Determine the metadata\n\n    var metadata = isCollection ? data.results[0].__metadata : data.__metadata; // Get the object type\n\n    var objType = metadata && metadata.type ? metadata.type : obj.targetInfo.endpoint;\n    var isV2 = ((_b = (_a = obj === null || obj === void 0 ? void 0 : obj.parent) === null || _a === void 0 ? void 0 : _a.targetInfo) === null || _b === void 0 ? void 0 : _b.endpoint.startsWith("_api/v2.0/")) || obj["@odata.context"] || objType.startsWith("@odata.context") ? true : false;\n\n    if (isV2) {\n      // See if we are overriding the object type\n      if (resultsObjType) {\n        // Set the object type\n        objType = resultsObjType;\n      } else {\n        // Get the object type from the context\n        var metadataType = obj["@odata.context"] || objType;\n        var values = metadataType.split("_api/v2.0/$metadata#");\n\n        if (values.length > 1) {\n          objType = values[1];\n        } else {\n          values = metadataType.split("/");\n          objType = values[values.length - 1].split("?")[0];\n        }\n      } // Get the methods for this object type\n\n\n      methods = mapper_1.MapperV2[objType];\n      console.log("[gd-sprest] v2 response detected. Type is: " + objType, methods);\n    } // Else, get the methods from the default mapper, otherwise get it from the custom mapper\n    else if ((methods = mapper_1.Mapper[objType + (isCollection ? ".Collection" : "")]) == null) {\n      // Determine the object type\n      var values = objType.split(\'/\');\n      objType = values[values.length - 1];\n      values = objType.split(\'.\');\n      objType = values[values.length - 1].toLowerCase();\n      objType += isCollection ? "s" : ""; // See if this is a graph request\n\n      if (/^graph/.test(objType)) {// Do nothing\n      } // Else, see if the base is a field\n      else if ((/^field/.test(objType) || /fields?$/.test(objType)) && objType != "fieldlinks" && objType != "fields") {\n        // Update the type\n        objType = "field" + (isCollection ? "s" : "");\n      } // Else, see if the base is an item\n      else if (/item$/.test(objType)) {\n        // Update the type\n        objType = "listitem";\n      } // Else, see if the base is an item collection\n      else if (/items$/.test(objType)) {\n        // Update the type\n        objType = "items";\n      } // Else, see if this is a tenant app\n      else if (/corporatecatalogappmetadata/.test(objType)) {\n        // Update the type\n        objType = "tenantapp";\n      } // Else, see if this is a tenant app collection\n      else if (/corporatecatalogappmetadatas/.test(objType)) {\n        // Update the type\n        objType = "tenantapps";\n      } // Get the methods for the base object\n\n\n      methods = mapper_1.Mapper_Custom[objType];\n    } // Ensure methods exist\n\n\n    if (methods) {\n      // Parse the methods\n      for (var methodName in methods) {\n        // Get the method information\n        var methodInfo = methods[methodName] ? methods[methodName] : {}; // See if the base is the "Properties" definition for the object\n\n        if (methodName == "properties") {\n          // Parse the properties\n          for (var _i = 0, methodInfo_1 = methodInfo; _i < methodInfo_1.length; _i++) {\n            var property = methodInfo_1[_i];\n            var propInfo = property.split("|"); // Get the metadata type\n\n            var propName = propInfo[0];\n            var propType = propInfo.length > 1 ? propInfo[1] : null;\n            var subPropName = propInfo.length > 2 ? propInfo[2] : null;\n            var subPropType = propInfo.length > 3 ? propInfo[3] : null; // See if the property is null or is a collection\n\n            if (obj[propName] == null || obj[propName].__deferred && obj[propName].__deferred.uri) {\n              // See if the base property has a sub-property defined for it\n              if (propInfo.length == 4) {\n                // Update the \' char in the property name\n                subPropName = subPropName.replace(/\'/g, "\\\\\'"); // Add the property\n\n                obj[propName] = new Function("name", "name = name ? \'" + propName + subPropName + "\'.replace(/\\\\[Name\\\\]/g, name.toString().replace(/\\\'/g, \\"\'\'\\")) : null;" + "return this.getProperty(name ? name : \'" + propName + "\', name ? \'" + subPropType + "\' : \'" + propType + "\');");\n              } else {\n                // Add the property\n                obj[propName] = new Function("return this.getProperty(\'" + propName + "\', \'" + propType + "\');");\n              }\n            }\n          } // Continue the loop\n\n\n          continue;\n        } // See if the base object has a dynamic metadata type\n\n\n        if (typeof methodInfo.metadataType === "function") {\n          // Clone the object properties\n          methodInfo = JSON.parse(JSON.stringify(methodInfo)); // Set the metadata type\n\n          methodInfo.metadataType = methods[methodName].metadataType(obj);\n        } // Add the method to the object\n\n\n        obj[methodName] = obj[methodName] || new Function("return this.executeMethod(\'" + methodName + "\', " + JSON.stringify(methodInfo) + ", arguments);");\n      }\n    }\n  },\n  // Method to add properties to the base object\n  addProperties: function addProperties(base, data) {\n    // Parse the data properties\n    for (var key in data) {\n      var value = data[key]; // Skip the results\n\n      if (key == "results") {\n        continue;\n      } // See if this is the metadata\n\n\n      if (key == "__metadata") {\n        // Set the etag value and continue\n        base["etag"] = value["etag"];\n        continue;\n      } // See if the base is a collection property\n\n\n      if (value && value.__deferred && value.__deferred.uri) {\n        // Generate a method for the base property\n        base["get_" + key] = base["get_" + key] ? base["get_" + key] : new Function("return this.getCollection(\'" + key + "\', arguments);");\n      } else {\n        // Set the property, based on the property name\n        switch (key) {\n          case "ClientPeoplePickerResolveUser":\n          case "ClientPeoplePickerSearchUser":\n            base[key] = JSON.parse(value);\n            break;\n\n          default:\n            // Append the property to the base object\n            base[key] = value;\n            break;\n        } // See if the base is a collection\n\n\n        if (base[key] && base[key].results) {\n          // Ensure the collection is an object\n          if (base[key].results.length == 0 || _typeof(base[key].results[0]) === "object") {\n            // Create the base property as a new request\n            var objCollection = new base_1.Base(base.targetInfo);\n            objCollection["results"] = base[key].results; // See no results exist\n\n            if (objCollection["results"].length == 0) {\n              // Set the metadata type to the key\n              objCollection["__metadata"] = {\n                type: key\n              };\n            } // Update the endpoint for the base request to point to the base property\n\n\n            objCollection.targetInfo.endpoint = (objCollection.targetInfo.endpoint.split("?")[0] + "/" + key).replace(/\\//g, "/"); // Add the methods\n\n            exports.Request.addMethods(objCollection, objCollection); // Update the data collection\n\n            helper_1.Helper.updateDataCollection(base, objCollection["results"]); // Update the expanded properties\n\n            helper_1.Helper.updateExpandedProperties(base); // Update the property\n\n            base[key] = objCollection;\n          }\n        }\n      }\n    }\n  },\n  // Method to execute the request\n  execute: function execute(base, args) {\n    var reject = null;\n    var resolve = null;\n    var waitFl = false; // Parse the arguments\n\n    for (var i = 0; i < args.length; i++) {\n      var arg = args[i]; // Check the type\n\n      switch (_typeof(arg)) {\n        case "boolean":\n          // Set the wait flag\n          waitFl = arg;\n          break;\n\n        case "function":\n          // See if the resolve method exists\n          if (resolve) {\n            // Set the reject method\n            reject = arg;\n          } else {\n            // Set the resolve method\n            resolve = arg;\n          }\n\n          break;\n      }\n    } // Set the base\n\n\n    base.base = base.base || base; // Set the base responses\n\n    base.base.responses = base.base.responses || []; // Set the base wait flags\n\n    base.base.waitFlags = base.base.waitFlags || []; // Set the response index\n\n    base.responseIndex = base.base.responses.length; // Add this object to the responses\n\n    base.base.responses.push(base); // See if we are waiting for the responses to complete\n\n    if (waitFl) {\n      // Wait for the responses to execute\n      exports.Request.waitForRequestsToComplete(base, function () {\n        // Execute this request\n        exports.Request.executeRequest(base, true, function (response, errorFl) {\n          // See if there was an error\n          if (errorFl) {\n            // Set the wait flag\n            base.base.waitFlags[base.responseIndex] = true; // Reject the request\n\n            reject ? reject(response) : null;\n          } // Else, see if there is a resolve method\n          else if (resolve) {\n            // Execute the callback and see if it returns a promise\n            var returnVal = resolve(response);\n            var waitFunc = returnVal ? returnVal.done || returnVal.then : null;\n\n            if (waitFunc && typeof waitFunc === "function") {\n              // Wait for the promise to complete\n              waitFunc(function () {\n                // Set the wait flag\n                base.base.waitFlags[base.responseIndex] = true; // Set the base to this object, and clear requests\n                // This will ensure requests from this object do not conflict w/ this request\n\n                base.base = base;\n                base.base.responses = [];\n                base.base.waitFlags = []; // Reset the base\n\n                base.base = (base.parent ? base.parent.base : null) || base.base;\n              }); // Do nothing\n\n              return;\n            } // Set the wait flag\n\n\n            base.base.waitFlags[base.responseIndex] = true; // Set the base to this object, and clear requests\n            // This will ensure requests from this object do not conflict w/ this request\n\n            base.base = base;\n            base.base.responses = [];\n            base.base.waitFlags = []; // Reset the base\n\n            base.base = (base.parent ? base.parent.base : null) || base.base;\n          } else {\n            // Set the wait flag\n            base.base.waitFlags[base.responseIndex] = true;\n          }\n        });\n      }, base.responseIndex);\n    } else {\n      // Execute this request\n      exports.Request.executeRequest(base, true, function (response, errorFl) {\n        // See if there was an error\n        if (errorFl) {\n          // Set the wait flag\n          base.base.waitFlags[base.responseIndex] = true; // Reject the request\n\n          reject ? reject(response) : null;\n        } else {\n          // Execute the resolve and see if it returns a promise\n          var returnVal = resolve ? resolve(response) : null;\n\n          if (returnVal && typeof returnVal.done === "function") {\n            // Wait for the promise to complete\n            returnVal.done(function () {\n              // Set the wait flag\n              base.base.waitFlags[base.responseIndex] = true;\n            });\n          } else {\n            // Set the wait flag\n            base.base.waitFlags[base.responseIndex] = true;\n          }\n        }\n      });\n    } // See if this is a query request\n\n\n    if (base.targetInfo.requestType == requestType_1.RequestType.OData) {\n      // Return the parent for chaining purposes\n      return base.parent;\n    } // Return this object\n\n\n    return base;\n  },\n  // Method to execute the request\n  executeRequest: function executeRequest(base, asyncFl, callback) {\n    // Execution method\n    var execute = function execute(targetInfo, batchIdx, onComplete) {\n      // See if this is an asynchronous request\n      if (asyncFl) {\n        // See if the not a batch request, and it already exists\n        if (base.xhr && !isBatchRequest) {\n          // Execute the callback\n          callback ? callback(base, false) : null;\n        } else {\n          // Create the request\n          base.xhr = new xhrRequest_1.XHRRequest(asyncFl, targetInfo, function () {\n            // Update the response and status\n            base.response = base.xhr.response;\n            base.status = base.xhr.status;\n            var errorFl = !(base.status >= 200 && base.status < 300); // See if we are returning a file buffer\n\n            if (base.requestType == requestType_1.RequestType.GetBuffer) {\n              // Execute the callback\n              callback ? callback(base.response, errorFl) : null;\n            } else {\n              // Update the data object\n              exports.Request.updateDataObject(base, isBatchRequest, batchIdx); // Ensure this isn\'t a batch request\n\n              if (!isBatchRequest) {\n                // See if this is an xml response\n                if (base.xml) {\n                  // Execute the callback\n                  callback ? callback(base, errorFl) : null;\n                } else {\n                  // Validate the data collection\n                  exports.Request.validateDataCollectionResults(base).then( // Success\n                  function () {\n                    // Execute the callback\n                    callback ? callback(base, errorFl) : null;\n                  }, // Error\n                  function () {\n                    // Execute the callback and set the error flag\n                    callback ? callback(base, true) : null;\n                  });\n                }\n              }\n            } // Call the event\n\n\n            onComplete ? onComplete() : null;\n          });\n        }\n      } // Else, see if we already executed this request\n      else if (base.xhr) {\n        return base;\n      } // Else, we haven\'t executed this request\n      else {\n        // Create the request\n        base.xhr = new xhrRequest_1.XHRRequest(asyncFl, targetInfo); // Update the response and status\n\n        base.response = base.xhr.response;\n        base.status = base.xhr.status; // See if we are returning a file buffer\n\n        if (base.requestType == requestType_1.RequestType.GetBuffer) {\n          // Return the response\n          return base.response;\n        } // Update the base object\n\n\n        exports.Request.updateDataObject(base, isBatchRequest, batchIdx); // See if the base is a collection and has more results\n\n        if (base["d"] && base["d"].__next) {\n          // Add the "next" method to get the next set of results\n          base["next"] = new Function("return this.getNextSetOfResults();");\n        } // Call the event\n\n\n        onComplete ? onComplete() : null; // Return the base object\n\n        return base;\n      }\n    }; // See if this is a batch request\n\n\n    var isBatchRequest = base.base && base.base.batchRequests && base.base.batchRequests.length > 0;\n\n    if (isBatchRequest) {\n      var batchIdx_1 = 0; // Parse the requests\n\n      executor_1.Executor(base.base.batchRequests, function (batchRequest) {\n        // Return a promise\n        return new Promise(function (resolve) {\n          // Execute the request\n          execute(batch_1.Batch.getTargetInfo(base.targetInfo.url, batchRequest, base.targetInfo.requestDigest), batchIdx_1++, function () {\n            // Resolve the request\n            resolve(null);\n          });\n        });\n      }).then(function () {\n        // Execute the callback if it exists\n        callback ? callback(base.base.batchRequests, false) : null; // Clear the batch requests\n\n        base.base.batchRequests = null;\n      });\n    } else {\n      // Execute the request\n      return execute(new targetInfo_1.TargetInfo(base.targetInfo));\n    }\n  },\n  // Method to parse the xml\n  parseXML: function parseXML(xml, objData) {\n    if (objData === void 0) {\n      objData = {};\n    }\n\n    var results = null; // See if the element has children\n\n    if (xml.hasChildNodes()) {\n      // Parse the child nodes\n      for (var i = 0; i < xml.childNodes.length; i++) {\n        var childNode = xml.childNodes[i];\n        var childPropName = childNode.nodeName.replace("d:", ""); // See if this is a text element\n\n        if (childPropName == "#text") {\n          // Return the value\n          return childNode.nodeValue;\n        } // Else, see if this is a collection\n        else if (childPropName == "element") {\n          // Ensure the results exist\n          results = results || []; // Append the object\n\n          results.push(exports.Request.parseXML(childNode));\n        } else {\n          // Read the value properties\n          var childType = childNode.getAttribute("m:type"); // Get the value\n\n          var value = exports.Request.parseXML(childNode);\n\n          if (value) {\n            // Update the value based on the type\n            switch (childType) {\n              // Boolean\n              case "Edm.Boolean":\n                value = value ? true : false;\n                break;\n              // Float\n\n              case "Edm.Decimal":\n              case "Edm.Double":\n                value = parseFloat(value);\n                break;\n              // Integer\n\n              case "Edm.Int16":\n              case "Edm.Int32":\n              case "Edm.Int64":\n                value = parseInt(value);\n                break;\n            }\n          } // Parse the node\n\n\n          objData[childPropName] = value;\n        }\n      }\n    } else {\n      // Return the property value\n      return xml.nodeValue;\n    } // Return the collection if it exists, otherwise the object\n\n\n    return results ? {\n      results: results\n    } : objData;\n  },\n  // Method to convert the input arguments into an object\n  updateDataObject: function updateDataObject(base, isBatchRequest, batchIdx) {\n    if (isBatchRequest === void 0) {\n      isBatchRequest = false;\n    }\n\n    if (batchIdx === void 0) {\n      batchIdx = 0;\n    } // Ensure the request was successful\n\n\n    if (base.status >= 200 && base.status < 300) {\n      // Return if we are expecting a buffer\n      if (base.requestType == requestType_1.RequestType.GetBuffer) {\n        return;\n      } // Parse the responses\n\n\n      var batchRequestIdx = 0;\n      var responses = isBatchRequest ? base.response.split("\\n") : [base.response];\n\n      for (var i = 0; i < responses.length; i++) {\n        var data = null; // Set the response\n\n        var response = responses[i];\n        response = response === "" && !isBatchRequest ? "{}" : response; // Set the xml flag\n\n        var isXML = response.indexOf("<?xml") == 0;\n\n        if (isXML) {\n          // Append the response while data exists\n          while (responses[i + 1] && responses[i + 1].indexOf("--batchresponse") != 0) {\n            // Append the response\n            response += responses[++i];\n          }\n        } // Try to convert the response\n\n\n        try {\n          data = isXML ? response.replace(/\\r/g, \'\') : JSON.parse(response);\n        } catch (ex) {\n          continue;\n        } // Set the object based on the request type\n\n\n        var obj = isBatchRequest ? Object.create(base) : base; // Set the exists flag\n\n        obj["existsFl"] = typeof obj["Exists"] === "boolean" ? obj["Exists"] : data.error == null; // See if this is xml\n\n        if (isXML) {\n          var objData = {}; // Convert the xml to an object\n\n          var parser = new DOMParser();\n          var xmlDoc = parser.parseFromString(data, "application/xml").firstChild;\n          objData[xmlDoc.nodeName.replace("d:", "")] = exports.Request.parseXML(xmlDoc); // See if this is a REST request\n\n          if (objData.entry && objData.entry.content && objData.entry.content["m:properties"]) {\n            // Set the object to the properties\n            objData = objData.entry.content["m:properties"]; // Update the metadata\n\n            helper_1.Helper.updateMetadata(obj, objData); // Update the base object\'s properties\n\n            exports.Request.addProperties(obj, objData); // Add the methods\n\n            exports.Request.addMethods(obj, objData, objData["@odata.context"]); // Update the data collection\n\n            helper_1.Helper.updateDataCollection(obj, objData["results"]); // Update the expanded properties\n\n            helper_1.Helper.updateExpandedProperties(obj); // Update the search results\n\n            helper_1.Helper.updateSearchResults(obj);\n          } else {\n            // Update the object\n            obj = __assign(__assign({}, obj), objData); // Update the search results\n\n            helper_1.Helper.updateSearchResults(obj);\n          }\n        } // Else, see if the data properties exists\n        else if (data.d) {\n          // Save a reference to it\n          obj["d"] = data.d; // Update the metadata\n\n          helper_1.Helper.updateMetadata(obj, data.d); // Update the base object\'s properties\n\n          exports.Request.addProperties(obj, data.d); // Add the methods\n\n          exports.Request.addMethods(obj, data.d, data["@odata.context"]); // Update the data collection\n\n          helper_1.Helper.updateDataCollection(obj, data.d.results); // Update the expanded properties\n\n          helper_1.Helper.updateExpandedProperties(obj); // Update the search results\n\n          helper_1.Helper.updateSearchResults(obj);\n        } // Else, see if this is a graph request\n        else if (data["@odata.context"]) {\n          // Save a reference to it\n          obj["d"] = data; // Update the base object\'s properties\n\n          exports.Request.addProperties(obj, data); // Add the methods\n\n          exports.Request.addMethods(obj, data, data["@odata.context"]); // Update the data collection\n\n          helper_1.Helper.updateDataCollection(obj, data.value);\n        } else {\n          // Update the base object\'s properties\n          exports.Request.addProperties(obj, data);\n        } // See if the batch request exists\n\n\n        if (isBatchRequest) {\n          // Get the batch request\n          var batchRequest = base.base.batchRequests[batchIdx][batchRequestIdx++];\n\n          if (batchRequest == null) {\n            // Update the batch indexes\n            batchIdx++;\n            batchRequestIdx = 0; // Ensure the requests exist\n\n            if (base.base.batchRequests[batchIdx]) {\n              // Update the batch request\n              var batch = base.base.batchRequests[batchIdx];\n              batchRequest = batch ? batch[batchRequestIdx++] : null;\n            } else {\n              break;\n            }\n          } // Ensure the batch request exists\n\n\n          if (batchRequest) {\n            // Set the response object\n            batchRequest.response = obj;\n          }\n        }\n      } // See if this was a batch request\n\n\n      if (isBatchRequest) {\n        // Process the callbacks\n        batch_1.Batch.processCallbacks(base.base.batchRequests[batchIdx]);\n      }\n    }\n  },\n  // Method to validate the data collection results\n  validateDataCollectionResults: function validateDataCollectionResults(base) {\n    // Return a promise\n    return new Promise(function (resolve, reject) {\n      // Method to validate the request\n      var request = function request(xhr, resolve) {\n        // Validate the response\n        if (xhr && xhr.status < 400 && typeof xhr.response === "string" && xhr.response.length > 0) {\n          // Try to convert the response and ensure the data property exists\n          var data = null;\n\n          try {\n            data = JSON.parse(xhr.response);\n          } // Reject the request\n          catch (_a) {\n            reject();\n            return;\n          } // Set the next item flag\n\n\n          base.nextFl = data.d && data.d.__next; // See if there are more items to get\n\n          if (base.nextFl) {\n            // See if we are getting all items in the base request\n            if (base.getAllItemsFl) {\n              // Create the target information to query the next set of results\n              var targetInfo = Object.create(base.targetInfo);\n              targetInfo.endpoint = "";\n              targetInfo.url = data.d.__next; // Create a new object\n\n              new xhrRequest_1.XHRRequest(true, new targetInfo_1.TargetInfo(targetInfo), function (xhr) {\n                // Convert the response and ensure the data property exists\n                var data = JSON.parse(xhr.response);\n\n                if (data.d) {\n                  // Update the data collection\n                  helper_1.Helper.updateDataCollection(base, data.d.results); // Update the expanded properties\n\n                  helper_1.Helper.updateExpandedProperties(base); // Append the raw data results\n\n                  base["d"].results = base["d"].results.concat(data.d.results); // Validate the data collection\n\n                  request(xhr, resolve);\n                } else {\n                  // Resolve the promise\n                  resolve();\n                }\n              });\n            } else {\n              // Add a method to get the next set of results\n              base["next"] = new Function("return this.getNextSetOfResults();"); // Resolve the promise\n\n              resolve();\n            }\n          } else {\n            // Resolve the promise\n            resolve();\n          }\n        } else {\n          // Resolve the promise\n          resolve();\n        }\n      }; // Execute the request\n\n\n      request(base.xhr, resolve);\n    });\n  },\n  // Method to wait for the parent requests to complete\n  waitForRequestsToComplete: function waitForRequestsToComplete(base, callback, requestIdx) {\n    // Ensure a callback exists and is a function\n    if (typeof callback === "function") {\n      // Loop until the requests have completed\n      var intervalId_1 = lib_1.ContextInfo.window.setInterval(function () {\n        var counter = 0; // Parse the responses to the requests\n\n        for (var i = 0; i < base.base.responses.length; i++) {\n          var response = base.base.responses[i]; // See if we are waiting until a specified index\n\n          if (requestIdx == counter++) {\n            break;\n          } // Return if the request hasn\'t completed\n\n\n          if (response.xhr == null || !response.xhr.completedFl) {\n            return;\n          } // Ensure the wait flag is set for the previous request\n\n\n          if (counter > 0 && base.base.waitFlags[counter - 1] != true) {\n            return;\n          }\n        } // Clear the interval\n\n\n        lib_1.ContextInfo.window.clearInterval(intervalId_1); // Execute the callback\n\n        callback();\n      }, 10);\n    }\n  }\n};\n\n//# sourceURL=webpack://gd-sprest/./build/utils/request.js?')},"./build/utils/requestType.js":function(__unused_webpack_module,exports){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.RequestType = void 0;\n/**\r\n * Request Type\r\n */\n\nexports.RequestType = {\n  // Requests\n  Custom: 0,\n  Delete: 1,\n  Merge: 2,\n  OData: 3,\n  // Get Requests\n  Get: 10,\n  GetBuffer: 11,\n  GetWithArgs: 12,\n  GetWithArgsInBody: 13,\n  GetWithArgsInQS: 14,\n  GetWithArgsInQSAsVar: 15,\n  GetWithArgsValueOnly: 16,\n  GetReplace: 17,\n  // Graph Requests\n  GraphGet: 20,\n  GraphGetReplace: 21,\n  GraphPost: 22,\n  GraphPostReplace: 23,\n  // Post Requests\n  Post: 30,\n  PostBodyNoArgs: 31,\n  PostWithArgs: 32,\n  PostWithArgsAndData: 33,\n  PostWithArgsInBody: 34,\n  PostWithArgsInQS: 35,\n  PostWithArgsInQSAsVar: 36,\n  PostWithArgsValueOnly: 37,\n  PostReplace: 38,\n  PostReplaceWithData: 39\n};\n\n//# sourceURL=webpack://gd-sprest/./build/utils/requestType.js?')},"./build/utils/targetInfo.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.TargetInfo = void 0;\n\nvar lib_1 = __webpack_require__(/*! ../lib */ "./build/lib/index.js");\n\nvar _1 = __webpack_require__(/*! . */ "./build/utils/index.js");\n/**\r\n * Target Information\r\n */\n\n\nvar TargetInfo =\n/** @class */\nfunction () {\n  /*********************************************************************************************************************************/\n  // Constructor\n\n  /*********************************************************************************************************************************/\n  function TargetInfo(props) {\n    // Default the properties\n    this.props = props || {};\n    this.requestData = this.props.data;\n    this.requestHeaders = this.props.requestHeader;\n    this.requestMethod = this.props.method ? this.props.method : "GET"; // See if this is a graph request\n\n    if (this.isGraph) {\n      // Set the request method\n      this.requestMethod = this.props.requestType == _1.RequestType.Get || this.props.requestType == _1.RequestType.GetReplace || this.props.requestType == _1.RequestType.GetWithArgsValueOnly || this.props.requestType == _1.RequestType.GraphGet || this.props.requestType == _1.RequestType.GraphGetReplace || this.props.requestType == _1.RequestType.OData ? "GET" : "POST"; // Set the request url\n\n      this.requestUrl = this.props.endpoint;\n    } else {\n      // Set the request url\n      this.setRESTRequestUrl();\n    }\n  }\n\n  Object.defineProperty(TargetInfo.prototype, "isBatchRequest", {\n    // Flag to determine if this is a batch request\n    get: function get() {\n      return this.props.endpoint == "$batch";\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(TargetInfo.prototype, "isGraph", {\n    // Flag to determine if this is a graph request\n    get: function get() {\n      var _a;\n\n      return ((_a = this.props.endpoint) === null || _a === void 0 ? void 0 : _a.startsWith("_api/v2.0/")) || this.props.requestType == _1.RequestType.GraphGet || this.props.requestType == _1.RequestType.GraphPost || this.props.requestType == _1.RequestType.GraphGetReplace || this.props.requestType == _1.RequestType.GraphPostReplace;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  /*********************************************************************************************************************************/\n  // Methods\n\n  /*********************************************************************************************************************************/\n  // Method to get the domain url\n\n  TargetInfo.prototype.getDomainUrl = function () {\n    var url = lib_1.ContextInfo.document ? lib_1.ContextInfo.document.location.href : ""; // See if this is an app web\n\n    if (lib_1.ContextInfo.isAppWeb) {\n      // Set the url to the host url\n      url = TargetInfo.getQueryStringValue("SPHostUrl") + "";\n    } // Split the url and validate it\n\n\n    url = url.split(\'/\');\n\n    if (url && url.length >= 2) {\n      // Set the url\n      url = url[0] + "//" + url[2];\n    } // Return the url\n\n\n    return url;\n  }; // Method to get a query string value\n\n\n  TargetInfo.getQueryStringValue = function (key) {\n    // Get the query string\n    var queryString = lib_1.ContextInfo.existsFl && lib_1.ContextInfo.document ? lib_1.ContextInfo.document.location.href.split(\'?\') : [""];\n    queryString = queryString.length > 1 ? queryString[1] : queryString[0]; // Parse the values\n\n    var values = queryString.split(\'&\');\n\n    for (var i = 0; i < values.length; i++) {\n      var keyValue = values[i].split(\'=\'); // Ensure a value exists\n\n      if (keyValue.length == 1) {\n        continue;\n      } // See if this is the key we are looking for\n\n\n      if (decodeURIComponent(keyValue[0]) == key) {\n        return decodeURIComponent(keyValue[1]);\n      }\n    } // Key was not found\n\n\n    return null;\n  }; // Method to set the request url for the REST API\n\n\n  TargetInfo.prototype.setRESTRequestUrl = function () {\n    var endpoint = this.props.endpoint ? "/" + this.props.endpoint : "";\n    var hostUrl = TargetInfo.getQueryStringValue("SPHostUrl");\n    var qs = (endpoint.indexOf("?") === -1 ? "?" : "&") + "@target=\'{{Target}}\'";\n    var template = "{{Url}}"; // See if the template is targeting the _vti_bin\n\n    if (endpoint.indexOf("_vti_bin") >= 0) {\n      template += "/{{EndPoint}}{{TargetUrl}}";\n    } else {\n      template += this.props.endpoint ? "/_api/{{EndPoint}}{{TargetUrl}}" : "";\n    } // See if we are defaulting the url for the app web\n\n\n    if (lib_1.ContextInfo.existsFl && lib_1.ContextInfo.window.$REST && lib_1.ContextInfo.window.$REST.DefaultRequestToHostFl && lib_1.ContextInfo.isAppWeb && !this.props.overrideDefaultRequestToHostFl && this.props.url == null) {\n      // Default the url to the host web\n      this.props.url = hostUrl;\n    } // Ensure the url exists\n\n\n    if (this.props.url == null) {\n      // Default the url to the current site/web url\n      this.props.url = this.props.defaultToWebFl == false ? lib_1.ContextInfo.siteAbsoluteUrl : lib_1.ContextInfo.webAbsoluteUrl;\n    } // Else, see if the url already contains the full request\n    else if (/\\/_api\\//.test(this.props.url)) {\n      // Get the url\n      var url = this.props.url.toLowerCase().split("/_api/"); // See if this is the app web and we are executing against a different web\n\n      if (lib_1.ContextInfo.isAppWeb && url[0] != lib_1.ContextInfo.webAbsoluteUrl.toLowerCase()) {\n        // Set the request url\n        this.requestUrl = lib_1.ContextInfo.webAbsoluteUrl + "/_api/SP.AppContextSite(@target)/" + url[1] + endpoint + qs.replace(/{{Target}}/g, url[0]);\n      } else {\n        // Set the request url\n        this.requestUrl = this.props.url + (this.props.endpoint ? "/" + this.props.endpoint : "");\n      }\n\n      return;\n    } // See if this is a relative url\n\n\n    if (this.props.url.indexOf("http") != 0) {\n      // Add the domain\n      this.props.url = this.getDomainUrl() + this.props.url;\n    } // See if this is the app web, and we are executing against a different web\n\n\n    if (lib_1.ContextInfo.isAppWeb && this.props.url != lib_1.ContextInfo.webAbsoluteUrl) {\n      // Set the request url\n      this.requestUrl = template.replace(/{{Url}}/g, lib_1.ContextInfo.webAbsoluteUrl).replace(/{{EndPoint}}/g, "SP.AppContextSite(@target)" + endpoint).replace(/{{TargetUrl}}/g, qs.replace(/{{Target}}/g, this.props.url));\n    } else {\n      // Set the request url\n      this.requestUrl = template.replace(/{{Url}}/g, this.props.url).replace(/{{EndPoint}}/g, this.props.endpoint).replace(/{{TargetUrl}}/g, "");\n    }\n  };\n\n  return TargetInfo;\n}();\n\nexports.TargetInfo = TargetInfo;\n\n//# sourceURL=webpack://gd-sprest/./build/utils/targetInfo.js?')},"./build/utils/xhrRequest.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.XHRRequest = void 0;\n\nvar lib_1 = __webpack_require__(/*! ../lib */ "./build/lib/index.js");\n/**\r\n * XML HTTP Request Class\r\n */\n\n\nvar XHRRequest =\n/** @class */\nfunction () {\n  /*********************************************************************************************************************************/\n  // Constructor\n\n  /*********************************************************************************************************************************/\n  function XHRRequest(asyncFl, targetInfo, callback, executeFl) {\n    if (executeFl === void 0) {\n      executeFl = true;\n    } // Default the properties\n\n\n    this.asyncFl = asyncFl;\n    this.executeFl = executeFl;\n    this.headers = {};\n    this.onRequestCompleted = callback || targetInfo.props.callback;\n    this.targetInfo = targetInfo; // Create the request\n\n    this.xhr = this.createXHR();\n\n    if (this.xhr) {\n      // Execute the request\n      this.execute();\n    } else {\n      // Default the headers\n      this.defaultHeaders();\n    }\n  }\n\n  Object.defineProperty(XHRRequest.prototype, "completedFl", {\n    /*********************************************************************************************************************************/\n    // Public Properties\n\n    /*********************************************************************************************************************************/\n    // Flag indicating the request has completed\n    get: function get() {\n      return this.xhr ? this.xhr.readyState == 4 : false;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(XHRRequest.prototype, "response", {\n    // The response\n    get: function get() {\n      return this.xhr ? this.xhr.response : null;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(XHRRequest.prototype, "request", {\n    // The xml http request\n    get: function get() {\n      return this.xhr ? this.xhr : null;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(XHRRequest.prototype, "requestData", {\n    // The data send in the body of the request\n    get: function get() {\n      return this.targetInfo.requestData;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(XHRRequest.prototype, "requestHeaders", {\n    // The request headers\n    get: function get() {\n      return this.headers;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(XHRRequest.prototype, "requestInfo", {\n    // The request information\n    get: function get() {\n      // Return the request information\n      return {\n        data: this.targetInfo.requestData,\n        headers: this.headers,\n        method: this.targetInfo.requestMethod,\n        url: this.targetInfo.requestUrl\n      };\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(XHRRequest.prototype, "requestUrl", {\n    // The request url\n    get: function get() {\n      return this.xhr ? this.xhr.responseURL : null;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  Object.defineProperty(XHRRequest.prototype, "status", {\n    // The request status\n    get: function get() {\n      return this.xhr ? this.xhr.status : null;\n    },\n    enumerable: false,\n    configurable: true\n  });\n  /*********************************************************************************************************************************/\n  // Private Methods\n\n  /*********************************************************************************************************************************/\n  // Method to create the xml http request\n\n  XHRRequest.prototype.createXHR = function () {\n    // See if the generic object doesn\'t exist\n    if (typeof XMLHttpRequest !== "undefined") {\n      // Create an instance of the xml http request object\n      return new XMLHttpRequest();\n    } // Try to create the request\n\n\n    try {\n      return new ActiveXObject("Msxml2.XMLHTTP.6.0");\n    } catch (e) {} // Try to create the request\n\n\n    try {\n      return new ActiveXObject("Msxml2.XMLHTTP.3.0");\n    } catch (e) {} // Try to create the request\n\n\n    try {\n      return new ActiveXObject("Microsoft.XMLHTTP");\n    } catch (e) {} // Log an error\n\n\n    console.error("This browser does not support xml http requests.");\n  }; // Method to default the request headers\n\n\n  XHRRequest.prototype.defaultHeaders = function (requestDigest) {\n    var ifMatchExists = false; // See if the custom headers exist\n\n    if (this.targetInfo.requestHeaders) {\n      // Parse the custom headers\n      for (var header in this.targetInfo.requestHeaders) {\n        // Add the header\n        this.xhr ? this.xhr.setRequestHeader(header, this.targetInfo.requestHeaders[header]) : null;\n        this.headers[header] = this.targetInfo.requestHeaders[header]; // See if this is the "IF-MATCH" header\n\n        ifMatchExists = ifMatchExists || header.toUpperCase() == "IF-MATCH";\n      }\n    } else {\n      // See if this is a graph request\n      if (this.targetInfo.isGraph) {\n        // Set the default headers\n        this.xhr ? this.xhr.setRequestHeader("Accept", "application/json") : null;\n        this.xhr ? this.xhr.setRequestHeader("Content-Type", "application/json") : null;\n        this.headers["Accept"] = "application/json";\n        this.headers["Content-Type"] = "application/json";\n      } else {\n        // Set the default headers\n        this.xhr ? this.xhr.setRequestHeader("Accept", "application/json;odata=verbose") : null;\n        this.xhr ? this.xhr.setRequestHeader("Content-Type", "application/json;odata=verbose") : null;\n        this.headers["Accept"] = "application/json;odata=verbose";\n        this.headers["Content-Type"] = "application/json;odata=verbose";\n      }\n    } // See if we are disabling cache\n\n\n    if (this.targetInfo.props.disableCache) {\n      // Add the header\n      this.xhr ? this.xhr.setRequestHeader("Cache-Control", "no-cache") : null;\n      this.headers["Cache-Control"] = "no-cache";\n    } // See if this is a graph request\n\n\n    if (this.targetInfo.isGraph) {\n      // Ensure the access token exists\n      if (this.targetInfo.props.accessToken) {\n        // Set the authorization\n        this.xhr ? this.xhr.setRequestHeader("Authorization", "Bearer " + this.targetInfo.props.accessToken) : null;\n        this.headers["Authorization"] = "Bearer " + this.targetInfo.props.accessToken;\n      } else {\n        // Set the request digest\n        this.xhr ? this.xhr.setRequestHeader("X-RequestDigest", requestDigest) : null;\n        requestDigest ? this.headers["X-RequestDigest"] = requestDigest : null;\n      }\n    } else {\n      // See if custom headers were not defined\n      if (this.targetInfo.requestHeaders == null) {\n        // Set the method by default\n        this.xhr ? this.xhr.setRequestHeader("X-HTTP-Method", this.targetInfo.requestMethod) : null;\n        this.headers["X-HTTP-Method"] = this.targetInfo.requestMethod;\n      } // Set the request digest\n\n\n      this.xhr ? this.xhr.setRequestHeader("X-RequestDigest", requestDigest) : null;\n      requestDigest ? this.headers["X-RequestDigest"] = requestDigest : null; // See if we are deleting or updating the data\n\n      if (this.targetInfo.requestMethod == "DELETE" || this.targetInfo.requestMethod == "MERGE" && !ifMatchExists) {\n        // Append the header for deleting/updating\n        this.xhr ? this.xhr.setRequestHeader("IF-MATCH", "*") : null;\n        this.headers["IF-MATCH"] = "*";\n      }\n    }\n  }; // Method to execute the xml http request\n\n\n  XHRRequest.prototype.execute = function () {\n    var _this = this; // Set the request digest\n\n\n    var requestDigest = this.targetInfo.props.requestDigest || "";\n\n    if (requestDigest == "") {\n      // Get the request digest\n      requestDigest = lib_1.ContextInfo.document ? lib_1.ContextInfo.document.querySelector("#__REQUESTDIGEST") : "";\n      requestDigest = requestDigest ? requestDigest.value : lib_1.ContextInfo.formDigestValue;\n    } // See if we are targeting the context endpoint\n\n\n    if (this.targetInfo.props.endpoint == "contextinfo") {\n      // Execute the request\n      this.executeRequest(requestDigest);\n    } // See if this is a post request and the request digest does not exist\n    else if (this.targetInfo.requestMethod != "GET" && requestDigest == "") {\n      // See if this is a synchronous request\n      if (!this.asyncFl) {\n        // Log\n        console.info("[gd-sprest] POST requests must include the request digest information for synchronous requests. This is due to the modern page not including this information on the page.");\n      } else {\n        // Get the context information\n        lib_1.ContextInfo.getWeb(this.targetInfo.props.url || document.location.pathname.substr(0, document.location.pathname.lastIndexOf(\'/\'))).execute(function (contextInfo) {\n          // Execute the request\n          _this.executeRequest(contextInfo.GetContextWebInformation.FormDigestValue);\n        });\n      }\n    } else {\n      // Execute the request\n      this.executeRequest(requestDigest);\n    }\n  }; // Method to execute the xml http request\n\n\n  XHRRequest.prototype.executeRequest = function (requestDigest) {\n    var _this = this; // Ensure the xml http request exists\n\n\n    if (this.xhr == null) {\n      return null;\n    } // Open the request\n\n\n    this.xhr.open(this.targetInfo.requestMethod == "GET" ? "GET" : "POST", this.targetInfo.requestUrl, this.asyncFl); // See if we are making an asynchronous request\n\n    if (this.asyncFl) {\n      // Set the state change event\n      this.xhr.onreadystatechange = function () {\n        // See if the request has finished\n        if (_this.xhr.readyState == 4) {\n          // Execute the request completed event\n          _this.onRequestCompleted ? _this.onRequestCompleted(_this) : null;\n        }\n      };\n    } // See if we the response type is an array buffer\n    // Note - Updating the response type is only allow for asynchronous requests. Any error will be thrown otherwise.\n\n\n    if (this.targetInfo.props.bufferFl && this.asyncFl) {\n      // Set the response type\n      this.xhr.responseType = "arraybuffer";\n    } else {\n      // Default the headers\n      this.defaultHeaders(requestDigest); // Ensure the arguments passed is defaulted as a string, unless it\'s an array buffer\n\n      if (this.targetInfo.requestData && typeof this.targetInfo.requestData !== "string") {\n        // Stringify the data object, if it\'s not an array buffer\n        this.targetInfo.requestData = this.targetInfo.requestData.byteLength ? this.targetInfo.requestData : JSON.stringify(this.targetInfo.requestData);\n      }\n    } // See if we are executing the request\n\n\n    if (this.executeFl) {\n      // Execute the request\n      this.targetInfo.props.bufferFl || this.targetInfo.requestData == null ? this.xhr.send() : this.xhr.send(this.targetInfo.requestData);\n    }\n  };\n\n  return XHRRequest;\n}();\n\nexports.XHRRequest = XHRRequest;\n\n//# sourceURL=webpack://gd-sprest/./build/utils/xhrRequest.js?')},"./build/v2/drive.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.drive = void 0;\n\nvar contextInfo_1 = __webpack_require__(/*! ../lib/contextInfo */ "./build/lib/contextInfo.js");\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js");\n/**\r\n * Drive\r\n * Returns the default library for a site.\r\n */\n\n\nexports.drive = function (id, targetInfo) {\n  var _a;\n\n  var drive = new utils_1.Base(targetInfo); // Default the properties\n\n  drive.targetInfo.defaultToWebFl = true;\n  drive.targetInfo.requestType = utils_1.RequestType.GraphGet; // See if an endpoint is not defined\n\n  if (drive.targetInfo.endpoint == undefined) {\n    // Default the endpoint\n    drive.targetInfo.endpoint = "_api/v2.0/sites/" + (id || ((_a = contextInfo_1.ContextInfo === null || contextInfo_1.ContextInfo === void 0 ? void 0 : contextInfo_1.ContextInfo.siteId) === null || _a === void 0 ? void 0 : _a.replace(/[{}]/g, \'\')) + "/drive"); // Add the methods\n    // TODO\n  } // Return the default library\n\n\n  return drive;\n};\n\n//# sourceURL=webpack://gd-sprest/./build/v2/drive.js?')},"./build/v2/drives.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.drives = void 0;\n\nvar contextInfo_1 = __webpack_require__(/*! ../lib/contextInfo */ "./build/lib/contextInfo.js");\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js");\n/**\r\n * Drives\r\n * Returns the libraries for a site.\r\n */\n\n\nexports.drives = function (id, targetInfo) {\n  var _a;\n\n  var drives = new utils_1.Base(targetInfo); // Default the properties\n\n  drives.targetInfo.defaultToWebFl = true;\n  drives.targetInfo.requestType = utils_1.RequestType.GraphGet; // See if an endpoint is not defined\n\n  if (drives.targetInfo.endpoint == undefined) {\n    // Default the endpoint\n    drives.targetInfo.endpoint = "_api/v2.0/sites/" + (id || ((_a = contextInfo_1.ContextInfo === null || contextInfo_1.ContextInfo === void 0 ? void 0 : contextInfo_1.ContextInfo.siteId) === null || _a === void 0 ? void 0 : _a.replace(/[{}]/g, \'\')) + "/drives"); // Add the methods\n    // TODO\n  } // Return the libraries\n\n\n  return drives;\n};\n\n//# sourceURL=webpack://gd-sprest/./build/v2/drives.js?')},"./build/v2/index.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nvar __createBinding = this && this.__createBinding || (Object.create ? function (o, m, k, k2) {\n  if (k2 === undefined) k2 = k;\n  Object.defineProperty(o, k2, {\n    enumerable: true,\n    get: function get() {\n      return m[k];\n    }\n  });\n} : function (o, m, k, k2) {\n  if (k2 === undefined) k2 = k;\n  o[k2] = m[k];\n});\n\nvar __exportStar = this && this.__exportStar || function (m, exports) {\n  for (var p in m) {\n    if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);\n  }\n};\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\n\n__exportStar(__webpack_require__(/*! ./drive */ "./build/v2/drive.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./drives */ "./build/v2/drives.js"), exports);\n\n__exportStar(__webpack_require__(/*! ./sites */ "./build/v2/sites.js"), exports);\n\n//# sourceURL=webpack://gd-sprest/./build/v2/index.js?')},"./build/v2/sites.js":function(__unused_webpack_module,exports,__webpack_require__){"use strict";eval('\n\nObject.defineProperty(exports, "__esModule", ({\n  value: true\n}));\nexports.sites = void 0;\n\nvar contextInfo_1 = __webpack_require__(/*! ../lib/contextInfo */ "./build/lib/contextInfo.js");\n\nvar utils_1 = __webpack_require__(/*! ../utils */ "./build/utils/index.js");\n/**\r\n * Sites\r\n * The sites graph endpoint.\r\n */\n\n\nexports.sites = function (id, targetInfo) {\n  var _a;\n\n  var sites = new utils_1.Base(targetInfo); // Default the properties\n\n  sites.targetInfo.defaultToWebFl = true;\n  sites.targetInfo.endpoint = "_api/v2.0/sites/" + (id || ((_a = contextInfo_1.ContextInfo === null || contextInfo_1.ContextInfo === void 0 ? void 0 : contextInfo_1.ContextInfo.siteId) === null || _a === void 0 ? void 0 : _a.replace(/[{}]/g, \'\')));\n  sites.targetInfo.requestType = utils_1.RequestType.GraphGet; // Add the methods\n\n  utils_1.Request.addMethods(sites, {\n    __metadata: {\n      type: "@odata.context/_api/v2.0/$metadata#sites"\n    }\n  }); // Return the sites\n\n  return sites;\n};\n/** Returns the current web. */\n\n\nexports.sites.getCurrentWeb = function () {\n  return exports.sites().sites(contextInfo_1.ContextInfo.webId.replace(/^\\{|\\}$/g, \'\'));\n};\n/** Returns a list by its title from the current web. */\n\n\nexports.sites.getList = function (title) {\n  return exports.sites().lists(title);\n};\n\n//# sourceURL=webpack://gd-sprest/./build/v2/sites.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/es/promise/index.js":function(module,__unused_webpack_exports,__webpack_require__){eval('__webpack_require__(/*! ../../modules/es.aggregate-error */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.aggregate-error.js");\n__webpack_require__(/*! ../../modules/es.array.iterator */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.array.iterator.js");\n__webpack_require__(/*! ../../modules/es.object.to-string */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.object.to-string.js");\n__webpack_require__(/*! ../../modules/es.promise */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.js");\n__webpack_require__(/*! ../../modules/es.promise.all-settled */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.all-settled.js");\n__webpack_require__(/*! ../../modules/es.promise.any */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.any.js");\n__webpack_require__(/*! ../../modules/es.promise.finally */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.finally.js");\n__webpack_require__(/*! ../../modules/es.string.iterator */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.string.iterator.js");\nvar path = __webpack_require__(/*! ../../internals/path */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/path.js");\n\nmodule.exports = path.Promise;\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/es/promise/index.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/a-callable.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var isCallable = __webpack_require__(/*! ../internals/is-callable */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-callable.js");\nvar tryToString = __webpack_require__(/*! ../internals/try-to-string */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/try-to-string.js");\n\nvar $TypeError = TypeError;\n\n// `Assert: IsCallable(argument) is true`\nmodule.exports = function (argument) {\n  if (isCallable(argument)) return argument;\n  throw $TypeError(tryToString(argument) + \' is not a function\');\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/a-callable.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/a-constructor.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var isConstructor = __webpack_require__(/*! ../internals/is-constructor */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-constructor.js");\nvar tryToString = __webpack_require__(/*! ../internals/try-to-string */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/try-to-string.js");\n\nvar $TypeError = TypeError;\n\n// `Assert: IsConstructor(argument) is true`\nmodule.exports = function (argument) {\n  if (isConstructor(argument)) return argument;\n  throw $TypeError(tryToString(argument) + \' is not a constructor\');\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/a-constructor.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/a-possible-prototype.js":function(module,__unused_webpack_exports,__webpack_require__){eval("var isCallable = __webpack_require__(/*! ../internals/is-callable */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-callable.js\");\n\nvar $String = String;\nvar $TypeError = TypeError;\n\nmodule.exports = function (argument) {\n  if (typeof argument == 'object' || isCallable(argument)) return argument;\n  throw $TypeError(\"Can't set \" + $String(argument) + ' as a prototype');\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/a-possible-prototype.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/add-to-unscopables.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var wellKnownSymbol = __webpack_require__(/*! ../internals/well-known-symbol */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/well-known-symbol.js");\nvar create = __webpack_require__(/*! ../internals/object-create */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-create.js");\nvar defineProperty = (__webpack_require__(/*! ../internals/object-define-property */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-define-property.js").f);\n\nvar UNSCOPABLES = wellKnownSymbol(\'unscopables\');\nvar ArrayPrototype = Array.prototype;\n\n// Array.prototype[@@unscopables]\n// https://tc39.es/ecma262/#sec-array.prototype-@@unscopables\nif (ArrayPrototype[UNSCOPABLES] == undefined) {\n  defineProperty(ArrayPrototype, UNSCOPABLES, {\n    configurable: true,\n    value: create(null)\n  });\n}\n\n// add a key to Array.prototype[@@unscopables]\nmodule.exports = function (key) {\n  ArrayPrototype[UNSCOPABLES][key] = true;\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/add-to-unscopables.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/an-instance.js":function(module,__unused_webpack_exports,__webpack_require__){eval("var isPrototypeOf = __webpack_require__(/*! ../internals/object-is-prototype-of */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-is-prototype-of.js\");\n\nvar $TypeError = TypeError;\n\nmodule.exports = function (it, Prototype) {\n  if (isPrototypeOf(Prototype, it)) return it;\n  throw $TypeError('Incorrect invocation');\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/an-instance.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/an-object.js":function(module,__unused_webpack_exports,__webpack_require__){eval("var isObject = __webpack_require__(/*! ../internals/is-object */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-object.js\");\n\nvar $String = String;\nvar $TypeError = TypeError;\n\n// `Assert: Type(argument) is Object`\nmodule.exports = function (argument) {\n  if (isObject(argument)) return argument;\n  throw $TypeError($String(argument) + ' is not an object');\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/an-object.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/array-includes.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var toIndexedObject = __webpack_require__(/*! ../internals/to-indexed-object */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-indexed-object.js");\nvar toAbsoluteIndex = __webpack_require__(/*! ../internals/to-absolute-index */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-absolute-index.js");\nvar lengthOfArrayLike = __webpack_require__(/*! ../internals/length-of-array-like */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/length-of-array-like.js");\n\n// `Array.prototype.{ indexOf, includes }` methods implementation\nvar createMethod = function (IS_INCLUDES) {\n  return function ($this, el, fromIndex) {\n    var O = toIndexedObject($this);\n    var length = lengthOfArrayLike(O);\n    var index = toAbsoluteIndex(fromIndex, length);\n    var value;\n    // Array#includes uses SameValueZero equality algorithm\n    // eslint-disable-next-line no-self-compare -- NaN check\n    if (IS_INCLUDES && el != el) while (length > index) {\n      value = O[index++];\n      // eslint-disable-next-line no-self-compare -- NaN check\n      if (value != value) return true;\n    // Array#indexOf ignores holes, Array#includes - not\n    } else for (;length > index; index++) {\n      if ((IS_INCLUDES || index in O) && O[index] === el) return IS_INCLUDES || index || 0;\n    } return !IS_INCLUDES && -1;\n  };\n};\n\nmodule.exports = {\n  // `Array.prototype.includes` method\n  // https://tc39.es/ecma262/#sec-array.prototype.includes\n  includes: createMethod(true),\n  // `Array.prototype.indexOf` method\n  // https://tc39.es/ecma262/#sec-array.prototype.indexof\n  indexOf: createMethod(false)\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/array-includes.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/array-slice.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var uncurryThis = __webpack_require__(/*! ../internals/function-uncurry-this */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-uncurry-this.js");\n\nmodule.exports = uncurryThis([].slice);\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/array-slice.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/check-correctness-of-iteration.js":function(module,__unused_webpack_exports,__webpack_require__){eval("var wellKnownSymbol = __webpack_require__(/*! ../internals/well-known-symbol */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/well-known-symbol.js\");\n\nvar ITERATOR = wellKnownSymbol('iterator');\nvar SAFE_CLOSING = false;\n\ntry {\n  var called = 0;\n  var iteratorWithReturn = {\n    next: function () {\n      return { done: !!called++ };\n    },\n    'return': function () {\n      SAFE_CLOSING = true;\n    }\n  };\n  iteratorWithReturn[ITERATOR] = function () {\n    return this;\n  };\n  // eslint-disable-next-line es-x/no-array-from, no-throw-literal -- required for testing\n  Array.from(iteratorWithReturn, function () { throw 2; });\n} catch (error) { /* empty */ }\n\nmodule.exports = function (exec, SKIP_CLOSING) {\n  if (!SKIP_CLOSING && !SAFE_CLOSING) return false;\n  var ITERATION_SUPPORT = false;\n  try {\n    var object = {};\n    object[ITERATOR] = function () {\n      return {\n        next: function () {\n          return { done: ITERATION_SUPPORT = true };\n        }\n      };\n    };\n    exec(object);\n  } catch (error) { /* empty */ }\n  return ITERATION_SUPPORT;\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/check-correctness-of-iteration.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/classof-raw.js":function(module,__unused_webpack_exports,__webpack_require__){eval("var uncurryThis = __webpack_require__(/*! ../internals/function-uncurry-this */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-uncurry-this.js\");\n\nvar toString = uncurryThis({}.toString);\nvar stringSlice = uncurryThis(''.slice);\n\nmodule.exports = function (it) {\n  return stringSlice(toString(it), 8, -1);\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/classof-raw.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/classof.js":function(module,__unused_webpack_exports,__webpack_require__){eval("var TO_STRING_TAG_SUPPORT = __webpack_require__(/*! ../internals/to-string-tag-support */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-string-tag-support.js\");\nvar isCallable = __webpack_require__(/*! ../internals/is-callable */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-callable.js\");\nvar classofRaw = __webpack_require__(/*! ../internals/classof-raw */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/classof-raw.js\");\nvar wellKnownSymbol = __webpack_require__(/*! ../internals/well-known-symbol */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/well-known-symbol.js\");\n\nvar TO_STRING_TAG = wellKnownSymbol('toStringTag');\nvar $Object = Object;\n\n// ES3 wrong here\nvar CORRECT_ARGUMENTS = classofRaw(function () { return arguments; }()) == 'Arguments';\n\n// fallback for IE11 Script Access Denied error\nvar tryGet = function (it, key) {\n  try {\n    return it[key];\n  } catch (error) { /* empty */ }\n};\n\n// getting tag from ES6+ `Object.prototype.toString`\nmodule.exports = TO_STRING_TAG_SUPPORT ? classofRaw : function (it) {\n  var O, tag, result;\n  return it === undefined ? 'Undefined' : it === null ? 'Null'\n    // @@toStringTag case\n    : typeof (tag = tryGet(O = $Object(it), TO_STRING_TAG)) == 'string' ? tag\n    // builtinTag case\n    : CORRECT_ARGUMENTS ? classofRaw(O)\n    // ES3 arguments fallback\n    : (result = classofRaw(O)) == 'Object' && isCallable(O.callee) ? 'Arguments' : result;\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/classof.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/clear-error-stack.js":function(module,__unused_webpack_exports,__webpack_require__){eval("var uncurryThis = __webpack_require__(/*! ../internals/function-uncurry-this */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-uncurry-this.js\");\n\nvar $Error = Error;\nvar replace = uncurryThis(''.replace);\n\nvar TEST = (function (arg) { return String($Error(arg).stack); })('zxcasd');\nvar V8_OR_CHAKRA_STACK_ENTRY = /\\n\\s*at [^:]*:[^\\n]*/;\nvar IS_V8_OR_CHAKRA_STACK = V8_OR_CHAKRA_STACK_ENTRY.test(TEST);\n\nmodule.exports = function (stack, dropEntries) {\n  if (IS_V8_OR_CHAKRA_STACK && typeof stack == 'string' && !$Error.prepareStackTrace) {\n    while (dropEntries--) stack = replace(stack, V8_OR_CHAKRA_STACK_ENTRY, '');\n  } return stack;\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/clear-error-stack.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/copy-constructor-properties.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var hasOwn = __webpack_require__(/*! ../internals/has-own-property */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/has-own-property.js");\nvar ownKeys = __webpack_require__(/*! ../internals/own-keys */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/own-keys.js");\nvar getOwnPropertyDescriptorModule = __webpack_require__(/*! ../internals/object-get-own-property-descriptor */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-get-own-property-descriptor.js");\nvar definePropertyModule = __webpack_require__(/*! ../internals/object-define-property */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-define-property.js");\n\nmodule.exports = function (target, source, exceptions) {\n  var keys = ownKeys(source);\n  var defineProperty = definePropertyModule.f;\n  var getOwnPropertyDescriptor = getOwnPropertyDescriptorModule.f;\n  for (var i = 0; i < keys.length; i++) {\n    var key = keys[i];\n    if (!hasOwn(target, key) && !(exceptions && hasOwn(exceptions, key))) {\n      defineProperty(target, key, getOwnPropertyDescriptor(source, key));\n    }\n  }\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/copy-constructor-properties.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/correct-prototype-getter.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var fails = __webpack_require__(/*! ../internals/fails */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/fails.js");\n\nmodule.exports = !fails(function () {\n  function F() { /* empty */ }\n  F.prototype.constructor = null;\n  // eslint-disable-next-line es-x/no-object-getprototypeof -- required for testing\n  return Object.getPrototypeOf(new F()) !== F.prototype;\n});\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/correct-prototype-getter.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/create-iterator-constructor.js":function(module,__unused_webpack_exports,__webpack_require__){"use strict";eval('\nvar IteratorPrototype = (__webpack_require__(/*! ../internals/iterators-core */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/iterators-core.js").IteratorPrototype);\nvar create = __webpack_require__(/*! ../internals/object-create */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-create.js");\nvar createPropertyDescriptor = __webpack_require__(/*! ../internals/create-property-descriptor */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/create-property-descriptor.js");\nvar setToStringTag = __webpack_require__(/*! ../internals/set-to-string-tag */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/set-to-string-tag.js");\nvar Iterators = __webpack_require__(/*! ../internals/iterators */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/iterators.js");\n\nvar returnThis = function () { return this; };\n\nmodule.exports = function (IteratorConstructor, NAME, next, ENUMERABLE_NEXT) {\n  var TO_STRING_TAG = NAME + \' Iterator\';\n  IteratorConstructor.prototype = create(IteratorPrototype, { next: createPropertyDescriptor(+!ENUMERABLE_NEXT, next) });\n  setToStringTag(IteratorConstructor, TO_STRING_TAG, false, true);\n  Iterators[TO_STRING_TAG] = returnThis;\n  return IteratorConstructor;\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/create-iterator-constructor.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/create-non-enumerable-property.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var DESCRIPTORS = __webpack_require__(/*! ../internals/descriptors */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/descriptors.js");\nvar definePropertyModule = __webpack_require__(/*! ../internals/object-define-property */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-define-property.js");\nvar createPropertyDescriptor = __webpack_require__(/*! ../internals/create-property-descriptor */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/create-property-descriptor.js");\n\nmodule.exports = DESCRIPTORS ? function (object, key, value) {\n  return definePropertyModule.f(object, key, createPropertyDescriptor(1, value));\n} : function (object, key, value) {\n  object[key] = value;\n  return object;\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/create-non-enumerable-property.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/create-property-descriptor.js":function(module){eval("module.exports = function (bitmap, value) {\n  return {\n    enumerable: !(bitmap & 1),\n    configurable: !(bitmap & 2),\n    writable: !(bitmap & 4),\n    value: value\n  };\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/create-property-descriptor.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/define-built-in.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var isCallable = __webpack_require__(/*! ../internals/is-callable */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-callable.js");\nvar definePropertyModule = __webpack_require__(/*! ../internals/object-define-property */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-define-property.js");\nvar makeBuiltIn = __webpack_require__(/*! ../internals/make-built-in */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/make-built-in.js");\nvar defineGlobalProperty = __webpack_require__(/*! ../internals/define-global-property */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/define-global-property.js");\n\nmodule.exports = function (O, key, value, options) {\n  if (!options) options = {};\n  var simple = options.enumerable;\n  var name = options.name !== undefined ? options.name : key;\n  if (isCallable(value)) makeBuiltIn(value, name, options);\n  if (options.global) {\n    if (simple) O[key] = value;\n    else defineGlobalProperty(key, value);\n  } else {\n    try {\n      if (!options.unsafe) delete O[key];\n      else if (O[key]) simple = true;\n    } catch (error) { /* empty */ }\n    if (simple) O[key] = value;\n    else definePropertyModule.f(O, key, {\n      value: value,\n      enumerable: false,\n      configurable: !options.nonConfigurable,\n      writable: !options.nonWritable\n    });\n  } return O;\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/define-built-in.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/define-global-property.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var global = __webpack_require__(/*! ../internals/global */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/global.js");\n\n// eslint-disable-next-line es-x/no-object-defineproperty -- safe\nvar defineProperty = Object.defineProperty;\n\nmodule.exports = function (key, value) {\n  try {\n    defineProperty(global, key, { value: value, configurable: true, writable: true });\n  } catch (error) {\n    global[key] = value;\n  } return value;\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/define-global-property.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/define-iterator.js":function(module,__unused_webpack_exports,__webpack_require__){"use strict";eval('\nvar $ = __webpack_require__(/*! ../internals/export */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/export.js");\nvar call = __webpack_require__(/*! ../internals/function-call */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-call.js");\nvar IS_PURE = __webpack_require__(/*! ../internals/is-pure */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-pure.js");\nvar FunctionName = __webpack_require__(/*! ../internals/function-name */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-name.js");\nvar isCallable = __webpack_require__(/*! ../internals/is-callable */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-callable.js");\nvar createIteratorConstructor = __webpack_require__(/*! ../internals/create-iterator-constructor */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/create-iterator-constructor.js");\nvar getPrototypeOf = __webpack_require__(/*! ../internals/object-get-prototype-of */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-get-prototype-of.js");\nvar setPrototypeOf = __webpack_require__(/*! ../internals/object-set-prototype-of */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-set-prototype-of.js");\nvar setToStringTag = __webpack_require__(/*! ../internals/set-to-string-tag */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/set-to-string-tag.js");\nvar createNonEnumerableProperty = __webpack_require__(/*! ../internals/create-non-enumerable-property */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/create-non-enumerable-property.js");\nvar defineBuiltIn = __webpack_require__(/*! ../internals/define-built-in */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/define-built-in.js");\nvar wellKnownSymbol = __webpack_require__(/*! ../internals/well-known-symbol */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/well-known-symbol.js");\nvar Iterators = __webpack_require__(/*! ../internals/iterators */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/iterators.js");\nvar IteratorsCore = __webpack_require__(/*! ../internals/iterators-core */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/iterators-core.js");\n\nvar PROPER_FUNCTION_NAME = FunctionName.PROPER;\nvar CONFIGURABLE_FUNCTION_NAME = FunctionName.CONFIGURABLE;\nvar IteratorPrototype = IteratorsCore.IteratorPrototype;\nvar BUGGY_SAFARI_ITERATORS = IteratorsCore.BUGGY_SAFARI_ITERATORS;\nvar ITERATOR = wellKnownSymbol(\'iterator\');\nvar KEYS = \'keys\';\nvar VALUES = \'values\';\nvar ENTRIES = \'entries\';\n\nvar returnThis = function () { return this; };\n\nmodule.exports = function (Iterable, NAME, IteratorConstructor, next, DEFAULT, IS_SET, FORCED) {\n  createIteratorConstructor(IteratorConstructor, NAME, next);\n\n  var getIterationMethod = function (KIND) {\n    if (KIND === DEFAULT && defaultIterator) return defaultIterator;\n    if (!BUGGY_SAFARI_ITERATORS && KIND in IterablePrototype) return IterablePrototype[KIND];\n    switch (KIND) {\n      case KEYS: return function keys() { return new IteratorConstructor(this, KIND); };\n      case VALUES: return function values() { return new IteratorConstructor(this, KIND); };\n      case ENTRIES: return function entries() { return new IteratorConstructor(this, KIND); };\n    } return function () { return new IteratorConstructor(this); };\n  };\n\n  var TO_STRING_TAG = NAME + \' Iterator\';\n  var INCORRECT_VALUES_NAME = false;\n  var IterablePrototype = Iterable.prototype;\n  var nativeIterator = IterablePrototype[ITERATOR]\n    || IterablePrototype[\'@@iterator\']\n    || DEFAULT && IterablePrototype[DEFAULT];\n  var defaultIterator = !BUGGY_SAFARI_ITERATORS && nativeIterator || getIterationMethod(DEFAULT);\n  var anyNativeIterator = NAME == \'Array\' ? IterablePrototype.entries || nativeIterator : nativeIterator;\n  var CurrentIteratorPrototype, methods, KEY;\n\n  // fix native\n  if (anyNativeIterator) {\n    CurrentIteratorPrototype = getPrototypeOf(anyNativeIterator.call(new Iterable()));\n    if (CurrentIteratorPrototype !== Object.prototype && CurrentIteratorPrototype.next) {\n      if (!IS_PURE && getPrototypeOf(CurrentIteratorPrototype) !== IteratorPrototype) {\n        if (setPrototypeOf) {\n          setPrototypeOf(CurrentIteratorPrototype, IteratorPrototype);\n        } else if (!isCallable(CurrentIteratorPrototype[ITERATOR])) {\n          defineBuiltIn(CurrentIteratorPrototype, ITERATOR, returnThis);\n        }\n      }\n      // Set @@toStringTag to native iterators\n      setToStringTag(CurrentIteratorPrototype, TO_STRING_TAG, true, true);\n      if (IS_PURE) Iterators[TO_STRING_TAG] = returnThis;\n    }\n  }\n\n  // fix Array.prototype.{ values, @@iterator }.name in V8 / FF\n  if (PROPER_FUNCTION_NAME && DEFAULT == VALUES && nativeIterator && nativeIterator.name !== VALUES) {\n    if (!IS_PURE && CONFIGURABLE_FUNCTION_NAME) {\n      createNonEnumerableProperty(IterablePrototype, \'name\', VALUES);\n    } else {\n      INCORRECT_VALUES_NAME = true;\n      defaultIterator = function values() { return call(nativeIterator, this); };\n    }\n  }\n\n  // export additional methods\n  if (DEFAULT) {\n    methods = {\n      values: getIterationMethod(VALUES),\n      keys: IS_SET ? defaultIterator : getIterationMethod(KEYS),\n      entries: getIterationMethod(ENTRIES)\n    };\n    if (FORCED) for (KEY in methods) {\n      if (BUGGY_SAFARI_ITERATORS || INCORRECT_VALUES_NAME || !(KEY in IterablePrototype)) {\n        defineBuiltIn(IterablePrototype, KEY, methods[KEY]);\n      }\n    } else $({ target: NAME, proto: true, forced: BUGGY_SAFARI_ITERATORS || INCORRECT_VALUES_NAME }, methods);\n  }\n\n  // define iterator\n  if ((!IS_PURE || FORCED) && IterablePrototype[ITERATOR] !== defaultIterator) {\n    defineBuiltIn(IterablePrototype, ITERATOR, defaultIterator, { name: DEFAULT });\n  }\n  Iterators[NAME] = defaultIterator;\n\n  return methods;\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/define-iterator.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/descriptors.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var fails = __webpack_require__(/*! ../internals/fails */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/fails.js");\n\n// Detect IE8\'s incomplete defineProperty implementation\nmodule.exports = !fails(function () {\n  // eslint-disable-next-line es-x/no-object-defineproperty -- required for testing\n  return Object.defineProperty({}, 1, { get: function () { return 7; } })[1] != 7;\n});\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/descriptors.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/document-create-element.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var global = __webpack_require__(/*! ../internals/global */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/global.js");\nvar isObject = __webpack_require__(/*! ../internals/is-object */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-object.js");\n\nvar document = global.document;\n// typeof document.createElement is \'object\' in old IE\nvar EXISTS = isObject(document) && isObject(document.createElement);\n\nmodule.exports = function (it) {\n  return EXISTS ? document.createElement(it) : {};\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/document-create-element.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-is-browser.js":function(module,__unused_webpack_exports,__webpack_require__){eval("var IS_DENO = __webpack_require__(/*! ../internals/engine-is-deno */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-is-deno.js\");\nvar IS_NODE = __webpack_require__(/*! ../internals/engine-is-node */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-is-node.js\");\n\nmodule.exports = !IS_DENO && !IS_NODE\n  && typeof window == 'object'\n  && typeof document == 'object';\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-is-browser.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-is-deno.js":function(module){eval("/* global Deno -- Deno case */\nmodule.exports = typeof Deno == 'object' && Deno && typeof Deno.version == 'object';\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-is-deno.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-is-ios-pebble.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var userAgent = __webpack_require__(/*! ../internals/engine-user-agent */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-user-agent.js");\nvar global = __webpack_require__(/*! ../internals/global */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/global.js");\n\nmodule.exports = /ipad|iphone|ipod/i.test(userAgent) && global.Pebble !== undefined;\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-is-ios-pebble.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-is-ios.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var userAgent = __webpack_require__(/*! ../internals/engine-user-agent */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-user-agent.js");\n\nmodule.exports = /(?:ipad|iphone|ipod).*applewebkit/i.test(userAgent);\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-is-ios.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-is-node.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var classof = __webpack_require__(/*! ../internals/classof-raw */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/classof-raw.js");\nvar global = __webpack_require__(/*! ../internals/global */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/global.js");\n\nmodule.exports = classof(global.process) == \'process\';\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-is-node.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-is-webos-webkit.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var userAgent = __webpack_require__(/*! ../internals/engine-user-agent */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-user-agent.js");\n\nmodule.exports = /web0s(?!.*chrome)/i.test(userAgent);\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-is-webos-webkit.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-user-agent.js":function(module,__unused_webpack_exports,__webpack_require__){eval("var getBuiltIn = __webpack_require__(/*! ../internals/get-built-in */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/get-built-in.js\");\n\nmodule.exports = getBuiltIn('navigator', 'userAgent') || '';\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-user-agent.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-v8-version.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var global = __webpack_require__(/*! ../internals/global */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/global.js");\nvar userAgent = __webpack_require__(/*! ../internals/engine-user-agent */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-user-agent.js");\n\nvar process = global.process;\nvar Deno = global.Deno;\nvar versions = process && process.versions || Deno && Deno.version;\nvar v8 = versions && versions.v8;\nvar match, version;\n\nif (v8) {\n  match = v8.split(\'.\');\n  // in old Chrome, versions of V8 isn\'t V8 = Chrome / 10\n  // but their correct versions are not interesting for us\n  version = match[0] > 0 && match[0] < 4 ? 1 : +(match[0] + match[1]);\n}\n\n// BrowserFS NodeJS `process` polyfill incorrectly set `.v8` to `0.0`\n// so check `userAgent` even if `.v8` exists, but 0\nif (!version && userAgent) {\n  match = userAgent.match(/Edge\\/(\\d+)/);\n  if (!match || match[1] >= 74) {\n    match = userAgent.match(/Chrome\\/(\\d+)/);\n    if (match) version = +match[1];\n  }\n}\n\nmodule.exports = version;\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-v8-version.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/enum-bug-keys.js":function(module){eval("// IE8- don't enum bug keys\nmodule.exports = [\n  'constructor',\n  'hasOwnProperty',\n  'isPrototypeOf',\n  'propertyIsEnumerable',\n  'toLocaleString',\n  'toString',\n  'valueOf'\n];\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/enum-bug-keys.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/error-stack-installable.js":function(module,__unused_webpack_exports,__webpack_require__){eval("var fails = __webpack_require__(/*! ../internals/fails */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/fails.js\");\nvar createPropertyDescriptor = __webpack_require__(/*! ../internals/create-property-descriptor */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/create-property-descriptor.js\");\n\nmodule.exports = !fails(function () {\n  var error = Error('a');\n  if (!('stack' in error)) return true;\n  // eslint-disable-next-line es-x/no-object-defineproperty -- safe\n  Object.defineProperty(error, 'stack', createPropertyDescriptor(1, 7));\n  return error.stack !== 7;\n});\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/error-stack-installable.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/export.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var global = __webpack_require__(/*! ../internals/global */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/global.js");\nvar getOwnPropertyDescriptor = (__webpack_require__(/*! ../internals/object-get-own-property-descriptor */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-get-own-property-descriptor.js").f);\nvar createNonEnumerableProperty = __webpack_require__(/*! ../internals/create-non-enumerable-property */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/create-non-enumerable-property.js");\nvar defineBuiltIn = __webpack_require__(/*! ../internals/define-built-in */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/define-built-in.js");\nvar defineGlobalProperty = __webpack_require__(/*! ../internals/define-global-property */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/define-global-property.js");\nvar copyConstructorProperties = __webpack_require__(/*! ../internals/copy-constructor-properties */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/copy-constructor-properties.js");\nvar isForced = __webpack_require__(/*! ../internals/is-forced */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-forced.js");\n\n/*\n  options.target         - name of the target object\n  options.global         - target is the global object\n  options.stat           - export as static methods of target\n  options.proto          - export as prototype methods of target\n  options.real           - real prototype method for the `pure` version\n  options.forced         - export even if the native feature is available\n  options.bind           - bind methods to the target, required for the `pure` version\n  options.wrap           - wrap constructors to preventing global pollution, required for the `pure` version\n  options.unsafe         - use the simple assignment of property instead of delete + defineProperty\n  options.sham           - add a flag to not completely full polyfills\n  options.enumerable     - export as enumerable property\n  options.dontCallGetSet - prevent calling a getter on target\n  options.name           - the .name of the function if it does not match the key\n*/\nmodule.exports = function (options, source) {\n  var TARGET = options.target;\n  var GLOBAL = options.global;\n  var STATIC = options.stat;\n  var FORCED, target, key, targetProperty, sourceProperty, descriptor;\n  if (GLOBAL) {\n    target = global;\n  } else if (STATIC) {\n    target = global[TARGET] || defineGlobalProperty(TARGET, {});\n  } else {\n    target = (global[TARGET] || {}).prototype;\n  }\n  if (target) for (key in source) {\n    sourceProperty = source[key];\n    if (options.dontCallGetSet) {\n      descriptor = getOwnPropertyDescriptor(target, key);\n      targetProperty = descriptor && descriptor.value;\n    } else targetProperty = target[key];\n    FORCED = isForced(GLOBAL ? key : TARGET + (STATIC ? \'.\' : \'#\') + key, options.forced);\n    // contained in target\n    if (!FORCED && targetProperty !== undefined) {\n      if (typeof sourceProperty == typeof targetProperty) continue;\n      copyConstructorProperties(sourceProperty, targetProperty);\n    }\n    // add a flag to not completely full polyfills\n    if (options.sham || (targetProperty && targetProperty.sham)) {\n      createNonEnumerableProperty(sourceProperty, \'sham\', true);\n    }\n    defineBuiltIn(target, key, sourceProperty, options);\n  }\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/export.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/fails.js":function(module){eval("module.exports = function (exec) {\n  try {\n    return !!exec();\n  } catch (error) {\n    return true;\n  }\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/fails.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-apply.js":function(module,__unused_webpack_exports,__webpack_require__){eval("var NATIVE_BIND = __webpack_require__(/*! ../internals/function-bind-native */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-bind-native.js\");\n\nvar FunctionPrototype = Function.prototype;\nvar apply = FunctionPrototype.apply;\nvar call = FunctionPrototype.call;\n\n// eslint-disable-next-line es-x/no-reflect -- safe\nmodule.exports = typeof Reflect == 'object' && Reflect.apply || (NATIVE_BIND ? call.bind(apply) : function () {\n  return call.apply(apply, arguments);\n});\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-apply.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-bind-context.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var uncurryThis = __webpack_require__(/*! ../internals/function-uncurry-this */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-uncurry-this.js");\nvar aCallable = __webpack_require__(/*! ../internals/a-callable */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/a-callable.js");\nvar NATIVE_BIND = __webpack_require__(/*! ../internals/function-bind-native */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-bind-native.js");\n\nvar bind = uncurryThis(uncurryThis.bind);\n\n// optional / simple context binding\nmodule.exports = function (fn, that) {\n  aCallable(fn);\n  return that === undefined ? fn : NATIVE_BIND ? bind(fn, that) : function (/* ...args */) {\n    return fn.apply(that, arguments);\n  };\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-bind-context.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-bind-native.js":function(module,__unused_webpack_exports,__webpack_require__){eval("var fails = __webpack_require__(/*! ../internals/fails */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/fails.js\");\n\nmodule.exports = !fails(function () {\n  // eslint-disable-next-line es-x/no-function-prototype-bind -- safe\n  var test = (function () { /* empty */ }).bind();\n  // eslint-disable-next-line no-prototype-builtins -- safe\n  return typeof test != 'function' || test.hasOwnProperty('prototype');\n});\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-bind-native.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-call.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var NATIVE_BIND = __webpack_require__(/*! ../internals/function-bind-native */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-bind-native.js");\n\nvar call = Function.prototype.call;\n\nmodule.exports = NATIVE_BIND ? call.bind(call) : function () {\n  return call.apply(call, arguments);\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-call.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-name.js":function(module,__unused_webpack_exports,__webpack_require__){eval("var DESCRIPTORS = __webpack_require__(/*! ../internals/descriptors */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/descriptors.js\");\nvar hasOwn = __webpack_require__(/*! ../internals/has-own-property */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/has-own-property.js\");\n\nvar FunctionPrototype = Function.prototype;\n// eslint-disable-next-line es-x/no-object-getownpropertydescriptor -- safe\nvar getDescriptor = DESCRIPTORS && Object.getOwnPropertyDescriptor;\n\nvar EXISTS = hasOwn(FunctionPrototype, 'name');\n// additional protection from minified / mangled / dropped function names\nvar PROPER = EXISTS && (function something() { /* empty */ }).name === 'something';\nvar CONFIGURABLE = EXISTS && (!DESCRIPTORS || (DESCRIPTORS && getDescriptor(FunctionPrototype, 'name').configurable));\n\nmodule.exports = {\n  EXISTS: EXISTS,\n  PROPER: PROPER,\n  CONFIGURABLE: CONFIGURABLE\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-name.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-uncurry-this.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var NATIVE_BIND = __webpack_require__(/*! ../internals/function-bind-native */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-bind-native.js");\n\nvar FunctionPrototype = Function.prototype;\nvar bind = FunctionPrototype.bind;\nvar call = FunctionPrototype.call;\nvar uncurryThis = NATIVE_BIND && bind.bind(call, call);\n\nmodule.exports = NATIVE_BIND ? function (fn) {\n  return fn && uncurryThis(fn);\n} : function (fn) {\n  return fn && function () {\n    return call.apply(fn, arguments);\n  };\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-uncurry-this.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/get-built-in.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var global = __webpack_require__(/*! ../internals/global */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/global.js");\nvar isCallable = __webpack_require__(/*! ../internals/is-callable */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-callable.js");\n\nvar aFunction = function (argument) {\n  return isCallable(argument) ? argument : undefined;\n};\n\nmodule.exports = function (namespace, method) {\n  return arguments.length < 2 ? aFunction(global[namespace]) : global[namespace] && global[namespace][method];\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/get-built-in.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/get-iterator-method.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var classof = __webpack_require__(/*! ../internals/classof */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/classof.js");\nvar getMethod = __webpack_require__(/*! ../internals/get-method */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/get-method.js");\nvar Iterators = __webpack_require__(/*! ../internals/iterators */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/iterators.js");\nvar wellKnownSymbol = __webpack_require__(/*! ../internals/well-known-symbol */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/well-known-symbol.js");\n\nvar ITERATOR = wellKnownSymbol(\'iterator\');\n\nmodule.exports = function (it) {\n  if (it != undefined) return getMethod(it, ITERATOR)\n    || getMethod(it, \'@@iterator\')\n    || Iterators[classof(it)];\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/get-iterator-method.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/get-iterator.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var call = __webpack_require__(/*! ../internals/function-call */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-call.js");\nvar aCallable = __webpack_require__(/*! ../internals/a-callable */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/a-callable.js");\nvar anObject = __webpack_require__(/*! ../internals/an-object */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/an-object.js");\nvar tryToString = __webpack_require__(/*! ../internals/try-to-string */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/try-to-string.js");\nvar getIteratorMethod = __webpack_require__(/*! ../internals/get-iterator-method */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/get-iterator-method.js");\n\nvar $TypeError = TypeError;\n\nmodule.exports = function (argument, usingIterator) {\n  var iteratorMethod = arguments.length < 2 ? getIteratorMethod(argument) : usingIterator;\n  if (aCallable(iteratorMethod)) return anObject(call(iteratorMethod, argument));\n  throw $TypeError(tryToString(argument) + \' is not iterable\');\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/get-iterator.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/get-method.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var aCallable = __webpack_require__(/*! ../internals/a-callable */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/a-callable.js");\n\n// `GetMethod` abstract operation\n// https://tc39.es/ecma262/#sec-getmethod\nmodule.exports = function (V, P) {\n  var func = V[P];\n  return func == null ? undefined : aCallable(func);\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/get-method.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/global.js":function(module,__unused_webpack_exports,__webpack_require__){eval("var check = function (it) {\n  return it && it.Math == Math && it;\n};\n\n// https://github.com/zloirock/core-js/issues/86#issuecomment-115759028\nmodule.exports =\n  // eslint-disable-next-line es-x/no-global-this -- safe\n  check(typeof globalThis == 'object' && globalThis) ||\n  check(typeof window == 'object' && window) ||\n  // eslint-disable-next-line no-restricted-globals -- safe\n  check(typeof self == 'object' && self) ||\n  check(typeof __webpack_require__.g == 'object' && __webpack_require__.g) ||\n  // eslint-disable-next-line no-new-func -- fallback\n  (function () { return this; })() || Function('return this')();\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/global.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/has-own-property.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var uncurryThis = __webpack_require__(/*! ../internals/function-uncurry-this */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-uncurry-this.js");\nvar toObject = __webpack_require__(/*! ../internals/to-object */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-object.js");\n\nvar hasOwnProperty = uncurryThis({}.hasOwnProperty);\n\n// `HasOwnProperty` abstract operation\n// https://tc39.es/ecma262/#sec-hasownproperty\n// eslint-disable-next-line es-x/no-object-hasown -- safe\nmodule.exports = Object.hasOwn || function hasOwn(it, key) {\n  return hasOwnProperty(toObject(it), key);\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/has-own-property.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/hidden-keys.js":function(module){eval("module.exports = {};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/hidden-keys.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/host-report-errors.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var global = __webpack_require__(/*! ../internals/global */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/global.js");\n\nmodule.exports = function (a, b) {\n  var console = global.console;\n  if (console && console.error) {\n    arguments.length == 1 ? console.error(a) : console.error(a, b);\n  }\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/host-report-errors.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/html.js":function(module,__unused_webpack_exports,__webpack_require__){eval("var getBuiltIn = __webpack_require__(/*! ../internals/get-built-in */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/get-built-in.js\");\n\nmodule.exports = getBuiltIn('document', 'documentElement');\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/html.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/ie8-dom-define.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var DESCRIPTORS = __webpack_require__(/*! ../internals/descriptors */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/descriptors.js");\nvar fails = __webpack_require__(/*! ../internals/fails */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/fails.js");\nvar createElement = __webpack_require__(/*! ../internals/document-create-element */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/document-create-element.js");\n\n// Thanks to IE8 for its funny defineProperty\nmodule.exports = !DESCRIPTORS && !fails(function () {\n  // eslint-disable-next-line es-x/no-object-defineproperty -- required for testing\n  return Object.defineProperty(createElement(\'div\'), \'a\', {\n    get: function () { return 7; }\n  }).a != 7;\n});\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/ie8-dom-define.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/indexed-object.js":function(module,__unused_webpack_exports,__webpack_require__){eval("var uncurryThis = __webpack_require__(/*! ../internals/function-uncurry-this */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-uncurry-this.js\");\nvar fails = __webpack_require__(/*! ../internals/fails */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/fails.js\");\nvar classof = __webpack_require__(/*! ../internals/classof-raw */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/classof-raw.js\");\n\nvar $Object = Object;\nvar split = uncurryThis(''.split);\n\n// fallback for non-array-like ES3 and non-enumerable old V8 strings\nmodule.exports = fails(function () {\n  // throws an error in rhino, see https://github.com/mozilla/rhino/issues/346\n  // eslint-disable-next-line no-prototype-builtins -- safe\n  return !$Object('z').propertyIsEnumerable(0);\n}) ? function (it) {\n  return classof(it) == 'String' ? split(it, '') : $Object(it);\n} : $Object;\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/indexed-object.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/inspect-source.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var uncurryThis = __webpack_require__(/*! ../internals/function-uncurry-this */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-uncurry-this.js");\nvar isCallable = __webpack_require__(/*! ../internals/is-callable */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-callable.js");\nvar store = __webpack_require__(/*! ../internals/shared-store */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/shared-store.js");\n\nvar functionToString = uncurryThis(Function.toString);\n\n// this helper broken in `core-js@3.4.1-3.4.4`, so we can\'t use `shared` helper\nif (!isCallable(store.inspectSource)) {\n  store.inspectSource = function (it) {\n    return functionToString(it);\n  };\n}\n\nmodule.exports = store.inspectSource;\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/inspect-source.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/install-error-cause.js":function(module,__unused_webpack_exports,__webpack_require__){eval("var isObject = __webpack_require__(/*! ../internals/is-object */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-object.js\");\nvar createNonEnumerableProperty = __webpack_require__(/*! ../internals/create-non-enumerable-property */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/create-non-enumerable-property.js\");\n\n// `InstallErrorCause` abstract operation\n// https://tc39.es/proposal-error-cause/#sec-errorobjects-install-error-cause\nmodule.exports = function (O, options) {\n  if (isObject(options) && 'cause' in options) {\n    createNonEnumerableProperty(O, 'cause', options.cause);\n  }\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/install-error-cause.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/internal-state.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var NATIVE_WEAK_MAP = __webpack_require__(/*! ../internals/native-weak-map */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/native-weak-map.js");\nvar global = __webpack_require__(/*! ../internals/global */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/global.js");\nvar uncurryThis = __webpack_require__(/*! ../internals/function-uncurry-this */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-uncurry-this.js");\nvar isObject = __webpack_require__(/*! ../internals/is-object */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-object.js");\nvar createNonEnumerableProperty = __webpack_require__(/*! ../internals/create-non-enumerable-property */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/create-non-enumerable-property.js");\nvar hasOwn = __webpack_require__(/*! ../internals/has-own-property */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/has-own-property.js");\nvar shared = __webpack_require__(/*! ../internals/shared-store */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/shared-store.js");\nvar sharedKey = __webpack_require__(/*! ../internals/shared-key */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/shared-key.js");\nvar hiddenKeys = __webpack_require__(/*! ../internals/hidden-keys */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/hidden-keys.js");\n\nvar OBJECT_ALREADY_INITIALIZED = \'Object already initialized\';\nvar TypeError = global.TypeError;\nvar WeakMap = global.WeakMap;\nvar set, get, has;\n\nvar enforce = function (it) {\n  return has(it) ? get(it) : set(it, {});\n};\n\nvar getterFor = function (TYPE) {\n  return function (it) {\n    var state;\n    if (!isObject(it) || (state = get(it)).type !== TYPE) {\n      throw TypeError(\'Incompatible receiver, \' + TYPE + \' required\');\n    } return state;\n  };\n};\n\nif (NATIVE_WEAK_MAP || shared.state) {\n  var store = shared.state || (shared.state = new WeakMap());\n  var wmget = uncurryThis(store.get);\n  var wmhas = uncurryThis(store.has);\n  var wmset = uncurryThis(store.set);\n  set = function (it, metadata) {\n    if (wmhas(store, it)) throw new TypeError(OBJECT_ALREADY_INITIALIZED);\n    metadata.facade = it;\n    wmset(store, it, metadata);\n    return metadata;\n  };\n  get = function (it) {\n    return wmget(store, it) || {};\n  };\n  has = function (it) {\n    return wmhas(store, it);\n  };\n} else {\n  var STATE = sharedKey(\'state\');\n  hiddenKeys[STATE] = true;\n  set = function (it, metadata) {\n    if (hasOwn(it, STATE)) throw new TypeError(OBJECT_ALREADY_INITIALIZED);\n    metadata.facade = it;\n    createNonEnumerableProperty(it, STATE, metadata);\n    return metadata;\n  };\n  get = function (it) {\n    return hasOwn(it, STATE) ? it[STATE] : {};\n  };\n  has = function (it) {\n    return hasOwn(it, STATE);\n  };\n}\n\nmodule.exports = {\n  set: set,\n  get: get,\n  has: has,\n  enforce: enforce,\n  getterFor: getterFor\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/internal-state.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-array-iterator-method.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var wellKnownSymbol = __webpack_require__(/*! ../internals/well-known-symbol */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/well-known-symbol.js");\nvar Iterators = __webpack_require__(/*! ../internals/iterators */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/iterators.js");\n\nvar ITERATOR = wellKnownSymbol(\'iterator\');\nvar ArrayPrototype = Array.prototype;\n\n// check on default Array iterator\nmodule.exports = function (it) {\n  return it !== undefined && (Iterators.Array === it || ArrayPrototype[ITERATOR] === it);\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-array-iterator-method.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-callable.js":function(module){eval("// `IsCallable` abstract operation\n// https://tc39.es/ecma262/#sec-iscallable\nmodule.exports = function (argument) {\n  return typeof argument == 'function';\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-callable.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-constructor.js":function(module,__unused_webpack_exports,__webpack_require__){eval("var uncurryThis = __webpack_require__(/*! ../internals/function-uncurry-this */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-uncurry-this.js\");\nvar fails = __webpack_require__(/*! ../internals/fails */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/fails.js\");\nvar isCallable = __webpack_require__(/*! ../internals/is-callable */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-callable.js\");\nvar classof = __webpack_require__(/*! ../internals/classof */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/classof.js\");\nvar getBuiltIn = __webpack_require__(/*! ../internals/get-built-in */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/get-built-in.js\");\nvar inspectSource = __webpack_require__(/*! ../internals/inspect-source */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/inspect-source.js\");\n\nvar noop = function () { /* empty */ };\nvar empty = [];\nvar construct = getBuiltIn('Reflect', 'construct');\nvar constructorRegExp = /^\\s*(?:class|function)\\b/;\nvar exec = uncurryThis(constructorRegExp.exec);\nvar INCORRECT_TO_STRING = !constructorRegExp.exec(noop);\n\nvar isConstructorModern = function isConstructor(argument) {\n  if (!isCallable(argument)) return false;\n  try {\n    construct(noop, empty, argument);\n    return true;\n  } catch (error) {\n    return false;\n  }\n};\n\nvar isConstructorLegacy = function isConstructor(argument) {\n  if (!isCallable(argument)) return false;\n  switch (classof(argument)) {\n    case 'AsyncFunction':\n    case 'GeneratorFunction':\n    case 'AsyncGeneratorFunction': return false;\n  }\n  try {\n    // we can't check .prototype since constructors produced by .bind haven't it\n    // `Function#toString` throws on some built-it function in some legacy engines\n    // (for example, `DOMQuad` and similar in FF41-)\n    return INCORRECT_TO_STRING || !!exec(constructorRegExp, inspectSource(argument));\n  } catch (error) {\n    return true;\n  }\n};\n\nisConstructorLegacy.sham = true;\n\n// `IsConstructor` abstract operation\n// https://tc39.es/ecma262/#sec-isconstructor\nmodule.exports = !construct || fails(function () {\n  var called;\n  return isConstructorModern(isConstructorModern.call)\n    || !isConstructorModern(Object)\n    || !isConstructorModern(function () { called = true; })\n    || called;\n}) ? isConstructorLegacy : isConstructorModern;\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-constructor.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-forced.js":function(module,__unused_webpack_exports,__webpack_require__){eval("var fails = __webpack_require__(/*! ../internals/fails */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/fails.js\");\nvar isCallable = __webpack_require__(/*! ../internals/is-callable */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-callable.js\");\n\nvar replacement = /#|\\.prototype\\./;\n\nvar isForced = function (feature, detection) {\n  var value = data[normalize(feature)];\n  return value == POLYFILL ? true\n    : value == NATIVE ? false\n    : isCallable(detection) ? fails(detection)\n    : !!detection;\n};\n\nvar normalize = isForced.normalize = function (string) {\n  return String(string).replace(replacement, '.').toLowerCase();\n};\n\nvar data = isForced.data = {};\nvar NATIVE = isForced.NATIVE = 'N';\nvar POLYFILL = isForced.POLYFILL = 'P';\n\nmodule.exports = isForced;\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-forced.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-object.js":function(module,__unused_webpack_exports,__webpack_require__){eval("var isCallable = __webpack_require__(/*! ../internals/is-callable */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-callable.js\");\n\nmodule.exports = function (it) {\n  return typeof it == 'object' ? it !== null : isCallable(it);\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-object.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-pure.js":function(module){eval("module.exports = false;\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-pure.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-symbol.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var getBuiltIn = __webpack_require__(/*! ../internals/get-built-in */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/get-built-in.js");\nvar isCallable = __webpack_require__(/*! ../internals/is-callable */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-callable.js");\nvar isPrototypeOf = __webpack_require__(/*! ../internals/object-is-prototype-of */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-is-prototype-of.js");\nvar USE_SYMBOL_AS_UID = __webpack_require__(/*! ../internals/use-symbol-as-uid */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/use-symbol-as-uid.js");\n\nvar $Object = Object;\n\nmodule.exports = USE_SYMBOL_AS_UID ? function (it) {\n  return typeof it == \'symbol\';\n} : function (it) {\n  var $Symbol = getBuiltIn(\'Symbol\');\n  return isCallable($Symbol) && isPrototypeOf($Symbol.prototype, $Object(it));\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-symbol.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/iterate.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var bind = __webpack_require__(/*! ../internals/function-bind-context */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-bind-context.js");\nvar call = __webpack_require__(/*! ../internals/function-call */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-call.js");\nvar anObject = __webpack_require__(/*! ../internals/an-object */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/an-object.js");\nvar tryToString = __webpack_require__(/*! ../internals/try-to-string */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/try-to-string.js");\nvar isArrayIteratorMethod = __webpack_require__(/*! ../internals/is-array-iterator-method */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-array-iterator-method.js");\nvar lengthOfArrayLike = __webpack_require__(/*! ../internals/length-of-array-like */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/length-of-array-like.js");\nvar isPrototypeOf = __webpack_require__(/*! ../internals/object-is-prototype-of */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-is-prototype-of.js");\nvar getIterator = __webpack_require__(/*! ../internals/get-iterator */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/get-iterator.js");\nvar getIteratorMethod = __webpack_require__(/*! ../internals/get-iterator-method */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/get-iterator-method.js");\nvar iteratorClose = __webpack_require__(/*! ../internals/iterator-close */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/iterator-close.js");\n\nvar $TypeError = TypeError;\n\nvar Result = function (stopped, result) {\n  this.stopped = stopped;\n  this.result = result;\n};\n\nvar ResultPrototype = Result.prototype;\n\nmodule.exports = function (iterable, unboundFunction, options) {\n  var that = options && options.that;\n  var AS_ENTRIES = !!(options && options.AS_ENTRIES);\n  var IS_RECORD = !!(options && options.IS_RECORD);\n  var IS_ITERATOR = !!(options && options.IS_ITERATOR);\n  var INTERRUPTED = !!(options && options.INTERRUPTED);\n  var fn = bind(unboundFunction, that);\n  var iterator, iterFn, index, length, result, next, step;\n\n  var stop = function (condition) {\n    if (iterator) iteratorClose(iterator, \'normal\', condition);\n    return new Result(true, condition);\n  };\n\n  var callFn = function (value) {\n    if (AS_ENTRIES) {\n      anObject(value);\n      return INTERRUPTED ? fn(value[0], value[1], stop) : fn(value[0], value[1]);\n    } return INTERRUPTED ? fn(value, stop) : fn(value);\n  };\n\n  if (IS_RECORD) {\n    iterator = iterable.iterator;\n  } else if (IS_ITERATOR) {\n    iterator = iterable;\n  } else {\n    iterFn = getIteratorMethod(iterable);\n    if (!iterFn) throw $TypeError(tryToString(iterable) + \' is not iterable\');\n    // optimisation for array iterators\n    if (isArrayIteratorMethod(iterFn)) {\n      for (index = 0, length = lengthOfArrayLike(iterable); length > index; index++) {\n        result = callFn(iterable[index]);\n        if (result && isPrototypeOf(ResultPrototype, result)) return result;\n      } return new Result(false);\n    }\n    iterator = getIterator(iterable, iterFn);\n  }\n\n  next = IS_RECORD ? iterable.next : iterator.next;\n  while (!(step = call(next, iterator)).done) {\n    try {\n      result = callFn(step.value);\n    } catch (error) {\n      iteratorClose(iterator, \'throw\', error);\n    }\n    if (typeof result == \'object\' && result && isPrototypeOf(ResultPrototype, result)) return result;\n  } return new Result(false);\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/iterate.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/iterator-close.js":function(module,__unused_webpack_exports,__webpack_require__){eval("var call = __webpack_require__(/*! ../internals/function-call */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-call.js\");\nvar anObject = __webpack_require__(/*! ../internals/an-object */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/an-object.js\");\nvar getMethod = __webpack_require__(/*! ../internals/get-method */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/get-method.js\");\n\nmodule.exports = function (iterator, kind, value) {\n  var innerResult, innerError;\n  anObject(iterator);\n  try {\n    innerResult = getMethod(iterator, 'return');\n    if (!innerResult) {\n      if (kind === 'throw') throw value;\n      return value;\n    }\n    innerResult = call(innerResult, iterator);\n  } catch (error) {\n    innerError = true;\n    innerResult = error;\n  }\n  if (kind === 'throw') throw value;\n  if (innerError) throw innerResult;\n  anObject(innerResult);\n  return value;\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/iterator-close.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/iterators-core.js":function(module,__unused_webpack_exports,__webpack_require__){"use strict";eval('\nvar fails = __webpack_require__(/*! ../internals/fails */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/fails.js");\nvar isCallable = __webpack_require__(/*! ../internals/is-callable */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-callable.js");\nvar create = __webpack_require__(/*! ../internals/object-create */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-create.js");\nvar getPrototypeOf = __webpack_require__(/*! ../internals/object-get-prototype-of */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-get-prototype-of.js");\nvar defineBuiltIn = __webpack_require__(/*! ../internals/define-built-in */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/define-built-in.js");\nvar wellKnownSymbol = __webpack_require__(/*! ../internals/well-known-symbol */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/well-known-symbol.js");\nvar IS_PURE = __webpack_require__(/*! ../internals/is-pure */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-pure.js");\n\nvar ITERATOR = wellKnownSymbol(\'iterator\');\nvar BUGGY_SAFARI_ITERATORS = false;\n\n// `%IteratorPrototype%` object\n// https://tc39.es/ecma262/#sec-%iteratorprototype%-object\nvar IteratorPrototype, PrototypeOfArrayIteratorPrototype, arrayIterator;\n\n/* eslint-disable es-x/no-array-prototype-keys -- safe */\nif ([].keys) {\n  arrayIterator = [].keys();\n  // Safari 8 has buggy iterators w/o `next`\n  if (!(\'next\' in arrayIterator)) BUGGY_SAFARI_ITERATORS = true;\n  else {\n    PrototypeOfArrayIteratorPrototype = getPrototypeOf(getPrototypeOf(arrayIterator));\n    if (PrototypeOfArrayIteratorPrototype !== Object.prototype) IteratorPrototype = PrototypeOfArrayIteratorPrototype;\n  }\n}\n\nvar NEW_ITERATOR_PROTOTYPE = IteratorPrototype == undefined || fails(function () {\n  var test = {};\n  // FF44- legacy iterators case\n  return IteratorPrototype[ITERATOR].call(test) !== test;\n});\n\nif (NEW_ITERATOR_PROTOTYPE) IteratorPrototype = {};\nelse if (IS_PURE) IteratorPrototype = create(IteratorPrototype);\n\n// `%IteratorPrototype%[@@iterator]()` method\n// https://tc39.es/ecma262/#sec-%iteratorprototype%-@@iterator\nif (!isCallable(IteratorPrototype[ITERATOR])) {\n  defineBuiltIn(IteratorPrototype, ITERATOR, function () {\n    return this;\n  });\n}\n\nmodule.exports = {\n  IteratorPrototype: IteratorPrototype,\n  BUGGY_SAFARI_ITERATORS: BUGGY_SAFARI_ITERATORS\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/iterators-core.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/iterators.js":function(module){eval("module.exports = {};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/iterators.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/length-of-array-like.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var toLength = __webpack_require__(/*! ../internals/to-length */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-length.js");\n\n// `LengthOfArrayLike` abstract operation\n// https://tc39.es/ecma262/#sec-lengthofarraylike\nmodule.exports = function (obj) {\n  return toLength(obj.length);\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/length-of-array-like.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/make-built-in.js":function(module,__unused_webpack_exports,__webpack_require__){eval("var fails = __webpack_require__(/*! ../internals/fails */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/fails.js\");\nvar isCallable = __webpack_require__(/*! ../internals/is-callable */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-callable.js\");\nvar hasOwn = __webpack_require__(/*! ../internals/has-own-property */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/has-own-property.js\");\nvar DESCRIPTORS = __webpack_require__(/*! ../internals/descriptors */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/descriptors.js\");\nvar CONFIGURABLE_FUNCTION_NAME = (__webpack_require__(/*! ../internals/function-name */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-name.js\").CONFIGURABLE);\nvar inspectSource = __webpack_require__(/*! ../internals/inspect-source */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/inspect-source.js\");\nvar InternalStateModule = __webpack_require__(/*! ../internals/internal-state */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/internal-state.js\");\n\nvar enforceInternalState = InternalStateModule.enforce;\nvar getInternalState = InternalStateModule.get;\n// eslint-disable-next-line es-x/no-object-defineproperty -- safe\nvar defineProperty = Object.defineProperty;\n\nvar CONFIGURABLE_LENGTH = DESCRIPTORS && !fails(function () {\n  return defineProperty(function () { /* empty */ }, 'length', { value: 8 }).length !== 8;\n});\n\nvar TEMPLATE = String(String).split('String');\n\nvar makeBuiltIn = module.exports = function (value, name, options) {\n  if (String(name).slice(0, 7) === 'Symbol(') {\n    name = '[' + String(name).replace(/^Symbol\\(([^)]*)\\)/, '$1') + ']';\n  }\n  if (options && options.getter) name = 'get ' + name;\n  if (options && options.setter) name = 'set ' + name;\n  if (!hasOwn(value, 'name') || (CONFIGURABLE_FUNCTION_NAME && value.name !== name)) {\n    if (DESCRIPTORS) defineProperty(value, 'name', { value: name, configurable: true });\n    else value.name = name;\n  }\n  if (CONFIGURABLE_LENGTH && options && hasOwn(options, 'arity') && value.length !== options.arity) {\n    defineProperty(value, 'length', { value: options.arity });\n  }\n  try {\n    if (options && hasOwn(options, 'constructor') && options.constructor) {\n      if (DESCRIPTORS) defineProperty(value, 'prototype', { writable: false });\n    // in V8 ~ Chrome 53, prototypes of some methods, like `Array.prototype.values`, are non-writable\n    } else if (value.prototype) value.prototype = undefined;\n  } catch (error) { /* empty */ }\n  var state = enforceInternalState(value);\n  if (!hasOwn(state, 'source')) {\n    state.source = TEMPLATE.join(typeof name == 'string' ? name : '');\n  } return value;\n};\n\n// add fake Function#toString for correct work wrapped methods / constructors with methods like LoDash isNative\n// eslint-disable-next-line no-extend-native -- required\nFunction.prototype.toString = makeBuiltIn(function toString() {\n  return isCallable(this) && getInternalState(this).source || inspectSource(this);\n}, 'toString');\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/make-built-in.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/math-trunc.js":function(module){eval("var ceil = Math.ceil;\nvar floor = Math.floor;\n\n// `Math.trunc` method\n// https://tc39.es/ecma262/#sec-math.trunc\n// eslint-disable-next-line es-x/no-math-trunc -- safe\nmodule.exports = Math.trunc || function trunc(x) {\n  var n = +x;\n  return (n > 0 ? floor : ceil)(n);\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/math-trunc.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/microtask.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var global = __webpack_require__(/*! ../internals/global */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/global.js");\nvar bind = __webpack_require__(/*! ../internals/function-bind-context */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-bind-context.js");\nvar getOwnPropertyDescriptor = (__webpack_require__(/*! ../internals/object-get-own-property-descriptor */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-get-own-property-descriptor.js").f);\nvar macrotask = (__webpack_require__(/*! ../internals/task */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/task.js").set);\nvar IS_IOS = __webpack_require__(/*! ../internals/engine-is-ios */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-is-ios.js");\nvar IS_IOS_PEBBLE = __webpack_require__(/*! ../internals/engine-is-ios-pebble */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-is-ios-pebble.js");\nvar IS_WEBOS_WEBKIT = __webpack_require__(/*! ../internals/engine-is-webos-webkit */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-is-webos-webkit.js");\nvar IS_NODE = __webpack_require__(/*! ../internals/engine-is-node */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-is-node.js");\n\nvar MutationObserver = global.MutationObserver || global.WebKitMutationObserver;\nvar document = global.document;\nvar process = global.process;\nvar Promise = global.Promise;\n// Node.js 11 shows ExperimentalWarning on getting `queueMicrotask`\nvar queueMicrotaskDescriptor = getOwnPropertyDescriptor(global, \'queueMicrotask\');\nvar queueMicrotask = queueMicrotaskDescriptor && queueMicrotaskDescriptor.value;\n\nvar flush, head, last, notify, toggle, node, promise, then;\n\n// modern engines have queueMicrotask method\nif (!queueMicrotask) {\n  flush = function () {\n    var parent, fn;\n    if (IS_NODE && (parent = process.domain)) parent.exit();\n    while (head) {\n      fn = head.fn;\n      head = head.next;\n      try {\n        fn();\n      } catch (error) {\n        if (head) notify();\n        else last = undefined;\n        throw error;\n      }\n    } last = undefined;\n    if (parent) parent.enter();\n  };\n\n  // browsers with MutationObserver, except iOS - https://github.com/zloirock/core-js/issues/339\n  // also except WebOS Webkit https://github.com/zloirock/core-js/issues/898\n  if (!IS_IOS && !IS_NODE && !IS_WEBOS_WEBKIT && MutationObserver && document) {\n    toggle = true;\n    node = document.createTextNode(\'\');\n    new MutationObserver(flush).observe(node, { characterData: true });\n    notify = function () {\n      node.data = toggle = !toggle;\n    };\n  // environments with maybe non-completely correct, but existent Promise\n  } else if (!IS_IOS_PEBBLE && Promise && Promise.resolve) {\n    // Promise.resolve without an argument throws an error in LG WebOS 2\n    promise = Promise.resolve(undefined);\n    // workaround of WebKit ~ iOS Safari 10.1 bug\n    promise.constructor = Promise;\n    then = bind(promise.then, promise);\n    notify = function () {\n      then(flush);\n    };\n  // Node.js without promises\n  } else if (IS_NODE) {\n    notify = function () {\n      process.nextTick(flush);\n    };\n  // for other environments - macrotask based on:\n  // - setImmediate\n  // - MessageChannel\n  // - window.postMessage\n  // - onreadystatechange\n  // - setTimeout\n  } else {\n    // strange IE + webpack dev server bug - use .bind(global)\n    macrotask = bind(macrotask, global);\n    notify = function () {\n      macrotask(flush);\n    };\n  }\n}\n\nmodule.exports = queueMicrotask || function (fn) {\n  var task = { fn: fn, next: undefined };\n  if (last) last.next = task;\n  if (!head) {\n    head = task;\n    notify();\n  } last = task;\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/microtask.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/native-symbol.js":function(module,__unused_webpack_exports,__webpack_require__){eval('/* eslint-disable es-x/no-symbol -- required for testing */\nvar V8_VERSION = __webpack_require__(/*! ../internals/engine-v8-version */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-v8-version.js");\nvar fails = __webpack_require__(/*! ../internals/fails */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/fails.js");\n\n// eslint-disable-next-line es-x/no-object-getownpropertysymbols -- required for testing\nmodule.exports = !!Object.getOwnPropertySymbols && !fails(function () {\n  var symbol = Symbol();\n  // Chrome 38 Symbol has incorrect toString conversion\n  // `get-own-property-symbols` polyfill symbols converted to object are not Symbol instances\n  return !String(symbol) || !(Object(symbol) instanceof Symbol) ||\n    // Chrome 38-40 symbols are not inherited from DOM collections prototypes to instances\n    !Symbol.sham && V8_VERSION && V8_VERSION < 41;\n});\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/native-symbol.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/native-weak-map.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var global = __webpack_require__(/*! ../internals/global */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/global.js");\nvar isCallable = __webpack_require__(/*! ../internals/is-callable */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-callable.js");\nvar inspectSource = __webpack_require__(/*! ../internals/inspect-source */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/inspect-source.js");\n\nvar WeakMap = global.WeakMap;\n\nmodule.exports = isCallable(WeakMap) && /native code/.test(inspectSource(WeakMap));\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/native-weak-map.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/new-promise-capability.js":function(module,__unused_webpack_exports,__webpack_require__){"use strict";eval("\nvar aCallable = __webpack_require__(/*! ../internals/a-callable */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/a-callable.js\");\n\nvar PromiseCapability = function (C) {\n  var resolve, reject;\n  this.promise = new C(function ($$resolve, $$reject) {\n    if (resolve !== undefined || reject !== undefined) throw TypeError('Bad Promise constructor');\n    resolve = $$resolve;\n    reject = $$reject;\n  });\n  this.resolve = aCallable(resolve);\n  this.reject = aCallable(reject);\n};\n\n// `NewPromiseCapability` abstract operation\n// https://tc39.es/ecma262/#sec-newpromisecapability\nmodule.exports.f = function (C) {\n  return new PromiseCapability(C);\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/new-promise-capability.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/normalize-string-argument.js":function(module,__unused_webpack_exports,__webpack_require__){eval("var toString = __webpack_require__(/*! ../internals/to-string */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-string.js\");\n\nmodule.exports = function (argument, $default) {\n  return argument === undefined ? arguments.length < 2 ? '' : $default : toString(argument);\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/normalize-string-argument.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-create.js":function(module,__unused_webpack_exports,__webpack_require__){eval("/* global ActiveXObject -- old IE, WSH */\nvar anObject = __webpack_require__(/*! ../internals/an-object */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/an-object.js\");\nvar definePropertiesModule = __webpack_require__(/*! ../internals/object-define-properties */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-define-properties.js\");\nvar enumBugKeys = __webpack_require__(/*! ../internals/enum-bug-keys */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/enum-bug-keys.js\");\nvar hiddenKeys = __webpack_require__(/*! ../internals/hidden-keys */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/hidden-keys.js\");\nvar html = __webpack_require__(/*! ../internals/html */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/html.js\");\nvar documentCreateElement = __webpack_require__(/*! ../internals/document-create-element */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/document-create-element.js\");\nvar sharedKey = __webpack_require__(/*! ../internals/shared-key */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/shared-key.js\");\n\nvar GT = '>';\nvar LT = '<';\nvar PROTOTYPE = 'prototype';\nvar SCRIPT = 'script';\nvar IE_PROTO = sharedKey('IE_PROTO');\n\nvar EmptyConstructor = function () { /* empty */ };\n\nvar scriptTag = function (content) {\n  return LT + SCRIPT + GT + content + LT + '/' + SCRIPT + GT;\n};\n\n// Create object with fake `null` prototype: use ActiveX Object with cleared prototype\nvar NullProtoObjectViaActiveX = function (activeXDocument) {\n  activeXDocument.write(scriptTag(''));\n  activeXDocument.close();\n  var temp = activeXDocument.parentWindow.Object;\n  activeXDocument = null; // avoid memory leak\n  return temp;\n};\n\n// Create object with fake `null` prototype: use iframe Object with cleared prototype\nvar NullProtoObjectViaIFrame = function () {\n  // Thrash, waste and sodomy: IE GC bug\n  var iframe = documentCreateElement('iframe');\n  var JS = 'java' + SCRIPT + ':';\n  var iframeDocument;\n  iframe.style.display = 'none';\n  html.appendChild(iframe);\n  // https://github.com/zloirock/core-js/issues/475\n  iframe.src = String(JS);\n  iframeDocument = iframe.contentWindow.document;\n  iframeDocument.open();\n  iframeDocument.write(scriptTag('document.F=Object'));\n  iframeDocument.close();\n  return iframeDocument.F;\n};\n\n// Check for document.domain and active x support\n// No need to use active x approach when document.domain is not set\n// see https://github.com/es-shims/es5-shim/issues/150\n// variation of https://github.com/kitcambridge/es5-shim/commit/4f738ac066346\n// avoid IE GC bug\nvar activeXDocument;\nvar NullProtoObject = function () {\n  try {\n    activeXDocument = new ActiveXObject('htmlfile');\n  } catch (error) { /* ignore */ }\n  NullProtoObject = typeof document != 'undefined'\n    ? document.domain && activeXDocument\n      ? NullProtoObjectViaActiveX(activeXDocument) // old IE\n      : NullProtoObjectViaIFrame()\n    : NullProtoObjectViaActiveX(activeXDocument); // WSH\n  var length = enumBugKeys.length;\n  while (length--) delete NullProtoObject[PROTOTYPE][enumBugKeys[length]];\n  return NullProtoObject();\n};\n\nhiddenKeys[IE_PROTO] = true;\n\n// `Object.create` method\n// https://tc39.es/ecma262/#sec-object.create\n// eslint-disable-next-line es-x/no-object-create -- safe\nmodule.exports = Object.create || function create(O, Properties) {\n  var result;\n  if (O !== null) {\n    EmptyConstructor[PROTOTYPE] = anObject(O);\n    result = new EmptyConstructor();\n    EmptyConstructor[PROTOTYPE] = null;\n    // add \"__proto__\" for Object.getPrototypeOf polyfill\n    result[IE_PROTO] = O;\n  } else result = NullProtoObject();\n  return Properties === undefined ? result : definePropertiesModule.f(result, Properties);\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-create.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-define-properties.js":function(__unused_webpack_module,exports,__webpack_require__){eval('var DESCRIPTORS = __webpack_require__(/*! ../internals/descriptors */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/descriptors.js");\nvar V8_PROTOTYPE_DEFINE_BUG = __webpack_require__(/*! ../internals/v8-prototype-define-bug */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/v8-prototype-define-bug.js");\nvar definePropertyModule = __webpack_require__(/*! ../internals/object-define-property */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-define-property.js");\nvar anObject = __webpack_require__(/*! ../internals/an-object */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/an-object.js");\nvar toIndexedObject = __webpack_require__(/*! ../internals/to-indexed-object */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-indexed-object.js");\nvar objectKeys = __webpack_require__(/*! ../internals/object-keys */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-keys.js");\n\n// `Object.defineProperties` method\n// https://tc39.es/ecma262/#sec-object.defineproperties\n// eslint-disable-next-line es-x/no-object-defineproperties -- safe\nexports.f = DESCRIPTORS && !V8_PROTOTYPE_DEFINE_BUG ? Object.defineProperties : function defineProperties(O, Properties) {\n  anObject(O);\n  var props = toIndexedObject(Properties);\n  var keys = objectKeys(Properties);\n  var length = keys.length;\n  var index = 0;\n  var key;\n  while (length > index) definePropertyModule.f(O, key = keys[index++], props[key]);\n  return O;\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-define-properties.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-define-property.js":function(__unused_webpack_module,exports,__webpack_require__){eval("var DESCRIPTORS = __webpack_require__(/*! ../internals/descriptors */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/descriptors.js\");\nvar IE8_DOM_DEFINE = __webpack_require__(/*! ../internals/ie8-dom-define */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/ie8-dom-define.js\");\nvar V8_PROTOTYPE_DEFINE_BUG = __webpack_require__(/*! ../internals/v8-prototype-define-bug */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/v8-prototype-define-bug.js\");\nvar anObject = __webpack_require__(/*! ../internals/an-object */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/an-object.js\");\nvar toPropertyKey = __webpack_require__(/*! ../internals/to-property-key */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-property-key.js\");\n\nvar $TypeError = TypeError;\n// eslint-disable-next-line es-x/no-object-defineproperty -- safe\nvar $defineProperty = Object.defineProperty;\n// eslint-disable-next-line es-x/no-object-getownpropertydescriptor -- safe\nvar $getOwnPropertyDescriptor = Object.getOwnPropertyDescriptor;\nvar ENUMERABLE = 'enumerable';\nvar CONFIGURABLE = 'configurable';\nvar WRITABLE = 'writable';\n\n// `Object.defineProperty` method\n// https://tc39.es/ecma262/#sec-object.defineproperty\nexports.f = DESCRIPTORS ? V8_PROTOTYPE_DEFINE_BUG ? function defineProperty(O, P, Attributes) {\n  anObject(O);\n  P = toPropertyKey(P);\n  anObject(Attributes);\n  if (typeof O === 'function' && P === 'prototype' && 'value' in Attributes && WRITABLE in Attributes && !Attributes[WRITABLE]) {\n    var current = $getOwnPropertyDescriptor(O, P);\n    if (current && current[WRITABLE]) {\n      O[P] = Attributes.value;\n      Attributes = {\n        configurable: CONFIGURABLE in Attributes ? Attributes[CONFIGURABLE] : current[CONFIGURABLE],\n        enumerable: ENUMERABLE in Attributes ? Attributes[ENUMERABLE] : current[ENUMERABLE],\n        writable: false\n      };\n    }\n  } return $defineProperty(O, P, Attributes);\n} : $defineProperty : function defineProperty(O, P, Attributes) {\n  anObject(O);\n  P = toPropertyKey(P);\n  anObject(Attributes);\n  if (IE8_DOM_DEFINE) try {\n    return $defineProperty(O, P, Attributes);\n  } catch (error) { /* empty */ }\n  if ('get' in Attributes || 'set' in Attributes) throw $TypeError('Accessors not supported');\n  if ('value' in Attributes) O[P] = Attributes.value;\n  return O;\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-define-property.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-get-own-property-descriptor.js":function(__unused_webpack_module,exports,__webpack_require__){eval('var DESCRIPTORS = __webpack_require__(/*! ../internals/descriptors */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/descriptors.js");\nvar call = __webpack_require__(/*! ../internals/function-call */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-call.js");\nvar propertyIsEnumerableModule = __webpack_require__(/*! ../internals/object-property-is-enumerable */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-property-is-enumerable.js");\nvar createPropertyDescriptor = __webpack_require__(/*! ../internals/create-property-descriptor */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/create-property-descriptor.js");\nvar toIndexedObject = __webpack_require__(/*! ../internals/to-indexed-object */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-indexed-object.js");\nvar toPropertyKey = __webpack_require__(/*! ../internals/to-property-key */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-property-key.js");\nvar hasOwn = __webpack_require__(/*! ../internals/has-own-property */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/has-own-property.js");\nvar IE8_DOM_DEFINE = __webpack_require__(/*! ../internals/ie8-dom-define */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/ie8-dom-define.js");\n\n// eslint-disable-next-line es-x/no-object-getownpropertydescriptor -- safe\nvar $getOwnPropertyDescriptor = Object.getOwnPropertyDescriptor;\n\n// `Object.getOwnPropertyDescriptor` method\n// https://tc39.es/ecma262/#sec-object.getownpropertydescriptor\nexports.f = DESCRIPTORS ? $getOwnPropertyDescriptor : function getOwnPropertyDescriptor(O, P) {\n  O = toIndexedObject(O);\n  P = toPropertyKey(P);\n  if (IE8_DOM_DEFINE) try {\n    return $getOwnPropertyDescriptor(O, P);\n  } catch (error) { /* empty */ }\n  if (hasOwn(O, P)) return createPropertyDescriptor(!call(propertyIsEnumerableModule.f, O, P), O[P]);\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-get-own-property-descriptor.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-get-own-property-names.js":function(__unused_webpack_module,exports,__webpack_require__){eval("var internalObjectKeys = __webpack_require__(/*! ../internals/object-keys-internal */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-keys-internal.js\");\nvar enumBugKeys = __webpack_require__(/*! ../internals/enum-bug-keys */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/enum-bug-keys.js\");\n\nvar hiddenKeys = enumBugKeys.concat('length', 'prototype');\n\n// `Object.getOwnPropertyNames` method\n// https://tc39.es/ecma262/#sec-object.getownpropertynames\n// eslint-disable-next-line es-x/no-object-getownpropertynames -- safe\nexports.f = Object.getOwnPropertyNames || function getOwnPropertyNames(O) {\n  return internalObjectKeys(O, hiddenKeys);\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-get-own-property-names.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-get-own-property-symbols.js":function(__unused_webpack_module,exports){eval("// eslint-disable-next-line es-x/no-object-getownpropertysymbols -- safe\nexports.f = Object.getOwnPropertySymbols;\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-get-own-property-symbols.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-get-prototype-of.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var hasOwn = __webpack_require__(/*! ../internals/has-own-property */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/has-own-property.js");\nvar isCallable = __webpack_require__(/*! ../internals/is-callable */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-callable.js");\nvar toObject = __webpack_require__(/*! ../internals/to-object */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-object.js");\nvar sharedKey = __webpack_require__(/*! ../internals/shared-key */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/shared-key.js");\nvar CORRECT_PROTOTYPE_GETTER = __webpack_require__(/*! ../internals/correct-prototype-getter */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/correct-prototype-getter.js");\n\nvar IE_PROTO = sharedKey(\'IE_PROTO\');\nvar $Object = Object;\nvar ObjectPrototype = $Object.prototype;\n\n// `Object.getPrototypeOf` method\n// https://tc39.es/ecma262/#sec-object.getprototypeof\n// eslint-disable-next-line es-x/no-object-getprototypeof -- safe\nmodule.exports = CORRECT_PROTOTYPE_GETTER ? $Object.getPrototypeOf : function (O) {\n  var object = toObject(O);\n  if (hasOwn(object, IE_PROTO)) return object[IE_PROTO];\n  var constructor = object.constructor;\n  if (isCallable(constructor) && object instanceof constructor) {\n    return constructor.prototype;\n  } return object instanceof $Object ? ObjectPrototype : null;\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-get-prototype-of.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-is-prototype-of.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var uncurryThis = __webpack_require__(/*! ../internals/function-uncurry-this */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-uncurry-this.js");\n\nmodule.exports = uncurryThis({}.isPrototypeOf);\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-is-prototype-of.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-keys-internal.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var uncurryThis = __webpack_require__(/*! ../internals/function-uncurry-this */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-uncurry-this.js");\nvar hasOwn = __webpack_require__(/*! ../internals/has-own-property */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/has-own-property.js");\nvar toIndexedObject = __webpack_require__(/*! ../internals/to-indexed-object */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-indexed-object.js");\nvar indexOf = (__webpack_require__(/*! ../internals/array-includes */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/array-includes.js").indexOf);\nvar hiddenKeys = __webpack_require__(/*! ../internals/hidden-keys */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/hidden-keys.js");\n\nvar push = uncurryThis([].push);\n\nmodule.exports = function (object, names) {\n  var O = toIndexedObject(object);\n  var i = 0;\n  var result = [];\n  var key;\n  for (key in O) !hasOwn(hiddenKeys, key) && hasOwn(O, key) && push(result, key);\n  // Don\'t enum bug & hidden keys\n  while (names.length > i) if (hasOwn(O, key = names[i++])) {\n    ~indexOf(result, key) || push(result, key);\n  }\n  return result;\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-keys-internal.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-keys.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var internalObjectKeys = __webpack_require__(/*! ../internals/object-keys-internal */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-keys-internal.js");\nvar enumBugKeys = __webpack_require__(/*! ../internals/enum-bug-keys */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/enum-bug-keys.js");\n\n// `Object.keys` method\n// https://tc39.es/ecma262/#sec-object.keys\n// eslint-disable-next-line es-x/no-object-keys -- safe\nmodule.exports = Object.keys || function keys(O) {\n  return internalObjectKeys(O, enumBugKeys);\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-keys.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-property-is-enumerable.js":function(__unused_webpack_module,exports){"use strict";eval("\nvar $propertyIsEnumerable = {}.propertyIsEnumerable;\n// eslint-disable-next-line es-x/no-object-getownpropertydescriptor -- safe\nvar getOwnPropertyDescriptor = Object.getOwnPropertyDescriptor;\n\n// Nashorn ~ JDK8 bug\nvar NASHORN_BUG = getOwnPropertyDescriptor && !$propertyIsEnumerable.call({ 1: 2 }, 1);\n\n// `Object.prototype.propertyIsEnumerable` method implementation\n// https://tc39.es/ecma262/#sec-object.prototype.propertyisenumerable\nexports.f = NASHORN_BUG ? function propertyIsEnumerable(V) {\n  var descriptor = getOwnPropertyDescriptor(this, V);\n  return !!descriptor && descriptor.enumerable;\n} : $propertyIsEnumerable;\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-property-is-enumerable.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-set-prototype-of.js":function(module,__unused_webpack_exports,__webpack_require__){eval('/* eslint-disable no-proto -- safe */\nvar uncurryThis = __webpack_require__(/*! ../internals/function-uncurry-this */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-uncurry-this.js");\nvar anObject = __webpack_require__(/*! ../internals/an-object */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/an-object.js");\nvar aPossiblePrototype = __webpack_require__(/*! ../internals/a-possible-prototype */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/a-possible-prototype.js");\n\n// `Object.setPrototypeOf` method\n// https://tc39.es/ecma262/#sec-object.setprototypeof\n// Works with __proto__ only. Old v8 can\'t work with null proto objects.\n// eslint-disable-next-line es-x/no-object-setprototypeof -- safe\nmodule.exports = Object.setPrototypeOf || (\'__proto__\' in {} ? function () {\n  var CORRECT_SETTER = false;\n  var test = {};\n  var setter;\n  try {\n    // eslint-disable-next-line es-x/no-object-getownpropertydescriptor -- safe\n    setter = uncurryThis(Object.getOwnPropertyDescriptor(Object.prototype, \'__proto__\').set);\n    setter(test, []);\n    CORRECT_SETTER = test instanceof Array;\n  } catch (error) { /* empty */ }\n  return function setPrototypeOf(O, proto) {\n    anObject(O);\n    aPossiblePrototype(proto);\n    if (CORRECT_SETTER) setter(O, proto);\n    else O.__proto__ = proto;\n    return O;\n  };\n}() : undefined);\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-set-prototype-of.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-to-string.js":function(module,__unused_webpack_exports,__webpack_require__){"use strict";eval("\nvar TO_STRING_TAG_SUPPORT = __webpack_require__(/*! ../internals/to-string-tag-support */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-string-tag-support.js\");\nvar classof = __webpack_require__(/*! ../internals/classof */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/classof.js\");\n\n// `Object.prototype.toString` method implementation\n// https://tc39.es/ecma262/#sec-object.prototype.tostring\nmodule.exports = TO_STRING_TAG_SUPPORT ? {}.toString : function toString() {\n  return '[object ' + classof(this) + ']';\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-to-string.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/ordinary-to-primitive.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var call = __webpack_require__(/*! ../internals/function-call */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-call.js");\nvar isCallable = __webpack_require__(/*! ../internals/is-callable */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-callable.js");\nvar isObject = __webpack_require__(/*! ../internals/is-object */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-object.js");\n\nvar $TypeError = TypeError;\n\n// `OrdinaryToPrimitive` abstract operation\n// https://tc39.es/ecma262/#sec-ordinarytoprimitive\nmodule.exports = function (input, pref) {\n  var fn, val;\n  if (pref === \'string\' && isCallable(fn = input.toString) && !isObject(val = call(fn, input))) return val;\n  if (isCallable(fn = input.valueOf) && !isObject(val = call(fn, input))) return val;\n  if (pref !== \'string\' && isCallable(fn = input.toString) && !isObject(val = call(fn, input))) return val;\n  throw $TypeError("Can\'t convert object to primitive value");\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/ordinary-to-primitive.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/own-keys.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var getBuiltIn = __webpack_require__(/*! ../internals/get-built-in */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/get-built-in.js");\nvar uncurryThis = __webpack_require__(/*! ../internals/function-uncurry-this */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-uncurry-this.js");\nvar getOwnPropertyNamesModule = __webpack_require__(/*! ../internals/object-get-own-property-names */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-get-own-property-names.js");\nvar getOwnPropertySymbolsModule = __webpack_require__(/*! ../internals/object-get-own-property-symbols */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-get-own-property-symbols.js");\nvar anObject = __webpack_require__(/*! ../internals/an-object */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/an-object.js");\n\nvar concat = uncurryThis([].concat);\n\n// all object keys, includes non-enumerable and symbols\nmodule.exports = getBuiltIn(\'Reflect\', \'ownKeys\') || function ownKeys(it) {\n  var keys = getOwnPropertyNamesModule.f(anObject(it));\n  var getOwnPropertySymbols = getOwnPropertySymbolsModule.f;\n  return getOwnPropertySymbols ? concat(keys, getOwnPropertySymbols(it)) : keys;\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/own-keys.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/path.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var global = __webpack_require__(/*! ../internals/global */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/global.js");\n\nmodule.exports = global;\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/path.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/perform.js":function(module){eval("module.exports = function (exec) {\n  try {\n    return { error: false, value: exec() };\n  } catch (error) {\n    return { error: true, value: error };\n  }\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/perform.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/promise-constructor-detection.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var global = __webpack_require__(/*! ../internals/global */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/global.js");\nvar NativePromiseConstructor = __webpack_require__(/*! ../internals/promise-native-constructor */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/promise-native-constructor.js");\nvar isCallable = __webpack_require__(/*! ../internals/is-callable */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-callable.js");\nvar isForced = __webpack_require__(/*! ../internals/is-forced */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-forced.js");\nvar inspectSource = __webpack_require__(/*! ../internals/inspect-source */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/inspect-source.js");\nvar wellKnownSymbol = __webpack_require__(/*! ../internals/well-known-symbol */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/well-known-symbol.js");\nvar IS_BROWSER = __webpack_require__(/*! ../internals/engine-is-browser */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-is-browser.js");\nvar IS_DENO = __webpack_require__(/*! ../internals/engine-is-deno */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-is-deno.js");\nvar IS_PURE = __webpack_require__(/*! ../internals/is-pure */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-pure.js");\nvar V8_VERSION = __webpack_require__(/*! ../internals/engine-v8-version */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-v8-version.js");\n\nvar NativePromisePrototype = NativePromiseConstructor && NativePromiseConstructor.prototype;\nvar SPECIES = wellKnownSymbol(\'species\');\nvar SUBCLASSING = false;\nvar NATIVE_PROMISE_REJECTION_EVENT = isCallable(global.PromiseRejectionEvent);\n\nvar FORCED_PROMISE_CONSTRUCTOR = isForced(\'Promise\', function () {\n  var PROMISE_CONSTRUCTOR_SOURCE = inspectSource(NativePromiseConstructor);\n  var GLOBAL_CORE_JS_PROMISE = PROMISE_CONSTRUCTOR_SOURCE !== String(NativePromiseConstructor);\n  // V8 6.6 (Node 10 and Chrome 66) have a bug with resolving custom thenables\n  // https://bugs.chromium.org/p/chromium/issues/detail?id=830565\n  // We can\'t detect it synchronously, so just check versions\n  if (!GLOBAL_CORE_JS_PROMISE && V8_VERSION === 66) return true;\n  // We need Promise#{ catch, finally } in the pure version for preventing prototype pollution\n  if (IS_PURE && !(NativePromisePrototype[\'catch\'] && NativePromisePrototype[\'finally\'])) return true;\n  // We can\'t use @@species feature detection in V8 since it causes\n  // deoptimization and performance degradation\n  // https://github.com/zloirock/core-js/issues/679\n  if (!V8_VERSION || V8_VERSION < 51 || !/native code/.test(PROMISE_CONSTRUCTOR_SOURCE)) {\n    // Detect correctness of subclassing with @@species support\n    var promise = new NativePromiseConstructor(function (resolve) { resolve(1); });\n    var FakePromise = function (exec) {\n      exec(function () { /* empty */ }, function () { /* empty */ });\n    };\n    var constructor = promise.constructor = {};\n    constructor[SPECIES] = FakePromise;\n    SUBCLASSING = promise.then(function () { /* empty */ }) instanceof FakePromise;\n    if (!SUBCLASSING) return true;\n  // Unhandled rejections tracking support, NodeJS Promise without it fails @@species test\n  } return !GLOBAL_CORE_JS_PROMISE && (IS_BROWSER || IS_DENO) && !NATIVE_PROMISE_REJECTION_EVENT;\n});\n\nmodule.exports = {\n  CONSTRUCTOR: FORCED_PROMISE_CONSTRUCTOR,\n  REJECTION_EVENT: NATIVE_PROMISE_REJECTION_EVENT,\n  SUBCLASSING: SUBCLASSING\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/promise-constructor-detection.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/promise-native-constructor.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var global = __webpack_require__(/*! ../internals/global */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/global.js");\n\nmodule.exports = global.Promise;\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/promise-native-constructor.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/promise-resolve.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var anObject = __webpack_require__(/*! ../internals/an-object */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/an-object.js");\nvar isObject = __webpack_require__(/*! ../internals/is-object */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-object.js");\nvar newPromiseCapability = __webpack_require__(/*! ../internals/new-promise-capability */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/new-promise-capability.js");\n\nmodule.exports = function (C, x) {\n  anObject(C);\n  if (isObject(x) && x.constructor === C) return x;\n  var promiseCapability = newPromiseCapability.f(C);\n  var resolve = promiseCapability.resolve;\n  resolve(x);\n  return promiseCapability.promise;\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/promise-resolve.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/promise-statics-incorrect-iteration.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var NativePromiseConstructor = __webpack_require__(/*! ../internals/promise-native-constructor */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/promise-native-constructor.js");\nvar checkCorrectnessOfIteration = __webpack_require__(/*! ../internals/check-correctness-of-iteration */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/check-correctness-of-iteration.js");\nvar FORCED_PROMISE_CONSTRUCTOR = (__webpack_require__(/*! ../internals/promise-constructor-detection */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/promise-constructor-detection.js").CONSTRUCTOR);\n\nmodule.exports = FORCED_PROMISE_CONSTRUCTOR || !checkCorrectnessOfIteration(function (iterable) {\n  NativePromiseConstructor.all(iterable).then(undefined, function () { /* empty */ });\n});\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/promise-statics-incorrect-iteration.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/queue.js":function(module){eval("var Queue = function () {\n  this.head = null;\n  this.tail = null;\n};\n\nQueue.prototype = {\n  add: function (item) {\n    var entry = { item: item, next: null };\n    if (this.head) this.tail.next = entry;\n    else this.head = entry;\n    this.tail = entry;\n  },\n  get: function () {\n    var entry = this.head;\n    if (entry) {\n      this.head = entry.next;\n      if (this.tail === entry) this.tail = null;\n      return entry.item;\n    }\n  }\n};\n\nmodule.exports = Queue;\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/queue.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/require-object-coercible.js":function(module){eval('var $TypeError = TypeError;\n\n// `RequireObjectCoercible` abstract operation\n// https://tc39.es/ecma262/#sec-requireobjectcoercible\nmodule.exports = function (it) {\n  if (it == undefined) throw $TypeError("Can\'t call method on " + it);\n  return it;\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/require-object-coercible.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/set-species.js":function(module,__unused_webpack_exports,__webpack_require__){"use strict";eval('\nvar getBuiltIn = __webpack_require__(/*! ../internals/get-built-in */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/get-built-in.js");\nvar definePropertyModule = __webpack_require__(/*! ../internals/object-define-property */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-define-property.js");\nvar wellKnownSymbol = __webpack_require__(/*! ../internals/well-known-symbol */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/well-known-symbol.js");\nvar DESCRIPTORS = __webpack_require__(/*! ../internals/descriptors */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/descriptors.js");\n\nvar SPECIES = wellKnownSymbol(\'species\');\n\nmodule.exports = function (CONSTRUCTOR_NAME) {\n  var Constructor = getBuiltIn(CONSTRUCTOR_NAME);\n  var defineProperty = definePropertyModule.f;\n\n  if (DESCRIPTORS && Constructor && !Constructor[SPECIES]) {\n    defineProperty(Constructor, SPECIES, {\n      configurable: true,\n      get: function () { return this; }\n    });\n  }\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/set-species.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/set-to-string-tag.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var defineProperty = (__webpack_require__(/*! ../internals/object-define-property */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-define-property.js").f);\nvar hasOwn = __webpack_require__(/*! ../internals/has-own-property */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/has-own-property.js");\nvar wellKnownSymbol = __webpack_require__(/*! ../internals/well-known-symbol */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/well-known-symbol.js");\n\nvar TO_STRING_TAG = wellKnownSymbol(\'toStringTag\');\n\nmodule.exports = function (target, TAG, STATIC) {\n  if (target && !STATIC) target = target.prototype;\n  if (target && !hasOwn(target, TO_STRING_TAG)) {\n    defineProperty(target, TO_STRING_TAG, { configurable: true, value: TAG });\n  }\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/set-to-string-tag.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/shared-key.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var shared = __webpack_require__(/*! ../internals/shared */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/shared.js");\nvar uid = __webpack_require__(/*! ../internals/uid */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/uid.js");\n\nvar keys = shared(\'keys\');\n\nmodule.exports = function (key) {\n  return keys[key] || (keys[key] = uid(key));\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/shared-key.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/shared-store.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var global = __webpack_require__(/*! ../internals/global */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/global.js");\nvar defineGlobalProperty = __webpack_require__(/*! ../internals/define-global-property */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/define-global-property.js");\n\nvar SHARED = \'__core-js_shared__\';\nvar store = global[SHARED] || defineGlobalProperty(SHARED, {});\n\nmodule.exports = store;\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/shared-store.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/shared.js":function(module,__unused_webpack_exports,__webpack_require__){eval("var IS_PURE = __webpack_require__(/*! ../internals/is-pure */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-pure.js\");\nvar store = __webpack_require__(/*! ../internals/shared-store */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/shared-store.js\");\n\n(module.exports = function (key, value) {\n  return store[key] || (store[key] = value !== undefined ? value : {});\n})('versions', []).push({\n  version: '3.24.1',\n  mode: IS_PURE ? 'pure' : 'global',\n  copyright: '© 2014-2022 Denis Pushkarev (zloirock.ru)',\n  license: 'https://github.com/zloirock/core-js/blob/v3.24.1/LICENSE',\n  source: 'https://github.com/zloirock/core-js'\n});\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/shared.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/species-constructor.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var anObject = __webpack_require__(/*! ../internals/an-object */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/an-object.js");\nvar aConstructor = __webpack_require__(/*! ../internals/a-constructor */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/a-constructor.js");\nvar wellKnownSymbol = __webpack_require__(/*! ../internals/well-known-symbol */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/well-known-symbol.js");\n\nvar SPECIES = wellKnownSymbol(\'species\');\n\n// `SpeciesConstructor` abstract operation\n// https://tc39.es/ecma262/#sec-speciesconstructor\nmodule.exports = function (O, defaultConstructor) {\n  var C = anObject(O).constructor;\n  var S;\n  return C === undefined || (S = anObject(C)[SPECIES]) == undefined ? defaultConstructor : aConstructor(S);\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/species-constructor.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/string-multibyte.js":function(module,__unused_webpack_exports,__webpack_require__){eval("var uncurryThis = __webpack_require__(/*! ../internals/function-uncurry-this */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-uncurry-this.js\");\nvar toIntegerOrInfinity = __webpack_require__(/*! ../internals/to-integer-or-infinity */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-integer-or-infinity.js\");\nvar toString = __webpack_require__(/*! ../internals/to-string */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-string.js\");\nvar requireObjectCoercible = __webpack_require__(/*! ../internals/require-object-coercible */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/require-object-coercible.js\");\n\nvar charAt = uncurryThis(''.charAt);\nvar charCodeAt = uncurryThis(''.charCodeAt);\nvar stringSlice = uncurryThis(''.slice);\n\nvar createMethod = function (CONVERT_TO_STRING) {\n  return function ($this, pos) {\n    var S = toString(requireObjectCoercible($this));\n    var position = toIntegerOrInfinity(pos);\n    var size = S.length;\n    var first, second;\n    if (position < 0 || position >= size) return CONVERT_TO_STRING ? '' : undefined;\n    first = charCodeAt(S, position);\n    return first < 0xD800 || first > 0xDBFF || position + 1 === size\n      || (second = charCodeAt(S, position + 1)) < 0xDC00 || second > 0xDFFF\n        ? CONVERT_TO_STRING\n          ? charAt(S, position)\n          : first\n        : CONVERT_TO_STRING\n          ? stringSlice(S, position, position + 2)\n          : (first - 0xD800 << 10) + (second - 0xDC00) + 0x10000;\n  };\n};\n\nmodule.exports = {\n  // `String.prototype.codePointAt` method\n  // https://tc39.es/ecma262/#sec-string.prototype.codepointat\n  codeAt: createMethod(false),\n  // `String.prototype.at` method\n  // https://github.com/mathiasbynens/String.prototype.at\n  charAt: createMethod(true)\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/string-multibyte.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/task.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var global = __webpack_require__(/*! ../internals/global */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/global.js");\nvar apply = __webpack_require__(/*! ../internals/function-apply */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-apply.js");\nvar bind = __webpack_require__(/*! ../internals/function-bind-context */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-bind-context.js");\nvar isCallable = __webpack_require__(/*! ../internals/is-callable */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-callable.js");\nvar hasOwn = __webpack_require__(/*! ../internals/has-own-property */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/has-own-property.js");\nvar fails = __webpack_require__(/*! ../internals/fails */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/fails.js");\nvar html = __webpack_require__(/*! ../internals/html */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/html.js");\nvar arraySlice = __webpack_require__(/*! ../internals/array-slice */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/array-slice.js");\nvar createElement = __webpack_require__(/*! ../internals/document-create-element */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/document-create-element.js");\nvar validateArgumentsLength = __webpack_require__(/*! ../internals/validate-arguments-length */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/validate-arguments-length.js");\nvar IS_IOS = __webpack_require__(/*! ../internals/engine-is-ios */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-is-ios.js");\nvar IS_NODE = __webpack_require__(/*! ../internals/engine-is-node */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-is-node.js");\n\nvar set = global.setImmediate;\nvar clear = global.clearImmediate;\nvar process = global.process;\nvar Dispatch = global.Dispatch;\nvar Function = global.Function;\nvar MessageChannel = global.MessageChannel;\nvar String = global.String;\nvar counter = 0;\nvar queue = {};\nvar ONREADYSTATECHANGE = \'onreadystatechange\';\nvar location, defer, channel, port;\n\ntry {\n  // Deno throws a ReferenceError on `location` access without `--location` flag\n  location = global.location;\n} catch (error) { /* empty */ }\n\nvar run = function (id) {\n  if (hasOwn(queue, id)) {\n    var fn = queue[id];\n    delete queue[id];\n    fn();\n  }\n};\n\nvar runner = function (id) {\n  return function () {\n    run(id);\n  };\n};\n\nvar listener = function (event) {\n  run(event.data);\n};\n\nvar post = function (id) {\n  // old engines have not location.origin\n  global.postMessage(String(id), location.protocol + \'//\' + location.host);\n};\n\n// Node.js 0.9+ & IE10+ has setImmediate, otherwise:\nif (!set || !clear) {\n  set = function setImmediate(handler) {\n    validateArgumentsLength(arguments.length, 1);\n    var fn = isCallable(handler) ? handler : Function(handler);\n    var args = arraySlice(arguments, 1);\n    queue[++counter] = function () {\n      apply(fn, undefined, args);\n    };\n    defer(counter);\n    return counter;\n  };\n  clear = function clearImmediate(id) {\n    delete queue[id];\n  };\n  // Node.js 0.8-\n  if (IS_NODE) {\n    defer = function (id) {\n      process.nextTick(runner(id));\n    };\n  // Sphere (JS game engine) Dispatch API\n  } else if (Dispatch && Dispatch.now) {\n    defer = function (id) {\n      Dispatch.now(runner(id));\n    };\n  // Browsers with MessageChannel, includes WebWorkers\n  // except iOS - https://github.com/zloirock/core-js/issues/624\n  } else if (MessageChannel && !IS_IOS) {\n    channel = new MessageChannel();\n    port = channel.port2;\n    channel.port1.onmessage = listener;\n    defer = bind(port.postMessage, port);\n  // Browsers with postMessage, skip WebWorkers\n  // IE8 has postMessage, but it\'s sync & typeof its postMessage is \'object\'\n  } else if (\n    global.addEventListener &&\n    isCallable(global.postMessage) &&\n    !global.importScripts &&\n    location && location.protocol !== \'file:\' &&\n    !fails(post)\n  ) {\n    defer = post;\n    global.addEventListener(\'message\', listener, false);\n  // IE8-\n  } else if (ONREADYSTATECHANGE in createElement(\'script\')) {\n    defer = function (id) {\n      html.appendChild(createElement(\'script\'))[ONREADYSTATECHANGE] = function () {\n        html.removeChild(this);\n        run(id);\n      };\n    };\n  // Rest old browsers\n  } else {\n    defer = function (id) {\n      setTimeout(runner(id), 0);\n    };\n  }\n}\n\nmodule.exports = {\n  set: set,\n  clear: clear\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/task.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-absolute-index.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var toIntegerOrInfinity = __webpack_require__(/*! ../internals/to-integer-or-infinity */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-integer-or-infinity.js");\n\nvar max = Math.max;\nvar min = Math.min;\n\n// Helper for a popular repeating case of the spec:\n// Let integer be ? ToInteger(index).\n// If integer < 0, let result be max((length + integer), 0); else let result be min(integer, length).\nmodule.exports = function (index, length) {\n  var integer = toIntegerOrInfinity(index);\n  return integer < 0 ? max(integer + length, 0) : min(integer, length);\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-absolute-index.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-indexed-object.js":function(module,__unused_webpack_exports,__webpack_require__){eval('// toObject with fallback for non-array-like ES3 strings\nvar IndexedObject = __webpack_require__(/*! ../internals/indexed-object */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/indexed-object.js");\nvar requireObjectCoercible = __webpack_require__(/*! ../internals/require-object-coercible */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/require-object-coercible.js");\n\nmodule.exports = function (it) {\n  return IndexedObject(requireObjectCoercible(it));\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-indexed-object.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-integer-or-infinity.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var trunc = __webpack_require__(/*! ../internals/math-trunc */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/math-trunc.js");\n\n// `ToIntegerOrInfinity` abstract operation\n// https://tc39.es/ecma262/#sec-tointegerorinfinity\nmodule.exports = function (argument) {\n  var number = +argument;\n  // eslint-disable-next-line no-self-compare -- NaN check\n  return number !== number || number === 0 ? 0 : trunc(number);\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-integer-or-infinity.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-length.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var toIntegerOrInfinity = __webpack_require__(/*! ../internals/to-integer-or-infinity */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-integer-or-infinity.js");\n\nvar min = Math.min;\n\n// `ToLength` abstract operation\n// https://tc39.es/ecma262/#sec-tolength\nmodule.exports = function (argument) {\n  return argument > 0 ? min(toIntegerOrInfinity(argument), 0x1FFFFFFFFFFFFF) : 0; // 2 ** 53 - 1 == 9007199254740991\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-length.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-object.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var requireObjectCoercible = __webpack_require__(/*! ../internals/require-object-coercible */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/require-object-coercible.js");\n\nvar $Object = Object;\n\n// `ToObject` abstract operation\n// https://tc39.es/ecma262/#sec-toobject\nmodule.exports = function (argument) {\n  return $Object(requireObjectCoercible(argument));\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-object.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-primitive.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var call = __webpack_require__(/*! ../internals/function-call */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-call.js");\nvar isObject = __webpack_require__(/*! ../internals/is-object */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-object.js");\nvar isSymbol = __webpack_require__(/*! ../internals/is-symbol */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-symbol.js");\nvar getMethod = __webpack_require__(/*! ../internals/get-method */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/get-method.js");\nvar ordinaryToPrimitive = __webpack_require__(/*! ../internals/ordinary-to-primitive */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/ordinary-to-primitive.js");\nvar wellKnownSymbol = __webpack_require__(/*! ../internals/well-known-symbol */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/well-known-symbol.js");\n\nvar $TypeError = TypeError;\nvar TO_PRIMITIVE = wellKnownSymbol(\'toPrimitive\');\n\n// `ToPrimitive` abstract operation\n// https://tc39.es/ecma262/#sec-toprimitive\nmodule.exports = function (input, pref) {\n  if (!isObject(input) || isSymbol(input)) return input;\n  var exoticToPrim = getMethod(input, TO_PRIMITIVE);\n  var result;\n  if (exoticToPrim) {\n    if (pref === undefined) pref = \'default\';\n    result = call(exoticToPrim, input, pref);\n    if (!isObject(result) || isSymbol(result)) return result;\n    throw $TypeError("Can\'t convert object to primitive value");\n  }\n  if (pref === undefined) pref = \'number\';\n  return ordinaryToPrimitive(input, pref);\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-primitive.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-property-key.js":function(module,__unused_webpack_exports,__webpack_require__){eval("var toPrimitive = __webpack_require__(/*! ../internals/to-primitive */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-primitive.js\");\nvar isSymbol = __webpack_require__(/*! ../internals/is-symbol */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-symbol.js\");\n\n// `ToPropertyKey` abstract operation\n// https://tc39.es/ecma262/#sec-topropertykey\nmodule.exports = function (argument) {\n  var key = toPrimitive(argument, 'string');\n  return isSymbol(key) ? key : key + '';\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-property-key.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-string-tag-support.js":function(module,__unused_webpack_exports,__webpack_require__){eval("var wellKnownSymbol = __webpack_require__(/*! ../internals/well-known-symbol */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/well-known-symbol.js\");\n\nvar TO_STRING_TAG = wellKnownSymbol('toStringTag');\nvar test = {};\n\ntest[TO_STRING_TAG] = 'z';\n\nmodule.exports = String(test) === '[object z]';\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-string-tag-support.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-string.js":function(module,__unused_webpack_exports,__webpack_require__){eval("var classof = __webpack_require__(/*! ../internals/classof */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/classof.js\");\n\nvar $String = String;\n\nmodule.exports = function (argument) {\n  if (classof(argument) === 'Symbol') throw TypeError('Cannot convert a Symbol value to a string');\n  return $String(argument);\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-string.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/try-to-string.js":function(module){eval("var $String = String;\n\nmodule.exports = function (argument) {\n  try {\n    return $String(argument);\n  } catch (error) {\n    return 'Object';\n  }\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/try-to-string.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/uid.js":function(module,__unused_webpack_exports,__webpack_require__){eval("var uncurryThis = __webpack_require__(/*! ../internals/function-uncurry-this */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-uncurry-this.js\");\n\nvar id = 0;\nvar postfix = Math.random();\nvar toString = uncurryThis(1.0.toString);\n\nmodule.exports = function (key) {\n  return 'Symbol(' + (key === undefined ? '' : key) + ')_' + toString(++id + postfix, 36);\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/uid.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/use-symbol-as-uid.js":function(module,__unused_webpack_exports,__webpack_require__){eval("/* eslint-disable es-x/no-symbol -- required for testing */\nvar NATIVE_SYMBOL = __webpack_require__(/*! ../internals/native-symbol */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/native-symbol.js\");\n\nmodule.exports = NATIVE_SYMBOL\n  && !Symbol.sham\n  && typeof Symbol.iterator == 'symbol';\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/use-symbol-as-uid.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/v8-prototype-define-bug.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var DESCRIPTORS = __webpack_require__(/*! ../internals/descriptors */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/descriptors.js");\nvar fails = __webpack_require__(/*! ../internals/fails */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/fails.js");\n\n// V8 ~ Chrome 36-\n// https://bugs.chromium.org/p/v8/issues/detail?id=3334\nmodule.exports = DESCRIPTORS && fails(function () {\n  // eslint-disable-next-line es-x/no-object-defineproperty -- required for testing\n  return Object.defineProperty(function () { /* empty */ }, \'prototype\', {\n    value: 42,\n    writable: false\n  }).prototype != 42;\n});\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/v8-prototype-define-bug.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/validate-arguments-length.js":function(module){eval("var $TypeError = TypeError;\n\nmodule.exports = function (passed, required) {\n  if (passed < required) throw $TypeError('Not enough arguments');\n  return passed;\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/validate-arguments-length.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/well-known-symbol.js":function(module,__unused_webpack_exports,__webpack_require__){eval('var global = __webpack_require__(/*! ../internals/global */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/global.js");\nvar shared = __webpack_require__(/*! ../internals/shared */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/shared.js");\nvar hasOwn = __webpack_require__(/*! ../internals/has-own-property */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/has-own-property.js");\nvar uid = __webpack_require__(/*! ../internals/uid */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/uid.js");\nvar NATIVE_SYMBOL = __webpack_require__(/*! ../internals/native-symbol */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/native-symbol.js");\nvar USE_SYMBOL_AS_UID = __webpack_require__(/*! ../internals/use-symbol-as-uid */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/use-symbol-as-uid.js");\n\nvar WellKnownSymbolsStore = shared(\'wks\');\nvar Symbol = global.Symbol;\nvar symbolFor = Symbol && Symbol[\'for\'];\nvar createWellKnownSymbol = USE_SYMBOL_AS_UID ? Symbol : Symbol && Symbol.withoutSetter || uid;\n\nmodule.exports = function (name) {\n  if (!hasOwn(WellKnownSymbolsStore, name) || !(NATIVE_SYMBOL || typeof WellKnownSymbolsStore[name] == \'string\')) {\n    var description = \'Symbol.\' + name;\n    if (NATIVE_SYMBOL && hasOwn(Symbol, name)) {\n      WellKnownSymbolsStore[name] = Symbol[name];\n    } else if (USE_SYMBOL_AS_UID && symbolFor) {\n      WellKnownSymbolsStore[name] = symbolFor(description);\n    } else {\n      WellKnownSymbolsStore[name] = createWellKnownSymbol(description);\n    }\n  } return WellKnownSymbolsStore[name];\n};\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/well-known-symbol.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.aggregate-error.constructor.js":function(__unused_webpack_module,__unused_webpack_exports,__webpack_require__){"use strict";eval('\nvar $ = __webpack_require__(/*! ../internals/export */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/export.js");\nvar isPrototypeOf = __webpack_require__(/*! ../internals/object-is-prototype-of */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-is-prototype-of.js");\nvar getPrototypeOf = __webpack_require__(/*! ../internals/object-get-prototype-of */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-get-prototype-of.js");\nvar setPrototypeOf = __webpack_require__(/*! ../internals/object-set-prototype-of */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-set-prototype-of.js");\nvar copyConstructorProperties = __webpack_require__(/*! ../internals/copy-constructor-properties */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/copy-constructor-properties.js");\nvar create = __webpack_require__(/*! ../internals/object-create */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-create.js");\nvar createNonEnumerableProperty = __webpack_require__(/*! ../internals/create-non-enumerable-property */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/create-non-enumerable-property.js");\nvar createPropertyDescriptor = __webpack_require__(/*! ../internals/create-property-descriptor */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/create-property-descriptor.js");\nvar clearErrorStack = __webpack_require__(/*! ../internals/clear-error-stack */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/clear-error-stack.js");\nvar installErrorCause = __webpack_require__(/*! ../internals/install-error-cause */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/install-error-cause.js");\nvar iterate = __webpack_require__(/*! ../internals/iterate */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/iterate.js");\nvar normalizeStringArgument = __webpack_require__(/*! ../internals/normalize-string-argument */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/normalize-string-argument.js");\nvar wellKnownSymbol = __webpack_require__(/*! ../internals/well-known-symbol */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/well-known-symbol.js");\nvar ERROR_STACK_INSTALLABLE = __webpack_require__(/*! ../internals/error-stack-installable */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/error-stack-installable.js");\n\nvar TO_STRING_TAG = wellKnownSymbol(\'toStringTag\');\nvar $Error = Error;\nvar push = [].push;\n\nvar $AggregateError = function AggregateError(errors, message /* , options */) {\n  var options = arguments.length > 2 ? arguments[2] : undefined;\n  var isInstance = isPrototypeOf(AggregateErrorPrototype, this);\n  var that;\n  if (setPrototypeOf) {\n    that = setPrototypeOf(new $Error(), isInstance ? getPrototypeOf(this) : AggregateErrorPrototype);\n  } else {\n    that = isInstance ? this : create(AggregateErrorPrototype);\n    createNonEnumerableProperty(that, TO_STRING_TAG, \'Error\');\n  }\n  if (message !== undefined) createNonEnumerableProperty(that, \'message\', normalizeStringArgument(message));\n  if (ERROR_STACK_INSTALLABLE) createNonEnumerableProperty(that, \'stack\', clearErrorStack(that.stack, 1));\n  installErrorCause(that, options);\n  var errorsArray = [];\n  iterate(errors, push, { that: errorsArray });\n  createNonEnumerableProperty(that, \'errors\', errorsArray);\n  return that;\n};\n\nif (setPrototypeOf) setPrototypeOf($AggregateError, $Error);\nelse copyConstructorProperties($AggregateError, $Error, { name: true });\n\nvar AggregateErrorPrototype = $AggregateError.prototype = create($Error.prototype, {\n  constructor: createPropertyDescriptor(1, $AggregateError),\n  message: createPropertyDescriptor(1, \'\'),\n  name: createPropertyDescriptor(1, \'AggregateError\')\n});\n\n// `AggregateError` constructor\n// https://tc39.es/ecma262/#sec-aggregate-error-constructor\n$({ global: true, constructor: true, arity: 2 }, {\n  AggregateError: $AggregateError\n});\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.aggregate-error.constructor.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.aggregate-error.js":function(__unused_webpack_module,__unused_webpack_exports,__webpack_require__){eval('// TODO: Remove this module from `core-js@4` since it\'s replaced to module below\n__webpack_require__(/*! ../modules/es.aggregate-error.constructor */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.aggregate-error.constructor.js");\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.aggregate-error.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.array.iterator.js":function(module,__unused_webpack_exports,__webpack_require__){"use strict";eval("\nvar toIndexedObject = __webpack_require__(/*! ../internals/to-indexed-object */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-indexed-object.js\");\nvar addToUnscopables = __webpack_require__(/*! ../internals/add-to-unscopables */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/add-to-unscopables.js\");\nvar Iterators = __webpack_require__(/*! ../internals/iterators */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/iterators.js\");\nvar InternalStateModule = __webpack_require__(/*! ../internals/internal-state */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/internal-state.js\");\nvar defineProperty = (__webpack_require__(/*! ../internals/object-define-property */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-define-property.js\").f);\nvar defineIterator = __webpack_require__(/*! ../internals/define-iterator */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/define-iterator.js\");\nvar IS_PURE = __webpack_require__(/*! ../internals/is-pure */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-pure.js\");\nvar DESCRIPTORS = __webpack_require__(/*! ../internals/descriptors */ \"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/descriptors.js\");\n\nvar ARRAY_ITERATOR = 'Array Iterator';\nvar setInternalState = InternalStateModule.set;\nvar getInternalState = InternalStateModule.getterFor(ARRAY_ITERATOR);\n\n// `Array.prototype.entries` method\n// https://tc39.es/ecma262/#sec-array.prototype.entries\n// `Array.prototype.keys` method\n// https://tc39.es/ecma262/#sec-array.prototype.keys\n// `Array.prototype.values` method\n// https://tc39.es/ecma262/#sec-array.prototype.values\n// `Array.prototype[@@iterator]` method\n// https://tc39.es/ecma262/#sec-array.prototype-@@iterator\n// `CreateArrayIterator` internal method\n// https://tc39.es/ecma262/#sec-createarrayiterator\nmodule.exports = defineIterator(Array, 'Array', function (iterated, kind) {\n  setInternalState(this, {\n    type: ARRAY_ITERATOR,\n    target: toIndexedObject(iterated), // target\n    index: 0,                          // next index\n    kind: kind                         // kind\n  });\n// `%ArrayIteratorPrototype%.next` method\n// https://tc39.es/ecma262/#sec-%arrayiteratorprototype%.next\n}, function () {\n  var state = getInternalState(this);\n  var target = state.target;\n  var kind = state.kind;\n  var index = state.index++;\n  if (!target || index >= target.length) {\n    state.target = undefined;\n    return { value: undefined, done: true };\n  }\n  if (kind == 'keys') return { value: index, done: false };\n  if (kind == 'values') return { value: target[index], done: false };\n  return { value: [index, target[index]], done: false };\n}, 'values');\n\n// argumentsList[@@iterator] is %ArrayProto_values%\n// https://tc39.es/ecma262/#sec-createunmappedargumentsobject\n// https://tc39.es/ecma262/#sec-createmappedargumentsobject\nvar values = Iterators.Arguments = Iterators.Array;\n\n// https://tc39.es/ecma262/#sec-array.prototype-@@unscopables\naddToUnscopables('keys');\naddToUnscopables('values');\naddToUnscopables('entries');\n\n// V8 ~ Chrome 45- bug\nif (!IS_PURE && DESCRIPTORS && values.name !== 'values') try {\n  defineProperty(values, 'name', { value: 'values' });\n} catch (error) { /* empty */ }\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.array.iterator.js?")},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.object.to-string.js":function(__unused_webpack_module,__unused_webpack_exports,__webpack_require__){eval('var TO_STRING_TAG_SUPPORT = __webpack_require__(/*! ../internals/to-string-tag-support */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-string-tag-support.js");\nvar defineBuiltIn = __webpack_require__(/*! ../internals/define-built-in */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/define-built-in.js");\nvar toString = __webpack_require__(/*! ../internals/object-to-string */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-to-string.js");\n\n// `Object.prototype.toString` method\n// https://tc39.es/ecma262/#sec-object.prototype.tostring\nif (!TO_STRING_TAG_SUPPORT) {\n  defineBuiltIn(Object.prototype, \'toString\', toString, { unsafe: true });\n}\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.object.to-string.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.all-settled.js":function(__unused_webpack_module,__unused_webpack_exports,__webpack_require__){"use strict";eval('\nvar $ = __webpack_require__(/*! ../internals/export */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/export.js");\nvar call = __webpack_require__(/*! ../internals/function-call */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-call.js");\nvar aCallable = __webpack_require__(/*! ../internals/a-callable */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/a-callable.js");\nvar newPromiseCapabilityModule = __webpack_require__(/*! ../internals/new-promise-capability */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/new-promise-capability.js");\nvar perform = __webpack_require__(/*! ../internals/perform */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/perform.js");\nvar iterate = __webpack_require__(/*! ../internals/iterate */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/iterate.js");\n\n// `Promise.allSettled` method\n// https://tc39.es/ecma262/#sec-promise.allsettled\n$({ target: \'Promise\', stat: true }, {\n  allSettled: function allSettled(iterable) {\n    var C = this;\n    var capability = newPromiseCapabilityModule.f(C);\n    var resolve = capability.resolve;\n    var reject = capability.reject;\n    var result = perform(function () {\n      var promiseResolve = aCallable(C.resolve);\n      var values = [];\n      var counter = 0;\n      var remaining = 1;\n      iterate(iterable, function (promise) {\n        var index = counter++;\n        var alreadyCalled = false;\n        remaining++;\n        call(promiseResolve, C, promise).then(function (value) {\n          if (alreadyCalled) return;\n          alreadyCalled = true;\n          values[index] = { status: \'fulfilled\', value: value };\n          --remaining || resolve(values);\n        }, function (error) {\n          if (alreadyCalled) return;\n          alreadyCalled = true;\n          values[index] = { status: \'rejected\', reason: error };\n          --remaining || resolve(values);\n        });\n      });\n      --remaining || resolve(values);\n    });\n    if (result.error) reject(result.value);\n    return capability.promise;\n  }\n});\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.all-settled.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.all.js":function(__unused_webpack_module,__unused_webpack_exports,__webpack_require__){"use strict";eval('\nvar $ = __webpack_require__(/*! ../internals/export */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/export.js");\nvar call = __webpack_require__(/*! ../internals/function-call */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-call.js");\nvar aCallable = __webpack_require__(/*! ../internals/a-callable */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/a-callable.js");\nvar newPromiseCapabilityModule = __webpack_require__(/*! ../internals/new-promise-capability */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/new-promise-capability.js");\nvar perform = __webpack_require__(/*! ../internals/perform */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/perform.js");\nvar iterate = __webpack_require__(/*! ../internals/iterate */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/iterate.js");\nvar PROMISE_STATICS_INCORRECT_ITERATION = __webpack_require__(/*! ../internals/promise-statics-incorrect-iteration */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/promise-statics-incorrect-iteration.js");\n\n// `Promise.all` method\n// https://tc39.es/ecma262/#sec-promise.all\n$({ target: \'Promise\', stat: true, forced: PROMISE_STATICS_INCORRECT_ITERATION }, {\n  all: function all(iterable) {\n    var C = this;\n    var capability = newPromiseCapabilityModule.f(C);\n    var resolve = capability.resolve;\n    var reject = capability.reject;\n    var result = perform(function () {\n      var $promiseResolve = aCallable(C.resolve);\n      var values = [];\n      var counter = 0;\n      var remaining = 1;\n      iterate(iterable, function (promise) {\n        var index = counter++;\n        var alreadyCalled = false;\n        remaining++;\n        call($promiseResolve, C, promise).then(function (value) {\n          if (alreadyCalled) return;\n          alreadyCalled = true;\n          values[index] = value;\n          --remaining || resolve(values);\n        }, reject);\n      });\n      --remaining || resolve(values);\n    });\n    if (result.error) reject(result.value);\n    return capability.promise;\n  }\n});\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.all.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.any.js":function(__unused_webpack_module,__unused_webpack_exports,__webpack_require__){"use strict";eval('\nvar $ = __webpack_require__(/*! ../internals/export */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/export.js");\nvar call = __webpack_require__(/*! ../internals/function-call */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-call.js");\nvar aCallable = __webpack_require__(/*! ../internals/a-callable */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/a-callable.js");\nvar getBuiltIn = __webpack_require__(/*! ../internals/get-built-in */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/get-built-in.js");\nvar newPromiseCapabilityModule = __webpack_require__(/*! ../internals/new-promise-capability */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/new-promise-capability.js");\nvar perform = __webpack_require__(/*! ../internals/perform */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/perform.js");\nvar iterate = __webpack_require__(/*! ../internals/iterate */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/iterate.js");\n\nvar PROMISE_ANY_ERROR = \'No one promise resolved\';\n\n// `Promise.any` method\n// https://tc39.es/ecma262/#sec-promise.any\n$({ target: \'Promise\', stat: true }, {\n  any: function any(iterable) {\n    var C = this;\n    var AggregateError = getBuiltIn(\'AggregateError\');\n    var capability = newPromiseCapabilityModule.f(C);\n    var resolve = capability.resolve;\n    var reject = capability.reject;\n    var result = perform(function () {\n      var promiseResolve = aCallable(C.resolve);\n      var errors = [];\n      var counter = 0;\n      var remaining = 1;\n      var alreadyResolved = false;\n      iterate(iterable, function (promise) {\n        var index = counter++;\n        var alreadyRejected = false;\n        remaining++;\n        call(promiseResolve, C, promise).then(function (value) {\n          if (alreadyRejected || alreadyResolved) return;\n          alreadyResolved = true;\n          resolve(value);\n        }, function (error) {\n          if (alreadyRejected || alreadyResolved) return;\n          alreadyRejected = true;\n          errors[index] = error;\n          --remaining || reject(new AggregateError(errors, PROMISE_ANY_ERROR));\n        });\n      });\n      --remaining || reject(new AggregateError(errors, PROMISE_ANY_ERROR));\n    });\n    if (result.error) reject(result.value);\n    return capability.promise;\n  }\n});\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.any.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.catch.js":function(__unused_webpack_module,__unused_webpack_exports,__webpack_require__){"use strict";eval('\nvar $ = __webpack_require__(/*! ../internals/export */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/export.js");\nvar IS_PURE = __webpack_require__(/*! ../internals/is-pure */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-pure.js");\nvar FORCED_PROMISE_CONSTRUCTOR = (__webpack_require__(/*! ../internals/promise-constructor-detection */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/promise-constructor-detection.js").CONSTRUCTOR);\nvar NativePromiseConstructor = __webpack_require__(/*! ../internals/promise-native-constructor */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/promise-native-constructor.js");\nvar getBuiltIn = __webpack_require__(/*! ../internals/get-built-in */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/get-built-in.js");\nvar isCallable = __webpack_require__(/*! ../internals/is-callable */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-callable.js");\nvar defineBuiltIn = __webpack_require__(/*! ../internals/define-built-in */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/define-built-in.js");\n\nvar NativePromisePrototype = NativePromiseConstructor && NativePromiseConstructor.prototype;\n\n// `Promise.prototype.catch` method\n// https://tc39.es/ecma262/#sec-promise.prototype.catch\n$({ target: \'Promise\', proto: true, forced: FORCED_PROMISE_CONSTRUCTOR, real: true }, {\n  \'catch\': function (onRejected) {\n    return this.then(undefined, onRejected);\n  }\n});\n\n// makes sure that native promise-based APIs `Promise#catch` properly works with patched `Promise#then`\nif (!IS_PURE && isCallable(NativePromiseConstructor)) {\n  var method = getBuiltIn(\'Promise\').prototype[\'catch\'];\n  if (NativePromisePrototype[\'catch\'] !== method) {\n    defineBuiltIn(NativePromisePrototype, \'catch\', method, { unsafe: true });\n  }\n}\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.catch.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.constructor.js":function(__unused_webpack_module,__unused_webpack_exports,__webpack_require__){"use strict";eval('\nvar $ = __webpack_require__(/*! ../internals/export */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/export.js");\nvar IS_PURE = __webpack_require__(/*! ../internals/is-pure */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-pure.js");\nvar IS_NODE = __webpack_require__(/*! ../internals/engine-is-node */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/engine-is-node.js");\nvar global = __webpack_require__(/*! ../internals/global */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/global.js");\nvar call = __webpack_require__(/*! ../internals/function-call */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-call.js");\nvar defineBuiltIn = __webpack_require__(/*! ../internals/define-built-in */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/define-built-in.js");\nvar setPrototypeOf = __webpack_require__(/*! ../internals/object-set-prototype-of */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/object-set-prototype-of.js");\nvar setToStringTag = __webpack_require__(/*! ../internals/set-to-string-tag */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/set-to-string-tag.js");\nvar setSpecies = __webpack_require__(/*! ../internals/set-species */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/set-species.js");\nvar aCallable = __webpack_require__(/*! ../internals/a-callable */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/a-callable.js");\nvar isCallable = __webpack_require__(/*! ../internals/is-callable */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-callable.js");\nvar isObject = __webpack_require__(/*! ../internals/is-object */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-object.js");\nvar anInstance = __webpack_require__(/*! ../internals/an-instance */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/an-instance.js");\nvar speciesConstructor = __webpack_require__(/*! ../internals/species-constructor */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/species-constructor.js");\nvar task = (__webpack_require__(/*! ../internals/task */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/task.js").set);\nvar microtask = __webpack_require__(/*! ../internals/microtask */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/microtask.js");\nvar hostReportErrors = __webpack_require__(/*! ../internals/host-report-errors */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/host-report-errors.js");\nvar perform = __webpack_require__(/*! ../internals/perform */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/perform.js");\nvar Queue = __webpack_require__(/*! ../internals/queue */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/queue.js");\nvar InternalStateModule = __webpack_require__(/*! ../internals/internal-state */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/internal-state.js");\nvar NativePromiseConstructor = __webpack_require__(/*! ../internals/promise-native-constructor */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/promise-native-constructor.js");\nvar PromiseConstructorDetection = __webpack_require__(/*! ../internals/promise-constructor-detection */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/promise-constructor-detection.js");\nvar newPromiseCapabilityModule = __webpack_require__(/*! ../internals/new-promise-capability */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/new-promise-capability.js");\n\nvar PROMISE = \'Promise\';\nvar FORCED_PROMISE_CONSTRUCTOR = PromiseConstructorDetection.CONSTRUCTOR;\nvar NATIVE_PROMISE_REJECTION_EVENT = PromiseConstructorDetection.REJECTION_EVENT;\nvar NATIVE_PROMISE_SUBCLASSING = PromiseConstructorDetection.SUBCLASSING;\nvar getInternalPromiseState = InternalStateModule.getterFor(PROMISE);\nvar setInternalState = InternalStateModule.set;\nvar NativePromisePrototype = NativePromiseConstructor && NativePromiseConstructor.prototype;\nvar PromiseConstructor = NativePromiseConstructor;\nvar PromisePrototype = NativePromisePrototype;\nvar TypeError = global.TypeError;\nvar document = global.document;\nvar process = global.process;\nvar newPromiseCapability = newPromiseCapabilityModule.f;\nvar newGenericPromiseCapability = newPromiseCapability;\n\nvar DISPATCH_EVENT = !!(document && document.createEvent && global.dispatchEvent);\nvar UNHANDLED_REJECTION = \'unhandledrejection\';\nvar REJECTION_HANDLED = \'rejectionhandled\';\nvar PENDING = 0;\nvar FULFILLED = 1;\nvar REJECTED = 2;\nvar HANDLED = 1;\nvar UNHANDLED = 2;\n\nvar Internal, OwnPromiseCapability, PromiseWrapper, nativeThen;\n\n// helpers\nvar isThenable = function (it) {\n  var then;\n  return isObject(it) && isCallable(then = it.then) ? then : false;\n};\n\nvar callReaction = function (reaction, state) {\n  var value = state.value;\n  var ok = state.state == FULFILLED;\n  var handler = ok ? reaction.ok : reaction.fail;\n  var resolve = reaction.resolve;\n  var reject = reaction.reject;\n  var domain = reaction.domain;\n  var result, then, exited;\n  try {\n    if (handler) {\n      if (!ok) {\n        if (state.rejection === UNHANDLED) onHandleUnhandled(state);\n        state.rejection = HANDLED;\n      }\n      if (handler === true) result = value;\n      else {\n        if (domain) domain.enter();\n        result = handler(value); // can throw\n        if (domain) {\n          domain.exit();\n          exited = true;\n        }\n      }\n      if (result === reaction.promise) {\n        reject(TypeError(\'Promise-chain cycle\'));\n      } else if (then = isThenable(result)) {\n        call(then, result, resolve, reject);\n      } else resolve(result);\n    } else reject(value);\n  } catch (error) {\n    if (domain && !exited) domain.exit();\n    reject(error);\n  }\n};\n\nvar notify = function (state, isReject) {\n  if (state.notified) return;\n  state.notified = true;\n  microtask(function () {\n    var reactions = state.reactions;\n    var reaction;\n    while (reaction = reactions.get()) {\n      callReaction(reaction, state);\n    }\n    state.notified = false;\n    if (isReject && !state.rejection) onUnhandled(state);\n  });\n};\n\nvar dispatchEvent = function (name, promise, reason) {\n  var event, handler;\n  if (DISPATCH_EVENT) {\n    event = document.createEvent(\'Event\');\n    event.promise = promise;\n    event.reason = reason;\n    event.initEvent(name, false, true);\n    global.dispatchEvent(event);\n  } else event = { promise: promise, reason: reason };\n  if (!NATIVE_PROMISE_REJECTION_EVENT && (handler = global[\'on\' + name])) handler(event);\n  else if (name === UNHANDLED_REJECTION) hostReportErrors(\'Unhandled promise rejection\', reason);\n};\n\nvar onUnhandled = function (state) {\n  call(task, global, function () {\n    var promise = state.facade;\n    var value = state.value;\n    var IS_UNHANDLED = isUnhandled(state);\n    var result;\n    if (IS_UNHANDLED) {\n      result = perform(function () {\n        if (IS_NODE) {\n          process.emit(\'unhandledRejection\', value, promise);\n        } else dispatchEvent(UNHANDLED_REJECTION, promise, value);\n      });\n      // Browsers should not trigger `rejectionHandled` event if it was handled here, NodeJS - should\n      state.rejection = IS_NODE || isUnhandled(state) ? UNHANDLED : HANDLED;\n      if (result.error) throw result.value;\n    }\n  });\n};\n\nvar isUnhandled = function (state) {\n  return state.rejection !== HANDLED && !state.parent;\n};\n\nvar onHandleUnhandled = function (state) {\n  call(task, global, function () {\n    var promise = state.facade;\n    if (IS_NODE) {\n      process.emit(\'rejectionHandled\', promise);\n    } else dispatchEvent(REJECTION_HANDLED, promise, state.value);\n  });\n};\n\nvar bind = function (fn, state, unwrap) {\n  return function (value) {\n    fn(state, value, unwrap);\n  };\n};\n\nvar internalReject = function (state, value, unwrap) {\n  if (state.done) return;\n  state.done = true;\n  if (unwrap) state = unwrap;\n  state.value = value;\n  state.state = REJECTED;\n  notify(state, true);\n};\n\nvar internalResolve = function (state, value, unwrap) {\n  if (state.done) return;\n  state.done = true;\n  if (unwrap) state = unwrap;\n  try {\n    if (state.facade === value) throw TypeError("Promise can\'t be resolved itself");\n    var then = isThenable(value);\n    if (then) {\n      microtask(function () {\n        var wrapper = { done: false };\n        try {\n          call(then, value,\n            bind(internalResolve, wrapper, state),\n            bind(internalReject, wrapper, state)\n          );\n        } catch (error) {\n          internalReject(wrapper, error, state);\n        }\n      });\n    } else {\n      state.value = value;\n      state.state = FULFILLED;\n      notify(state, false);\n    }\n  } catch (error) {\n    internalReject({ done: false }, error, state);\n  }\n};\n\n// constructor polyfill\nif (FORCED_PROMISE_CONSTRUCTOR) {\n  // 25.4.3.1 Promise(executor)\n  PromiseConstructor = function Promise(executor) {\n    anInstance(this, PromisePrototype);\n    aCallable(executor);\n    call(Internal, this);\n    var state = getInternalPromiseState(this);\n    try {\n      executor(bind(internalResolve, state), bind(internalReject, state));\n    } catch (error) {\n      internalReject(state, error);\n    }\n  };\n\n  PromisePrototype = PromiseConstructor.prototype;\n\n  // eslint-disable-next-line no-unused-vars -- required for `.length`\n  Internal = function Promise(executor) {\n    setInternalState(this, {\n      type: PROMISE,\n      done: false,\n      notified: false,\n      parent: false,\n      reactions: new Queue(),\n      rejection: false,\n      state: PENDING,\n      value: undefined\n    });\n  };\n\n  // `Promise.prototype.then` method\n  // https://tc39.es/ecma262/#sec-promise.prototype.then\n  Internal.prototype = defineBuiltIn(PromisePrototype, \'then\', function then(onFulfilled, onRejected) {\n    var state = getInternalPromiseState(this);\n    var reaction = newPromiseCapability(speciesConstructor(this, PromiseConstructor));\n    state.parent = true;\n    reaction.ok = isCallable(onFulfilled) ? onFulfilled : true;\n    reaction.fail = isCallable(onRejected) && onRejected;\n    reaction.domain = IS_NODE ? process.domain : undefined;\n    if (state.state == PENDING) state.reactions.add(reaction);\n    else microtask(function () {\n      callReaction(reaction, state);\n    });\n    return reaction.promise;\n  });\n\n  OwnPromiseCapability = function () {\n    var promise = new Internal();\n    var state = getInternalPromiseState(promise);\n    this.promise = promise;\n    this.resolve = bind(internalResolve, state);\n    this.reject = bind(internalReject, state);\n  };\n\n  newPromiseCapabilityModule.f = newPromiseCapability = function (C) {\n    return C === PromiseConstructor || C === PromiseWrapper\n      ? new OwnPromiseCapability(C)\n      : newGenericPromiseCapability(C);\n  };\n\n  if (!IS_PURE && isCallable(NativePromiseConstructor) && NativePromisePrototype !== Object.prototype) {\n    nativeThen = NativePromisePrototype.then;\n\n    if (!NATIVE_PROMISE_SUBCLASSING) {\n      // make `Promise#then` return a polyfilled `Promise` for native promise-based APIs\n      defineBuiltIn(NativePromisePrototype, \'then\', function then(onFulfilled, onRejected) {\n        var that = this;\n        return new PromiseConstructor(function (resolve, reject) {\n          call(nativeThen, that, resolve, reject);\n        }).then(onFulfilled, onRejected);\n      // https://github.com/zloirock/core-js/issues/640\n      }, { unsafe: true });\n    }\n\n    // make `.constructor === Promise` work for native promise-based APIs\n    try {\n      delete NativePromisePrototype.constructor;\n    } catch (error) { /* empty */ }\n\n    // make `instanceof Promise` work for native promise-based APIs\n    if (setPrototypeOf) {\n      setPrototypeOf(NativePromisePrototype, PromisePrototype);\n    }\n  }\n}\n\n$({ global: true, constructor: true, wrap: true, forced: FORCED_PROMISE_CONSTRUCTOR }, {\n  Promise: PromiseConstructor\n});\n\nsetToStringTag(PromiseConstructor, PROMISE, false, true);\nsetSpecies(PROMISE);\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.constructor.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.finally.js":function(__unused_webpack_module,__unused_webpack_exports,__webpack_require__){"use strict";eval('\nvar $ = __webpack_require__(/*! ../internals/export */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/export.js");\nvar IS_PURE = __webpack_require__(/*! ../internals/is-pure */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-pure.js");\nvar NativePromiseConstructor = __webpack_require__(/*! ../internals/promise-native-constructor */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/promise-native-constructor.js");\nvar fails = __webpack_require__(/*! ../internals/fails */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/fails.js");\nvar getBuiltIn = __webpack_require__(/*! ../internals/get-built-in */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/get-built-in.js");\nvar isCallable = __webpack_require__(/*! ../internals/is-callable */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-callable.js");\nvar speciesConstructor = __webpack_require__(/*! ../internals/species-constructor */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/species-constructor.js");\nvar promiseResolve = __webpack_require__(/*! ../internals/promise-resolve */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/promise-resolve.js");\nvar defineBuiltIn = __webpack_require__(/*! ../internals/define-built-in */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/define-built-in.js");\n\nvar NativePromisePrototype = NativePromiseConstructor && NativePromiseConstructor.prototype;\n\n// Safari bug https://bugs.webkit.org/show_bug.cgi?id=200829\nvar NON_GENERIC = !!NativePromiseConstructor && fails(function () {\n  // eslint-disable-next-line unicorn/no-thenable -- required for testing\n  NativePromisePrototype[\'finally\'].call({ then: function () { /* empty */ } }, function () { /* empty */ });\n});\n\n// `Promise.prototype.finally` method\n// https://tc39.es/ecma262/#sec-promise.prototype.finally\n$({ target: \'Promise\', proto: true, real: true, forced: NON_GENERIC }, {\n  \'finally\': function (onFinally) {\n    var C = speciesConstructor(this, getBuiltIn(\'Promise\'));\n    var isFunction = isCallable(onFinally);\n    return this.then(\n      isFunction ? function (x) {\n        return promiseResolve(C, onFinally()).then(function () { return x; });\n      } : onFinally,\n      isFunction ? function (e) {\n        return promiseResolve(C, onFinally()).then(function () { throw e; });\n      } : onFinally\n    );\n  }\n});\n\n// makes sure that native promise-based APIs `Promise#finally` properly works with patched `Promise#then`\nif (!IS_PURE && isCallable(NativePromiseConstructor)) {\n  var method = getBuiltIn(\'Promise\').prototype[\'finally\'];\n  if (NativePromisePrototype[\'finally\'] !== method) {\n    defineBuiltIn(NativePromisePrototype, \'finally\', method, { unsafe: true });\n  }\n}\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.finally.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.js":function(__unused_webpack_module,__unused_webpack_exports,__webpack_require__){eval('// TODO: Remove this module from `core-js@4` since it\'s split to modules listed below\n__webpack_require__(/*! ../modules/es.promise.constructor */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.constructor.js");\n__webpack_require__(/*! ../modules/es.promise.all */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.all.js");\n__webpack_require__(/*! ../modules/es.promise.catch */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.catch.js");\n__webpack_require__(/*! ../modules/es.promise.race */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.race.js");\n__webpack_require__(/*! ../modules/es.promise.reject */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.reject.js");\n__webpack_require__(/*! ../modules/es.promise.resolve */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.resolve.js");\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.race.js":function(__unused_webpack_module,__unused_webpack_exports,__webpack_require__){"use strict";eval('\nvar $ = __webpack_require__(/*! ../internals/export */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/export.js");\nvar call = __webpack_require__(/*! ../internals/function-call */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-call.js");\nvar aCallable = __webpack_require__(/*! ../internals/a-callable */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/a-callable.js");\nvar newPromiseCapabilityModule = __webpack_require__(/*! ../internals/new-promise-capability */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/new-promise-capability.js");\nvar perform = __webpack_require__(/*! ../internals/perform */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/perform.js");\nvar iterate = __webpack_require__(/*! ../internals/iterate */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/iterate.js");\nvar PROMISE_STATICS_INCORRECT_ITERATION = __webpack_require__(/*! ../internals/promise-statics-incorrect-iteration */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/promise-statics-incorrect-iteration.js");\n\n// `Promise.race` method\n// https://tc39.es/ecma262/#sec-promise.race\n$({ target: \'Promise\', stat: true, forced: PROMISE_STATICS_INCORRECT_ITERATION }, {\n  race: function race(iterable) {\n    var C = this;\n    var capability = newPromiseCapabilityModule.f(C);\n    var reject = capability.reject;\n    var result = perform(function () {\n      var $promiseResolve = aCallable(C.resolve);\n      iterate(iterable, function (promise) {\n        call($promiseResolve, C, promise).then(capability.resolve, reject);\n      });\n    });\n    if (result.error) reject(result.value);\n    return capability.promise;\n  }\n});\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.race.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.reject.js":function(__unused_webpack_module,__unused_webpack_exports,__webpack_require__){"use strict";eval('\nvar $ = __webpack_require__(/*! ../internals/export */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/export.js");\nvar call = __webpack_require__(/*! ../internals/function-call */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/function-call.js");\nvar newPromiseCapabilityModule = __webpack_require__(/*! ../internals/new-promise-capability */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/new-promise-capability.js");\nvar FORCED_PROMISE_CONSTRUCTOR = (__webpack_require__(/*! ../internals/promise-constructor-detection */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/promise-constructor-detection.js").CONSTRUCTOR);\n\n// `Promise.reject` method\n// https://tc39.es/ecma262/#sec-promise.reject\n$({ target: \'Promise\', stat: true, forced: FORCED_PROMISE_CONSTRUCTOR }, {\n  reject: function reject(r) {\n    var capability = newPromiseCapabilityModule.f(this);\n    call(capability.reject, undefined, r);\n    return capability.promise;\n  }\n});\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.reject.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.resolve.js":function(__unused_webpack_module,__unused_webpack_exports,__webpack_require__){"use strict";eval('\nvar $ = __webpack_require__(/*! ../internals/export */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/export.js");\nvar getBuiltIn = __webpack_require__(/*! ../internals/get-built-in */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/get-built-in.js");\nvar IS_PURE = __webpack_require__(/*! ../internals/is-pure */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/is-pure.js");\nvar NativePromiseConstructor = __webpack_require__(/*! ../internals/promise-native-constructor */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/promise-native-constructor.js");\nvar FORCED_PROMISE_CONSTRUCTOR = (__webpack_require__(/*! ../internals/promise-constructor-detection */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/promise-constructor-detection.js").CONSTRUCTOR);\nvar promiseResolve = __webpack_require__(/*! ../internals/promise-resolve */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/promise-resolve.js");\n\nvar PromiseConstructorWrapper = getBuiltIn(\'Promise\');\nvar CHECK_WRAPPER = IS_PURE && !FORCED_PROMISE_CONSTRUCTOR;\n\n// `Promise.resolve` method\n// https://tc39.es/ecma262/#sec-promise.resolve\n$({ target: \'Promise\', stat: true, forced: IS_PURE || FORCED_PROMISE_CONSTRUCTOR }, {\n  resolve: function resolve(x) {\n    return promiseResolve(CHECK_WRAPPER && this === PromiseConstructorWrapper ? NativePromiseConstructor : this, x);\n  }\n});\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.promise.resolve.js?')},"./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.string.iterator.js":function(__unused_webpack_module,__unused_webpack_exports,__webpack_require__){"use strict";eval('\nvar charAt = (__webpack_require__(/*! ../internals/string-multibyte */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/string-multibyte.js").charAt);\nvar toString = __webpack_require__(/*! ../internals/to-string */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/to-string.js");\nvar InternalStateModule = __webpack_require__(/*! ../internals/internal-state */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/internal-state.js");\nvar defineIterator = __webpack_require__(/*! ../internals/define-iterator */ "./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/internals/define-iterator.js");\n\nvar STRING_ITERATOR = \'String Iterator\';\nvar setInternalState = InternalStateModule.set;\nvar getInternalState = InternalStateModule.getterFor(STRING_ITERATOR);\n\n// `String.prototype[@@iterator]` method\n// https://tc39.es/ecma262/#sec-string.prototype-@@iterator\ndefineIterator(String, \'String\', function (iterated) {\n  setInternalState(this, {\n    type: STRING_ITERATOR,\n    string: toString(iterated),\n    index: 0\n  });\n// `%StringIteratorPrototype%.next` method\n// https://tc39.es/ecma262/#sec-%stringiteratorprototype%.next\n}, function next() {\n  var state = getInternalState(this);\n  var string = state.string;\n  var index = state.index;\n  var point;\n  if (index >= string.length) return { value: undefined, done: true };\n  point = charAt(string, index);\n  state.index += point.length;\n  return { value: point, done: false };\n});\n\n\n//# sourceURL=webpack://gd-sprest/./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/modules/es.string.iterator.js?')}},__webpack_module_cache__={};function __webpack_require__(e){var n=__webpack_module_cache__[e];if(void 0!==n)return n.exports;var t=__webpack_module_cache__[e]={exports:{}};return __webpack_modules__[e].call(t.exports,t,t.exports,__webpack_require__),t.exports}__webpack_require__.g=function(){if("object"==typeof globalThis)return globalThis;try{return this||new Function("return this")()}catch(e){if("object"==typeof window)return window}}(),__webpack_require__("./node_modules/.pnpm/core-js@3.24.1/node_modules/core-js/es/promise/index.js");var __webpack_exports__=__webpack_require__("./build/index.js")})();